Package: Rdistance
Type: Package
Title: Density and Abundance from Distance-Sampling Surveys
Version: 4.4.3
Date: 2026-05-13
Language: en-US
Maintainer: Trent McDonald <trent@mcdonalddatasciences.com>
Authors@R: c(person("Trent", "McDonald", role=c("cre","aut"), email="trent@mcdonalddatasciences.com"), 
  person("Jason", "Carlisle", role="aut", email="jason.carlisle@wyo.gov"), 
  person("Aidan", "McDonald", role="aut", email="aidan@mcdcentral.org", comment="point transect methods"), 
  person("Ryan", "Nielson", role="ctb", comment="smoothed likelihood"), 
  person("Ben", "Augustine", role="ctb", comment="maximization method"), 
  person("James", "Griswald", role="ctb", comment="maximization method"), 
  person("Patrick", "McKann", role="ctb", comment="maximization method"), 
  person("Lacey", "Jeroue", role="ctb", comment="vignettes"), 
  person("Hoffman", "Abigail", role="ctb", comment="vignettes"), 
  person("Kleinsausser", "Michael", role="ctb", comment="vignettes"),
  person("Joel", "Reynolds", role="ctb", comment="Gamma likelihood"), 
  person("Pham", "Quang", role="ctb", comment="Gamma likelihood"), 
  person("Earl", "Becker", role="ctb", comment="Gamma likelihood"), 
  person("Aaron", "Christ", role="ctb", comment="Gamma likelihood"), 
  person("Brook", "Russelland", role="ctb", comment="Gamma likelihood"), 
  person("Stefan", "Emmons", role="ctb", comment="Automated tests"),
  person("Will", "McDonald", role="ctb", comment="Automated tests"),
  person("Reid", "Olson", role="ctb", comment="Automated tests and bug fixes"))
Description: Distance-sampling (<doi:10.1007/978-3-319-19219-2>) 
  is a field survey and analytical method that estimates density and 
  abundance of survey targets (e.g., animals) when 
  detection probability declines with observation distance. 
  Distance-sampling is popular in ecology, 
  especially when survey targets are observed from aerial platforms (e.g., 
  airplane or drone), surface vessels (e.g., boat or truck), or along 
  walking transects. Analysis involves fitting smooth (parametric) curves to 
  histograms of observation distances and using those functions to 
  adjust density estimates for missed targets.  Routines included here 
  fit curves to observation distance histograms, estimate effective 
  sampling area, density of targets in surveyed areas, and the abundance 
  of targets in a surrounding study area. Confidence interval estimation
  uses built-in bootstrap resampling. Help files are extensive and have been 
  vetted by multiple authors. Many tutorials are available on the package's 
  website (URL below).
License: GNU General Public License
URL: https://mcdonalddatasciences.com/Rdistance.html
BugReports: https://github.com/tmcd82070/Rdistance/issues
LazyData: TRUE
Suggests: testthat (>= 3.0.0),
Depends: R (>= 4.1.0), units
Imports: graphics, stats, utils, crayon, withr, tidyr, dplyr, progress,
        tibble, tidyselect, dfoptim, expint, multidplyr
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-13 23:26:52 UTC; trent
Author: Trent McDonald [cre, aut],
  Jason Carlisle [aut],
  Aidan McDonald [aut] (point transect methods),
  Ryan Nielson [ctb] (smoothed likelihood),
  Ben Augustine [ctb] (maximization method),
  James Griswald [ctb] (maximization method),
  Patrick McKann [ctb] (maximization method),
  Lacey Jeroue [ctb] (vignettes),
  Hoffman Abigail [ctb] (vignettes),
  Kleinsausser Michael [ctb] (vignettes),
  Joel Reynolds [ctb] (Gamma likelihood),
  Pham Quang [ctb] (Gamma likelihood),
  Earl Becker [ctb] (Gamma likelihood),
  Aaron Christ [ctb] (Gamma likelihood),
  Brook Russelland [ctb] (Gamma likelihood),
  Stefan Emmons [ctb] (Automated tests),
  Will McDonald [ctb] (Automated tests),
  Reid Olson [ctb] (Automated tests and bug fixes)
Repository: CRAN
Date/Publication: 2026-05-14 13:50:18 UTC
Built: R 4.5.2; ; 2026-05-14 14:22:19 UTC; unix

Package: Rdistance
Type: Package
Title: Density and Abundance from Distance-Sampling Surveys
Version: 4.4.3
Date: 2026-05-13
Language: en-US
Maintainer: Trent McDonald <trent@mcdonalddatasciences.com>
Authors@R: c(person("Trent", "McDonald", role=c("cre","aut"), email="trent@mcdonalddatasciences.com"), 
  person("Jason", "Carlisle", role="aut", email="jason.carlisle@wyo.gov"), 
  person("Aidan", "McDonald", role="aut", email="aidan@mcdcentral.org", comment="point transect methods"), 
  person("Ryan", "Nielson", role="ctb", comment="smoothed likelihood"), 
  person("Ben", "Augustine", role="ctb", comment="maximization method"), 
  person("James", "Griswald", role="ctb", comment="maximization method"), 
  person("Patrick", "McKann", role="ctb", comment="maximization method"), 
  person("Lacey", "Jeroue", role="ctb", comment="vignettes"), 
  person("Hoffman", "Abigail", role="ctb", comment="vignettes"), 
  person("Kleinsausser", "Michael", role="ctb", comment="vignettes"),
  person("Joel", "Reynolds", role="ctb", comment="Gamma likelihood"), 
  person("Pham", "Quang", role="ctb", comment="Gamma likelihood"), 
  person("Earl", "Becker", role="ctb", comment="Gamma likelihood"), 
  person("Aaron", "Christ", role="ctb", comment="Gamma likelihood"), 
  person("Brook", "Russelland", role="ctb", comment="Gamma likelihood"), 
  person("Stefan", "Emmons", role="ctb", comment="Automated tests"),
  person("Will", "McDonald", role="ctb", comment="Automated tests"),
  person("Reid", "Olson", role="ctb", comment="Automated tests and bug fixes"))
Description: Distance-sampling (<doi:10.1007/978-3-319-19219-2>) 
  is a field survey and analytical method that estimates density and 
  abundance of survey targets (e.g., animals) when 
  detection probability declines with observation distance. 
  Distance-sampling is popular in ecology, 
  especially when survey targets are observed from aerial platforms (e.g., 
  airplane or drone), surface vessels (e.g., boat or truck), or along 
  walking transects. Analysis involves fitting smooth (parametric) curves to 
  histograms of observation distances and using those functions to 
  adjust density estimates for missed targets.  Routines included here 
  fit curves to observation distance histograms, estimate effective 
  sampling area, density of targets in surveyed areas, and the abundance 
  of targets in a surrounding study area. Confidence interval estimation
  uses built-in bootstrap resampling. Help files are extensive and have been 
  vetted by multiple authors. Many tutorials are available on the package's 
  website (URL below).
License: GNU General Public License
URL: https://mcdonalddatasciences.com/Rdistance.html
BugReports: https://github.com/tmcd82070/Rdistance/issues
LazyData: TRUE
Suggests: testthat (>= 3.0.0),
Depends: R (>= 4.1.0), units
Imports: graphics, stats, utils, crayon, withr, tidyr, dplyr, progress,
        tibble, tidyselect, dfoptim, expint, multidplyr
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-13 23:26:52 UTC; trent
Author: Trent McDonald [cre, aut],
  Jason Carlisle [aut],
  Aidan McDonald [aut] (point transect methods),
  Ryan Nielson [ctb] (smoothed likelihood),
  Ben Augustine [ctb] (maximization method),
  James Griswald [ctb] (maximization method),
  Patrick McKann [ctb] (maximization method),
  Lacey Jeroue [ctb] (vignettes),
  Hoffman Abigail [ctb] (vignettes),
  Kleinsausser Michael [ctb] (vignettes),
  Joel Reynolds [ctb] (Gamma likelihood),
  Pham Quang [ctb] (Gamma likelihood),
  Earl Becker [ctb] (Gamma likelihood),
  Aaron Christ [ctb] (Gamma likelihood),
  Brook Russelland [ctb] (Gamma likelihood),
  Stefan Emmons [ctb] (Automated tests),
  Will McDonald [ctb] (Automated tests),
  Reid Olson [ctb] (Automated tests and bug fixes)
Repository: CRAN
Date/Publication: 2026-05-14 13:50:18 UTC
Built: R 4.6.0; ; 2026-05-14 15:50:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(AIC,dfunc)
S3method(coef,dfunc)
S3method(lines,dfunc)
S3method(model.matrix,dfunc)
S3method(plot,dfunc)
S3method(predict,dfunc)
S3method(print,abund)
S3method(print,dfunc)
S3method(summary,abund)
S3method(summary,dfunc)
S3method(summary,rowwise_df)
export("%#%")
export(EDR)
export(ESW)
export(Gamma.like)
export(Gamma.start.limits)
export(GammaModes)
export(GammaReparam)
export(RdistDf)
export(abundEstim)
export(autoDistSamp)
export(bcCI)
export(bootstrap)
export(bspline.expansion)
export(checkUnits)
export(cosine.expansion)
export(dE.multi)
export(dE.single)
export(dfuncEstim)
export(differentiableLikelihoods)
export(distances)
export(dropUnits)
export(effectiveDistance)
export(effort)
export(estimateN)
export(expandW)
export(expansionTerms)
export(groupSizes)
export(gxEstim)
export(halfnorm.like)
export(halfnorm.start.limits)
export(hazrate.like)
export(hazrate.start.limits)
export(hermite.expansion)
export(huber.cumFunc)
export(huber.like)
export(huber.start.limits)
export(integrateDfuncs)
export(integrateGammaLines)
export(integrateHalfnormLines)
export(integrateHalfnormPoints)
export(integrateHazrateLines)
export(integrateHuberLines)
export(integrateNegexpLines)
export(integrateNegexpPoints)
export(integrateNumeric)
export(integrateOneStepLines)
export(integrateOneStepNumeric)
export(integrateOneStepPoints)
export(integrateTriangleLines)
export(is.RdistDf)
export(is.Unitless)
export(is.points)
export(is.smoothed)
export(likeParamNames)
export(maximize.g)
export(mlEstimates)
export(nLL)
export(negexp.like)
export(negexp.start.limits)
export(observationType)
export(oneBsIter)
export(oneStep.like)
export(oneStep.start.limits)
export(parseModel)
export(perpDists)
export(predDensity)
export(predDfuncs)
export(predLikelihood)
export(secondDeriv)
export(setOptimizer)
export(setUnits)
export(simple.expansion)
export(simpsonCoefs)
export(sine.expansion)
export(startLimits)
export(transectType)
export(triangle.like)
export(triangle.start.limits)
export(unnest)
export(varcovarEstim)
exportPattern(`%.+%`)
import(units)

#' @title Distance function values at observations
#' 
#' @description
#' An internal prediction function to predict (compute) 
#' the values of distance functions at a set of observed values. 
#' Unlike \code{predDfuncs}, which evaluates distance
#' functions at EVERY input distance, this routine evaluates 
#' distance functions at only ONE distance. This is what's 
#' appropriate for likelihood computation.
#' This version allows for matrix inputs and 
#' uses matrix operations, and is thus faster than earlier
#' looping versions.
#' 
#' @inheritParams predict.dfunc 
#' @inheritParams predDfuncs
#' 
#' @details
#' 
#' Assuming \code{L} is the vector returned by this function, 
#' the negative log likelihood is \code{-sum(log(L / I), na.rm=T)}, 
#' where \code{I} is the integration constant, or 
#' area under the likelihood between 
#' \code{w.lo} and \code{w.hi}. 
#' Note that returned likelihood values for distances less 
#' than \code{w.lo} or greater than \code{w.hi} are \code{NA}; 
#' hence, \code{na.rm=TRUE} in the sum. 
#' 
#' @return A vector of distance function values, of length 
#' n = number of observed distances = length(distances(x)). 
#' Elements in \code{distances(x)} correspond, in order, 
#' to values in the returned vector.
#'  
#' @export
predLikelihood <- function(object
                         , params
                          ){
  
  d <- Rdistance::distances(object) # these should have the right units, don't check

  # Note: w.lo is subtracted in predDfuncs
  
  # This call to predDfuncs results in application of 
  # all params to all distances, i.e., an n X n matrix.  We only 
  # want the diagonal of this matrix for the likelihood. This 
  # method computes way too many values (all the off-diagonals)
  # but I cannot figure out a way to reduce computations without 
  # resorting to a loop and keeping the predict dfuncs methods.
  # This is something that could be modified in the future.
  
  # cat(colorize("in predLikelihood...\n", col="yellow", bg = "bgRed"))
  # cat(colorize(paste("dim(params) = ", paste(dim(params), collapse=","), "\n"),  col="yellow", bg = "bgRed"))
  # cat(colorize(paste("length(d) = ", paste(length(d), collapse=","), "\n"), col="yellow", bg = "bgRed"))
  
  y <- Rdistance::predDfuncs(object = object
                                , params = params
                                , distances = d
                                , isSmooth = is.smoothed(object))
  y <- diag(y)
 
  return( y )  
}

#' @title Huber Cumulative Function
#' 
#' @description
#' Computes the cumulative function of the `huber` likelihood. The 
#' cumulative function is proportional to the `huber` 
#' cumulative distribution function, differing only by appropriate
#' scaling constant. 
#' 
#' @param x A vector of distances. Can have units or not
#' (i.e., regular numeric). 
#' 
#' @param t1 A vector of values for the \eqn{\theta_1}{T1} parameter 
#' of the `huber` likelihood. If \code{x} has units, \code{t1} must have 
#' compatible units.
#' 
#' @param t2 A vector of values for the \eqn{\theta_2}{T2} parameter 
#' of the `huber` likelihood. If \code{x} has units, \code{t2} must have 
#' compatible units.
#' 
#' @param p A vector of values for the \eqn{p} parameter of the 
#' `huber` likelihood. If \code{x} has units, \code{p} must have 
#' units of '[1]' (i.e., \code{setUnits(p,1)}). 
#' 
#' @param w A vector of maximum strip widths, the maximum distance. 
#' If \code{x} has units, \code{x} must have 
#' compatible units.
#'  
#' @return A vector of values from the `huber` cumulative function. 
#' The `huber` cumulative function is
#' \deqn{
#' F(x|\theta_1,\theta_2,p) = \int_0^x f(y|\theta_1,\theta_2,p) dy,}{
#' F(x|T1,T2,p) = Area Under f(y|T1,T2,p) from 0 to x,
#' }
#' where \eqn{f(y|\theta_1,\theta_2,p)}{f(y|T1,T2,p)} is Rdistance's `huber` 
#' likelihood. The only difference between this \emph{cumulative function}, 
#' and the \emph{cumulative distribution function}
#' is the scaling constant. That is, the maximum of the \emph{cumulative 
#' function} is greater than 1 while the maximum \emph{cumulative distribution function}
#' is exactly 1.
#' 
#' @seealso \code{\link{huber.like}}
#' 
#' @examples 
#' d <- -10:210
#' 
#' # Cumulative function
#' fd <- huber.cumFunc(d, 125, 25, .05, 200) 
#' plot(d, fd, type="l")
#' 
#' # Cumulative distribution function
#' Fd <- fd / huber.cumFunc(200, 125, 25, .05, 200)
#'
#' @export 
huber.cumFunc <- function(x, t1, t2, p, w){
  if( inherits(x, "units") ){
    zero <- setUnits(0, units(x))
    one <- setUnits(1, "1")
  } else {
    zero <- 0
    one <- 1
  }
  
  # All vectors must be same length due to subsetting used below
  n <- length(x)
  if(length(t1) == 1){
    t1 <- rep(t1, n)
  } else if(length(t1) != n){
    stop(paste0("Length of t1 vector must be 1 or same as x, which is length ", n, ". "
                , "t1 is length ", length(t1), "."))
  }
  if(length(t2) == 1){
    t2 <- rep(t2, n)
  } else if(length(t2) != n){
    stop(paste0("Length of t2 vector must be 1 or same as x, which is length ", n, ". "
              , "t2 is length ", length(t2), "."))
  }
  if(length(p) == 1){
    p <- rep(p, n)
  } else if(length(p) != n){
    stop(paste0("Length of p vector must be 1 or same as x, which is length ", n, ". "
                , "p is length ", length(p), "."))
  }
  if(length(w) == 1){
    w <- rep(w, n)
  } else if(length(w) != n){
    stop(paste0("Length of w vector must be 1 or same as x, which is length ", n, ". "
                , "w is length ", length(w), "."))
  }

  # Initiation
  out <- rep(zero, length(x))
  greater0 <- x > zero
  
  x <- pmin(x, w)
  
  T2 <- t1 + t2
  m <- t1*(T2 - 0.5*t1)
  K <- (one - p) / (2*m)

  # Part 1: everyone over 0  
  x1 <- pmin(x, t1)[greater0]
  KGT <- K[greater0]
  out[greater0] <- x1 - KGT * (x1^3 / 3)
  
  # Part 2: t1 < x < T2; but x=min(x,w) 
  greaterT1 <- x > t1
  x2 <- pmin(x, T2)[greaterT1]
  tGT <- t1[greaterT1]
  KGT <- K[greaterT1]
  gAtT1 <- one - KGT * tGT^2
  gAtX  <- one - KGT * tGT * (2*x2 - tGT) #(1-p)*(t1*(x - 0.5*t1))/m
  f2 <- (x2 - tGT) * (gAtT1 + gAtX) / 2 # trapazoid, t1 to x
  out[greaterT1] <- out[greaterT1] + f2
  
  # Part 3: T2 < x < w
  greaterT2 <- x > T2
  f3 <- (x[greaterT2] - T2[greaterT2])*p[greaterT2]
  out[greaterT2] <- out[greaterT2] + f3
  
  return(out)
  
}

#' @title Huber distance function
#' 
#' @description 
#' Computes the Huber likelihood for use as a distance function. 
#' The Huber likelihood is a mixture of an inverted Huber loss 
#' and a uniform distribution.  
#' The distance function is quadratic from the lowest distance to 
#' its first parameter, linear from the first parameter
#' to the sum of its first and second parameter, and constant 
#' after that. 
#' 
#' @inheritParams halfnorm.like
#' 
#' @inherit halfnorm.like return seealso
#' 
#' @details
#' The 'huber' likelihood is an inverted version of the 
#' Huber loss function, mixed with a uniform at the upper end, and contains
#' three canonical parameters. 
#' The 'huber' distance function is a negative quadratic 
#' between \code{0} and its first parameter \eqn{\theta_1}{T1},
#' linear between \eqn{\theta_1}{T1} and \eqn{\theta_1 + \theta_2}{T1 + T2}, and 
#' constant  after \eqn{\theta_1 + \theta_2}{T1 + T2}.
#' Specifically, the 'huber' likelihood is, 
#' \deqn{
#'    f(d|\theta_1,\theta_2, p) = \left(1 - \frac{(1-p)h(d)}{m}\right) \,
#'      I(0 \leq d \leq \Theta) \  + \  p \, I(\Theta < d \leq w)
#' }
#' where
#' \eqn{\Theta = \theta_1 + \theta_2}{T = T1 + T2}, \eqn{w} = \code{w.hi - w.lo},
#' \eqn{
#'  m = \theta_1(\Theta - 0.5\theta_1)
#' }{
#'  m = T1(T - 0.5(T1))
#' }
#' and \emph{h(d)} is Huber loss between 0 and \eqn{\Theta}{T}, i.e.,
#' \deqn{
#'  h(d)  = 0.5d^2\, I(0 \leq d \leq \theta_1) \ + \ 
#'          \theta_1(d - 0.5\theta_1)\, I(\theta_1 < d \leq \Theta).
#' }
#' 
#' The first parameter, \eqn{\theta_1}{T1}, is related to covariates, 
#' i.e., \eqn{log(\theta_1) = \beta_0 + \beta_1x_1 + \ldots + \beta_qx_q}{
#' log(T_1) = B_0 + B_1(x_1) + ... + B_q(x_q)}.  \eqn{\theta_2}{T2} and \eqn{p} 
#' are constant across covariate values.
#' 
#' @seealso See \href{https://mcdonalddatasciences.com/Rdistance.html}{Rdistance tutorials}
#' for a method to generate random observations from the Huber likelihood.
#' 
#' @examples
#' 
#' t1 <- c(65,80,120)
#' t2 <- 60
#' p  <- 0.05
#' a <- matrix(c(log(t1)
#'        , rep(t2,3)
#'        , rep(p,3))
#'        , 3,3)
#' d <- seq(0, 200, length=201)
#' X <- matrix(1,length(d),1)
#' y <- huber.like(a, d, X)
#' 
#' # Plot showing covariate effects 
#' plot(range(d), range(y$L.unscaled)
#'   , type = "n", xlab = "d", ylab = "Huber(d)")
#' for(i in 1:3){
#'   # Distance functions
#'   lines(d
#'     , y$L.unscaled[,i]
#'     , col = i
#'     , lwd= 2)
#'   # Quadradic to linear transitions
#'   points(exp(a[i,1])
#'     , y$L.unscaled[(t1[i]-0.1) < d & d < (t1[i]+0.01),i]
#'     , pch = 16
#'     , col = i )
#'   # Linear to constant transition
#'   Theta <- exp(a[i,1]) + a[i,2]
#'   points(Theta
#'     , y$L.unscaled[(Theta-0.1) < d & d < (Theta+0.1),i]
#'     , pch = 15
#'     , col = i )
#' }
#' 
#' @export
#' 
huber.like <- function(a
                       , dist
                       , covars 
                       , w.hi = NULL){
  
  # What's in a? : 
  #     a = [(Intercept), b1, ..., bp, range, <expansion coef>]
  
  q <- nCovars(covars)
  
  if(is.matrix(a)){
    k <- nrow(a) 
    beta <- a[,1:q, drop = FALSE]  # k X q
    p <- a[1, (q+1):(q+2)]     # 1 X 2
  } else {
    k <- 1
    beta <- matrix(a[1:q], nrow = 1) # 1 X q
    p <- a[(q+1):(q+2)]     # 1 X 2
  }
  
  s <- covars %*% t(beta) # (nXq) %*% (qXk) = nXk
  
  theta2 <- p[1]
  p <- p[2]
  
  # theta1 is location of transition between 
  # squared trend and linear. 
  # theta2 is distance between theta1 and range, 

  if(p < 0 | p > 1 | theta2 < 0){
    h <- matrix(NA, nrow = length(dist), ncol = k)
  } else {
    theta1 <- exp(s)  # link function here
    dist <- dropUnits(dist)
    range <- theta1 + theta2
    
    # Huber loss function
    h <- ifelse( dist <= theta1
                , 0.5 * dist^2
                , theta1*(dist - 0.5*theta1)
    )
    
    # Flip it over
    mx <- theta1*(range - 0.5*theta1)
    h <-  1 - (h / mx)*(1-p)
    h <- ifelse( dist > range
               , p
               , h)
  }

  # Integrate under the function and scale
  # integral0a <- beta^2*range - (2/3)*beta^3
  # integralar <-  0.5*beta*(range^2 + beta^2 - 2*range*beta)
  # areaUnder <- integral0a + integralar
  # 
  # h <- h / areaUnder
  
  return(
    list(L.unscaled = h, 
         params = cbind(par1 = s
                      , par2 = theta2
                      , par3 = p
                      )
    )
  )
  
}

#' @title Brewer's Sparrow site data
#' 
#' @name sparrowSiteData
#' 
#' @description  
#' Site data from line transect surveys for 
#' Brewer's sparrow on 72 transects 
#' located on a 4105 km^2 study 
#' area in central Wyoming. Data were collected by 
#' Dr. Jason Carlisle of the Wyoming Cooperative Fish & Wildlife
#' Research Unit in 2012. Each transect was 500 meters long.
#' 
#' @docType data
#' 
#' @format A data.frame containing 72 rows and 8 columns.  Each row represents
#' a site (transect) surveyed.  Column descriptions: 
#' 
#' \enumerate{ 
#'   \item `siteID`: Factor (72 levels), the site or transect surveyed.  
#'   \item `length`: Number, the length in meters (\[m\]) of each transect.  
#'   \item `observer`: Factor (five levels), identity of the observer 
#'   who surveyed the transect.
#'   \item `bare`: Number, the mean bare ground cover (%) within 100 \[m\] of each transect.
#'   \item `herb`: Number, the mean herbaceous cover (%) within 100 \[m\] of
#'   each transect.
#'   \item `shrub`: Number, the mean shrub cover (%) within
#'   100 m of each transect.
#'   \item `height`: Number, the mean shrub height 
#'   \[cm\] within 100 m of each transect.
#'   \item `shrubclass`: Factor (two levels), shrub class is "Low" when 
#'   shrub cover is < 10%, "High" otherwise.  
#' }
#' 
#' @seealso [sparrowDetectionData()]
#' 
#' @source The Brewer's sparrow data are a subset of the data collected 
#' by Jason Carlisle and various field technicians for his Ph.D. from the 
#' Department of Ecology, University of Wyoming, in 2017.  This portion of 
#' Jason's work was funded by the Wyoming Game and Fish Department through agreements 
#' with the University of Wyoming's  Cooperative Fish & Wildlife
#' Research Unit (2012). 
#' 
#' @references Carlisle, J.D. 2017. The effect of sage-grouse conservation on
#' wildlife species of concern: Implications for the umbrella species concept.
#' Dissertation. University of Wyoming, Laramie, Wyoming, USA.
#' 
#' Carlisle, J. D., and A. D. Chalfoun. 2020. The abundance of Greater 
#' Sage-Grouse as a proxy for the abundance of sagebrush-associated songbirds 
#' in Wyoming, USA. *Avian Conservation and Ecology* 15(2):16. 
#' \doi{10.5751/ACE-01702-150216}
#' 
NULL

#' @title Integrate Line-transect Triangle function 
#' 
#' @description
#' Compute exact integral of the triangle distance function for line 
#' transects. 
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' Returned integrals are
#' \deqn{\int_0^{w} (\frac{p}{\theta_i}I(0\leq x \leq \theta_i) + 
#'    \frac{1-p}{w - \theta_i}I(\theta_i < x \leq w)) dx = \frac{\theta_i}{p},}{
#' Integral((p/Theta)I(0<=x<=Theta) + ((1-p)/(w-Theta))I(Theta<x<=w)) = Theta / p,} 
#' where \eqn{w = w.hi - w.lo}, \eqn{\theta_i}{Theta} is the estimated one-step
#' distance function
#' threshold for the i-th observed distance, and \eqn{p}{p} is the estimated 
#' one-step proportion. 
#' 
#' @seealso \code{\link{integrateNumeric}}; \code{\link{integrateNegexpLines}}; 
#' \code{\link{integrateHalfnormLines}} 
#' 
#' @examples
#' 
#' w <- 250
#' T <- 160
#' p <- 0.4
#' obj <- matrix(c(T,p), 1, 2)
#' 
#' integrateTriangleLines(obj
#'   , w.lo = units::set_units(0,"m")
#'   , w.hi = units::set_units(w,"m")
#'   , Units = "m")
#'   
#' # same
#' (1 + p) * T / 2 + p * (w - T)
#' 
#' # check by numeric integration
#' triLike <- function(d, T, p, wl, wh) { 
#'   y <- triangle.like(a = c(log(T), p)
#'            , dist = d - wl
#'            , covars = matrix(1, length(d))
#'            , w.hi = wh - wl)$L.unscaled
#'   y
#' }
#' integrate(triLike, lower = 0, upper = w, T = T, p = p, wl = 0, wh = w)
#'   
#' @export
#' 
integrateTriangleLines <- function(object
                                  , newdata = NULL
                                  , w.lo = NULL
                                  , w.hi = NULL
                                  , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    object <- stats::predict(object = object
                        , newdata = newdata
                        , type = "parameters"
    )
  } 
  
  Theta <- setUnits(object[,1], Units)
  p <- object[,2]

  outArea <- (Theta - w.lo) * (1 + p) / 2 + # this is trapazoid part
    p * (w.hi - Theta) # horizontal part
  
  outArea 
  
}

#' @title Estimate g(0) or g(x)
#' 
#' @description Estimate distance function scaling factor
#' , g(0) or g(x), for a specified distance function.
#' 
#' @param fit An estimated `dfunc` object.  See `dfuncEstim`.
#'   
#' @details This routine scales sightability such that 
#' g(`x.scl`) = `g.x.scl`, where g() is the sightability function.
#' Specification of `x.scl` and `g.x.scl` covers several estimation cases:
#' \enumerate{  
#'   \item **g(0) = 1** : (the default) Inputs are `x.scl` = 0, `g.x.scl` = 1. 
#'   If `w.lo` > 0, `x.scl` will be set to `w.lo`
#'   so technically this case is g(`w.low`) = 1.
#'   
#'   \item **User supplied probability at specified distance**: Inputs are 
#'   `x.scl` = a number greater than or equal 
#'   to `w.lo`, `g.x.scl` = a number between 0 and 1.  This case 
#'   covers situations where sightability on the transect (distance 0) is 
#'   not perfect.  This case  assumes researchers have an independent 
#'   estimate of sightability at distance 
#'   `x.scl` off the transect.  For example, researchers could be
#'   using multiple 
#'   observers to estimate that sightability at distance `x.scl` 
#'   is `g.x.scl`. 
#'   
#'   \item **Maximum sightability specified**: Inputs 
#'   are `x.scl`="max", `g.x.scl` =  a number 
#'   between 0 and 1.  In this case, 
#'   g() is scaled such that its maximum value is `g.x.scl`.  
#'   This routine computes the distance at which g() is maximum, sets 
#'   g()'s height there to `g.x.scl`, and returns `x.max` where 
#'   x.max is the distance at which g is maximized. This case covers the 
#'   common aerial survey situation where maximum sightability is slightly 
#'   off the transect, but the distance at which the maximum occurs 
#'   is unknown. 
#'   
#'   \item **Double observer system**: Inputs are 
#'   `x.scl`="max", `g.x.scl` = \<a data frame\>. 
#'   In this case, g(*x*) = *h*, where *x* is the distance that 
#'   maximizes g and *h* is the height of g() at *x* 
#'   computed from the double observer data frame (see below for 
#'   structure of the double observer data frame).
#'   
#'   
#'   \item **Distance of independence specified, height computed from double 
#'   observer system**: Inputs are 
#'   `x.scl` = a number greater than or equal to `w.lo`
#'   `g.x.scl` = a data frame.  In this case, g(`x.scl`) = *h*, 
#'   where *h* is computed from the double observer data frame 
#'   (see below for structure of the double observer data frame). 
#'
#' }   
#'   
#'   When `x.scl`, `g.x.scl`, or `observer` are NULL, the routine 
#'   will look for and use `$call.x.scl`, or `$call.g.x.scl`, or 
#'   `$call.observer` components of the `fit` object for whichever 
#'   of these three parameters is missing.  Later, different 
#'   values can be specified in a direct call to `F.gx.estim` 
#'   without having to re-estimate the distance function. Because of this feature, 
#'   the default values in `dfuncEstim` are `x.scl` = 0 and 
#'   `g.x.scl` = 1 and `observer` = "both". 
#'   
#' @section Structure of the double observer data frame:  
#' When `g.x.scl` is a data frame, it is assumed to contain 
#'   the components `$obsby.1` and `$obsby.2` (no flexibility on names). 
#'   Each row in the data frame contains data from one sighted target. 
#'   The `$obsby.1` and `$obsby.2` components are 
#'   TRUE/FALSE (logical) vectors indicating whether 
#'   observer 1 (`obsby.1`) or observer 2 (`obsby.2`) spotted the target.
#'   
#' @return A list comprised of the following components:
#'   \item{x.scl}{The value of x (distance) at which g() is evaluated.  }
#'   \item{comp2 }{The estimated value of g() when evaluated at `x.scl`.  }
#'   
#' 
#' @seealso [dfuncEstim()]
#' 
#' @examples 
#' 
#' data(sparrowDf)
#' fit <- dfuncEstim(sparrowDf, dist ~ groupsize(groupsize))
#' gxEstim(fit)
#'   
#' fit <- dfuncEstim(sparrowDf, dist ~ groupsize(groupsize)
#'                 , x.scl = 50 %m%.
#'                 , g.x.scl = 0.75)
#' gxEstim(fit)
#' plot(fit)
#' abline(h=0.75)
#' abline(v=50%m%.)
#'   
#' @export

gxEstim <- function( fit ){

  #   Will need measurement units later ----
  distUnits <- fit$outputUnits
  
  #   Compute x (the point to evaluate g() at) ----
  # if( is.null( fit$x.scl ) ){
  #     x.scl <- fit$call.x.scl
  # }
  
  #   Compute height of g() at x.scl ----
  # if( is.null( g.x.scl ) ){
  #     g.x.scl <- fit$call.g.x.scl
  # }
  
  #   Double observers? ----
  # if( is.null( observer ) ){
  #     observer <- fit$call.observer
  # }
  
  x.scl <- fit$x.scl
  
  if( is.character(x.scl) ){
    if( x.scl == "max" ){
      if( fit$likelihood %in% c("halfnorm", "negexp", "hazrate", "triangle", "huber") & 
                 fit$expansions == 0 ){
        # All these are monotonically negative
        x.scl <- fit$w.lo 
      } 
    } else {
        # x.scl != "max"
        x.scl <- setUnits(NA, fit$outputUnits)
        warning("Invalid character string for x.scl specified in gxEstim. x.scl set to missing.")
    }
  } 

  #   --------------------------------------------------------------------------------------
  #   Now compute g(x)
  
  if( Rdistance::observationType(fit) != "single" ){
      #   Compute g(x) from double observer data
      stop("Double observer systems not implemented.")
      #g.x.scl <- doubleObsProb( fit )
  } else {
      # Single observer; but, g(x) could be less than 1
      # g(x) is specified, nothing to do except check validity
      if( length(fit$g.x.scl) > 1 ){
          g.x.scl <- fit$g.x.scl[1]
          warning(paste("Vector of g(x) values found in gxEstim. Only the first value,", g.x.scl, "has been used."))
      }
      if( fit$g.x.scl < 0 ){
          g.x.scl <- 0
          warning("Impossible g.x.scl < 0 specified in gxEstim. g(x) has been reset to 0.")
      } else if( fit$g.x.scl > 1 ){
          g.x.scl <- 1
          warning("Impossible g(x) > 1 specified in gxEstim. g(x) reset to 1.")
      } else if( is.character(fit$g.x.scl) ){
          stop(paste0("g.x.scl cannot be character valued. Found '", g.x.scl, "'. Convert this to numeric."))
      } else {
          g.x.scl <- fit$g.x.scl
      }
  }

  list( x.scl = x.scl, g.x.scl = g.x.scl )
}

#' @title 'nlminb' optimizer
#' 
#' @description
#' Call R native function 'nlminb' to perform optimization. 
#' 
#' @inheritParams mlEstimates
#' 
#' @return 
#' A list with following named components:
#' \itemize{
#'   \item par = parameters
#'   \item loglik = objective function value at minimum
#'   \item convergence = 0 for yes, other for no
#'   \item iterations = number of iterations
#'   \item evaluations = function evaluations
#'   \item message = a convergence message
#'   \item varcovar = a variance covariance matrix of parameters
#'   \item limits = low and high limits
#' }
#' 
# Do Not export

Nlminb <- function(ml, strt.lims){
  
  contRl <- list(trace = getOption("Rdistance_trace")
                 , eval.max = getOption("Rdistance_evalMax")
                 , iter.max = getOption("Rdistance_maxIters")
                 , rel.tol = getOption("Rdistance_likeTol")
                 , x.tol = getOption("Rdistance_coefTol")
  )
  
  verboseLevel <- getOption("Rdistance_verbosity")
  if( verboseLevel >= 1 ){
    cat(colorize("NLMINB gradient maximization ----\n", col = "red"))
  }
  
  fit <- stats::nlminb(
    start = strt.lims$start
    , objective = nLL
    , lower = strt.lims$low
    , upper = strt.lims$high
    , verbosity = verboseLevel
    , control = contRl
    , ml = ml
  )
  
  names(fit$par) <- strt.lims$names
  if( ml$asymptoticSE ){
    fit$varcovar <- Rdistance::varcovarEstim(fit, ml)
  } else {
    fit$varcovar <- NULL
  }
  
  # final few things ----
  fit$limits <- strt.lims[c("low", "high")]
  
  # Flip over objective: object$logLike is true logLike, -LL was minimized
  names(fit)[names(fit) == "objective"] <- "loglik"
  fit$loglik <- -fit$loglik  

  fit
} 

#' @title Distance function maximum likelihood estimates
#' 
#' @description
#' Estimate parameters of a distance function using maximum likelihood.
#' 
#' @inheritParams startLimits
#' 
#' @param strt.lims A list containing start, low, and high limits for 
#' parameters of the requested likelihood. This list is typically produced
#' by a call to [startLimits()].
#' 
#' @return An Rdistance fitted model object. This object contains the 
#' raw object returned by the optimization routine (e.g., `nlming`), 
#' and additional components specific to Rdistance.
#' 
#' @export

mlEstimates <- function( ml
                       , strt.lims
                       ){

  # Missing observations: 
  #  (1) there can be missing responses (distances); but, Rdistance::distances drops them
  #      i.e., model.response(ml$mf) has missings (potentially), but Rdistance::distances(ml) does not
  #  (2) There are missing group sized in the model frame for transects with missing lengths. 
  #      These don't matter here.
  
  optimFunc <- ml$optimizer
  optimFunc <- gsub("_.+$", "", optimFunc)

  fit <- switch( optimFunc
      , "optim" = Optim(ml, strt.lims)
      , "nlminb" = Nlminb(ml, strt.lims)
      , "hookeJeeves" = HookeJeeves(ml, strt.lims)
      ,  stop(paste("Unknown optimizer function. Found", optimFunc))
  )
  
  fit 

}

#' @title Sage Thrasher site data. 
#' 
#' @name thrasherSiteData
#' 
#' @description Point transect data collected in central 
#' Wyoming from 120 points surveyed for Sage Thrashers by the Wyoming 
#' Cooperative Fish & Wildlife Research Unit in 2013.
#' 
#' @docType data
#' 
#' @format A data.frame containing 120 rows and 6 columns.  Each row represents
#' a surveyed site (point).  Column descriptions: 
#' \enumerate{ 
#' \item `siteID`: Factor (120 levels), the site or point surveyed.  
#' \item `observer`: Factor (six levels), identity of the observer who surveyed
#' the point.  
#' \item `bare`: Number, the mean bare ground cover (%)
#' within 100 m of each point.  
#' \item `herb`: Number, the mean herbaceous
#' cover (%) within 100 \[m\] of each point.  
#' \item `shrub`: Number, the mean
#' shrub cover (%) within 100 \[m\] of each point.  
#' \item `height`: Number,
#' the mean shrub height \[cm\] within 100 \[m\] of each point.  }
#' 
#' @seealso [thrasherDetectionData()]
#' 
#' @source The Sage Thrasher data are a subset of data collected 
#' by Jason Carlisle and field technicians for his Ph.D. from the 
#' Department of Ecology, University of Wyoming, in 2017.  This portion of 
#' Jason's work was funded by the Wyoming Game and Fish Department through agreements 
#' with the University of Wyoming's  Cooperative Fish & Wildlife
#' Research Unit (2012). 
#' 
#' @references Carlisle, J.D. 2017. The effect of sage-grouse conservation on
#' wildlife species of concern: implications for the umbrella species concept.
#' Dissertation. University of Wyoming, Laramie, Wyoming, USA.
#' 
#' Carlisle, J. D., A. D. Chalfoun, K. T. Smith, and J. L. Beck. 2018. 
#' Nontarget effects on songbirds from habitat manipulation for Greater 
#' Sage-Grouse: Implications for the umbrella species concept. 
#' *The Condor: Ornithological Applications* 120:439–455. 
#' \doi{10.1650/CONDOR-17-200.1}
#' 
NULL

#' @title Test whether object is unitless
#' 
#' @description Tests whether a 'units' object is actually 
#' unitless.  
#' Unitless objects, such as ratios, should be assigned
#' units of \[1\].  Often they are, but  
#' sometimes unitless ratios are assigned units like \[m/m\].
#' The `units` package should always convert \[m/m\] to 
#' \[1\], but it does not always. 
#' Sometimes units like \[m/m\] mess things up, so it is 
#' better to remove them before calculations. 
#' 
#' @param obj  A numeric scalar or vector, with or without units. 
#' 
#' @return TRUE if `obj` has units and they 
#' are either \[1\] or the denominator units equal
#' the numerator units.  Otherwise, return FALSE.
#' If `obj` does not have units, this routine
#' returns TRUE. 
#' 
#' @examples 
#' a <- setUnits(2, "m")
#' b <- a / a
#' is.Unitless(a)
#' is.Unitless(b)
#' is.Unitless(3)
#' 
#' @export
is.Unitless <- function(obj){
  
  if(!is.numeric(obj)){
    stop("'obj' must be numeric.")
  }
  
  if( !inherits(obj, "units") ){
    # Missing units = unitless
    return(TRUE)
  } else if( units(obj) == units::unitless ){
    return(TRUE)
  } else if (isTRUE(all.equal(units(obj)$numerator, units(obj)$denominator))) {
    return(TRUE)
  } else {
    return(FALSE)
  }
  NA
}

#' @title Plot parametric distance functions
#' 
#' @description
#' Plot method for parametric line and point transect distance functions. 
#' 
#' @inheritParams print.dfunc 
#'   
#' @param include.zero Boolean value specifying whether to include 0 on the x-axis 
#' of the plot.  A value of TRUE will include 0 on the left hand end of the x-axis
#' regardless of the range of distances.  A value of FALSE will plot only the
#' observation strip (\code{w.lo} to \code{w.hi}).
#'   
#' @param nbins Internally, this function uses \code{hist} to compute histogram
#'   bars for the plot. This argument is the \code{breaks} argument to
#'   \code{hist}.  This can be either a vector giving the breakpoints between
#'   bars, the suggested number of bars (a single number), a string naming
#'   an algorithm to compute the number of bars, or a function to compute the
#'   number of bars.  See \code{\link{hist}} for all options.
#'   
#' @param newdata Data frame (similar to \code{newdata} parameter 
#' of \code{\link{lm}}) containing new values for covariates in 
#' the distance function.
#' One distance function is computed and plotted for each row in the data frame. 
#' If \code{newdata} is NULL, a single distance function is plotted 
#' for mean values of all numeric covariates and mode values for all 
#' factor covariates. 
#'   
#' @param legend Logical scalar for whether to include a legend. 
#'   If TRUE, a legend will be included on the plot detailing
#'   the covariate values used to generate the plot.
#'   
#' @param plotBars Logical scalar for whether to plot the histogram 
#' of distances behind the distance function.  If FALSE, no histogram 
#' is plotted, only the distance function line(s).
#'  
#' @param circles Logical scalar indicating whether to plot 
#' small circles on the distance function(s) at the locations 
#' of observed distances. Note that one circle plots on each distance 
#' function for every observed distance even though covariates 
#' associated with the distances may not match the function. 
#' 
#' @param xlab Label for the x-axis
#' 
#' @param ylab Label for the y-axis
#' 
#' @param density If \code{plotBars=TRUE}, a vector giving the density of 
#' shading lines, in lines per inch, for the bars underneath 
#' the distance function, repeated as necessary to exceed the number 
#' of bars. Values of NULL or a number strictly less than 0 
#' mean solid fill using colors from parameter \code{col}. 
#' If \code{density = 0}, bars are not filled and only the borders are rendered. 
#' If \code{density} > 0, bars are shaded with colors and angles from 
#' parameters \code{col} and \code{angle}.
#' 
#' @param angle When \code{density} > 0, the slope of bar shading lines, 
#' given as an angle in degrees (counter-clockwise), repeated as necessary
#' to exceed the number of bars.
#' 
#' @param col A vector of bar fill colors or line colors when bars are 
#' drawn and \code{density != 0}, repeated as necessary to exceed the number
#' of bars. Also used for the bar borders when
#' \code{border = TRUE}.
#' 
#' @param circles Logical scalar requesting the location of detection distances
#' be plotted. If TRUE, open circles are plotted at predicted distance 
#' function heights associated with all detection distances. For computational
#' simplicity, all distances are plotted for EVERY covariate class even though
#' observed distances belong to only one covariate class. If FALSE, circles 
#' are not plotted. 
#'  
#' @param border The color of bar borders when bars are plotted, 
#' repeated as necessary to exceed the number of bars. A 
#' value of NA or FALSE means no borders. If bars are shaded with lines 
#' (i.e., \code{density>0}), \code{border = TRUE} uses the same 
#' color for the border as for the shading lines.  Otherwise, fill color
#' or shaded line color are specified in \code{col} while 
#' border color is specified in \code{border}.  
#' 
#' @param vertLines Logical scalar specifying whether to plot vertical 
#'  lines at \code{w.lo} and \code{w.hi} from 0 to the  
#'  distance function. 
#'  
#' @param col.dfunc Color of the distance function(s).
#' If only one distance function (one line) is being plotted, 
#' the default color is "red".
#' If covariates or \code{newdata} are present, 
#' the default value uses \code{graphics::rainbow(n)}, 
#' where \code{n} is the number 
#' of plotted distance functions.  Otherwise, \code{col.dfunc} 
#' is replicated to the required length.  Plot all 
#' distance functions in the same color by setting 
#' \code{col.dfunc} to
#' a scalar. Plot blue-red pairs of distance functions 
#' by setting \code{col.dfunc} = \code{c("blue", "red")}. Etc. 
#'   
#' @param lty.dfunc Line type of the distance function(s).
#' If covariates or \code{newdata} is present, 
#' the default uses line types  
#' to \code{1:n}, where \code{n} is the number 
#' of plotted distance functions.  Otherwise, \code{lty.dfunc} 
#' is replicated to the required length. Plot solid lines
#' by specifying \code{lty.dfunc = 1}. Plot solid-dashed line pairs
#' by specifying \code{lty.dfunc = c(1,2)}. Etc.
#' 
#' @param lwd.dfunc Line width of the distance function(s), replicated 
#' to the required length. Default is 2 for all lines.  
#' 
#' @param \dots When bars are plotted, this routine 
#'  uses \code{graphics::barplot} to draw the 
#'  plotting region and bars. When bars are not plotted,
#'  this routine sets up the plot with \code{graphics::plot}.
#'  \dots can be any argument to \code{barplot} or \code{plot} EXCEPT  
#'  \code{width}, \code{ylim}, \code{xlim}, 
#'  \code{density}, \code{angle}, and \code{space}. For example, 
#'  set the main title with \code{main = "Main Title"}.
#' 
#' @inherit plot.dfunc return
#' 
#' @seealso \code{\link{plot.dfunc}}
#' 
#' @examples
#' 
#' # Simulated data
#' set.seed(87654)
#' x <- rnorm(1000, mean=0, sd=20)
#' x <- x[x >= 0]
#' x <- setUnits(x, "ft")
#' Df <- data.frame(transectID = "A"
#'                , distance = x
#'                 ) |> 
#'   dplyr::nest_by( transectID
#'                , .key = "detections") |> 
#'   dplyr::mutate(length = setUnits(1,"mi"))
#' attr(Df, "detectionColumn") <- "detections"
#' attr(Df, "obsType") <- "single"
#' attr(Df, "transType") <- "line"
#' attr(Df, "effortColumn") <- "length"
#' is.RdistDf(Df) # TRUE
#' 
#' # Estimation
#' dfunc <- dfuncEstim(Df
#'                   , formula = distance~1
#'                   , likelihood="halfnorm")
#' plot(dfunc)
#' plot(dfunc, nbins=25)
#'
plot.dfunc.para <- function( x, 
                        include.zero=FALSE, 
                        nbins="Sturges", 
                        newdata = NULL, 
                        legend = TRUE, 
                        vertLines = TRUE,
                        plotBars = TRUE,
                        circles = FALSE,
                        density = -1,
                        angle = 45,
                        xlab = NULL,
                        ylab = NULL,
                        border = TRUE,
                        col = "grey85",
                        col.dfunc=NULL,
                        lty.dfunc=NULL,
                        lwd.dfunc=NULL,
                        ... ){
  
  # a constant used later
  zero <- units::as_units(0, x$outputUnits)
  
  d <- Rdistance::distances(x)
  whi <- x$w.hi
  wlo <- x$w.lo
  
  cnts <- graphics::hist( d
                , plot = FALSE
                , breaks = nbins
                , include.lowest = TRUE
                , warn.unused = FALSE)
  
  # hist should return breaks with units attached, but it does not
  cnts$breaks <- units::as_units(cnts$breaks, x$outputUnits)
  cnts$mids <- units::as_units(cnts$mids, x$outputUnits)
  xscl <- diff(cnts$breaks) 
  x.seq <- seq( x$w.lo, x$w.hi, length=getOption("Rdistance_intEvalPts") )
  
  
  # Fixup new data if missing ----
  if( is.null(newdata) ){
    
    # Function to return mode (=most frequent values) of a FACTOR.
    # only needed in this case.
    getmode <- function(v) {
      uniqv <- table(v)
      modeVal <- names(uniqv)[which.max(uniqv)] 
      if( is.factor(v) ){
        modeVal <- factor(modeVal, levels = levels(v))
      } 
      if( is.character(v) ){
        modeVal <- factor(modeVal, levels = names(uniqv))
      }
      modeVal
    }
    
    # Note: x$mf is the model frame. It has only non-missing values between w.lo and w.hi
    # x$mf[,-1] has covariates in un-expanded-for-indicator variables format 
    
    covNames <- labels(stats::terms(x$mf)) # Intercept not included here
    newdata <- matrix(NA, nrow = 1, ncol = length(covNames))
    colnames(newdata) <- covNames
    newdata <- data.frame(newdata)
    
    # Watch filtering of in-out strip observations here
    
    # origDist <- Rdistance::distances(x) 
    # inStrip <- (x$w.lo <= d) & (d <= x$w.hi)
    factor.names <- attr(stats::terms(x$mf), "dataClasses")
    factor.names <- names(factor.names)[ factor.names %in% c("factor","character") ]
    for( nm in covNames ) {
      if( nm %in% factor.names ) {
        newdata[,nm] <- getmode(x$mf[, nm]) # Use mode to predict
      } else {
        newdata[,nm] <- mean(x$mf[, nm]) # Use mean
      }
    }
  }

  # Add discrete likelihood points ----
  # add points just left and just right of breaks in discontinuous functions
  x.seq <- switch(x$likelihood
                  , "oneStep" = insertOneStepBreaks(obj = x
                                                  , newData = newdata
                                                  , xseq = x.seq)
                  , x.seq
  )
  
    
  # Predict distance functions ----
  # after here, y is a matrix, columns are distance functions.
  y <- stats::predict(object = x
                      , newdata = newdata
                      , distances = x.seq
                      , type = "dfuncs"
  )

  # Compute scaling factors ----
  if( Rdistance::is.points(x) ){
    y <- y * dropUnits(x.seq - x$w.lo)
    y <- t( t(y) / (colSums(y, na.rm = TRUE) * dropUnits(x.seq[2] - x.seq[1]))) # now y integrates to 1.0
    # don't need to modify ybarhgts because cnts$density integrates to 1.0 already
    ybarhgts <- cnts$density
    y.finite <- y[ y < Inf ]
    y.lims <- c(0, max( ybarhgts, y.finite, na.rm=TRUE ))
  } else {
    scaler <- Rdistance::effectiveDistance(object = x
                                           , newdata = newdata)
    # Note: scaler is correct even when g.x.scl != 1. Hence, no need to apply 
    # another scaler. i.e., this works when g.x.scl < 1
    ybarhgts <-  cnts$density * dropUnits(mean(scaler)) # now ybarhgts integrates to ESW 
    y.finite <- y[ y < Inf ]
    y.lims <- c(0, max( x$g.x.scl, ybarhgts, y.finite, na.rm=TRUE ))
  }

  # I DON'T THINK THIS IS NEEDED, BUT MAYBE  
  # if( include.zero & x$like.form == "hazrate" ){
  #   x.seq[1] <- x$w.lo
  # }
  
  
  if( include.zero ){
    x.limits <- c(zero , max(x.seq))
  } else {
    x.limits <- range(x.seq) 
  }
  
  # Default xlab ----
  if(is.null(xlab)){
    xlab <- paste0("Distance ", format(zero)) # zero cause it has units
    xlab <- sub("0 ", "", xlab) # erase 0 but leave units
  } 
  
  # Default ylab ----
  if(is.null(ylab)){
    if( Rdistance::is.points(x)){
      ylab <- "Observation density"
    } else {
      ylab <- "Probability of detection"
    }
  }
  
  # Main plot ----
  if(plotBars){
    if(x$w.lo != zero){
      ybarhgts <- c(NA,ybarhgts)
      xscl <- c(x$w.lo, xscl)
      # the following is because barplot draws the border
      # of the NA box when line density >= 0.  Makes no sense, but there it is.
      # This produces a line to 0 when w.lo > 0
      if( any(density > 0) ){
        warning("Line 'density' of bars cannot be positive when w.lo > 0. Set to 0 or negative.")
        density[ density > 0 ] <- -1
      }
      if( any(density == 0) ){
        # bars are supposed to be empty when density = 0, so reset to fill with background
        col[density == 0] = graphics::par()$bg
        density[density == 0] = -1
      }
    }
    bar.mids <- graphics::barplot( ybarhgts, 
                         width = dropUnits(xscl), 
                         ylim = y.lims, 
                         xlim = dropUnits(x.limits),
                         space = 0, 
                         density = density,
                         angle = angle,
                         col = col,
                         border = border,
                         xlab = xlab,
                         ylab = ylab,
                         ... )  
    xticks <- graphics::axTicks(1)
    graphics::axis( 1, at=xticks,  labels=xticks, line=.5, ... )
  } else {
    plot(1,1,type="n",
         ylim = y.lims, 
         xlim = x.limits, 
         xlab = xlab,
         ylab = ylab,
         bty = "n",
         ...)
  }
  
  #   Work out the colors, line types, and line widths ----
  nFunctions <- ncol(y)
  if( is.null(col.dfunc) ){
    # rainbow(1) is red, the default for one line
    col.dfunc <- grDevices::rainbow(nFunctions)
  } else if(length(col.dfunc) < nFunctions){
    col.dfunc <- rep(col.dfunc,ceiling(nFunctions/length(col.dfunc)))[1:nFunctions]
  }
  if( is.null(lty.dfunc) ){
    lty.dfunc <- rep(1,nFunctions)
  } else if(length(lty.dfunc) < nFunctions){
    lty.dfunc <- rep(lty.dfunc,ceiling(nFunctions/length(lty.dfunc)))[1:nFunctions]
  }
  if( is.null(lwd.dfunc) ){
    lwd.dfunc <- rep(2,nFunctions)
  } else if(length(lwd.dfunc) < nFunctions){
    lwd.dfunc <- rep(lwd.dfunc,ceiling(nFunctions/length(lwd.dfunc)))[1:nFunctions]
  }

  # Plot circles if called for ----
  if(circles){
    d <- Rdistance::distances(x)
    g <- apply(y, 2, FUN = function(y, x.seq, d){
      stats::approx(x.seq, y, xout = d)$y
    }
    , x.seq = x.seq
    , d = d )
    graphics::matpoints(d, g, pch = 1, lty = 1, col = 1, cex = 1)
  }
  
  # Draw distance functions ----
  for(i in 1:nFunctions){
    graphics::lines( x.seq, y[,i]
           , col=col.dfunc[i]
           , lwd=lwd.dfunc[i]
           , lty = lty.dfunc[i]
           , xpd = TRUE
    )
  }
  
  #   Add vertical lines at 0 and w if called for ----
  if(vertLines){
    graphics::lines( rep(x.seq[1], 2), c(0,y[1,1]), col=col.dfunc[1], lwd=lwd.dfunc[1],
           lty=lty.dfunc[1])
    graphics::lines( rep(x.seq[length(x.seq)], 2), c(0,y[length(x.seq), 1]), 
           col=col.dfunc[1], lwd=lwd.dfunc[1],
           lty=lty.dfunc[1] )
  }
  

  # Check convergence ----
  if( (x$likelihood != "smu") && x$convergence != 0 ){
    if( x$convergence == -1 ){
      mess <- "Solution failure"
    } else {
      mess <- "Convergence failure"
    }
    graphics::text( mean(x.seq), mean(y.lims), mess, cex=3, adj=.5, col="red")
    graphics::text( mean(x.seq), mean(y.lims), paste("\n\n\n", x$fit$message, sep=""), cex=1, adj=.5, col="black")
  }
  
  # Check for Probabilities > 1 ----
  if( !is.points(x) && any(y > 1) ){
    # some g(x) > 1, should'nt happen for line transects
    mess <- "Probabilities > 1"
    graphics::text( mean(x.seq), mean(y.lims), mess, cex=3, adj=.5, col="red")
    mess <- "g(x) > 1 should not happen"
    graphics::text( mean(x.seq), mean(y.lims), paste("\n\n\n", mess,sep=""), cex=1, adj=.5, col="black")
    mess <- paste0("Check g(", format(x$x.scl), ")= ", round(x$g.x.scl,2), " assumption")
    graphics::text( mean(x.seq), mean(y.lims), paste("\n\n\n\n\n", mess,sep=""), cex=1, adj=.5, col="black")
  } 
  
  # Check missing or <0 y values ----
  if( any(is.na(y)) | any(y < 0) ){
    #   invalid scaling, g0 is wrong
    mess <- "Probabilities < 0"
    graphics::text( mean(x.seq), mean(y.lims), mess, cex=3, adj=.5, col="red")
    mess <- paste("One or more parameters likely at boundary")
    graphics::text( mean(x.seq), mean(y.lims), paste("\n\n\n", mess,sep=""), cex=1, adj=.5, col="black")
  }
  
  # Legend ----
  if(legend & (x$nCovars != 0) ){
    for(j in 1:ncol(newdata)){
      if(is.numeric(newdata[,j])){
        newdata[,j]<-signif(newdata[,j], 3)
      }
    }
    nr <- nrow(newdata)
    v.names <- names(newdata)
    v.names <- rep(v.names, each=nr)
    v.vals <- c(as.matrix(newdata))
    leg <- matrix( paste(v.names, v.vals,sep="="), nr)
    Leg <- leg[,1]
    if( ncol(leg) >= 2){
      for(j in 2:ncol(leg)){
        Leg <- paste(Leg, leg[,j], sep=",") 
      }
    }
    
    
    #legend.factors <- vector(length = length(x$legend.names))
    
    graphics::legend('topright', legend = Leg, lty = lty.dfunc, lwd = lwd.dfunc, 
           col = col.dfunc, cex = 0.7)
  }
  
  # Clean up ----
  # x$yscl <- scaler   # this is g(x) / f(x) = ESW if lines. One for each row in newdata.  Might want this later.
  x$barHeights <- ybarhgts  # scaled to mean scaler.
  x$barWidths <- xscl
  x$predCovValues <- newdata
  
  invisible(x)
  
}

#' @title Set optimizing routine
#' 
#' @description
#' Checks that the maximum likelihood optimizing routine is appropriate 
#' for the requested likelihood function. That is, checks that gradient 
#' based maximization routines are only applied to smooth likelihoods, 
#' and that gradient-free methods are used for non-smooth likelihoods.
#' 
#' @inheritParams startLimits
#' 
#' @return If Rdistance options are changed, a list of the options and 
#' their original values.  The return can be used outside this routine 
#' to set options back to their state when this routine was entered.  
#' If no options changed, the return in NULL. 
#' 
#' @export
setOptimizer <- function(ml){
  
  out <- NULL
  optimizerAlgo <- getOption("Rdistance_optimizer")
  if( is.null( optimizerAlgo ) ){
    # Set default optimizer
    if( ml$likelihood %in% differentiableLikelihoods() ){
      optimizerAlgo <- "nlminb"
    } else {
      optimizerAlgo <- "optim_Nelder-Mead"
      nInts <- getOption("Rdistance_intEvalPts")
      if(nInts < 301){
        # bump up integral points
        options(Rdistance_intEvalPts = 301)
      }
      checkNEvalPts(getOption("Rdistance_intEvalPts")) # make sure coefs match
    }
  } else {
    # user has requested an alternate optimizer, check it.
    if( !(ml$likelihood %in% differentiableLikelihoods()) ){
      
      # Non-smooth likelihood case 
      if( !(optimizerAlgo %in% c("optim_Nelder-Mead", "optim_SANN", "hookeJeeves")) ){
            stop(paste("Gradient based optimization method"
                       , optimizerAlgo
                       , "cannot be used because likelihood"
                       , ml$likelihood
                       , "is not smooth (i.e., differentiable)."
                       , "Use 'options('Rdistance_optimizer' = 'optim_Nelder-Mead')'."
            ))
      }
    } else {
      
      # Smooth likelihood case
      # Check whether problem is univariate and Hooke-Jeeves is called for; HJ can't do univariate problems ----
      termLabs <- attr(stats::terms(ml$formula), "term.labels")
      termLabs <- termLabs[!grepl("groupsize\\(", termLabs)]
      if( (length(termLabs) == 0) && 
          (optimizerAlgo == "hookeJeeves") &&
          (ml$likelihood != "hazrate") ){
        stop(paste("Cannot estimate an intercept-only model using 'hookeJeeves'.",
                   "Reset optimizer with options(Rdistance_optimizer = 'nlminb'), or restart R",
                   "and re-attach Rdistance"))  
      } 
      
    }
  }
  
  optimizerAlgo
  
}

#' @title Brewer's Sparrow detection data frame
#' in Rdistance >4.0.0 format.
#' 
#' @name sparrowDf 
#' 
#' @description 
#' Detection data from line transect surveys for Brewer's sparrow on 72 transects 
#' located on a 4105 km^2 study area in central Wyoming collected by 
#' Dr. Jason Carlisle as part of his graduate work in the Wyoming Cooperative Fish & Wildlife
#' Research Unit in 2012. Each transect was 500 meters long.
#' 
#' @docType data
#' 
#' @format A rowwise tibble containing 72 rows and 9 columns, one of which 
#' is nested data frame of detections.  Each row represents
#' one transect.  The embedded data frame in column `detections` 
#' contains the detections made on the transect represented on that row. 
#' 
#' Column descriptions: 
#' \enumerate{ 
#'   \item `siteID`: Factor (72 levels), the transect identifier 
#'   for that row of the data frame. 
#'   \item `length`: The length, in meters \[m\], of each transect.  
#'   \item `observer`: Identity of the observer who surveyed the transect.
#'   \item `bare`: The mean bare ground cover (%) within 100 \[m\] of the transect.
#'   \item `herb`: The mean herbaceous cover (%) within 100 \[m\] of the transect.
#'   \item `shrub`: The mean shrub cover (%) within 100 \[m\] of the transect.
#'   \item `height`: The mean shrub height \[cm\] within 100 \[m\] of the transect.
#'   \item `shrubclass`: Shrub class factor.  Either "Low" when 
#'   shrub cover is < 10%, or "High" if cover >= 10%.  
#' }
#'  
#' The embedded data frame in column `detections` contains the following
#' variables:
#' \enumerate{
#'   \item `groupsize`: The number of individuals in the detected group.  
#'   \item `sightdist`: Distance \[m\] from observer to the detected group.  
#'   \item `sightangle`: Angle \[degrees\] from the transect 
#'   line to the detected group. Not bearing. Range 0 \[degrees\] to 90 \[degrees\].
#'   \item `dist`: Perpendicular, off-transect distance \[m\], from the
#'   transect to the detected group.  This is the distance used in analysis.
#'   Calculated using [perpDists()].  
#' }
#' 
#' @seealso [sparrowSiteData()], [sparrowDetectionData()],
#' [RdistDf()]
#' 
#' @source The Brewer's sparrow data are a subset of data collected 
#' by Jason Carlisle and various field technicians for his Ph.D. from the 
#' Department of Ecology, University of Wyoming, in 2017.  This portion of 
#' Jason's work was funded by the Wyoming Game and Fish Department through agreements 
#' with the University of Wyoming's  Cooperative Fish & Wildlife
#' Research Unit (2012). 
#' 
#' @examples
#' \dontrun{
#' # The following code generated 'sparrowDf'
#' data(sparrowDetectionData)
#' data(sparrowSiteData)
#' sparrowDf <- RdistDf(transectDf = sparrowSiteData
#'                    , detectionDf = sparrowDetectionData
#'                    , by = "siteID"
#'                    , pointSurvey = FALSE
#'                    , .effortCol = "length"
#'                     )
#' }
#' 
#' data(sparrowDf)
#' tidyr::unnest(sparrowDf, detections)  # only non-zero transects
#' Rdistance::unnest(sparrowDf) # with zero transects at the bottom
#' summary(sparrowDf,
#'   formula = dist ~ groupsize(groupsize)
#' )
#' 
#' @references Carlisle, J.D. 2017. The effect of sage-grouse conservation on
#' wildlife species of concern: implications for the umbrella species concept.
#' Dissertation. University of Wyoming, Laramie, Wyoming, USA.
#' 
#' Carlisle, J. D., and A. D. Chalfoun. 2020. The abundance of Greater 
#' Sage-Grouse as a proxy for the abundance of sagebrush-associated songbirds 
#' in Wyoming, USA. *Avian Conservation and Ecology* 15(2):16. 
#' \doi{10.5751/ACE-01702-150216}
#' 
NULL

#' @title Hermite expansion factors
#' 
#' @description Computes Hermite expansion terms for use in 
#' distance analysis. The Hermite (and other expansions) allow "wiggle" in
#' estimated distance functions. 
#'  
#' @inheritParams cosine.expansion
#'    
#' @details There are, in general, several expansions that can be called Hermite. Let \eqn{w = 4x - 2}{w = 4*x - 2}. 
#' Rdistance's Hermite expansions are:
#'   \itemize{
#'     \item **First term**: \deqn{h_1(w) = w + 2,}{h1(w) = w + 2,}
#'     \item **Second term**: \deqn{h_2(w) = w^2 - 4,}{h2(w) = w^2 - 4,}
#'     \item **Third term**: \deqn{h_3(w) = w^3 - 3w + 2,}{h3(w) = w^3 - 3*w + 2,}
#'     \item **Fourth term**: \deqn{h_4(w) = w^4 - 6w^2 + 8,}{h4(w) = w^4 - 6*w^2 + 8,}
#'   }
#'   The maximum number of expansion terms computed is 4.
#'   
#' @inherit cosine.expansion return
#'   
#' @seealso [dfuncEstim()]
#' , [cosine.expansion()]
#' , [sine.expansion()]
#' , [simple.expansion()].
#'   
#' @examples 
#' x <- matrix(seq(0, 1, length = 200), ncol = 1)
#' herm.expn <- hermite.expansion(x, 4)
#' plot(range(x), range(herm.expn), type="n")
#' matlines(x, herm.expn[,1,1:4], col=rainbow(4), lty = 1)
#' 
#' @export

hermite.expansion <- function(x, expansions){

    expansions <- as.integer(expansions)  
  
    if (expansions > 4){
        warning("Too many Hermite polynomial expansion terms. Only 4 used.")
        expansions = 4
    }
    
    if( expansions < 1 ) stop( "Number of expansions must be >= 1" )

    expansion = array(dim = c(nrow(x), ncol(x), expansions))
  
    x <- 4*x - 2

    expansion[,,1] = x + 2

    if(expansions >= 2){
        expansion[,,2] = x^2 - 4
    } 
    if(expansions >= 3){
        expansion[,,3] = x^3 - 3*x + 2
    } 
    if(expansions >= 4){
        expansion[,,4] =  x^4 - 6*x^2 + 8
    }         
    
    return(expansion)
            
}

#' @title Set number of cores 
#' 
#' @description Set the number of cores for parallel operations. 
#' Also convert integer 'parallel' to TRUE-FALSE for ease. 
#' 
#' @inheritParams abundEstim 
#' 
#' @return A list with components $parallel and $cores. $parallel 
#' is logical, T for parallel operations, F o.w.  $cores is the 
#' number of cores to use during parallel operations.  $cores is 
#' integer in the range 1, 2, ..., max(Available cores). 
#' 
#' @details Input \code{parallel} <= 0 is converted to 1. 
#' Input \code{parallel} > maxCores is converted to maxCores.
#' If input \code{parallel} is numeric, is first converted to integer
#' by rounding down. 
#' 

getNCores <- function(parallel){
  maxCores <- parallel::detectCores()
  if( is.logical(parallel) ){
    if( parallel ){
      cores2Use <- maxCores - 1
    } else {
      cores2Use <- 1
    }
  } else {
    if( parallel < 1 ){
      warning(paste0("Number of requested cores must be >= 1. Found "
                    , parallel
                    , ". Running on 1 core.")
              , immediate. = TRUE)
    } else if( parallel > maxCores ){
      warning(paste0("Number of requested cores must be <= "
                    , maxCores
                    , ". Found "
                     , parallel
                     , ". Running on "
                     , maxCores
                     ," cores.")
              , immediate. = TRUE)
      
    }
    if( parallel >= 2 ){
      cores2Use <- min(as.integer(parallel), maxCores)
      parallel <- TRUE
    } else {
      cores2Use <- 1
      parallel <- FALSE
    }
  }
  # At this point, parallel is T-F, cores2use is 1,...,max
  
  list(parallel=parallel, cores = cores2Use)
}

#' @title Estimate single-observer line-transect distance function
#' 
#' @description Fits a detection function to off-transect 
#' distances collected by a single observer. 
#' 
#' @inheritParams dfuncEstim
#' 
#' @param formula A standard formula object.  For example, `dist ~ 1`, 
#' `dist ~ covar1 + covar2`). The left-hand side (before `~`)
#' is the name of the vector containing off-transect or radial detection distances.  
#' The right-hand side contains the names of covariate 
#' vectors to fit in the detection
#' function, and potentially group sizes. Group sizes are specified 
#' by including `+ groupsize(<variable>)` in the RHS 
#' (see 'Group Sizes' section).
#' Covariates can be either detection level 
#' or transect level and can appear in  `data` or exist in the 
#' global working environment. Regular R scoping 
#' rules apply.  
#' 
#' 
#' @param likelihood String specifying the likelihood to fit. Built-in 
#' likelihoods at present are "halfnorm", "hazrate", and "negexp". 
#' 
#' @param w.lo Lower or left-truncation limit of the distances in distance data. 
#' This is the minimum possible off-transect distance. Default is 0.  If 
#' `w.lo` is greater than 0, it must have measurement units. 
#' See `help(unitHelpers)` for assistance assigning units.
#'  
#' @param w.hi Upper or right-truncation limit of the distances 
#' in `dist`. This is the maximum off-transect distance that 
#' could be observed. If unspecified (i.e., NULL), 
#' right-truncation is set to the maximum of the observed 
#' distances.  If `w.hi` is specified, it must have  
#' measurement units.  
#' See `help(unitHelpers)` for assistance assigning units.
#' 
#' @param expansions A scalar specifying the number of terms 
#' in `series` to compute. Depending on the series, 
#' this could be 0 through 5.  The default of 0 equates 
#' to no expansion terms of any type.  No expansion terms 
#' are allowed (i.e., `expansions` is forced to 0) if 
#' covariates are present in the detection function 
#' (i.e., right-hand side of `formula` includes
#' something other than `1`). 
#' 
#' @param series If `expansions` > 0, this string 
#' specifies the type of expansion to use. Valid values at 
#' present are 'simple', 'hermite', and 'cosine'. 
#' 
#' @param x.scl The x coordinate (a distance) at which the 
#'   detection function will be scaled.  `g.x.scl` can be a distance
#'   or the string "max".  
#'   When `x.scl` is specified (i.e., not 0 or "max"), it must have measurement 
#'   units assigned. 
#'   See `help(unitHelpers)` for assistance assigning units. 
#'   
#' @param g.x.scl Height of the distance function at coordinate `x`. 
#'   The distance function 
#'   will be scaled so that g(`x.scl`) = `g.x.scl`. 
#'   If `g.x.scl` is not 
#'   a data frame, it must be a numeric value (vector of length 1) 
#'   between 0 and 1. 
#' 
#' @param warn A logical scalar specifying whether to issue 
#' an R warning if the estimation did not converge or if one 
#' or more parameter estimates are at their boundaries.  
#' For estimation, `warn` should generally be left at
#' its default value of `TRUE`.  When computing bootstrap 
#' confidence intervals, setting `warn = FALSE` 
#' turns off annoying warnings when an iteration does 
#' not converge.  Regardless of `warn`, after 
#' completion all messages about 
#' convergence and boundary conditions are printed 
#' by `print.dfunc`, `print.abund`, and 
#' `plot.dfunc`. 
#' 
#' @param outputUnits A string specifying the symbolic measurement 
#' units for results. Valid units are listed in `units::valid_udunits()`.
#' The strings for common distance symbolic units are: 
#' "m" - meters, "ft" - feet, "cm" - centimeters, "mm" - 
#' millimeters, "mi" - miles, "nmile" - 
#' nautical miles ("nm" is nano meters), "in" - inches, 
#' "yd" - yards, "km" - kilometers, "fathom" - fathoms, 
#' "chains" - chains, and "furlong" - furlongs.  
#' If `outputUnits` is unspecified (NULL),
#' output units will be the same as those on 
#' distances in `data`.  
#' 
#' @param asymptoticSE Logical variable for whether to calculate 
#' asymptotic standard errors. The default (TRUE) estimates an
#' asymptotic variance-covariance matrix for parameters based on the 
#' likelihood's Hessian (2nd derivative). If maximization 
#' has been performed by Nlminb or HookesJeeves, the asymptotic 
#' Hessian is estimated using numeric second derivatives 
#' of the likelihood at the maximum likelihood solution. If 
#' maximization was performed by Optim, the last Hessian of 
#' the maximization is returned 
#' by Optim and used
#' (see [varcovarEstim()] and [secondDeriv()]). 
#' Asymptotic standard errors will not be estimated if 
#' `asymptoticSE = FALSE`. If not estimated, 
#' bootstrap iterations will run faster because the numeric Hessian, 
#' which is discarded during bootstrapping,
#' will not be calculated every iteration.
#'
#'
#' @section Group Sizes: 
#' To specify non-unity group sizes,  use `groupsize()`
#' on the RHS of `formula`. When group sizes are not all 1, they must appear in a column 
#' of the 'detections' list-column of `data`. 
#' For example, `d ~ habitat + groupsize(number)` specifies  
#' distances in column `d`, one covariate 
#' named `habitat`, and that column `number` 
#' contains the number of individuals 
#' associated with each detection.  If group sizes are not specified, 
#' all group sizes are assumed to be 1.
#' 
#' @section Contrasts: 
#' Factor contrasts in `Rdistance` are specified 
#' the same way as in `lm` or `glm`. 
#' By default, `Rdistance` uses 
#' contrasts in `getOption("contrasts")`.  To change contrasts, use a statement
#' like `options(contrasts = c(unordered = "contr.SAS", 
#' ordered = "contr.poly"))`.  Or, to set contrasts for a 
#' specific factor in the input data frame, use 
#' `contrasts(df$A) <- "contr.sum"` or similar. 
#' See [contrasts()] or the `contrasts.arg`
#' of [model.matrix()].
#' 
#' @section Transect types: 
#' `Rdistance` accommodates two kinds of transects: continuous and point.  
#' Detections can occur at any point on continuous transects.
#' `Rdistance` calls these 'line-transects' even though routes are not
#' necessarily a straight line.
#' On point transects, detections occur at a series of stops 
#' (points). `Rdisance` calls these point-transects. Transects are the basic 
#' sampling unit in both cases. `Rdistance` assumes each row of `data` 
#' contains information from one transect. See [RdistDf()] for 
#' more details. 
#' 
#' @section Measurement Units: 
#' As of `Rdistance` version 3.0.0, measurement units are 
#' require on all physical distances. 
#' Requiring units ensures that internal calculations and results 
#' (e.g., ESW and abundance) are correct 
#' and that output units are clear.   
#' Physical distances are required on
#' off-transect distances, radial distances, truncation distances 
#' (`w.lo`, unless it is zero; and `w.hi`, unless it is NULL), 
#' scale locations (`x.scl`, unless it is zero),
#' line-transect lengths, and study area size. All units are 
#' 1-dimensional except those on study area, which are 2-dimensional. 
#' 
#' Physical measurement units can vary. For example, 
#' off-transect distances can be meters ("m"), `w.hi` can be inches ("in"), 
#' and `w.lo` can be kilometers ("km").  Internally, all distances are 
#' converted to the units specified by `outputUnits` 
#' (or the units of input distances if 
#' `outputUnits` is NULL), and 
#' all output is reported 
#' in units of `outputUnits`. Valid conversions must exist between 
#' units or an error is thrown (e.g., meters cannot convert 
#' into hectares).
#'   
#' Measurement units can be assigned using one of Rdistance's 
#' unit helper routines (see `help(unitHelpers)`), Rdistance's 
#' [setUnits()] function, or [units::set_units()] 
#' See [units::valid_udunits()]
#' for a list of valid symbolic units. 
#' 
#' If measurements are truly unit-less, or measurement units are unknown, 
#' set `options(Rdist_requireUnits = FALSE)`.  This suppresses 
#' all unit checks and conversions.  Users are on their own here
#' and must make sure all inputs are scaled correctly so that internal 
#' computations are correct and output units are known. 
#'  
#' @details
#' Optimization and estimation controls can be modified using `options()`. 
#' See [RdistanceControls()].
#' 
#' @return  An object of class 'dfunc' with the following components:
#' 
#' * `par`: The vector of estimated parameter values. 
#'     Length of this vector is the sum of the following: 
#'    1. The number of columns of the design matrix. This equals the 
#'       number of covariates in the distance function plus one for the 
#'       intercept, assuming an intercept is included.
#'    1. The number of constant parameters in the distance function.
#'        Constant parameters are those not related to covariates.  For example, 
#'        the exponent 'k' parameter for hazard rate likelihood, or 
#'        the mixing fraction 'p' for the oneStep likelihood. This can be zero.
#'    1. The number of expansion functions called for.  This equals 
#'        the input `expansions`. 
#' *  `loglik`: The maximized value of the log likelihood.
#' *  `convergence`: The convergence code. This code 
#'     is returned by the optimizing routine (e.g., `optim` or `nlminb`).  
#'     Values other than 0 indicate suspect convergence.
#' *  `message`: If maximization did not converge (`convergence != 0`),
#'     this is the reason given by the optimizing routine.  
#' *  `varcovar`: The variance-covariance matrix for coefficients 
#'     of the distance function, either estimated by the inverse of 
#'     the fit's Hessian or by bootstrapping.  
#'     If the likelihood is smooth (i.e., those listed by 
#'     `Rdistance:::differentiableLikelihoods())`, 
#'     Rdistance initially estimates the variance-covariance matrix using the 
#'     second derivative of the log likelihood surface 
#'     at the final estimates, where second derivatives are estimated by 
#'     numeric differentiation (in routine [secondDeriv()]. 
#'     The variance-covariance matrix is re-set to NULL 
#'     if the Hessian is not positive-definite.  If bootstrap resampling
#'     has been performed (using [abundEstim()]), the variance-covariance
#'     matrix is re-estimated using the bootstrap values of parameters
#'     and automatically reset.  
#'     Error estimates derived from bootstrapping are generally 
#'     preferable to the asymptotic estimates, hence the automatic 
#'     re-set.  
#' *  `limits`: A list containing the lower and upper limits of parameters.
#' *  `evaluations`: The number of likelihood evaluations performed by the 
#'   optimizer.
#' *  `mf`: An R 'model frame' containing the detections (within the strip 
#'   or circle) used in the fit, covariates specified in the formula, 
#'   and groupsizes.  Column 'dist' contains the 
#'   observed distances. The intercept, if included in the model, is not 
#'   included as a column in this model frame. (Test whether an intercept 
#'   is included using `attr(terms(return$mf), "intercept")`). 
#'   Column `offset(...)` contains group sizes associated with 
#'   the values of `dist`. Name of the group size column is "offset(...)", 
#'   not "groupsize(...)", so that group sizes can be treated offsets in 
#'   other R routines.  The `mf` component is a proper `model.frame` and contains
#'   both terms and contrasts attributes. This model frame 
#'   contains only non-missing distances between `w.lo` and `w.hi`. 
#' *  `data`: The original nested data frame subset to information required 
#'   to complete distance estimation.  This data frame contains information 
#'   on replication (i.e., rows are sites and are re-sampled during bootstrapping),
#'   missing distances, missing transect lengths, and distances outside the observation 
#'   strip (below `w.lo` or above `w.hi`). 
#' *  `formula`: The distance function's formula.
#' *  `dataName`: Name of the original nested data frame.
#' *  `likelihood`: The name of the likelihood fitted to observation 
#'   distances. 
#' *  `w.lo`: Left-truncation value used during the fit.
#' *  `w.hi`: Right-truncation value used during the fit.
#' *  `expansions`: The number of expansion terms.
#' *  `series`: The type of expansion used during estimation. This is 
#'   only relevant if `expansions > 0`.
#' *  `x.scl`: The distance at which the function has been scaled to some value.  
#'   This is the *x* at which g(*x*) = `g.x.scl`. 
#' *  `g.x.scl`: The height of the distance function at distance `x.scl`. 
#' *  `outputUnits`: A list of type `symbolic_units` containing the 
#'   physical measurement units used during estimation. 
#' *  `asymptoticSE`: A logical scalar indication whether the 
#'   variance-covariance matrix in component `varcovar` is 
#'   asymptotic (TRUE; estimated from the Hessian) or bootstrap (FALSE; 
#'   estimated by bootstrap resampling).
#' *  `optimizer`: The optimizing routine used.
#' *  `call`: The original function call.
#' *  `nCovars`: The number of exogenous covariates fitted in the 
#'   distance function. Does not include the intercept. 
#' * `LhoodType`: The type of likelihood fitted. Currently, only 'parametric' 
#'   types are fitted.  
#'     
#'     
#'     
#' @references Buckland, S.T., D.R. Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers,
#'    and L. Thomas. (2001) *Introduction to distance sampling: estimating
#'    abundance of biological populations*. Oxford University Press, Oxford, UK.
#'     
#' @seealso [abundEstim()], [autoDistSamp()].
#' Likelihood-specific help files (e.g., [halfnorm.like()]). 
#' 
#' @examples 
#' # Load example sparrow data (line transect survey type)
#' data(sparrowDf)
#' 
#' dfunc <- dfuncEstim(data = sparrowDf
#'                   , formula = dist ~ 1)
#' dfunc
#' plot(dfunc)                   
#'
#' @export
dE.single <- function( data
                      , formula
                      , likelihood = "halfnorm"
                      , w.lo = setUnits(0,"m")
                      , w.hi = NULL
                      , expansions = 0
                      , series = "cosine"
                      , x.scl = w.lo
                      , g.x.scl = 1
                      , warn = TRUE
                      , outputUnits = NULL
                      , asymptoticSE = TRUE
 ){

  verboseLevel <- getOption("Rdistance_verbosity")
  
  # Parse the formula and make a model list ----
  # all parameters go into parseModel because they need to become
  # components for the output list, not just formula.
  # All checking is done in parseModel(), including 
  # check of units (via checkUnits()).
  
  modelList <- Rdistance::parseModel(data = data
                          , formula = formula
                          , likelihood = likelihood
                          , w.lo = w.lo
                          , w.hi = w.hi
                          , expansions = expansions
                          , series = series
                          , x.scl = x.scl
                          , g.x.scl = g.x.scl
                          , outputUnits = outputUnits
                          , asymptoticSE = asymptoticSE
                        )
  strt.lims <- Rdistance::startLimits(modelList)
  
  if(verboseLevel >= 2){
    cat(colorize("Starting values ----\n", col="red"))
    cat(colorize("   Start: "))
    cat(paste(paste(names(strt.lims$start), "=", colorize(strt.lims$start)), collapse=", "), "\n")
    cat(colorize("Lo Limit: "))
    cat(paste(paste(names(strt.lims$low), "=", colorize(strt.lims$low)), collapse=", "), "\n")
    cat(colorize("Hi Limit: "))
    cat(paste(paste(names(strt.lims$high), "=", colorize(strt.lims$high)), collapse=", "), "\n")
  }

  # Check whether need to use non-gradient optimizer ----
  modelList$optimizer <- setOptimizer(modelList)
  
  # Perform optimization
  fit <- mlEstimates( ml = modelList
                    , strt.lims = strt.lims
                    )

  # Assemble results
  ans <- c(fit, modelList)
  class(ans) <- "dfunc"

  if ( ans$likelihood != "Gamma" ){
    # not absolutely necessary. 
    # Could estimate these later in print and plot methods.
    # but this saves a little time.
    gx <- gxEstim(ans)
    ans$x.scl <- gx$x.scl
    ans$g.x.scl <- gx$g.x.scl
  } 

  # ---- Check parameter boundaries ----
  fuzz <- getOption("Rdistance_fuzz")
  if (ans$convergence != 0) {
    if (warn) warning(ans$message)
    low.bound <- FALSE
    high.bound <- FALSE
  } else {
    low.bound <- ans$par <= (ans$limits$low + fuzz)
    high.bound <- ans$par >= (ans$limits$high - fuzz)
  }
  if (any(low.bound)) {
    # if we are here, model converged but to limit
    ans$convergence <- -1
    messL <- paste(paste(strt.lims$names[low.bound], "parameter at lower boundary.")
    , collapse = "; ")
    ans$message <- messL
    if (warn) warning(ans$message)
    } 
  else {
    messL <- NULL
  }
  if (any(high.bound)) {
    ans$convergence <- -1
    messH <- paste(paste(strt.lims$names[high.bound], "parameter at upper boundary.")
    , collapse = "; ")
    ans$message <- c(messL, messH)
    if (warn) warning(ans$message)
  }
  
  ans

} # end function

#' @title Print abundance estimates
#' 
#' @description Print an object of class \code{c("abund","dfunc")} 
#' produced by \code{abundEstim}.
#' 
#' @param x An object output by \code{abundEstim}.  This is a distance 
#' function object augmented with abundance estimates, and has 
#' class \code{c("abund", "dfunc")}.
#'   
#' @param \dots Included for compatibility to other print methods.  
#' Ignored here.
#' 
#' @return 0 is invisibly returned
#' 
#' @seealso \code{\link{dfuncEstim}}, \code{\link{abundEstim}}, 
#' \code{\link{summary.dfunc}}, \code{\link{print.dfunc}}, 
#' \code{\link{summary.abund}}
#' 
#' @examples
#' # Load example sparrow data (line transect survey type)
#' data(sparrowDf)
#' 
#' # Fit half-normal detection function
#' dfunc <- sparrowDf |> dfuncEstim(formula=dist~groupsize(groupsize))
#' 
#' # Estimate abundance given a detection function
#' fit <- abundEstim(object = dfunc
#'                 , area = setUnits(4105, "km^2")
#'                 , ci = NULL)
#' print(fit)
#' summary(fit)
#' 
#' \dontrun{
#' # Bootstrap confidence intervals (500 iterations)
#' # Requires ~4 min
#' fit <- abundEstim(object = dfunc
#'                 , area = setUnits(4105, "km^2")
#'                 , ci = 0.95
#'                 , plot.bs = TRUE
#'                 , showProgress = TRUE)
#' print(fit)
#' summary(fit)
#' }
#' 
#' @export

print.abund <- function( x
                       , ... ){

  print.dfunc( x, ... )
  cat("\n")
  hasCI <- all(!is.null(x$density.ci))
  
  # ---- Density printout ----
  mess <- c("Density in sampled area:")
  ptEst <- colorize(format(x$estimates$density))
  mess <- paste(mess, ptEst)
  cat(paste0(mess, "\n"))

  # ---- Abundance printout ----
  mess <- paste0( "Abundance in ", format(x$estimates$area), " study area:")
  ptEst <- colorize(format(x$estimates$abundance))
  mess <- paste(mess, ptEst)
  cat(paste0(mess, "\n"))

  invisible(0)
}

#' @title Add color to result if terminal accepts it
#' 
#' @description Add ANSI color to a string using the 
#' \code{crayon} package, if the R environment accepts color. 
#' This function is needed because some  
#' output cannot be colorized.  Color determination 
#' is made by \code{crayon::has_color()}. 
#' 
#' Rdistance results often include units, e.g., "25 [m]". 
#' Only colorize the numbers, not the units.  Everything between the 
#' first set of square brackets (\code{[...]}) is NOT colorized. 
#' Subsequent sets of brackets (e.g., "25 [m] + 30 [ft]") ARE colorized
#' (i.e., "[ft]" is color). 
#' 
#' @param STR A vector of strings to colorize. 
#' 
#' @param col A string specifying the desired foreground color.  
#' This is passed straight to \code{crayon::style} and so must be 
#' recognized as one of the 8 base crayon colors. i.e., 
#' "black", "red", "green", "yellow", "blue", "magenta", "cyan", 
#' "white", and "silver" (silver = gray). By default, numbers 
#' are styled in "green".
#' 
#' @param bg A string specifying the desired background color. 
#' Must be one of "bgBlack", "bgRed", "bgGreen", "bgYellow", "bgBlue"
#' "bgMagenta", "bgCyan", or "bgWhite".  By default, no background 
#' is applied. 
#' 
#' @return If color is not allowed in the terminal, the input 
#' string is returned unperturbed.  If color is allowed, the input 
#' string is returned with color and background ANSI code surrounding 
#' the initial part of the string from character 1 to the character 
#' before the [ in the first pair of []. 
#' 
#' @seealso \code{crayon::style}
#' 
#' 
# Do not export. In Rdistance namespace only
#
colorize <- function(STR, col=NULL, bg = NULL){
  if(crayon::has_color()){
    if( is.null(col) ){
      col <- "green"
    }
    uLoc <- regexpr("\\[(?:.(?!\\[))+\\]", STR, perl = TRUE)  
    u <- substring(STR, uLoc, uLoc + attr(uLoc, "match.length"))
    u[is.na(u)] <- ""
    uLoc <- ifelse(uLoc < 0, nchar(STR)+1, uLoc)
    STR1 <- substring(STR, 1, uLoc - 1)  # before [*]
    STR2 <- substring(STR, uLoc + nchar(u)) # after [*]
    STR2[is.na(STR2)] <- ""
    STR1 <- crayon::style(STR1, as = col, bg = bg)
    STR2 <- crayon::style(STR2, as = col, bg = bg)
    STR <- paste0(STR1, u, STR2)
  } 
  STR
}

# .onLoad ----

.onLoad <- function(libname, pkgname){

  # To speed numeric integration, store Simpson coefs in options()
  nEvalPts <- 101  # MUST BE ODD
  intCoefs <- simpsonCoefs( nEvalPts )

  # ** If you add an option here, MAKE SURE you add it 
  # to op.Rdist in function .onUnload below. **
  
  op <- options()
  op.Rdist <- list(
      Rdistance_evalMax   = 2000
    , Rdistance_maxIters  = 1000
    , Rdistance_likeTol   = (.Machine$double.eps)^(1/1.75)
    , Rdistance_coefTol   = (.Machine$double.eps)^(1/2)
    , Rdistance_hessEps   = (.Machine$double.eps)^(1/1.75)
    , Rdistance_trace     = 0
    , Rdistance_requireUnits = TRUE
    , Rdistance_maxBSFailPropForWarning = 0.2
    , Rdistance_negInf    = -1 / .Machine$double.neg.eps
    , Rdistance_posInf    =  1 / .Machine$double.neg.eps
    , Rdistance_fuzz      = 100 * .Machine$double.eps
    , Rdistance_zero      = .Machine$double.eps
    , Rdistance_warn      = FALSE
    , Rdistance_intEvalPts= nEvalPts  
    , Rdistance_intCoefs  = intCoefs # not really needed; Set in 'checkNEvalPts'
    , Rdistance_verbosity = 0
    , Rdistance_knownLikelihoods= c("halfnorm"
                                  , "negexp"
                                  , "hazrate"
                                  , "oneStep"
                                  , "Gamma"
                                  , "triangle"
                                  , "huber"
                                    )
  )
  toset <- !(names(op.Rdist) %in% names(op))
  if (any(toset)) options(op.Rdist[toset])
  
  invisible()
}

.onUnload <- function(libpath){
  
  # un-option Rdist options by full name, not just grep("Rdistance_", names(op))
  # in case user has an option starting with "Rdistance_".
  op.Rdist <- list(
      "Rdistance_evalMax"   = NULL
    , "Rdistance_maxIters"  = NULL 
    , "Rdistance_likeTol"   = NULL
    , "Rdistance_coefTol"   = NULL
    , "Rdistance_hessEps"   = NULL
    , "Rdistance_trace"     = NULL
    , "Rdistance_requireUnits" = NULL
    , "Rdistance_maxBSFailPropForWarning" = NULL
    , "Rdistance_negInf"    = NULL
    , "Rdistance_posInf"    = NULL
    , "Rdistance_fuzz"      = NULL
    , "Rdistance_zero"      = NULL
    , "Rdistance_warn"      = NULL
    , "Rdistance_intEvalPts"= NULL
    , "Rdistance_intCoefs"  = NULL
    , "Rdistance_knownLikelihoods" = NULL
    , "Rdistance_verbosity" = NULL
  )
  
  options(op.Rdist)
  
  invisible()
}


# .onAttach ----

.onAttach <- function(libname, pkgname){
  
  # Startup message
  v <- utils::packageVersion("Rdistance")  # this requires utils package, which is base
  
  packageStartupMessage(paste0(crayon::bold(
                                crayon::blue("Rdistance")
                               ), " (v", v ,")"))
  
  invisible()
}

#' @title Sage Thrasher detection data 
#' 
#' @name thrasherDetectionData
#' 
#' @description  
#' Point transect data collected in central Wyoming from 120 points
#' surveyed for Sage Thrashers by the Wyoming Cooperative Fish & Wildlife Research Unit in 2013.
#' 
#' @docType data
#' 
#' @format A data.frame containing 193 rows and 3 columns.  Each row represents
#' a detected group of thrashers.  Column descriptions: 
#' \enumerate{ 
#'   \item `siteID`: Factor (120 levels), the site or point where the detection
#'   was made.  
#'   \item `groupsize`: Number, the number of individuals within
#'   the detected group.  
#'   \item `dist`: Number, the radial distance (m) from
#'   the transect to the detected group.  This is the distance used in analysis.
#' }
#' 
#' @seealso [thrasherSiteData()]
#' 
#' @source The Sage Thrasher data are a subset of the data collected 
#' by Jason Carlisle and various field technicians for his Ph.D. from the 
#' Department of Ecology, University of Wyoming, in 2017.  This portion of 
#' Jason's work was funded by the Wyoming Game and Fish Department through agreements 
#' with the University of Wyoming's  Cooperative Fish & Wildlife
#' Research Unit (2012). 
#' 
#' @references Carlisle, J.D. 2017. The effect of sage-grouse conservation on
#' wildlife species of concern: implications for the umbrella species concept.
#' Dissertation. University of Wyoming, Laramie, Wyoming, USA.
#' 
#' Carlisle, J. D., A. D. Chalfoun, K. T. Smith, and J. L. Beck. 2018. 
#' Nontarget effects on songbirds from habitat manipulation for Greater 
#' Sage-Grouse: Implications for the umbrella species concept. 
#' *The Condor: Ornithological Applications* 120:439–455. 
#' \doi{10.1650/CONDOR-17-200.1}
#' 
NULL

#' @title Distance function starting values and limits
#' 
#' @description 
#' Returns starting values and limits (boundaries) for the parameters of 
#'   distance functions.  This function is called by 
#'   other routines in `Rdistance`, and is not intended to 
#'   be called by the user. Replace this function in the global 
#'   environment to change boundaries and starting values. 
#'
#' @param ml Either a Rdistance 'model frame' or an Rdistance 
#' 'fitted object'.  Both are of class "dfunc". 
#' Rdistance 'model frames' are lists containing components 
#' necessary to estimate a distance function, but no estimates. 
#' Rdistance 'model frames' are typically
#' produced by calls to [parseModel()]. 
#' Rdistance 'fitted objects'
#' are typically produced by calls to [dfuncEstim()].
#' 'Fitted objects' are 'model frames'
#' with additional components such as the parameters estimates, 
#' log likelihood value, convergence information, and the variance-
#' covariance matrix of the parameters. 
#' 
#' 
#'   
#' @return A list containing the following components
#'   \item{start}{Vector of starting values for parameters of the likelihood and expansion terms. }
#'   \item{lowlimit}{Vector of lower limits for the likelihood parameters and expansion terms.}
#'   \item{uplimit}{Vector of upper limits for the likelihood parameters and expansion terms.}
#'   \item{names}{Vector of names for the likelihood parameters and expansion terms.}
#'   
#' The length of each vector in the return is:
#'   `(Num expansions) + 1 + 1*(like %in% c("hazrate")) + (Num Covars)`.
#'  
#' @seealso [dfuncEstim()]
#' @examples 
#'   data(sparrowDf)
#'   
#'   # Half-normal start limits
#'   modList <- parseModel(
#'        data = sparrowDf
#'      , formula = dist ~ 1
#'      , likelihood = "halfnorm"
#'   )
#'   startLimits(modList)
#'   
#'   # Half-normal with expansions
#'   modList <- parseModel(
#'        data = sparrowDf
#'      , formula = dist ~ 1
#'      , likelihood = "halfnorm"
#'      , expansions = 3
#'   )
#'   startLimits(modList)
#'   
#'   # Hazard rate start limits
#'   modList$likelihood <- "hazrate"
#'   startLimits(modList)
#'   
#'   # Neg exp start limits
#'   modList$likelihood <- "negexp"
#'   startLimits(modList)
#'   
#'   
#' @export
startLimits <- function( ml ){

  # All start limit functions handled by likelihood-specific function
  like.sl <- match.fun( paste(ml$like, ".start.limits", sep="") )
  ans <- like.sl(ml)

  ans
  
}

#' @title Integrate Line-transect Huber function 
#' 
#' @description
#' Compute exact integral of the 'huber' distance function for line 
#' transects. 
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' 
#' @seealso [integrateNumeric()]; [integrateNegexpLines()]; 
#' [integrateHalfnormLines()] 
#' 
#' @examples
#' 
#' w <- 250
#' T1 <- 160
#' T2 <- 20
#' p <- 0.03
#' obj <- matrix(c(T1,T2,p), 1, 3)
#' 
#' integrateHuberLines(obj
#'   , w.lo = units::set_units(0,"m")
#'   , w.hi = units::set_units(w,"m")
#'   , Units = "m")
#'   
#' # same
#' huber.cumFunc(w,T1,T2,p,w)
#' 
#' # check by numeric integration
#' hubLike <- function(d, T1, T2, p, wl, wh) { 
#'   y <- huber.like(a = c(log(T1), T2, p)
#'            , dist = d - wl
#'            , covars = matrix(1, length(d))
#'            , w.hi = wh - wl)$L.unscaled
#'   y
#' }
#' integrate(hubLike, lower = 0, upper = w, T1 = T1
#'         , T2 = T2, p = p, wl = 0, wh = w)
#'   
#' @export
#' 
integrateHuberLines <- function(object
                              , newdata = NULL
                              , w.lo = NULL
                              , w.hi = NULL
                              , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    
    y <- stats::predict(object, newdata = newdata, type="dfunc")
    
    object <- stats::predict(object = object
                        , newdata = newdata
                        , type = "parameters"
    )
  } 
  
  theta1 <- setUnits(object[,1], Units)
  theta2 <- setUnits(object[,2], Units)
  p      <- setUnits(object[,3], "1")
  w      <- rep(w.hi - w.lo, nrow(object))

  outArea <- huber.cumFunc(x = w
                         , t1 = theta1 
                         , t2 = theta2 
                         , p = p 
                         , w = w
  )

  outArea 
  
}

#' @title Summarize abundance estimates
#' 
#' @description Summarize an object of class \code{c("abund","dfunc")} 
#' that is output by \code{abundEstim}.
#' 
#' @inheritParams print.abund
#'
#' @inheritParams summary.dfunc
#'    
#' @details
#' If the proportion of bootstrap iterations that failed is 
#' greater than \code{getOption("Rdistance_maxBSFailPropForWarning")}, 
#' a warning about the validity of CI's is issued and 
#' a diagnostic message printed.  Increasing this option to a number greater 
#' than 1 will kill the warning (e.g., \code{options(Rdistance_maxBSFailPropForWarning = 1.3)}), 
#' but ignoring a large number of non-convergent
#' bootstrap iterations may be a bad idea (i.e., validity of the CI is 
#' questionable). The default value for \code{Rdistance_maxBSFailPropForWarning}
#' is 0.2.
#' 
#' @param \dots Included for compatibility to other print methods.  
#' Ignored here.
#' 
#' @return 0 is invisibly returned.
#' 
#' @seealso \code{\link{dfuncEstim}}, \code{\link{abundEstim}}, 
#' \code{\link{summary.dfunc}}, \code{\link{print.dfunc}}, 
#' \code{\link{print.abund}}
#' 
#' @examples
#' # Load example sparrow data (line transect survey type)
#' data(sparrowDf)
#' 
#' # Fit half-normal detection function
#' dfunc <- sparrowDf |> dfuncEstim(formula=dist ~ 1 + offset(groupsize))
#' 
#' # Estimate abundance given the detection function
#' fit <- abundEstim(dfunc
#'                 , area = setUnits(4105, "km^2")
#'                 , ci=NULL)
#' summary(fit) # No confidence intervals
#'                 
#' \dontrun{
#' # With bootstrap confidence intervals 
#' # Requires ~3 min to complete
#' fit <- abundEstim(dfunc
#'                 , area = setUnits(4105, "km^2")
#'                 , ci=0.95)
#' 
#' summary(fit)
#' }
#' @export

summary.abund <- function( object
                         , criterion="AICc"
                         , ... ){
  
  summary.dfunc( object = object, criterion=criterion )
  cat("\n")
  hasCI <- !is.null(object$B) && (nrow(object$B) > 0)
  ests <- object$estimates
  
  # ---- Groupsize printout ----
  gs <- Rdistance::groupSizes(object)
  mess <- format(c(
                  "Surveyed Units:"
                , "Individuals seen:"
                , "Average group size:"
                , "Group size range:"), justify = "right")
  mess[1] <- paste0(" ", mess[1]) # pesky " " that happens with cat and \n
  avgGs <- c(
             colorize(format(ests$surveyedUnits))
           , paste( colorize(format(ests$nSeen)), "in", colorize(format(ests$nGroups)), "groups")
           , colorize( format( ests$avgGroupSize ))
           , paste(colorize(format( min(gs, na.rm = TRUE) ))
                          , "to"
                          , colorize(format( max(gs, na.rm = TRUE) ))))
  mess <- paste(mess, avgGs)
  cat(paste(mess, "\n"))
  
  if(hasCI){
    cat("\n")
  }
  
  # ---- Density printout ----
  if( hasCI ){
    mess <- c("Density in sampled area:", paste0(object$ci*100, "% CI:"))
    mess <- format(mess, justify = "right")
    mess[2] <- substring(mess[2], 2) # remove pesky " " that happens with cat and \n
    ci <- paste( colorize(format(ests$density_lo)), 
                 "to", 
                 colorize(format(ests$density_hi)) )
    ptEst <- colorize( format(ests$density) )
    mess <- paste(mess, c(ptEst, ci))
  } else {
    mess <- c("Density in sampled area:")
    ptEst <- colorize( format(ests$density) )
    mess <- paste(mess, ptEst)
  }
  cat(paste0(mess, "\n"))

  # ---- Abundance printout ----
  if(hasCI){
    cat("\n")  # blank line between for readability
  }
  if( hasCI ){
    mess <- c(paste0( "Abundance in ", format(ests$area), " study area:"), 
                      paste0(object$ci*100, "% CI:"))
    mess <- format(mess, justify = "right")
    mess[2] <- substring(mess[2], 2) # remove pesky " " that happens with cat and \n
    ci <- paste( colorize(format(ests$abundance_lo)), 
                 "to", 
                 colorize(format(ests$abundance_hi)) )
    ptEst <- colorize(format(ests$abundance))
    mess <- paste(mess, c(ptEst, ci))
  } else {
    mess <- paste0( "Abundance in ", format(ests$area), " study area:")
    ptEst <- colorize(format(ests$abundance))
    mess <- paste(mess, ptEst)
  }
  cat(paste0(mess, "\n"))
  
  if( hasCI ){
    nItersConverged <- sum(!is.na(object$B$density))
    if(nItersConverged < nrow(object$B)) {
      cat(paste("CI based on", nItersConverged, "of", nrow(object$B), 
                "successful bootstrap iterations\n"))
      convRatio <- nItersConverged / nrow(object$B)
      if((1.0-convRatio) > getOption("Rdistance_maxBSFailPropForWarning")) {
        warning("The proportion of non-convergent bootstrap iterations is high.", immediate. = TRUE)
        cat(paste0("The proportion of non-convergent bootstrap iterations exceeds ",
                  getOption("Rdistance_maxBSFailPropForWarning"), ".\n",
                  "It would be good to figure out why this happened (low detections, unstable dfunc form, etc.),\n",
                  "inspect the $B component of the abundance object (e.g., hist(object$B$density)), and decide whether the bootstrap CI is valid.\n"))
      }
    }
  }

  invisible(0)
}

#' @title Parse Rdistance model
#' 
#' @description 
#' Parse an 'Rdistance' formula and produce a list containing all model 
#' parameters.
#' This routine is not normally called directly by the user, but 
#' it might be helpful in simulations.  It is called 
#' internally from the model estimation routines.
#' 
#' @inheritParams dfuncEstim
#' @inheritParams dE.single
#' 
#' 
#' @return An Rdistance model frame, which is an object of class 
#' "dfunc". 
#' Rdistance model frames are lists containing distance 
#' model components but not estimates.  Model frames contain 
#' everything necessary to fit an Rdistance mode, such as covariates, 
#' minimum and maximum distances, the form of the likelihood, 
#' number of expansions, etc.  Rdistance model frames contain a 
#' subset of fitted Rdistance model components. 
#' 
#' @seealso \code{\link{RdistDf}}, which returns an 
#' Rdistance \emph{data} frame;
#' \code{\link{dfuncEstim}}, which returns an 
#' Rdistance \emph{fitted} model.
#' 
#' @examples
#' 
#' data(sparrowDf)
#'    
#' ml <- Rdistance::parseModel(sparrowDf
#'    , formula = dist ~ 1 + observer + groupsize(groupsize)
#'    , likelihood = "halfnorm"
#'    , w.lo = 0
#'    , w.hi = NULL
#'    , series = "cosine"
#'    , x.scl = 0
#'    , g.x.scl = 1
#'    , outputUnits = "m"
#'    )
#' class(ml)  # 'dfunc', but no estimated coefficients
#' print(ml)
#' print.default(ml)
#' 
#' @export
parseModel <- function(data
                          , formula = NULL
                          , likelihood = "halfnorm"
                          , w.lo = 0
                          , w.hi = NULL
                          , expansions = 0
                          , series = "cosine"
                          , x.scl = w.lo
                          , g.x.scl = 1
                          , outputUnits = NULL
                          , asymptoticSE = TRUE
                          ){

  # Check validity of data set ----
  if( !Rdistance::is.RdistDf(data, verbose = TRUE) ){
    stop(paste(deparse(substitute(data)), "is not an RdistDf. See help('RdistDf')"))
  }

  # Control parameters ----
  # if you want, could save control options in output object.
  # control <- options()[grep("Rdist_", names(options()))]
  
  # checkNEvalPts computes and sets Simpson coefficients in options()
  checkNEvalPts(getOption("Rdistance_intEvalPts")) # not exported
  
  # Check that we know the likelihood ----
  if( !( likelihood %in% getOption("Rdistance_knownLikelihoods")) ){
    stop(paste("Unknown likelihood. Likelihood must be one of"
             , paste(getOption("Rdistance_knownLikelihoods"), collapse = ", ")
             , "(case sensitive)."))
  }
  
  # Check for a response ----
  # Otherwise, as.character(formula) is length 2, not 3
  if( is.null(formula) ){
    stop("'formula' is required.")
  }
  if( !inherits(formula, "formula") ){
    stop("'formula' must be a formula object.")
  }
  mt <- stats::terms.formula(formula)
  if ( attr(mt, "response") == 0 ){
    stop("Formula must have a response on LHS of '~'.")
  }

  # Unnest the data frame ----
  # If a name in list column is the same as another in parent frame, 
  # unnest will throw an error; but, this is checked in is.RdistDf.
  detectionData <- tidyr::unnest(data
                                 , cols = attr(data, "detectionColumn"))
  
  # Fix up formula ----
  # I.e., add offset() if not present; change "groupsize" to "offset" if present.
  # Use "offset" because model.offset works with it. 
  # This may change detectionData by adding an offset with random name
  #
  # Much easier to convert "groupsize" to "offset" in formula because
  # model.frame and others treat offset correctly.
  formulaAtCall <- formula # save the original
  formulaChar <- as.character(formula) # [1] = "~"; [2] = LHS; [3] = RHS
  gsFormulaTxt <- "groupsize\\(" # string specifying groupsize in formula
  if( grepl( gsFormulaTxt, formulaChar[3]) ){
    # "groupsize(...)" specified
    formulaChar[3] <- gsub( gsFormulaTxt, "offset(", formulaChar[3] )
  } else {
    # "groupsize" not specified; add it.
    effColumn <- attr(data, "effortColumn")
    offsetVar <- withr::with_preserve_seed({
      set.seed(Sys.time())
      formatC(trunc(stats::runif(1, max = 10000000))
              , format="f"
              , digits=0
              , width = 7
              , flag = "0")
      })
    offsetVar <- paste0("gs_",offsetVar)
    formulaChar[3] <- paste0(formulaChar[3], " + offset(", offsetVar, ")")
    detectionData[[offsetVar]] <- ifelse(is.na(detectionData[[effColumn]]), NA, 1)
  }
  formula <- formula( paste(formulaChar[c(2,1,3)], collapse = " ") )


  # Evaluate model frame ----
  # Will evaluate model frame twice: here, to get response and offset from formula
  # so that we can check units. Later, to exclude observations outside the strip.
  #
  # Use na.exclude here so that predict includes NA for any missing cases 
  # later.  I.e., predict returns vector same size as detectionData always
  # Test whether NA cases were present with is.null(attr())
  mf <- stats::model.frame(
    formula = formula
    , data = detectionData
    , drop.unused.levels = TRUE
    , na.action = stats::na.pass
  )

  # A note on missing values  ----
  # Missing distances are okay.  They are observations of 
  # a target for which crew did not get a distance. Happens. 
  # These count toward "n" when computing density, but not 
  # when estimating distance functions.
  # Using na.pass preserves all missings in the model frame, 
  # this includes missing group sizes (for obs on missing transects). 
  # All of Rdistance must keep in mind that missing responses (distances)
  # could happen.

  # Check units ----    
  # We could check that offset, g.x.scl, and expansions DON'T have units
  mtNames <- as.character(attr(mt, "variables"))
  respName <- mtNames[ attr(mt, "response") + 1 ]
  offsetName <- mtNames[ attr(mt, "offset") + 1 ]
  dataWUnits <- list(data = detectionData
                     , w.lo = w.lo
                     , w.hi = w.hi
                     , respName = respName
                     , offsetName = offsetName
                     , x.scl = x.scl
                     , outputUnits = outputUnits
                     , dataName = deparse(substitute(data))
  )
  
  # CHANGE CHECKUNITS TO ACCEPT ML OBJECTS i.e., accept "RdistMl" object
  # so user can call directly more easier.
  if( getOption("Rdistance_requireUnits") ){
    dataWUnits <- Rdistance::checkUnits(dataWUnits)
  }
  

  # Truncate for w.lo and w.hi ----
  # This physically removes observations outside the strip. 
  # But, it keeps missing values of distances because their group sizes 
  # may count toward abundance. 
  # Use dataWUnits because checkUnits may have changed units, so original mf (and mt) may
  # not be good.
  d <- dplyr::pull(dataWUnits$data, dataWUnits$respName)  
  ind <- (dataWUnits$w.lo <= d) & (d <= dataWUnits$w.hi)  # could be NA's here from missing distances
  # ind <- ind & !is.na(ind)
  if( any(!ind, na.rm = TRUE) ){
    dataWUnits$data <- dataWUnits$data[is.na(ind) | ind, ]  
  }

  mf <- stats::model.frame(
      formula = formula
      , data = dataWUnits$data
      , drop.unused.levels = TRUE
      , na.action = stats::na.pass
    )    

  # Store a reduced data frame for abundance estimation later ----
  allVars <- c(all.vars( stats::terms(mf) ) # for covariates
               , attr(data, "detectionColumn")
               , attr(data, "effortCol")
               , dplyr::group_vars(data))
  modelData <- data |> 
    dplyr::select(dplyr::any_of(allVars))

  # Put everything in list ----    
  ml <- list(mf = mf
             , data = modelData
             , formula = formulaAtCall
             , dataName = dataWUnits$dataName
             , likelihood = likelihood
             , w.lo = dataWUnits$w.lo
             , w.hi = dataWUnits$w.hi
             , expansions = expansions
             , series = series
             , x.scl = dataWUnits$x.scl
             , g.x.scl = g.x.scl
             , outputUnits = dataWUnits$outputUnits
             , asymptoticSE = asymptoticSE
  )
  

  # Check x.scl, and override x.scl for Gamma likelihood ----
  if ( length(ml$x.scl) > 1 ){
    # at this point in the process, x.scl can only be a scaler or "max"
    warning(paste0("x.scl cannot have length >1. "
                 , "Found length "
                 , length(ml$x.scl)
                 , ". Only first element will used. "
                 , " x.scl has been set to "
                 , ml$x.scl[1]))
    ml$x.scl <- ml$x.scl[1]
  }
  
  # Check x.scl for Gamma, and x.scl >= w.lo ----
  if ( !is.character(ml$x.scl) ){

    if( ml$likelihood == "Gamma" ){
      isZero <- dropUnits(ml$x.scl) == 0  # the default for x.scl
      if ( !isZero ){
          # some non-default x.scl
          warning("Cannot specify g(0) for Gamma likelihood.  x.scl set to 'max'.")
      }
      ml$x.scl <- "max"
      
    } else {
      
      if ( ml$x.scl < ml$w.lo ){
        ml$x.scl <- ml$w.lo
        warning(paste0("x.scl must be >= w.lo. x.scl set to "
                     , format(ml$x.scl)
                     , " i.e., g("
                     , format(ml$x.scl)
                     , ") = "
                     , format(ml$g.x.scl)
                    ))
      }
      if ( ml$x.scl > ml$w.hi ){
        ml$x.scl <- ml$w.hi
        warning(paste0("x.scl must be <= w.hi. x.scl set to "
                       , format(ml$x.scl)
                       , " i.e., g("
                       , format(ml$x.scl)
                       , ") = "
                       , format(ml$g.x.scl)
        ))
      }
      
    }
  } else if( ml$x.scl != "max"){
    stop(paste0("x.scl must either be a number (with units) or string 'max'. Found "
                , ml$x.scl))
  }
  
  class(ml) <- "dfunc"

  ml
}

#' @title Differentiable likelihoods in Rdistance
#' 
#' @description Return a character vector of 
#' likelihoods which are differentiable and hence
#' the second derivative method can be used to estimate
#' variance-covariance.  Any likelihoods not in this list
#' must use bootstrapping. This vector is polled in 
#' a few Rdistance routines, notably \code{parseModel}
#' and \code{varcovarEstim}.
#' 
#' @return A character vector of differentiable 
#' likelihoods. 
#' 
#' @export

differentiableLikelihoods<-function(){
  
  c("halfnorm", "hazrate", "negexp", "Gamma")
  
}

#' @aliases lines.dfunc
#'   
#' @title lines.dfunc - Line plotting method for distance functions
#'   
#' @description Line plot method for objects of class '`dfunc`' 
#' that adds distance functions to an existing plot. 
#'   
#' @inheritParams plot.dfunc
#' @inheritParams print.dfunc
#' @inheritParams predict.dfunc
#' 
#' @param prob Logical scalar for whether to scale the distance function 
#' to be a density function (integrates to one). Default behavior is designed 
#' to be compatible with the plot method for distance functions 
#' ([plot.dfunc()]). By default, line transect distance 
#' functions are not scaled to a density and integrate to the effective strip width. 
#' In contrast, point transects distance functions are scaled to be densities 
#' by default. 
#' 
#' @param \dots Parameters passed to `lines.default` that control attributes like 
#' color, line width, line type, etc. 
#'   
#' @return A data frame containing the x and y coordinates of the 
#' plotted line(s) is returned invisibly.  X coordinates in the 
#' return are names `x`. Y coordinates in the return are named 
#' `y1, y2, ..., yn`, i.e., one column per returned 
#' distance function.  
#'   
#' @seealso [dfuncEstim()], [plot.dfunc()],
#'   [print.abund()]
#'   
#' @examples 
#' # a simulated RdistDf
#' set.seed(87654)
#' x <- rnorm(1000, mean=0, sd=20)
#' x <- x[x >= 0]
#' x <- setUnits(x, "mi")
#' Df <- data.frame(transectID = "A"
#'                , distance = x
#'                 ) |> 
#'   dplyr::nest_by( transectID
#'                , .key = "detections") |> 
#'   dplyr::mutate(length = setUnits(100,"km"))              
#' attr(Df, "detectionColumn") <- "detections"
#' attr(Df, "obsType") <- "single"
#' attr(Df, "transType") <- "line"
#' attr(Df,'effortColumn') <- "length"
#' is.RdistDf(Df)  # TRUE
#' 
#' dfunc <- Df |> dfuncEstim(distance ~ 1, likelihood="halfnorm")
#' plot(dfunc, nbins = 40, col="lightgrey", border=NA, vertLines=FALSE)
#' lines(dfunc, col="grey30", lwd=15)
#' lines(dfunc, col="grey90", lwd=5, lty = 2)
#' 
#' # Multiple lines 
#' data(sparrowDfuncObserver)
#' obsLevs <- levels(sparrowDfuncObserver$data$observer)
#' plot(sparrowDfuncObserver
#'    , vertLines = FALSE
#'    , lty = 0
#'    , plotBars = FALSE
#'    , main="Detection by observer"
#'    , legend = FALSE)
#' y <- lines(sparrowDfuncObserver
#'    , newdata = data.frame(observer = obsLevs)
#'    , col = palette.colors(length(obsLevs))
#'    , lty = 1
#'    , lwd = 4)
#' head(y) # values returned, with distances as column
#' 
#' 
#' @export
lines.dfunc <- function(x
                        , newdata = NULL
                        , prob = NULL
                        ,  ...) {
  
  x.seq <- seq(x$w.lo, x$w.hi, length = getOption("Rdistance_intEvalPts") )
  g.at.x0 <- x$g.x.scl
  x0 <- x$x.scl
  
  y <- stats::predict(
               object = x
             , newdata = newdata
             , distances = x.seq
             , type = "dfunc")
  
  # dfuncs are in columns.
  
  if( is.points(x) ){
    y <- y * dropUnits(x.seq - x$w.lo) # integrate to > 1
    if( is.null(prob) || (prob) ){
      y <- t( t(y) / (colSums(y, na.rm = TRUE) * dropUnits(x.seq[2] - x.seq[1]))) # now y integrates to 1.0
    }
  } else {
    if( !is.null(prob) && prob ){
      y <- t( t(y) / (colSums(y, na.rm = TRUE) * dropUnits(x.seq[2] - x.seq[1]))) # now y integrates to 1.0
    }
  }
  
  if( ncol(y) > 1 ){
    # newdata has >1 row
    graphics::matlines(x.seq, y, ...)
    dimnames(y)[[2]] <- paste0("y", 1:ncol(y))
    ans <- cbind(data.frame(x = x.seq), as.data.frame(y))
  } else {
    graphics::lines(x.seq, y[,1], ...)
    ans <- data.frame(x = x.seq, y )
  }

  invisible(ans)
  
}

#' @title Effort information
#'
#' @description 
#' Extract effort information from an Rdistance data frame. Effort is 
#' length of line-transects or number of points on point-transects.
#' 
#' @inheritParams transectType
#' 
#' @param ... Ignored
#' 
#' @return A vector containing effort. If line-transects, return is 
#' length of transects, with units.  If point-transects, return is 
#' number of points (integers, no units).  Vector length is
#' number of transects. If input is not an RdistDf or estimated distance
#' function, return is NULL.
#' 
#' @examples
#' 
#' data(sparrowDf)
#' effort(sparrowDf)
#' fit <- dfuncEstim(sparrowDf, dist ~ 1)
#' effort(fit)
#' 
#' @export
#' 
effort <- function(x, ...){
  if( inherits(x, "dfunc") ){
    return( dplyr::pull(x$data, attr(x$data, "effortColumn")) ) # or x$data[[attr(x$data, "effortColumn")]]
  } else if( is.RdistDf(x, verbose = TRUE) ){
    return( dplyr::pull(x, attr(x, "effortCol")) )
  } else {
    return(NULL)
  }
}

#' @title Type of observations
#' 
#' @description Return the type of observations (single or multiple observers) 
#' represented in either a fitted distance function or Rdistance data frame. 
#' 
#' @inheritParams transectType
#' 
#' @details 
#' This function is a simple helper function.  If `x` is an
#' estimated distance object, it polls the `obsType`
#' attribute of the object's Rdistance data frame. If `x` is an Rdistance nested data frame, it 
#' polls the `obsType` attribute. 
#' 
#' @return One of the following values: 
#' "single", "1given2", "2given1", or "both". If observation type 
#' has not been assigned, 
#' return is NULL.
#' 
#' @export
observationType <- function(x){
  if( inherits(x, "dfunc") ){
    return( attr(x$data, "obsType") )
  } else {
    return( attr(x, "obsType") )
  }
}

#' @title Integrate Negative exponential 
#' 
#' @description
#' Compute integral of the negative exponential distance function. 
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#'
#' @details 
#' 
#' Returned integrals are
#' \deqn{\int_0^{w} e^{-a_i x} dx = \frac{1}{a_i} (1 - e^{-a_i w}),}{
#' Integral( exp(-a*x) ) = (1 - exp(-a*w)) / a,}
#' where \eqn{w = w.hi - w.lo} and \eqn{a_i}{a} is the estimated 
#' negative exponential distance 
#' function parameter for the 
#' i-th observed distance. 
#'  
#' @seealso \code{\link{integrateNumeric}}; \code{\link{integrateNegexpPoints}}; 
#' \code{\link{integrateOneStepLines}} 
#' 
#' @examples
#' 
#' # Fake distance function object w/ minimum inputs for integration
#' d <- rep(1,4) %#% "m" # Only units needed, not values
#' obs <- factor(rep(c("obs1", "obs2"), 2))
#' beta <- c(-5, -0.5)
#' w.hi <- 125
#' w.lo <- 20
#' ml <- list(
#'     mf = model.frame(d ~ obs)
#'   , par = beta 
#'   , likelihood = "negexp"
#'   , w.lo = w.lo %#% "m"
#'   , w.hi = w.hi %#% "m"
#'   , expansions = 0
#' )
#' class(ml) <- "dfunc"
#' integrateNegexpLines(ml)
#' 
#' # Check: Integral of exp(-bx) from 0 to w.hi-w.lo
#' b <- c(exp(beta[1]), exp(beta[1] + beta[2]))
#' intgral <- (1 - exp(-b*(w.hi - w.lo))) / b
#' intgral
#' 
#' 
#' @export
#' 
integrateNegexpLines <- function(object
                            , newdata = NULL
                            , w.lo = NULL
                            , w.hi = NULL
                            , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    w.lo <- object$w.lo
    w.hi <- object$w.hi
    object <- stats::predict(object = object
                             , newdata = newdata
                             , type = "parameters"
    )
  } 
  
  # Remove units b/c cannot exp units object.
  w <- dropUnits(w.hi - w.lo)

  outArea <- (1 - exp(-object*w)) / object
  
  outArea <- setUnits(outArea, Units)
  
  outArea 
  
}

#' @title Integrate Point-survey One-step function
#' 
#' @description
#' Compute integral of the one-step distance function
#' for point-surveys. 
#' 
#' @param object Either an Rdistance fitted distance function
#' (an object that inherits from class "dfunc"; usually produced 
#' by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
#' distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
#' If a matrix, each row corresponds to a 
#' distance function and each column is a parameter. The first column is 
#' the parameter related to sighting covariates and must be transformed 
#' to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
#' be applied outside this routine).  If \code{object} is a matrix, 
#' it should not have measurement units because
#' only derived quantities (e.g., ESW) have units; Rdistance function 
#' parameters themselves never have units.
#' 
#' @param newdata A data frame containing new values for 
#' the distance function covariates. If NULL and 
#' \code{object} is a fitted distance function, the  
#' observed covariates stored in
#' \code{object} are used (behavior similar to \code{\link{predict.lm}}).
#' Argument \code{newdata} is ignored if \code{object} is a matrix.
#' 
#' @param w.lo Minimum sighting distance or left-truncation value
#' if \code{object} is a matrix.
#' Ignored if \code{object} 
#' is a fitted distance function. 
#' Must have physical measurement units. 
#' 
#' @param w.hi Maximum sighting distance or right-truncation value
#' if \code{object} is a matrix.
#' Ignored if \code{object} 
#' is a fitted distance function.
#' Must have physical measurement units. 
#' 
#' @param Units Physical units of sighting distances if 
#' \code{object} is a matrix. Sighting distance units can differ from units 
#' of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
#' is a fitted distance function.
#' 
#' @section Note:
#' Users will not normally call this function. It is called 
#' internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}. 
#' 
#' @details 
#' Returned integrals are
#' \deqn{\int_0^{w} x(\frac{p}{\theta_i}I(0\leq x \leq \theta_i) + \frac{1-p}{w - \theta_i}I(\theta_i < x \leq w)) dx = \frac{\theta_i}{2p}((1-p)w + \theta_i),}{
#' Integral(x((p/Theta)I(0<=x<=Theta) + ((1-p)/(w-Theta))I(Theta<x<=w))) = Theta ((1-p)w + Theta) / (2p),} 
#' where \eqn{w = w.hi - w.lo}, \eqn{\theta_i}{Theta} is the estimated one-step
#' distance function
#' threshold for the i-th observed distance, and \eqn{p}{p} is the estimated
#' one-step proportion.
#' 
#' @return A vector of areas under the distance functions represented in 
#' \code{object}. 
#' If \code{object} is a distance function and 
#' \code{newdata} is specified, the returned vector's length is 
#' \code{nrow(newdata)}.  If \code{object} is a distance function and 
#' \code{newdata} is NULL, 
#' returned vector's length is \code{length(distances(object))}. If 
#' \code{object} is a matrix, return's length is 
#' \code{nrow(object)}. 
#' 
#' 
#' @seealso \code{\link{integrateNumeric}}; \code{\link{integrateOneStepNumeric}}; 
#' \code{\link{integrateOneStepLines}} 
#' 
#' @examples
#' 
#' fit <- dfuncEstim(thrasherDf, dist~1, likelihood = "oneStep")
#' integrateOneStepPoints(fit, newdata = data.frame(`(Intercept)`=1))
#' EDR(fit, newdata = data.frame(`(Intercept)`=1))
#' 
#' # Check: 
#' Theta <- exp(fit$par[1])
#' Theta <- setUnits(Theta, "m")
#' p <- fit$par[2]
#' w.hi <- fit$w.hi
#' w.lo <- fit$w.lo
#' g.at0 <- w.lo
#' g.atT <- Theta
#' g.atTPlusFuzz <- (((1-p) * Theta) / ((w.hi - Theta) * p))*Theta
#' g.atWhi <- (((1-p) * Theta) / ((w.hi - Theta) * p))*w.hi
#' area.0.to.T <- (Theta - w.lo) * (g.atT - g.at0) / 2 # triangle; Theta^2/2
#' area.T.to.w <- (w.hi - Theta) * (g.atTPlusFuzz + g.atWhi) / 2 # trapazoid
#' area <- area.0.to.T + area.T.to.w
#' edr <- sqrt( 2*area )
#' 
#' @export
#' 
integrateOneStepPoints <- function(object
                            , newdata = NULL
                            , w.lo = NULL
                            , w.hi = NULL
                            , Units = NULL
                              ){

  # need this if b/c sometimes this is called from nLL (object is just a 
  # matrix of parameters) and other times it is called from EDR (object is 
  # fitted object)
  if( inherits(object, "dfunc") ){
    w.hi <- object$w.hi # override input if it's given
    w.lo <- object$w.lo
    Units <- object$outputUnits # override if given
    object <- stats::predict(object = object
                        , newdata = newdata
                        , type = "parameters"
    )
  } 
  
  Theta <- setUnits(object[,1], Units)
  p <- object[,2]
  w <- w.hi - w.lo # must have units, or calculation fails below.

  # Slower, but clearer:
  # fatT <- (((1-p) * Theta) / ((w.hi - Theta) * p)) # height of f from Theta to w.hi
  #   Triangle between 0 and Theta
  # part1 <- Theta * Theta / 2
  #   Trapazoid between Theta and w.hi
  # gAtT <- fatT * Theta
  # gAtw <- fatT * w.hi
  # part2 <- (gAtw + gAtT) * (w.hi - Theta) / 2 
  # outArea <- part1 + part2
  
  outArea <- 0.5 * Theta * ((1 - p) * w + Theta) / p
  
  outArea # units should be object$outputUnits^2
  
}

#' @title Summarize a distance function object
#' 
#' @description A summary method for distance functions. 
#' Distance functions are produced by 
#' \code{dfuncEstim} (class \code{dfunc}).
#' 
#' @inheritParams predict.dfunc 
#' 
#' @param \dots Included for compatibility with other print methods.  Ignored here.
#' 
#' @param criterion A string specifying the model fit criterion to print.
#' Must be one of "AICc" (the default), 
#' "AIC", or "BIC".  See \code{\link{AIC.dfunc}} for formulas. 
#' 
#' @details This function prints the following 
#' quantities:
#' \itemize{  
#'   \item \samp{Call} : The original function call.
#'   \item \samp{Coefficients} : A matrix of estimated coefficients, their 
#'   standard errors, and Wald Z tests.
#'   \item \samp{Strip} : The left (\code{w.lo}) and right (\code{w.hi}) truncation values.
#'   \item \samp{Effective strip width or detection radius} : ESW or EDR as computed by \code{effectiveDistance}.
#'   \item \samp{Probability of Detection} : Probability of detecting a single target in the strip. 
#'   \item \samp{Scaling} : The horizontal and vertical coordinates used to scale the distance function. 
#'   Usually, the horizontal coordinate is 0 and the vertical coordinate is 1 (i.e., g(0) = 1).
#'   \item \samp{Log likelihood} : Value of the maximized log likelihood.
#'   \item \samp{Criterion} : Value of the specified fit criterion (AIC, AICc, or BIC).
#' }
#' The number of digits used in the printout is 
#' controlled by \code{options()$digits}.
#' 
#' @return The input distance function object (\code{object}), invisibly, 
#' with the following additional components:
#' \itemize{
#'   \item \code{convMessage}: The convergence message. If the distance function
#'   is smoothed, the convergence message is NULL.
#'   \item \code{effDistance}: The ESW or EDR.
#'   \item \code{pDetect}: Probability of detection in the strip.
#'   \item \code{AIC}: AICc, AIC, or BIC of the fit, whichever was requested.
#'   \item \code{coefficients}: If the distance function has coefficients, this 
#'   is the coefficient matrix with standard errors, Wald Z values, and p values. 
#'   If the distance function is smoothed, it has no coefficients and this component 
#'   is NULL.
#' }
#' 
#' @seealso \code{\link{dfuncEstim}}, \code{\link{plot.dfunc}}, 
#' \code{\link{print.abund}}, \code{\link{print.abund}}
#' 
#' @examples
#' # Load example sparrow data (line transect survey type)
#' data(sparrowDf)
#' 
#' # Fit half-normal detection function
#' dfunc <- sparrowDf |> dfuncEstim(formula=dist~1)
#' 
#' # Print results
#' summary(dfunc)
#' summary(dfunc, criterion="BIC")
#' 
#' @export
summary.dfunc <- function( object, criterion="AICc", ... ){

  object <- print.dfunc(x = object, ...)
  
  # Convergence and likelihood line ----
  if( !(isSmooth <- is.smoothed(object)) ){
    # **** CHANGE THIS grepl IF SUCCESSFUL MESSAGE CHANGES IN PRINT.DFUNC
    if( grepl("(Asymptotic|Bootstrap) SE's", object$convMessage)){
      # b/c FAILURE mess printed in 'print.dfunc', but not Success
      # or "pending bootstrap" messages
      cat("\n")
      cat(paste("Message: ", object$convMessage,  "\n", sep=""))
    }

    if( object$expansions==0 ){
        mess <- ""
    } else {
        mess <- paste( "with", object$expansions, "expansion(s) of", 
                       casefold( object$series, upper=TRUE ), "series")
    }
    cat(paste("Function:", colorize(casefold(object$likelihood, upper=TRUE)), 
              mess, "\n") )
  } 
    
  # Strip line ----
  cat(paste("Strip:", colorize(format(object$w.lo)), "to", 
            colorize(format(object$w.hi)), "\n"))
  
  # Effective distance line ----
  effDist <- effectiveDistance(object)
  pDetect <- effDist / (object$w.hi - object$w.lo) 
  pDetect <- dropUnits(pDetect)  # units of pDetect should always be [1]
  interceptOnly <- intercept.only(object) # in Rdistance, not exported

  if( is.points(object) ){
    # Points
    pDetect <- pDetect^2 # needed later, for P(detect) line
    if( interceptOnly ){
      mess <- "Effective detection radius (EDR):"
    } else {
      mess <- "Average effective detection radius (EDR):"
    }
  } else {
    # Lines
    if( interceptOnly ){
      mess <- "Effective strip width (ESW):"
    } else {
      mess <- "Average effective strip width (ESW):"
    }
  }
  
  if(interceptOnly){
    efd <- effDist[1]
  } else {
    efd <- mean(effDist)
  }
  mess <- paste(mess, colorize(format(efd)))
  cat(paste(mess)) # no return here, wait for ciMess to print at end of line
  
  # EFD CI line ----
  if( interceptOnly && all(!is.null(object$effDistance.ci)) ){
    ciMess <- paste0(" "
      , object$alpha*100
      , "% CI: "
      , colorize(format(object$effDistance.ci[1]))
      , " to " 
      , colorize(format(object$effDistance.ci[2])) 
    ) 
  } else if( !interceptOnly ){
    ciMess <- paste0(
                  " (range "
                , colorize(format(min(effDist)))
                , " to "
                , colorize(format(max(effDist)))
                , ")"
                )
  } else {
    ciMess <- ""
  }
  cat(paste(ciMess,"\n"))
  
  # pDetect line ----
  if(interceptOnly){
    if( pDetect[1] > 1 ){
      cat(paste("Probability of detection:"
                , colorize(format(pDetect[1]), col = "red")
                , colorize("> 1", col = "red")))
    } else {
      cat(paste("Probability of detection:" 
                , colorize(format(pDetect[1]))
                ))
    }
  } else {
    if( any(pDetect > 1) ){
      ng1 <- sum(pDetect > 1)
      np <- length(pDetect)
      cat(paste("Average probability of detection:", 
                colorize(format(mean(pDetect))),
                colorize(paste(ng1, "of", np, "P(detect) > 1")
                         , col = "red")))
      
    } else {
      cat(paste0("Average probability of detection: " 
                , colorize(format(mean(pDetect)))
                , " (range "
                , colorize(format(min(pDetect)))
                , " to "
                , colorize(format(max(pDetect)))
                , ")"
                ))
    }
  }
  cat("\n")
    
  # Scaling line ----    
  cat(paste("Scaling: g(", 
            colorize(format(object$x.scl)), ") = ", 
            colorize(format(object$g.x.scl)), sep=""))
  if(any(pDetect > 1.0)){
    cat(colorize(" <- Check scaling", col = "red"))
  } 
  cat("\n")
    
  # Log likelihood line ----    
  cat(paste("Log likelihood:", 
            colorize(format(object$loglik)), "\n"))
  
  # AIC line ----
  if( !isSmooth ){
    aic <- AIC.dfunc(object,criterion=criterion) 
    cat(paste0(attr(aic,"criterion"),": ", 
               colorize(format(aic)), "\n"))
  }

  # Final assignments ----
  object$effDistance <- effDist
  object$pDetect <- pDetect
  object$AIC <- aic

  invisible(object)
}

#' @title Effective Detection Radius (EDR) for point transects
#'   
#' @description Computes Effective Detection Radius (EDR) for estimated 
#'   detection functions on point transects.  
#'   See \code{\link{ESW}} is for line transects. 
#'   
#' @inheritParams predict.dfunc
#'   
#' @details Effective Detection Radius is the integral under the 
#' detection function times distance. \if{latex}{I.e., 
#'     \deqn{EDR = \int_{w.lo}^{w.hi} xg(x)dx,} 
#'   where \eqn{g(x)} is the distance
#'   function scaled so that \eqn{g(x.scl) = g.x.scl}
#'   and \eqn{w.lo} and \eqn{w.hi} are the lower
#'   and upper truncation limits.  }
#'   
#' @inherit effectiveDistance return  
#' 
#'    
#' @seealso \code{\link{dfuncEstim}}, \code{\link{ESW}}, 
#' \code{\link{effectiveDistance}}
#'   
#' @examples
#' # Load example thrasher data (point transect survey type)
#' data(thrasherDf)
#' 
#' # Fit half-normal detection function
#' dfunc <- thrasherDf |> dfuncEstim(formula=dist~bare)
#' 
#' # Compute effective detection radius (EDR)
#' EDR(dfunc) # vector length 192
#' effectiveDistance(dfunc) # same
#' EDR(dfunc, newdata = data.frame(bare=30)) # vector length 1
#'   
#' @export

EDR <- function(object, newdata = NULL){
  
  # Issue error if the input detection function was fit to line-transect data
  if( !Rdistance::is.points(object) ){
    stop("EDR is for point transects only.  See ESW for the line-transect equivalent.")
  } 

  likExpan <- paste0(object$likelihood, "_", object$expansions)
  
  if( likExpan == "oneStep_0" ){
    edr <- integrateOneStepPoints(object, newdata = newdata)
    
  } else if( likExpan == "halfnorm_0" ){
    edr <- integrateHalfnormPoints(object, newdata = newdata)
    
  # Don't have hazrate_0 points, see integrateHazratePoints in code parking lot

  } else if( likExpan == "negexp_0" ){
    edr <- integrateNegexpPoints(object, newdata = newdata)
    
  } else if( grepl("oneStep", likExpan) ){
    edr <- integrateOneStepNumeric(object, newdata = newdata)
    
  } else {
    edr <- integrateNumeric(object, newdata = newdata)
  }

  edr <- dropUnits(edr)
  edr <- sqrt( 2 * edr )  # cannot sqrt units (unless like m^2 are assigned)
  edr <- setUnits(edr, object$outputUnits) # add back units

  
  edr
}

#' @title Numeric Integration of One-step Function
#' 
#' @description
#' Compute integral of the one-step distance function
#' using numeric integration. This function is only called for 
#' oneStep functions that contain expansion factors. 
#' 
#' @inheritParams integrateOneStepPoints
#' @inheritParams dE.single
#' @inheritParams integrateNumeric
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' The [oneStep.like()] function has an extremely large 
#' discontinuity at Theta. Accurate numeric integration requires 
#' inserting Theta and Theta+ (a value just larger than Theta) 
#' into the series of points being evaluated. Because this creates 
#' variable sized intervals, the Trapezoid rule must be used. 
#' Rdistance's Simpson's rule routine 
#' ([integrateNumeric()]) will not work for oneStep likelihoods 
#' that have expansions. 
#' 
#' @seealso [integrateNumeric()]; 
#' [integrateOneStepLines()]; [integrateOneStepPoints()]
#' 
#' @export
#' 
integrateOneStepNumeric <- function(object
                            , newdata = NULL
                            , w.lo = NULL
                            , w.hi = NULL
                            , Units = NULL
                            , expansions = NULL
                            , series = NULL
                            , isPoints = NULL
                              ){
  
  # we know we are in the oneStep with expansions > 0 case here
  likelihood = "oneStep"
  f.like <- utils::getFromNamespace(paste0( likelihood, ".like"), "Rdistance")    
  
  # need this if b/c sometimes this is called from nLL (object is just a 
  # matrix of parameters) and other times it is called from EDR (object is 
  # fitted object)
  if( inherits(object, "dfunc") ){
    w.lo <- object$w.lo
    w.hi <- object$w.hi # override input if it's given
    Units <- object$outputUnits # override if given
    expansions <- object$expansions
    series <- object$series
    isPoints <- is.points(object)
    object <- stats::predict(object = object
                        , newdata = newdata
                        , type = "parameters"
    )
    object[,1] <- log(object[,1]) # predict returns real param, f.like needs link value
    # If object has expansions, their coefficients come back from predict
  } 
  
  Theta <- setUnits(object[,1], Units) # link scale
  p <- object[1,2] # p is always constant, this saves some space
  if( expansions > 0 ){
    coefLocs <- (ncol(object)-(expansions-1)):(ncol(object))
    expCoefs <- object[1,coefLocs] # constant across observations
  }
  
  nInts <- getOption("Rdistance_intEvalPts") # odd not a requirement here
  
  # This really slows down oneStep with expansions, but, no other way
  # If there are covariates, Theta is potentially different on every row
  XIntOnly <- matrix(1, nrow = 2*nInts, ncol = 1) 
  outArea <- rep(NA, length(Theta))
  outArea <- setUnits(outArea, Units)
  zero <- setUnits(0, Units)
  
  uniqueTheta <- unique(Theta)
  thetaFuzz <- setUnits(getOption("Rdistance_fuzz"), Units)

  for(i in 1:length(uniqueTheta)){
    theta <- uniqueTheta[i]
    thetaNoUnits <- dropUnits( theta) 
    expTheta <- exp( thetaNoUnits ) 
    expTheta <- setUnits(expTheta, Units) # with units
    posTheta <- Theta == theta # use later
    
    # Key: insert theta and theta+ into distances, use unequal intervals
    d = c(seq(zero, expTheta, length=nInts) 
             , seq(expTheta + thetaFuzz, w.hi - w.lo, length=nInts ))
    dx <- diff(d)  

    y <- f.like(
      a = c(thetaNoUnits, p)
      , dist = d
      , covars = XIntOnly
      , w.hi = w.hi
    )
    y <- y$L.unscaled # (nInts x 1) = (length(d) X 1) in this case

    # yL <- y #debugging
    
    if( expansions > 0 ){
      W <- expTheta - w.lo
      exp.terms <- Rdistance::expansionTerms(a = c(0  # theta and p not used in expTerms
                                                   , 0
                                                   , expCoefs)
                                             , d = d
                                             , series = series
                                             , nexp = expansions
                                             , w = W)
      y <- y * exp.terms
      y[ !is.na(y) & (y <= 0) ] <- getOption("Rdistance_zero")
      
    }
    
    if(isPoints){
      y <- d * y  # element-wise
    }
    
    thetaArea <- sum( dx * (y[-1,] + y[-nrow(y), ]) / 2 ) 
    outArea[posTheta] <- thetaArea
  }
  
  # plot(d, y[,1], main = "in intOneStepNumeric", type="p")
  # 
  # lines(d, yL[,1], col = "red")
  # abline(v = expTheta)
  
  outArea # units should be object$outputUnits^2
  
}

#' @title Mixture of triangle and uniform likelihood
#' 
#' @description
#' Compute likelihood function for a mixture of a triangle and 
#' uniform distributions. 
#' 
#' @inheritParams halfnorm.like
#' 
#' @inherit halfnorm.like return seealso
#' 
#' @details Rdistance's \code{triangle} likelihood is a mixture of a 
#' triangle and uniform distribution. The 'triangle' density function
#' is  
#' \deqn{f(d|\theta, p) = (1 - \frac{1-p}{\theta}d) I(0 \leq d \leq \theta) + 
#'        p I(\theta \le d \leq w),}{
#'        f(d|T) = (1 - (1-p)*d/T)I(0<=d<=T) + p*I(T<d<=w),}
#' where 
#' \eqn{I(x)} is the indicator function for event \eqn{x}, 
#' and \eqn{w} is the nominal strip width 
#' (i.e., \code{w.hi - w.lo} in Rdistance). 
#' The unknown parameters to be estimated 
#' are \eqn{\theta}{T} and \eqn{p} 
#' (\eqn{w} is fixed - given by the user). 
#'  
#' Covariates influence values of \eqn{\theta}{T} 
#' via a log link function, i.e., \eqn{\theta = e^{x'b}}{T = exp(x'b)},
#' where \eqn{x} is the vector of covariate values 
#' associated with distance \eqn{d}, and \eqn{b}
#' is the vector of estimated coefficients. 
#' 
#' @examples
#' 
#' w <- 250
#' Theta <- 160
#' p <- 0.4
#' d <- seq(0,w,length = w+1)
#' y <- (1 - ((1-p)/Theta)*d)*(d <= Theta) + p*(d > Theta)
#' plot(d, y, type="l", ylim = c(0,1), xlab = "Distance", ylab = "Probability")
#' points(Theta, p, pch=16, col = "red")
#' lines(c(-10,Theta), c(p,p), lty = 2, col = "grey")
#' axis(2, at=p, label = "p", line = 1, srt = 0, tick = FALSE)
#' lines(c(Theta,Theta), c(-1,p), lty = 2, col = "grey")
#' axis(1, at=Theta, label = "Theta", line = 2, tick = FALSE)
#' 
#' Theta <- 25
#' p <- 0.2
#' y <- (1 - ((1-p)/Theta)*d)*(d <= Theta) + p*(d > Theta)
#' lines(d, y)
#' points(Theta, p, pch=16, col = "red")
#' lines(c(-10,Theta), c(p,p), lty = 2, col = "grey")
#' axis(2, at=p, label = "p", line = 1, srt = 0, tick = FALSE)
#' lines(c(Theta,Theta), c(-1,p), lty = 2, col = "grey")
#' axis(1, at=Theta, label = "Theta", line = 2, tick = FALSE)
#' 
#' # same as above
#' y <- triangle.like(a = c(log(Theta), p)
#'                  , dist = d
#'                  , covars = matrix(1, length(d))
#'                  , w.hi = 250)
#' lines(d, y$L.unscaled, col = "green")
#' 
#' @export
triangle.like <- function(a
                , dist
                , covars 
                , w.hi = NULL) {
  
  if(length(dim(dist)) >= 2 && dim(dist)[2] != 1 ){ 
    stop(paste("Argument 'dist' must be a vector or single-column matrix.",
               "Found array with", length(dim(dist)), "dimensions."))
  }
  q <- nCovars(covars) 
  if(is.matrix(a)){
    beta <- a[,1:q, drop = FALSE]  # k X q
    p <- a[1, q+1, drop = TRUE]     # 1 X 1
  } else {
    beta <- matrix(a[1:q], nrow = 1) # 1 X q
    p <- a[q+1]     # 1 X 1
  }
  s <- covars %*% t(beta) # (nXq) %*% (qXk) = nXk
  theta <- exp(s)  # link function here
  
  dist <- dropUnits(dist)

  if(is.null(w.hi)){
    w.hi <- max(dist)  # no units b/c removed above
  } else {
    w.hi <- dropUnits(w.hi) # already checked units
  }

  # 'p' is height for d > theta
  m <- (1 - p) / theta     # slope btwn 0 and theta
  key <- (1 - m*dist) * (0 <= dist & dist <= theta) + 
    p * (theta < dist & dist <= w.hi)
  
  return( list(L.unscaled = key, 
               params = cbind(s, p)) )

}

#' @title Detect intercept-only distance function
#' 
#' @description
#' Utility function to detect whether a distance function has 
#' covariates beyond the intercept. If the model contains an 
#' intercept-only, effective distance is constant across detections
#' and short-cuts can be implemented in code.
#' 
#' @inheritParams predict.dfunc
#'
#' @return TRUE if \code{object} contains an intercept-only.
#' FALSE if \code{object} contains at least one detection-level 
#' or transect-level covariate in the detection function.
#' 
# NOT EXPORTED

intercept.only <- function(object){
  
  mt <- stats::terms(object$mf)
  if( attr(mt,"intercept") == 0 ){
    # it has no intecept, so not "intecept only"
    return(FALSE)
  }
  
  if( length(attr(mt, "term.labels")) == 0 ){
    # It has an intercept but no other covariates
    return(TRUE)
  } else {
    # It has an intercept, but at least one other covariate
    return(FALSE)
  }
  
}

#' @title Tests for point surveys
#' 
#' @description
#' Determines whether a distance function is for a point survey 
#' or line survey. 
#' 
#' @inheritParams transectType
#' 
#' @return TRUE if the model frame or fitted distance function 
#' contains point surveys.  FALSE if the model frame or distance 
#' function contains line transect surveys. 
#' 
#' @export
#' 
is.points <- function(x){
  typ <- Rdistance::transectType(x)
  if(!is.null(typ)){
    return(typ == "point")
  } else {
    stop(paste("'transType' component of object x missing.  Cannot determine"
               , "whether data is from point or line transect surveys."))
  }
}
  

#' @title Modes of the Gamma distribution
#' 
#' @description
#' Compute mode (i.e., maximum) of Rdistance's version of the gamma 
#' distribution. 
#' 
#' @param params A matrix of Gamma distribution parameters. First column 
#' is the scale parameter, and is a function of covariates.
#' Second column is the shape parameter. 
#' 
#' @return A vector of the locations of the gamma modes associated 
#' with each row in the `params` matrix. 
#' 
#' @export
GammaModes <- function( params ){
  
  scl = exp(params[,1])
  shp = params[,2]
  
  b <- GammaReparam( shp = shp, scl = scl )
  
  shp <- b$shp
  scl <- b$scl
  
  mod <- (shp - 1) * scl
  
  mod

}

#' @title Integrate Hazard-rate line survey distance functions
#' 
#' @description
#' Compute integral of the hazard-rate distance function for 
#' line-transect surveys.
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' 
#' Returned integrals are
#' \deqn{\int_0^{w} (1 - e^{-(x/\sigma_i)^{-k}}) dx = w - \frac{\sigma_i}{k} \Gamma(-\frac{1}{k}, {\frac{\sigma_i}{w}}^{k}),}{
#' Integral( 1 - e^{-(x/\sigma_i)^(-k)} ) = w - (s/k)(Gamma(-1/k, (s/w)^k),}
#' where \eqn{w = w.hi - w.lo}, \eqn{\sigma_i}{s} and \eqn{k}{k} are estimated 
#' hazard-rate distance 
#' function parameters for the 
#' i-th observed distance, and \eqn{\Gamma()}{Gamma()} is the incomplete gamma 
#' function. 
#' Rdistance uses the 
#' incomplete gamma function implemented in 
#' \code{\link[expint]{gammainc}}, which for 
#' all intents and purposes is exact.
#' 
#' @seealso \code{\link{integrateNumeric}}; \code{\link{integrateNegexpLines}}; 
#' \code{\link{integrateOneStepLines}} 
#' 
#' @examples
#' 
#' # A pre-estimated hazard rate distance function: sparrowDfuncObserver
#' fit <- sparrowDfuncObserver
#' table(ESW(fit))
#' table(integrateHazrateLines(fit))
#' 
#' # Check: Integral of 1 - exp(-(x/s)^(-k)) from 0 to w.hi-w.lo
#' w <- dropUnits(fit$w.hi - fit$w.lo)
#' params <- predict(fit)
#' sigma <- params[,1]
#' minusk <- -params[,2]
#' 
#' outArea <- w + sigma * 
#'            expint::gammainc(1/minusk, (w/sigma)^(minusk)) / minusk
#' table(outArea)
#' 
#' @export
#' 
integrateHazrateLines <- function(object
                            , newdata = NULL
                            , w.lo = NULL
                            , w.hi = NULL
                            , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    w.lo <- object$w.lo
    w.hi <- object$w.hi
    object <- stats::predict(object = object
                             , newdata = newdata
                             , type = "parameters"
    )
  } 
  
  w <- dropUnits(w.hi - w.lo)
  sigma <- object[,1]
  minusk <- -object[,2]

  outArea <- w + sigma * expint::gammainc(1/minusk, (w/sigma)^(minusk)) / minusk
    
  outArea <- setUnits(outArea, Units)
  
  outArea 
  
}

#' @title Compute and print distance function integration
#' 
#' @description Check several integration characteristics, 
#' and report them to screen.  This was designed to print 
#' info when user asks for higher verbose level. The scaled
#' distance function should integrate to 1.0 every iteration of 
#' maximization, and this function checks that. 
#' 
#' @param ml The fitted object or model list
#' 
#' @param key The scaled distance function.  
#' 
#' @param f0 The value of f(0) when integrating line transects. 
#' This should be the ESW for the case. 
#' 
#' @param plot Logical scalar.  If TRUE, plot a diagnostic consisting
#' of the distance function and approximating points we use for quadrature. 
#' 
#' 
#' @return Nothing.  Prints information on integrals to the screen.
#' 
integrateKey <- function(ml, key, f0, plot = FALSE){
  
  nInts <- getOption("Rdistance_intEvalPts")
  intCoefs <- getOption("Rdistance_intCoefs")
  
  # nInts <- 501
  # intCoefs <- simpsonCoefs(nInts)
  
  # Make sure 0 is included in domain of key. We know g(0) and f(0)
  dObs <- c( setUnits(0,"m")
             , distances(ml) - ml$w.lo
  )
  dObs <- dropUnits(dObs)
  
  likExpan <- paste0(ml$likelihood, "_", ml$expansions, "_", transectType(ml))
  
  if( grepl("line", likExpan) && !grepl("Gamma", likExpan) ){
    # line survey g(0) = 1; f(0) = ESW
    keyUni <- c(dropUnits(f0) , key)
  } else {
    # Point survey, and Gamma, g(0) = f(0) = 0
    keyUni <- c( 0, dropUnits(key) )
  }
  
  dObsUni <- dObs[!duplicated(dObs)]
  keyUni <- keyUni[!duplicated(dObs)]
  
  ord <- order(dObsUni)
  dObsUni <- dObsUni[ord]
  keyUni <- keyUni[ord]
  
  # Simpson integral -----
  # seqx = seq(ml$w.lo, ml$w.hi, length=nInts) 
  # d <- dropUnits(seqx - ml$w.lo) 
  # dx <- seqx[2] - seqx[1]  
  
  # for f(w.hi), use f(x) at largest dObs (rule = 2) 
  # fy <- stats::approx(dObsUni, keyUni, xout = d, rule = c(1,2) )$y
  # keyIntegral <- sum(fy*intCoefs)*dx/3 
  
  # R integrate ----
  
  F <- function(x, d, fd){
    stats::approx(d, fd, xout = x, rule = c(1,2))$y
  }
  
  if( (length(dObsUni) > 1) && all(!is.na(keyUni)) && all(!is.infinite(keyUni)) ){
    tryCatch(keyIntegralR <- stats::integrate(f = F 
                              , lower = ml$w.lo
                              , upper = ml$w.hi
                              , d = dObsUni
                              , fd = keyUni)
             , error = function(e) e
    )
  } else {
    # really bad value of params, probably using optim without limits
    keyIntegralR <- list(
        value = "Missing or Infinite"
      , abs.err = Inf
    )
  }
  
  # Plot ----
  if( plot ){
    graphics::plot(dObsUni, keyUni, pch = 15,
                   xlab = "Distance",
                   ylab = "Likelihood"
    )
    seqx = seq(ml$w.lo, ml$w.hi, length=nInts) 
    d <- dropUnits(seqx - ml$w.lo)
    fy <- stats::approx(dObsUni, keyUni, xout = d, rule = c(1,2) )$y
    graphics::points(d, fy, pch = 16, cex = .5, col="red")
    graphics::legend("topright"
           , legend = c("Observation", paste(nInts, "Lin. Approx Pts."))
           , pch = c(15,16)
           , col = c("black", "red"))
  }
  
  # cat(paste0("    Simpson integral of key ("
  #           , colorize(nInts)
  #           , " points) : "
  #           , colorize(keyIntegral)
  #           , " (should be ~1)\n"
  #           ))
  cat(paste0("  base::integrate of key "
             , colorize(keyIntegralR$value)
             , " (+-"
             , colorize(formatC(keyIntegralR$abs.err, format="f", digits = 7))
             , ") (should be 1+-0.001)\n"
  ))
  
  invisible(1)
}

#' @title Estimate variance-covariance
#' 
#' @description
#' Estimate the variance-covariance matrix of parameters 
#' in the distance function. If the likelihood is 
#' differentiable, the variance-covariance matrix is 
#' estimated from the second derivative of the likelihood 
#' (i.e., the hessian).  If the likelihood is not 
#' differentiable, the variance-covariance matrix is 
#' a matrix of 0's that are interpreted as "pending" (i.e., 
#' pending bootstrapping). 
#' 
#' @inheritParams startLimits
#' @inheritParams print.dfunc
#' 
#' @return A square symmetric matrix estimating the 
#' variance-covariance matrix of parameters in \code{x}. 
#' Dimension of return is p X p, where p = \code{length(x$par)}.
#' 
#' @export
#' 
varcovarEstim <- function( x, ml ){
  
  # varcovar gets called from maximization routines (Nlminb, Optim, HookesJeeves)
  # AND 'abundEstim'. 
  
  nP <- length(x$par)
  nmsP <- names(x$par)

  verboseLevel <- getOption("Rdistance_verbosity")
  warn <- getOption("Rdistance_warn")
  
  if( verboseLevel >= 1 ){
    cat(colorize("VarCovar estimation ----\n", col = "red"))
  }
  
  if( !(ml$likelihood %in% differentiableLikelihoods()) ){
    # varcovar is PENDING bootstrapping ----
    if( verboseLevel >= 1 ){
      cat(colorize("  Bootstraps pending\n"))
    }
    
    varcovar <- NULL
  } else {
    # we want to estimate varcovar
    if( is.null(x$hessian) ){
      hessian <- Rdistance::secondDeriv(
          x = x$par
        , FUN = nLL
        , eps = getOption("Rdistance_hessEps")
        , ml = ml
        , verbosity = verboseLevel
      )
    } else {
      # Optim returns a hessian, others may, use it.
      if( verboseLevel >= 1 ){
        cat(colorize("  Using Hessian from final iteration\n"))
      }
      hessian <- x$hessian
    }
  
    if (x$convergence != 0) {
      if (warn) warning("Distance function did not converge, or converged to (Inf,-Inf)")
      varcovar <- matrix(NaN, nP, nP)
    } else if (!any(is.na(hessian)) & !any(is.infinite(hessian))){
      qrh <- base::qr(hessian)
      if (qrh$rank < nP) {
        if (warn) warning("Singular variance-covariance matrix.")
        varcovar <- matrix(NaN, nP, nP)
      } else {
        varcovar <- tryCatch(solve(hessian), error = function(e){NaN})
        if (length(varcovar) == 1 && is.nan(varcovar)){
          if (warn) warning("Singular variance-covariance matrix.")
          varcovar <- matrix(NaN, nP, nP)
        }
      }
    } else {
      if (warn) warning("Covariance matrix has Inf or NA elemetns.")
      varcovar <- matrix(NaN, nP, nP)
    }
    dimnames(varcovar) <- list(nmsP, nmsP)
  }
 
  
  varcovar
  
}

#' @title 'nlminb' optimizer
#' 
#' @description
#' Call R native function 'nlminb' to perform optimization. 
#' 
#' @inheritParams mlEstimates
#'
#' @inherit Nlminb return
#' 
# Do Not export

HookeJeeves <- function(ml, strt.lims){
  
  contRl <- list(info = getOption("Rdistance_trace") > 0
                 , maximize = FALSE
                 , target = Inf
                 , maxfeval = getOption("Rdistance_evalMax")
                 , tol = getOption("Rdistance_likeTol")
  )
  
  verboseLevel <- getOption("Rdistance_verbosity")
  if( verboseLevel >= 1 ){
    cat(colorize("HOOKEJEEVES non-gradient maximization ----\n", col = "red"))
  }
  
  fit <- dfoptim::hjkb(
      par = strt.lims$start
    , fn = nLL
    , lower = strt.lims$low
    , upper = strt.lims$high
    , control = contRl
    , verbosity = verboseLevel
    , ml = ml
  )
  
  names(fit$par) <- strt.lims$names
  if( ml$asymptoticSE ){
    fit$varcovar <- Rdistance::varcovarEstim(fit, ml)
  } else {
    fit$varcovar <- NULL
  }

  # final few things ----
  fit$limits <- strt.lims[c("low", "high")]
  
  # Flip over objective: object$logLike is true logLike, -LL was minimized
  names(fit)[names(fit) == "value"] <- "loglik"
  fit$loglik <- -fit$loglik  
  
  names(fit)[names(fit) == "feval"] <- "evaluations"
  names(fit)[names(fit) == "niter"] <- "iterations"
  
  if( fit$convergence == 0 ){
    fit$message <- "converged"
  } else {
    fit$message <- "did not converge"
  }
  
  fit
} 

#' @title Start and limit values for negexp distance function
#' 
#' @description Compute starting values and limits for the negative 
#' exponential distance
#' function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @export
negexp.start.limits <- function (ml){
  
  X <- stats::model.matrix(ml)
  dist <- Rdistance::distances(ml)  
  
  ncovars <- nCovars(X)

  zero <- getOption("Rdistance_zero")
  posInf <- getOption("Rdistance_posInf")
  negInf <- getOption("Rdistance_negInf")
  
  expan <- ml$expansions

  # there should not be any distances outside (w.lo,w.hi)
  d <- dist - ml$w.lo
  medDist <- stats::median(d)
  medDist <- dropUnits(medDist)
  
  if(is.null(medDist) || 
     is.na(medDist) || 
     is.infinite(medDist) || 
     (medDist <= zero)){
    w <- ml$w.hi - ml$w.lo
    medDist <- ml$w.lo + w / 2
    medDist <- dropUnits(medDist)
  } 
  
  startIntercept <- -log(medDist) # = log(1/medDist)
  
  # 1/medDist is MOM estimate of lambda (lambda = slope of neg exp)
  # log(1/medDist) is MOM estimate on link scale

  start <- c(startIntercept
             , rep(zero, ncovars - 1)
             , rep(zero, expan)
             )
  low   <- c(negInf
             , rep(negInf, ncovars - 1 )
             , rep(negInf, expan)
             )
  high  <- c(posInf
             , rep( posInf, ncovars - 1 )
             , rep( posInf, expan)
             )
  nms <- colnames(X)
  
  if(expan > 0){
    nms <- c(nms, paste( "a", 1:expan, sep=""))
  }
  
  names(start) <- nms
  names(low) <- nms
  names(high) <- nms
  
  list( start=start, low=low, high=high, names=nms )
  
}

#' @title Rdistance optimization control parameters.
#' 
#' @aliases control controls RdistanceControls
#' 
#' @concept control optimization
#' 
#' @description Optimization control parameters 
#' are set by calls to `options()` (see examples). 
#' Optimization parameters used in 
#' `Rdistance` are the following:  
#' 
#' \itemize{
#'   \item `Rdistance_maxIters`: The maximum number of optimization 
#' iterations allowed.
#' 
#'   \item `Rdistance_evalMax`: The maximum number of objective function
#' evaluations allowed.
#' 
#'   \item `Rdistance_likeTol`: Minimum change in the likelihood 
#' between iterations required optimization to continue.  
#' If the likelihood changes by less than this amount, 
#' optimization stops and a solution is declared. Iteration 
#' continues when likelihood changes exceed this value.
#' 
#'   \item `Rdistance_coefTol`: Minimum change in model coefficients 
#' between iterations for optimization to continue.  
#' If the sum of squared coefficient differences changes 
#' by less than this amount between iterations, 
#' optimization stops and a solution is declared. 
#'
#'   \item `Rdistance_optimizer`: Normally, this option is not set and 
#'   the default optimizer routines are used. The default optimization 
#'   routine for smooth likelihoods (i.e., listed by 
#'   [differentiableLikelihoods()]) is the gradient-based 
#'   method in [stats::nlminb()]. The default routine for
#'   non-smooth likelihoods is the Nelder-Mead method 
#'   in [stats::optim()]. 
#'   
#'   Not often, but occasionally optimizers fail on problem. Switching 
#'   algorithms can make a poorly behaved distance function converge, 
#'   particularly when parameters are near their boundaries.  
#'   If users wish to override the default optimization routine, 
#'   set this option to one of the following: 
#'   \itemize{
#'    \item "optim_\<method\>": Use the "\<method\>" method in stats::optim. 
#'    For example, "optim_Nelder-Mead" uses 
#'    the Nelder-Mead. See [stats::optim()] for 
#'    documentation of the six available methods. 
#'    \item "nlminb": Uses [stats::nlminb()], a finite-difference
#'    gradient based approach. 
#'    \item "hookeJeeves": Uses [dfoptim::hjkb()], a 
#'    derivative-free approach for continuous and discontinuous 
#'    likelihoods.
#'   }
#'
#'   \item `Rdistance_hessEps`: A vector of parameter distances used during 
#' computation of numeric second derivatives. These distances control
#' and determine variance estimates, and they may need revision when 
#' the maximum likelihood solution is near a parameter boundary. 
#' Should have length 
#' 1 or the number of parameters in the model. See function 
#' [secondDeriv()] for further details. 
#' 
#'   \item `Rdistance_trace`: Integer scalar for the level 
#'   of information printed to the console by the optimization 
#'   routine during maximization of the likelihood. All optimizer 
#'   routines interpret a value of 0 as 'do not print any information'
#'   or silent.  Higher values produce more information.  The 
#'   information produced varies among optimization routines. 
#'  
#'   \item `Rdistance_requireUnits`: A logical specifying whether measurement 
#' units are required on distances and areas.  If TRUE, 
#' measurement units are required on off-transect and radial 
#' distances in the input data frame.  Likewise, measurement 
#' units are required on truncation distances, scale location, 
#' transect lengths, and study area size. If FALSE, no units are 
#' required and input values are used as is.  The FALSE options is 
#' provided for rare cases when `Rdistance` functions are called
#' from other functions and the calling functions do not accommodate 
#' units.
#' 
#' Assign units with statement like `units(detectionDf$dist) <- "m"`
#' or `setUnits(w.hi, "km")` or `w.hi <- 150 %#% "m"` or 
#' `w.hi <- 150 %m%.`.  
#' Measurement units of 
#' the various physical quantities need not 
#' be equal because appropriate conversions occur internally.
#' An error is thrown if differing units are not compatible.  
#' For example, "m" (meters) cannot be converted into "ha" (hectares),
#' but "acres" can be converted into "ha".
#' `Rdistance` recognizes units listed in `units::`[units::valid_udunits()]. 
#' 
#'   \item `Rdistance_maxBSFailPropForWarning`: The proportion of bootstrap 
#' iterations that can fail without a warning. If the proportion 
#' of non-convergent bootstrap iterations exceeds this 
#' parameter, a warning about the validity of CI's is issued in 
#' the abundance print method. 
#' 
#' 
#' }
#' 
#' @examples 
#' # increase number of iterations
#' options(Rdistance_maxIters=2000)
#' 
#' # change optimizer and decrease tolerance
#' op <- options(list(Rdistance_optimizer="optim_Nelder-Mead"
#'                  , Rdistance_likeTol=1e-6)) 
#' 
#' # change back
#' options(op)
#' @name RdistanceControls
NULL

#' @export
#' 
#' @rdname unitHelpers
#' 
#' @returns For `dropUnits`, argument `x` with no units. If 
#' input `x` has no units, `x` is returned unchanged. 
#' 
dropUnits <- function(x){
  units::set_units(x, NULL)
}

#' @title Observation distances
#'
#' @description 
#' Extract the observation distances (i.e., responses for 
#' an Rdistance model) from an Rdistance model frame.
#' 
#' @inheritParams startLimits
#' 
#' @param ... Ignored
#' 
#' @param na.rm Whether to include or exclude missing distance values. 
#' In \code{ml}, the model list containing the model frame, missing 
#' values of the response (distance) are potentially present for two 
#' reasons: (1) they are outside the strip w.lo to w.hi, and (2) they
#' are missing because the crew did not get a distance for that observation.
#' 
#' 
#' @return A vector containing observation distances contained
#' in the Rdistance model frame.
#' 
#' @examples
#' 
#' data(sparrowDf)
#' sparrowModel <- parseModel( sparrowDf, dist ~ observer )
#' stats::model.response(sparrowModel$mf)
#' distances(sparrowModel) # same, but future-proof
#' 
#' @export
#' 
distances <- function(ml, na.rm = TRUE, ...){
  # ml$mf is a regular linear model frame, with terms, so this is easy
  y <- stats::model.response(ml$mf)
  
  # There could be missing distances b/c crew could not measure it or b/c
  # the obs is outside the strip.
  # We want to count this observation for abundance, but not distance function estimation
  if( na.rm ){
    y <- y[!is.na(y)]
  }
  y
}

#' @title Bias corrected bootstraps
#' 
#' @description
#' Calculate bias-corrected confidence intervals for bootstrap data
#' using methods in Manly textbook.
#' 
#' @param x.bs A vector of bootstrap estimates of some quantity. 
#' 
#' @param x A scalar of the original estimate of the quantity.
#' 
#' @param ci A scalar of the desired confidence interval 
#' coverage. 
#' 
#' @return A named vector containing the lower and upper endpoints of
#' the bias-corrected bootstrap confidence interval. 
#' 
#' @export
#' 
bcCI <- function(x.bs, x, ci = 0.95){
  p <- mean(x.bs > x, na.rm = TRUE)
  z.0 <- stats::qnorm(1 - p)
  z.alpha <- stats::qnorm(1 - ((1 - ci)/2))
  p.L <- stats::pnorm(2 * z.0 - z.alpha)
  p.H <- stats::pnorm(2 * z.0 + z.alpha)
  ans <- stats::quantile(x.bs[!is.na(x.bs)], p = c(p.L, p.H))
  names(ans) <- paste0(100*c( (1 - ci)/2, 1 - (1-ci)/2 ), "%")
  ans
}

#' @title Negative exponential likelihood
#' 
#' @description Computes the negative exponential distance function. 
#' 
#' @inheritParams halfnorm.like 
#' 
#' @details The negative exponential likelihood is 
#' \deqn{f(x|\alpha) = \exp(-\alpha x)}{f(x|a) = exp( -a*x )} where \eqn{\alpha}{a} is the 
#' slope parameter. 
#' 
#' @inherit halfnorm.like return seealso
#'    
#' @examples
#' d <- seq(0, 100, length=100)
#' covs <- matrix(1,length(d),1)
#' negexp.like(log(0.01), d, covs)
#' 
#' # Changing slope parameter
#' plot(d, negexp.like(log(0.1), d, covs)$L.unscaled, type="l", col="red")
#' lines(d, negexp.like(log(0.05), d, covs)$L.unscaled, col="blue")
#' 
#' @export

negexp.like <- function(a
                      , dist
                      , covars
                      , w.hi = NULL
                      ){

  # w.hi is ignored, but needed for compatability in other likelihoods
  # What's in a? : 
  #     a = [(Intercept), b1, ..., bp, <expansion coef>]
  if(length(dim(dist)) >= 2 && dim(dist)[2] != 1 ){ 
    stop(paste("Argument 'dist' must be a vector or single-column matrix.",
               "Found array with", length(dim(dist)), "dimensions."))
  }
  
  q <- nCovars(covars)
  if(is.matrix(a)){
    beta <- a[,1:q, drop = FALSE]  # k X q
  } else {
    beta <- matrix(a[1:q], nrow = 1) # 1 X q
  }
  s <- covars %*% t(beta) # (nXq) %*% (qXk) = nXk
  sigma <- exp(s)  # link function here

  dist <- dropUnits(dist)

  key = -sigma * dist  # (nXk) * (nXk)
  key <- exp(key)
  
  return( list(L.unscaled = key, 
               params = s)
  )  

}

#' @title Start and limit values for hazrate distance function
#' 
#' @description Compute starting values and limits for the hazard rate distance
#' function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @export
hazrate.start.limits <- function (ml){
  
  X <- stats::model.matrix(ml)
  dist <- Rdistance::distances(ml)  
  
  ncovars <- ncol(X)

  zero <- getOption("Rdistance_zero")
  posInf <- getOption("Rdistance_posInf")
  negInf <- getOption("Rdistance_negInf")
  
  expan <- ml$expansions
  dMin <- max( min(dist), ml$w.lo )
  dMax <- min( max(dist), ml$w.hi )
  w <- ml$w.hi - ml$w.lo
  medDist <- stats::median(dist)
  if(is.null(medDist) || is.na(medDist) || is.infinite(medDist)){
    medDist <- ml$w.lo + w / 2
  }
  
  # Only time dist will not have units is when user overrides requirement
  # Nonetheless, need to remove units b/c likelihood is unitless
  dMin <- dropUnits(dMin)
  dMax <- dropUnits(dMax)
  w <- dropUnits(w)
  medDist <- dropUnits(medDist)
  
  start <- c(log(0.8 * medDist)   # Sigma 
             , rep(zero, ncovars-1)    # any covars
             , 1               # k
             , rep(zero, expan))        # any expansions
  low   <- c(negInf
             , rep(negInf, ncovars-1)
             , 0.5
             , rep(negInf, expan))
  high  <- c(posInf
             , rep( posInf, ncovars-1)
             , 20 
             , rep( posInf, expan))
  nms <- c(colnames(X), "k")
    
  if(expan > 0){
    nms <- c(nms, paste( "a", 1:expan, sep=""))
  }

  names(start) <- nms
  names(low) <- nms
  names(high) <- nms
  
  list( start=start, low=low, high=high, names=nms )
  
}

#' @title Unit assignment helpers
#' 
#' @description
#' Helper functions for assigning physical measurement units in Rdistance. 
#' All are convenience wrappers for `units::`[units::set_units()]. 
#' 
#' @param x A numeric vector or matrix.  
#' 
#' @param u A string representing physical measurement units to 
#' assign to `x`, e.g., "m", "km", "m^2".  Valid units are  
#' listed in columns "(symbol|name)" of [units::valid_udunits()].
#' 
#' @returns For `%#%` and `setUnits`, argument `x` with 
#' units `u` attached. 
#' 
#' @examples 
#' 
#' 2 %#% "m" 
#' setUnits(2,"km")
#' x <- 2 %#% "km^2"
#' 10 %#% units(x)
#' 2 %#% "km^2" %#% "acres" # Convert km^2 to acres
#' x %#% "acres"            # Same
#' x %#% NULL    # Drop units
#' dropUnits(x)  # Same
#' 
#' # %#%'s precedence is below "^" but above "+" and "*"
#' # The following fails: 
#' # 2 %#% "m" / (2 %#% "ha") %#% "in/acre"
#' # The following succeeds:
#' (2 %#% "m" / (2 %#% "ha")) %#% "in/acre"
#' 1 %m%. ^ 2      # [m]
#' (1 %m%.) ^ 2    # [m^2]
#' 
#' @export
#' 
#' @aliases unitHelpers
#' 
#' @rdname unitHelpers
#' 
'%#%' <- function(x, u){
  units::set_units(x, u, mode = "standard")
}

#' @title Integrate Line-transect One-step function 
#' 
#' @description
#' Compute exact integral of the one-step distance function for line 
#' transects. 
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' Returned integrals are
#' \deqn{\int_0^{w} (\frac{p}{\theta_i}I(0\leq x \leq \theta_i) + 
#'    \frac{1-p}{w - \theta_i}I(\theta_i < x \leq w)) dx = \frac{\theta_i}{p},}{
#' Integral((p/Theta)I(0<=x<=Theta) + ((1-p)/(w-Theta))I(Theta<x<=w)) = Theta / p,} 
#' where \eqn{w = w.hi - w.lo}, \eqn{\theta_i}{Theta} is the estimated one-step
#' distance function
#' threshold for the i-th observed distance, and \eqn{p}{p} is the estimated 
#' one-step proportion. 
#' 
#' @seealso [integrateNumeric()]; [integrateNegexpLines()]; 
#' [integrateHalfnormLines()] 
#' 
#' @examples
#' 
#' # A oneStep distance function on simulated data
#' whi <- 250
#' T <- 100  # true threshold
#' p <- 0.85 # true proportion <T
#' n <- 200  # num simulated points
#' x <- c( runif(n*p, min=0, max=T), runif(n*(1-p), min=T, max=whi))
#' x <- setUnits(x, "m")
#' tranID <- sample(rep(1:10, each=n/10), replace=FALSE)
#' detectDf <- data.frame(transect = tranID, dist = x)
#' siteDf <- data.frame(transect = 1:10
#'                , length = rep(setUnits(10,"m"), 10))
#' distDf <- RdistDf(siteDf, detectDf)
#' 
#' # Estimation
#' fit <- dfuncEstim(distDf
#'                  , formula = dist ~ 1
#'                  , likelihood = "oneStep"
#'                  , w.hi = setUnits(whi, "m")
#'                  )
#' table(integrateOneStepLines(fit))
#' table(ESW(fit))
#' 
#' # Check:
#' T.hat <- exp(fit$par[1])
#' p.hat <- fit$par[2]
#' gAtT <- ((1-p.hat) * T.hat) / (p.hat * (whi - T.hat))
#' 
#' plot(fit)
#' abline(h = gAtT, col="blue")
#' 
#' areaLE.T <- (1.0) * T.hat
#' areaGT.T <- gAtT * (whi - T.hat)
#' areaLE.T + areaGT.T  # ESW
#' 
#' # Equivalent
#' T.hat / p.hat
#' 
#' @export
#' 
integrateOneStepLines <- function(object
                            , newdata = NULL
                            , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    object <- stats::predict(object = object
                        , newdata = newdata
                        , type = "parameters"
    )
  } 
  
  Theta <- setUnits(object[,1], Units)
  p <- object[,2]

  outArea <- Theta / p
  

  outArea 
  
}

#' @title Check for the presence of units
#' 
#' @description
#' Check for the presence of physical measurement units on key
#' columns of  
#' an `RdistDf` data frame. 
#' 
#' @param ml An `Rdistance` model list produced by 
#' [parseModel()] containing a list
#' of parameters for the distance model. 
#' 
#' @returns The input `ml` list, with units of various 
#' quantities converted to common units.  If a check fails, for example, 
#' a quantity does not have units, an error is thrown.
#' 
#' @export
#' 
checkUnits <- function(ml){

  # If we are here, we assume that requireUnits == TRUE
  
  # Units for dist ----
  dist <- dplyr::pull(ml$data, ml$respName)
  if ( !inherits(dist, "units") ){
    stop(paste("Measurement units required on distances.",
          "Assign units by attaching 'units' package then:\n",
          paste0("units(",ml$dataName,"$", ml$respName, ")"), "<- '<units of measurment>',\n",
          "for example 'm' (meters) or 'ft' (feet). See units::valid_udunits()"))
  } 
  # if we are here, dist has units
  # set units for output by converting dist units; 
  # w.lo, w.hi, and x.scl will all be converted later
  # We want outputUnits to be units(dist), not e.g., string "m"
  if ( !is.null(ml$outputUnits) ){
    ml$data <- ml$data |> 
      dplyr::mutate(dplyr::across(.cols = ml$respName
                                , .fns = ~ setUnits(.x, ml$outputUnits)))
    ml$outputUnits <- units(setUnits(1, ml$outputUnits))
  } else {
    ml$outputUnits <- units(dist)
  }

  # Units for w.lo ----
  if ( !inherits(ml$w.lo, "units") ){
      if ( is.null(ml$w.lo) || (ml$w.lo[1] != 0) ){
          stop(paste("Measurement units required on minimum distance (w.lo).",
              "Assign units with statement like",
              paste0("setUnits(", 
                     ifelse(is.function(ml$w.lo), "<value>", ml$w.lo),
              ", <units>)."),
              "See 'help(unitHelpers)'."))
      }
      # if we are here, w.lo is 0, it has no units, and we require units
      ml$w.lo <- setUnits(ml$w.lo, ml$outputUnits) # assign units to 0
  } 
  # if we are here, w.lo has units and we require units, convert to the output units
  # Technically, I don't think we need to do this.  As long as w.lo has units, we are good.
  # We do this here so that we don't do it later when we print units during output.
  ml$w.lo <- setUnits(ml$w.lo, ml$outputUnits)

  # Units on w.hi ----
  if (is.null(ml$w.hi)){
    ml$w.hi <- max(dist, na.rm = TRUE) # units flow through max() automatically
  } else if ( !inherits(ml$w.hi, "units") ){
    stop(paste("Measurement units required on maximum distance (w.hi).",
               "Assign units with statement like",
               paste0("setUnits(", 
                      ifelse(is.function(ml$w.hi), "<value>", ml$w.hi),
                      ", <units>)."),
               "See 'help(unitHelpers)'."))
  } 
  # if we are here, w.hi has units and we require them, convert to output units
  # Again, technically I don't think we need to do this.  Happens automatically in computations
  ml$w.hi <- setUnits(ml$w.hi, ml$outputUnits)

  # Units on x.scl ---- 
  if( !inherits(ml$x.scl, "units") ){
    if( (ml$x.scl[1] != 0) && (ml$x.scl[1] != "max")){
      stop(paste("Measurement units for x.scl are required.",
                 "Assign units with statement like",
                 paste0("setUnits(", 
                        ifelse(is.function(ml$x.scl), "<value>", ml$x.scl),
                        ", <units>)."),
                 "See 'help(unitHelpers)'."))
    }
    # if we are here, x.scl is either 0 (w/o units) or "max"
    if(!is.character(ml$x.scl)){
      ml$x.scl <- setUnits(ml$x.scl, ml$outputUnits)
    }
  } else {
    # if we are here, x.scl has units, is not "max", so we convert to the output units
    ml$x.scl <- setUnits(ml$x.scl, ml$outputUnits)
  }

  ml

}

#' @title Cosine expansion terms
#' 
#' @description 
#' Computes the cosine expansion terms that modify the 
#' shape of distance likelihood functions. 
#'   
#' @param x A numeric matrix of distances at which to evaluate 
#' the expansion series. For distance analysis, \code{x} should
#' be the proportion of the maximum sighting distance
#' at which a group was sighted, i.e., \eqn{x = d/w}, where \eqn{d}
#' is sighting distance and \eqn{w} is maximum sighting distance.
#'   
#' @param expansions A scalar specifying the number of 
#' expansion terms to compute. Must be one of the 
#' integers 1, 2, 3, 4, or 5.
#' 
#' @details The cosine expansion used here is:
#'   \itemize{
#'     \item \bold{First term}: \deqn{h_1(x)=\cos(2\pi x),}{h1(x) = cos(2*Pi*x),}
#'     \item \bold{Second term}: \deqn{h_2(x)=\cos(4\pi x),}{h2(x) = cos(4*Pi*x),}
#'     \item \bold{Third term}: \deqn{h_3(x)=\cos(6\pi x),}{h3(x) = cos(6*Pi*x),}
#'     \item \bold{Fourth term}: \deqn{h_4(x)=\cos(8\pi x),}{h4(x) = cos(8*Pi*x),}
#'     \item \bold{Fifth term}: \deqn{h_5(x)=\cos(10\pi x),}{h5(x) = cos(10*Pi*x),}
#'   }
#'   The maximum number of expansion terms is 5.
#'   
#' The cosine expansion frequency in Rdistance is 2*pi. Each term is two pi more than 
#' the previous. The sine expansion frequency in Rdistance is pi. Consequently, 
#' the sine and cosine expansions fit different models. 
#'   
#'   
#' @return A 3D array of size \code{nrow(x)} X \code{ncol(x)} X \code{expansions}.
#'   The 'pages' (3rd dimension) of this array are the cosine expansions of 
#'   \code{x}. i.e., page 1 is the first expansion term of \code{x}, 
#'   page 2 is the second expansion term of \code{x}, etc.
#'   
#' @seealso \code{\link{dfuncEstim}}, 
#'          \code{\link{hermite.expansion}}, 
#'          \code{\link{simple.expansion}}
#'   
#' @examples 
#' x <- matrix(seq(0, 1, length = 200), ncol = 1)
#' cos.expn <- cosine.expansion(x, 5)
#' plot(range(x), range(cos.expn), type="n")
#' matlines(x, cos.expn[,1,1:5], col=rainbow(5), lty = 1)
#' 
#' @export

cosine.expansion <- function(x, expansions){

  if (expansions > 5){
      warning("Too many Cosine polynomial expansion terms. Only 5 used.")
      expansions = 5
  }
  
  if( expansions < 1 ) stop( "Number of expansions must be >= 1" )
    
  expansion = array(dim = c(nrow(x), ncol(x), expansions))
  
  expansion[,,1] = cos(2*pi*x)
  
  # I realize I could do this in a for loop, but I think this is faster.    
  if(expansions >= 2){
    expansion[,,2] = cos(4*pi*x)
  }
  if(expansions >= 3){
    expansion[,,3] = cos(6*pi*x)
  }
  if(expansions >= 4){
    expansion[,,4]= cos(8*pi*x)
  }
  if(expansions >= 5){
    expansion[,,5] = cos(10*pi*x)
  }      
  
  # Cosine expansion range = 0 to -1
  expansion <- (expansion / 2) - 0.5
  
  return(expansion)
            
}

#' @title Reparameterize Gamma parameters for use in dgamma
#' 
#' @description Transform Rdistance's version of the Gamma distribution
#' parameters, which is that of Becker and Quan, into the version for 
#' use in R::dgamma() and elsewhere.
#' 
#' @param shp Rdistance's shape parameter
#' 
#' @param scl Rdistance's scale parameter.  This parameter 
#' is related to covariates via exp(x'B).
#' 
#' @return A list with components $shp and $scl, which are the 
#' re-parameterized versions of the input parameters suitable for 
#' us in R::dgamma().
#' 
#' @seealso 'Details' section of [Gamma.like()]  
#' for Rdistance's Gamma distribution
#' 
#' @examples
#' # Rdistance Gamma parameters
#' Rd.scl <- 50  # must be >0
#' Rd.shp <- 1.5 # must be >1
#' 
#' # dgamma parameters
#' dgParams <- GammaReparam(Rd.shp, Rd.scl)
#' dgParams
#' 
#' # Gamma distribution with (Rd.scl, Rd.shp) from 0 to 100
#' curve(dgamma(x, shape=dgParams$shp, scale = dgParams$scl)
#'            , from = 0
#'            , to = 100)
#'  
#' # Rdistance's version: same curve but scaled so maximum = 1
#' x <- seq(0, 100, length = 200) 
#' scl <- dgParams$scl
#' shp <- dgParams$shp
#' m <- (shp - 1) * scl
#' g <- (x / m)^(shp - 1) * exp(-(x - m) / scl) # distance function
#' plot(x, g, type = "l")
#' 
#' @export
#' 
GammaReparam <- function(shp, scl){
  
  k <- (1/gamma(shp)) * (((shp - 1)/exp(1))^(shp - 1))
  list(shp = shp
       , scl = scl * k)
  
}

#' @title Gamma distance function 
#' 
#' @description Evaluate the gamma distance function 
#' for sighting distances, potentially including covariates
#' and expansion terms
#' 
#' @inheritParams halfnorm.like 
#' 
#' @details 
#' 
#' The Rdistance implementation of a Gamma distance function follows 
#' Becker and Quang (2009). Rdistance's Gamma distance function is 
#' \deqn{f(d|\alpha, \sigma) = \left(\frac{d}{m}\right)^{\alpha - 1}e^{-(d-m)/\sigma},}{
#' f(d|a,s) =  (d/m)^{a-1} exp(-(d-m)/s),
#' }
#' where \eqn{\alpha}{a} is the \bold{shape} parameter, \eqn{\sigma}{s} is 
#' the \bold{scale} parameter, and \eqn{m = (\alpha-1)\sigma}{m = (a-1)s}.
#' \eqn{m} is the mode of the Gamma function, and in Rdistance it's 
#' scaled to have a maximum of 1.0 at \eqn{m}.  
#' The \bold{scale} parameter is a function of the shape parameter 
#' \emph{and} sighting covariates, i.e., 
#' \deqn{\sigma = k [exp(x'\beta)],}{s = k*exp(x'b),}
#' where \eqn{x} is a vector of covariate values associated with distance \eqn{d} 
#' (i.e., a row of \code{covars}), \eqn{\beta}{b} is a vector of the 
#' first \eqn{q} (=\code{ncol(covars)}) values of the first argument 
#' of the function (\code{a}), and 
#' \eqn{k} is a function of the shape parameter, i.e., 
#' \deqn{k = \frac{1}{\Gamma(\alpha)}  \left(\frac{a - 1}{e^1} \right)^{a - 1}.}{
#' k = (1/Gamma(a)) * (((a - 1)/exp(1))^(a - 1)).}  
#' The shape parameter \eqn{\alpha}{a} is the 
#' \eqn{q+1}-st value in the function's first argument and is constrained to 
#' be strictly greater than 1.0.
#'  
#' See Examples for use of \code{\link{GammaReparam}} to compute \eqn{\alpha}{a}
#' and \eqn{\sigma}{s} from fitted object coefficients.
#'   
#' @inherit halfnorm.like return seealso
#'    
#' @references Becker, E. F., and P. X. Quang, 2009. \emph{A Gamma-Shaped Detection Function for Line-Transect Surveys with Mark-Recapture and Covariate Data.}
#'   Journal of Agricultural, Biological, and Environmental Statistics 14(2):207-223.
#'   
#' @examples 
#' x <- seq(0, 100, length=100)
#' covars <- matrix(1,100,1)
#' 
#' # Plots showing changes in scale
#' plot(x, Gamma.like(c(log(20),2.5), x, covars)$L.unscaled, type="l", col="red")
#' lines(x, Gamma.like(c(log(40),2.5), x, covars)$L.unscaled, col="blue")
#' 
#' # Plots showing changes in shape
#' plot(x, Gamma.like(c(log(20),1.5), x, covars)$L.unscaled, type="l", col="red")
#' lines(x, Gamma.like(c(log(20),2.5), x, covars)$L.unscaled, col="blue")
#' lines(x, Gamma.like(c(log(20),4.5), x, covars)$L.unscaled, col="green")
#' 
#' # Roll-your-own plot, showing "re-parameterization":
#' # Assume fitted object coefficients are c(log(20), 4.5)
#' fit <- list(par = c(log(20), 4.5))
#' 
#' # The distance function is then,
#' gammaPar <- GammaReparam( scl = exp(fit$par[1])
#'                         , shp = fit$par[2] ) # returns scl=k*exp(x'B)
#' scl <- gammaPar$scl
#' shp <- gammaPar$shp
#' m <- (shp - 1) * scl
#' g <- (x / m)^(shp - 1) * exp(-(x - m) / scl) # distance function
#' lines(x, g, lwd = 3, lty = 2, col="green3")
#' 
#' @export
Gamma.like <- function(a
                       , dist
                       , covars
                       , w.hi = NULL
                       ){

  # What's in a? : 
  #   If no covariates: a = [Shape, Scale, <expansion coef>]
  #   If covariates:    a = [(Intercept), b1, ..., bp, Shape, <expansion coef>]
  
  # w.hi is ignored, but needed for compatability in other likelihoods
  # cat(paste("In", crayon::red("gamma.like"), "\n"))
  
  if(length(dim(dist)) >= 2 && dim(dist)[2] != 1 ){ 
    stop(paste("Argument 'dist' must be a vector or single-column matrix.",
               "Found array with", length(dim(dist)), "dimensions."))
  }
  
  q <- nCovars(covars)
  if(is.matrix(a)){
    beta <- a[,1:q, drop = FALSE]  # k X q
    shp <- a[1, q+1, drop = TRUE]     # 1 X 1
  } else {
    beta <- matrix(a[1:q], nrow = 1) # 1 X q
    shp <- a[q+1]     # 1 X 1
  }
  s <- covars %*% t(beta) # (nXq) %*% (qXk) = nXk
  scl <- exp(s)  # link function here

  # An oddity:  dgamma preserved units if scl is 1D vector; but, 
  #  strips units if scl has >= 2 cols. 
  
  dgamPars <- GammaReparam(shp, scl)

  m <- (dgamPars$shp - 1)*dgamPars$scl  # mode of Gamma
  d <- dropUnits(dist)
  
  key <- (d/m)^(dgamPars$shp - 1)*exp(-(d-m)/dgamPars$scl)

  # key <- stats::dgamma( dist, shape=dgamPars$shp, scale=dgamPars$scl )
  # 
  # # stats::dgamma returns a matrix if dgamPars$scl is a matrix with ncol(key) > 1
  # # stats::dgamma returns vector if $scl is vector or matrix with ncol(key) == 1
  # # fix this to return matrix always
  # if( ncol(scl) == 1 ){
  #   key <- matrix(key, ncol = 1)  # drops units too
  # }
  # 
  # # Scale like to have max 1
  # m <- (dgamPars$shp - 1)*dgamPars$scl
  # keyAtM <- stats::dgamma( m, shape=dgamPars$shp, scale=dgamPars$scl )
  # 
  # key <- key / keyAtM
  
  # Note: Mode of Gamma distribution is (dgamPars$shp - 1)*dgamPars$scl,
  # or scl * b * (shp - 1) where b = (1/gamma(shp)) * (((shp - 1)/exp(1))^(shp - 1)) 

  return( list(L.unscaled = key, 
               params = cbind(s, shp)))  # return params on log scale

}

#' @title Unknown error message
#' 
#' @description
#' Constructs a string stating what is "unknown" that is 
#' suitable for use in warning and error functions. 
#' 
#' @param txt Text.  The "unknown" we are looking for. 
#' 
#' @param attri Attribute description we are looking for.
#'
#' @return A descriptive string, suitable for warning or error.
#' 
errDataUnk <- function(txt, attri){
  paste0( "Unknown "
          , txt
          , ". Set "
          , txt
          , " using 'RdistDf',"
          , " or with "
          , 'attr(data, "'
          , attri
          , '") <- "value".'
          , " See help(RdistDf) for list of values."
  )
} 

#' @title Estimate multiple-observer line-transect distance functions
#' 
#' @description Fits a detection function to off-transect 
#' distances collected by multiple observers. 
#' 
#' @inheritParams dfuncEstim
#' @inheritParams dE.single
#' 
#' @inherit dE.single return
#' 
#' @export
#' 
dE.multi <- function( data
                         , formula
                         , likelihood = "halfnorm"
                         , w.lo = setUnits(0,"m")
                         , w.hi = NULL
                         , expansions = 0
                         , series = "cosine"
                         , x.scl = setUnits(0,"m")
                         , g.x.scl = 1
                         , warn = TRUE
                         , outputUnits = NULL){
  
  
  # The double-observer method hasn't been checked since updating to version 2.x
  # It's likely that an error would be raised if a user did try the double-observer option,
  # but I'm adding a warning here, just in case
    stop("The double-observer routines have not been tested in Rdistance
          versions >2.x, so they have been disabled for the time being.
          Contact the Rdistance authors if you need double observer analyses.")

}

#' @title Sage Thrasher detection data frame in Rdistance >4.0.0 format
#' 
#' @name thrasherDf
#' 
#' @description  
#' Point transect data collected in central Wyoming on 120 points
#' surveyed for Sage Thrashers by the Wyoming Cooperative Fish & Wildlife Research Unit in 2013.
#' 
#' @docType data
#'
#' @format A rowwise tibble containing 120 rows and 8 columns, one of which 
#' (i.e., 'detections') contains nested data frames of detections.  
#' Each row represents
#' one transect of one point.  
#' 
#' @format A data.frame containing 120 rows and 6 columns.  Each row represents
#' a surveyed site.  Each surveyed site is considered one transect of one point.  
#' Column descriptions: 
#' \enumerate{ 
#' \item \code{siteID}: Factor (120 levels), the site or point surveyed.  
#' \item \code{detections}: An embedded (nested) data frame containing 
#' detections made at that point.  Columns in the embedded data frame contain:
#'   \enumerate{
#'     \item \code{groupsize}: The number of individuals in
#'     the detected group.  
#'     \item \code{dist}: The radial distance (m) from
#'     the transect to the detected group.  
#'   }
#' \item \code{observer}: Factor (six levels), identity of the observer who surveyed
#' the point.  
#' \item \code{bare}: Number, the mean bare ground cover (\%)
#' within 100 m of each point.  
#' \item \code{herb}: Number, the mean herbaceous
#' cover (\%) within 100 m of each point.  
#' \item \code{shrub}: Number, the mean
#' shrub cover (\%) within 100 m of each point.  
#' \item \code{height}: Number,
#' the mean shrub height (cm) within 100 m of each point.  
#' \item \code{npoints}: The number of point counts on the transect.
#' }
#'  
#' 
#' @seealso \code{\link{thrasherSiteData}}, \code{\link{thrasherDetectionData}},
#' \code{\link{RdistDf}}
#' 
#' @source The sage thrasher data are a subset of data collected 
#' by Jason Carlisle and various field technicians for his Ph.D. from the 
#' Department of Ecology, University of Wyoming, in 2017.  This portion of 
#' Jason's work was funded by the Wyoming Game and Fish Department 
#' through agreements 
#' with the University of Wyoming's  Cooperative Fish & Wildlife
#' Research Unit (2012). 
#' 
#' @examples
#' data(thrasherDf)
#' 
#' is.RdistDf(thrasherDf)
#' 
#' summary(thrasherDf,
#'   formula = dist ~ groupsize(groupsize)
#' )
#' 
#' @references Carlisle, J.D. 2017. The effect of sage-grouse conservation on
#' wildlife species of concern: implications for the umbrella species concept.
#' Dissertation. University of Wyoming, Laramie, Wyoming, USA.
#' 
#' Carlisle, J. D., A. D. Chalfoun, K. T. Smith, and J. L. Beck. 2018. 
#' Nontarget effects on songbirds from habitat manipulation for Greater 
#' Sage-Grouse: Implications for the umbrella species concept. 
#' \emph{The Condor: Ornithological Applications} 120:439–455. 
#' \doi{10.1650/CONDOR-17-200.1}
#' 
NULL

#' @title Insert oneStep Likelihood breaks
#' 
#' @description Compute break points in a oneStep likelihood
#' and insert them into a sequence of distances.  The idea 
#' is to insert a point just left and just right of the breaks 
#' so that they plot as vertical lines. 
#' 
#' @param obj A fitted Rdistance model object
#' 
#' @param newData A data frame containing covariate values to 
#' use in prediction. 
#' 
#' @param xseq A vector of distances into which the break points 
#' are inserted. 
#' 
#' @return A vector like `xseq`, but with the break points 
#' inserted. 
#' 
#' 

# Do not export

insertOneStepBreaks <- function(obj 
                              , newData 
                              , xseq){
  
  parms <- stats::predict(object = obj
                 , newdata = newData
                 , type = "parameters")[, 1]
  parms <- setUnits(parms, units(xseq))
  parms <- parms + obj$w.lo # matrix + scalar
  
  epsilon <- matrix(c(-1,1) * getOption("Rdistance_fuzz")
                  , nrow = length(parms)
                  , ncol = 2
                  , byrow = TRUE)
  epsilon <- setUnits(epsilon, units(xseq))
  breaks <- parms + epsilon # vector length n + n X 2 matrix
  breaks <- c(t(breaks))
  xseq <- sort(c(xseq, breaks))
  xseq
}

#' @title Gamma.start.limits - Start and limit values for Gamma distance function
#' 
#' @description Compute starting values and limits for the Gamma distance
#' function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @export
Gamma.start.limits <- function (ml){
  
  X <- stats::model.matrix(ml)
  dist <- Rdistance::distances(ml)  
  
  expan <- ml$expansions
  ncovars <- nCovars(X)

  fuzz <- getOption("Rdistance_fuzz")
  zero <- getOption("Rdistance_zero")
  posInf <- getOption("Rdistance_posInf")
  negInf <- getOption("Rdistance_negInf")
  
  maxGammaAble <- 171 # max number s.t. gamma(x) < Inf
  
  d <- dist[ dist > units::set_units(0, "m") ] # even though 0 is fine, can't take log of it
  s <- dropUnits( log( mean(d, na.rm=TRUE) ) - mean( log(d), na.rm=TRUE ) ) # there are no NA's
  s2 <- (s-3)^2 + 24*s
  if( s2 < zero ){ 
    s2 <- zero 
  }
  r <- (3 - s + sqrt( s2 )) / (12*s)
  if( r <= 1.0 ) {
    r <- 1.01
  }
  b <- ( (r-1) / exp(1) )^(r-1) / gamma(r)
  lam <- mean(d,na.rm=TRUE) / (r * b)
  if( inherits(lam, "units")){
    # only time this does not fire is when user overides units
    lam <- dropUnits(lam)
  }

  expanStart <- rep(0, expan)
  expanLow <- rep(negInf, expan)
  expanHigh <- rep(posInf, expan)
  
  start <- c(log(lam), rep(0, ncovars-1), r, expanStart)
  low   <- c(zero, rep(negInf, ncovars-1), 1 + fuzz, expanLow)
  high  <- c(posInf, rep(posInf, ncovars-1), maxGammaAble, expanHigh)

  nms <- c(colnames(X), "Shape")
  if(expan > 0){
    nms <- c(nms, paste( "a", 1:expan, sep=""))
  }

  names(start) <- nms
  names(low) <- nms
  names(high) <- nms  
  
  list( start=start, low=low, high=high, names=nms )
 
}

#' @title Plot method for distance (detection) functions
#'   
#' @description Plot method for objects of class '`dfunc`'.  Objects of 
#' class '`dfunc`' are estimated distance functions produced by 
#' [dfuncEstim()]. 
#'   
#' @inheritDotParams plot.dfunc.para include.zero nbins newdata 
#'     legend vertLines plotBars density angle xlab ylab 
#'     border col col.dfunc lty.dfunc lwd.dfunc 
#'   
#' @inheritParams print.dfunc 
#' 
#' @details If `plotBars` is TRUE, a scaled histogram is plotted
#'  and the estimated distance function
#'   is plotted over the top of it.  When bars are plotted, 
#'   this routine uses `graphics::barplot` 
#'  for setting up the initial plotting region and
#'  most parameters to `graphics::barplot` can 
#'  be specified (exceptions noted above in description of '\dots').
#'  
#'  The form of the likelihood and any series
#'   expansions is printed in the main title (overwrite this with 
#'   `main="<my title>"`). Convergence of the distance
#'   function is checked.  If the distance function did not converge, a warning
#'   is printed over the top of the histogram.  If one or more parameter
#'   estimates are at their limits (likely indicating non-convergence or poor
#'   fit), another warning is printed. 
#'   
#'   
#' @return The input distance function is returned, with two additional
#'   components than can be used to reconstruct the plotted bars.  (To 
#'   obtain values of the plotted distance functions, use `predict`
#'   with `type = "distances"`.) 
#'   The additional components are:
#'   \item{barHeights}{A vector containing the scaled bar heights drawn 
#'   on the plot.}
#'   \item{barWidths}{A vector or scalar of the bar widths drawn on 
#'   the plot, with measurement units.  }
#'   Re-plot the bars with `barplot( return$barHeights, 
#'   width = return$barWidths )`.
#'   
#' @seealso [dfuncEstim()], [print.dfunc()],
#'   [print.abund()]
#'   
#' @examples 
#' # Simulated RdistDf
#' set.seed(87654)
#' x <- rnorm(1000, mean=0, sd=20)
#' x <- x[x >= 0]
#' x <- setUnits(x, "ft")
#' Df <- data.frame(transectID = "A"
#'                , distance = x
#'                 ) |> 
#'   dplyr::nest_by( transectID
#'                , .key = "detections") |> 
#'   dplyr::mutate(length = setUnits(1,"mi"))
#' attr(Df, "detectionColumn") <- "detections"
#' attr(Df, "obsType") <- "single"
#' attr(Df, "transType") <- "line"
#' attr(Df, "effortColumn") <- "length"
#' is.RdistDf(Df) # TRUE
#' 
#' dfunc <- Df |> dfuncEstim(distance ~ 1, likelihood="halfnorm")
#' plot(dfunc)
#' plot(dfunc, nbins=25)
#' 
#' # showing effects of plot parameters
#' plot(dfunc
#'   , col=c("red","blue","orange")
#'   , border="black"
#'   , xlab="Off-transect distance"
#'   , ylab="Prob"
#'   , vertLines = FALSE
#'   , main="Showing plot params")
#'  
#' plot(dfunc
#'    , col="purple"
#'    , density=30
#'    , angle=c(-45,0,45)
#'    , cex.axis=1.5
#'    , cex.lab=2
#'    , ylab="Probability") 
#' 
#' plot(dfunc
#'    , col=c("grey","lightgrey")
#'    , border=NA) 
#' 
#' plot(dfunc
#'    , col="grey"
#'    , border=0
#'    , col.dfunc="blue"
#'    , lty.dfunc=2
#'    , lwd.dfunc=4
#'    , vertLines=FALSE)
#' 
#' plot(dfunc
#'    , plotBars=FALSE
#'    , cex.axis=1.5
#'    , col.axis="blue")
#' rug(distances(dfunc))
#' 
#' # un-equal bin widths, nbins must span distances
#' plot(dfunc
#'    , nbins = c(0,2.5,5,7.5,10,15,25,50,70)
#' )
#' 
#' # Plot showing f(0)
#' hist(distances(dfunc)
#'    , n = 40
#'    , border = NA
#'    , prob = TRUE)
#' x <- seq(dfunc$w.lo, dfunc$w.hi, length=200)
#' g <- predict(dfunc, type="dfunc", distances = x, newdata = data.frame(a=1))
#' f <- g[,1] / ESW(dfunc)[1]
#' 
#' # Check integration:
#' sum(diff(x)*(f[-1] + f[-length(f)]) / 2) # Trapazoid rule; should be 1.0
#' lines(x, f) # hence, 1/f(0) = ESW
#' 
#' # Covariates: detection by observer
#' data(sparrowDfuncObserver) # pre-estimated model
#' 
#' obsLevs <- levels(sparrowDfuncObserver$data$observer)
#' plot(sparrowDfuncObserver
#'    , newdata = data.frame(observer = obsLevs)
#'    , vertLines = FALSE
#'    , col.dfunc = heat.colors(length(obsLevs))
#'    , col = c("grey","lightgrey")
#'    , border=NA
#'    , main="Detection by observer")
#' 
#' @export
plot.dfunc <- function( x
                        , ... ){

  # dispatch to other functions as follows
  is.smoothed <- Rdistance::is.smoothed(x)
  tType <- Rdistance::transectType(x)
  key <- paste0(tType, "_", is.smoothed)
  switch(key
       # , point_TRUE  = 
       # , line_TRUE   = plot.dfunc.smu(x, ...)
       , point_FALSE = 
       , line_FALSE  = plot.dfunc.para(x, ...)
       , warning( paste("Unrecognized transect type and smooth indicator. Found", key) )
  )

}

#' @title Distance Sampling Abundance Estimates 
#'   
#' @description Estimate abundance (or density) from an estimated detection
#'   function and supplemental information on observed group sizes, transect
#'   lengths, area surveyed, etc.  Computes confidence intervals on
#'   abundance (or density) using a the bias corrected bootstrap method.
#'   
#' @inheritParams dfuncEstim 
#' 
#' @inheritParams predict.dfunc 
#' 
#' @param area A scalar containing the total area of inference. Usually, this is 
#' study area size.  If \code{area} is NULL (the default), 
#' \code{area} will be set to 1 square unit of the output units and density estimates
#' will be produced. 
#' If \code{area} is not NULL, it must have measurement units 
#' assigned by the \code{units} package. 
#' The units on \code{area} must be convertible
#' to squared output units. Units 
#' on \code{area} must be two-dimensional. 
#' For example, if output units are "foo", 
#' units on area must be convertible to "foo^2" by the \code{units}
#' package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
#' others are acceptable.  
#'   
#' @param ci A scalar indicating the confidence level of confidence intervals. 
#'   Confidence intervals are computed using a bias corrected bootstrap
#'   method. If \code{ci = NULL} or \code{ci == NA}, confidence intervals 
#'   are not computed.
#'   
#' @param R The number of bootstrap iterations to conduct when \code{ci} is not
#'   NULL.
#'   
#' @param plot.bs A logical scalar indicating whether to plot individual
#'   bootstrap iterations.
#'   Ignored unless \code{parallel = FALSE}.
#'   
#' @param propUnitSurveyed A scalar or vector of real numbers between 0 and 1.  
#' The proportion of the default sampling unit that 
#'   was surveyed.  If both sides of line transects were observed, 
#'   \code{propUnitSurveyed}
#'   = 1.  If only a single side of line transects were observed, set 
#'   \code{propUnitSurveyed} = 0.5. For point transects, this should be set to 
#'   the proportion of each circle that was observed. Length must either be
#'   1 or the total number of transects in \code{x}.
#'   
#' @param showProgress A logical indicating whether to show a text-based
#'   progress bar during bootstrapping. Default is \code{TRUE}. 
#'   It is handy to shut off the 
#'   progress bar if running this within another function. 
#'   Ignored unless \code{parallel = FALSE}.
#'   
#' @param parallel A logical scalar, or a positive integer; ignored unless 
#'   confidence intervals are requested (i.e., \code{!is.null(ci)}).
#'   If TRUE, bootstrap iterations are 
#'   run in parallel using the maximum number of CPU cores minus 1. 
#'   The maximum number of CPU cores is reported by \code{parallel::detectCores()}.
#'   If a positive integer (1 <= \code{parallel} <= maximum cores), bootstrap 
#'   iterations are performed in parallel on that many cores. 
#'   If FALSE, bootstrap iterations are performed in series, and progress 
#'   will be shown if \code{showProgress == TRUE}. Parameters 
#'   \code{showProgress} and \code{plot.bs} are ignored when operating
#'   in parallel. 
#'   
#' @details The abundance estimate for line-transect surveys (if no covariates
#'    are included in the detection function and both sides of the transect 
#'    are observed) is 
#'    \deqn{N =\frac{n(A)}{2(ESW)(L)}}{%
#'          N = n*A / (2*ESW*L)} 
#'    where \emph{n} is total number of sighted individuals 
#'   (i.e., \code{sum(groupSizes(dfunc))}), \emph{L} is the total length of 
#'   surveyed transect (i.e., \code{sum(effort(dfunc))}),
#'   and \emph{ESW} is effective strip width
#'   computed from the estimated distance function (i.e., \code{ESW(dfunc)}).
#'   If only one side of transects were observed, the "2" in the denominator 
#'   is not present (or, replaced with a "1"). 
#'   
#'   The abundance estimate for point transect surveys (if no covariates are
#'   included) is 
#'    \deqn{N =\frac{n(A)}{\pi(ESR^2)(P)}}{%
#'          N = n*A / ((3.1415)*ESR^2*(P))} 
#'    where \emph{n} is total number of sighted individuals (i.e., \code{sum(groupSizes(dfunc))}),
#'    \emph{P} is the total number of surveyed points (i.e., \code{sum(effort(dfunc))}), 
#'    and \emph{ESR} is effective search radius 
#'    computed from the estimated distance function (i.e., \code{ESR(dfunc)}).
#'
#'  This routine, \code{abundEstim}, estimates abundance on the 
#'  entire study area.  Site-specific density estimates 
#'  are computed by 
#'  \code{predict(x, type = "density")}, which returns a 
#'  tibble containing density and abundance on the area surveyed by every
#'  transect. 
#'   
#' @section Bootstrap Confidence Intervals:
#' 
#'   Rdistance's nested data frames (produced by \code{\link{RdistDf}})
#'   contain all information required to estimate bootstrap CIs. 
#'   To compute bootstrap CIs, Rdistance resamples, with replacement,
#'   the rows of the \code{$data} component contained in Rdistance 
#'   fitted models. Rdistance assumes each row of \code{$data} 
#'   contains information on one transect.
#'   The \code{$data} component also contains 
#'   information on which observations inform the 
#'   detection function, which observations should be counted as 
#'   detected targets, 
#'   and which transects count toward transect length. 
#'   After resampling rows of \code{$data}, Rdistance 
#'   refits the distance function using non-missing distances, 
#'   recomputes the detected number of targets using non-missing 
#'   group sizes on transects with non-missing length, 
#'   and re-computes total transect length from transects 
#'   with non-missing lengths. 
#'   By default, \code{R} = 500 bootstrap iterations are 
#'   performed, after which bias
#'   corrected confidence intervals are computed (Manly, 1997, section 3.4).
#'   
#'   The distance function is not re-selected during bootstrap resampling. The 
#'   model of the input object is re-fitted every iteration.  
#'   
#'   During bootstrap iterations, the distance function can fail. 
#'   An iteration can fail for a two reasons:
#'   (1) no detections on the iteration, and (2) a bad configuration 
#'   of distances that push the distance function's parameters to their 
#'   limits. When an iteration fails, Rdistance 
#'   skips the iteration and effectively ignores the 
#'   failed iterations. 
#'   If the proportion of failed iterations is small 
#'   (less than 20\% by default), the resulting abundance confidence interval 
#'   is probably valid and no warning is issued.  If the proportion of 
#'   non-convergent iterations 
#'   is not small (exceeds 20\% by default), a warning is issued.  
#'   The warning can be modified  
#'   by re-setting option \code{"Rdistance_maxBSFailPropForWarning"} to
#'   the acceptable proportion of failures.. 
#'   Setting \code{options(Rdistance_masBSFailPropForWarning = 1.0)} will turn 
#'   suppress the warning. 
#'   Setting \code{options(Rdistance_masBSFailPropForWarning = 0.0)} will 
#'   warn if any iteration failed.  Results (density and effective 
#'   sampling distance) 
#'   from all successful iterations are contained in the  
#'   non-NA rows of data frame 'B' in the output object.  
#'   
#' @section Missing Transect Lengths:
#' 
#'   Transect lengths can be missing in the RdistDf object. 
#'   Missing length transects are equivalent
#'   to 0 [m] transects and do not count toward total surveyed units,
#'   nor do group sizes on these transects count toward total 
#'   detected individuals.  
#'   Use NA-length transects to include their associated distances 
#'   during distance function estimation, but not when estimating abundance. 
#'   For example, this allows estimation of abundance on one 
#'   study area using off-transect distances from another.  This allows 
#'   sightability to be estimated using two or more similar targets (e.g., 
#'   two similar species), but abundance to be estimated separate for each 
#'   target type.
#'   Include NA-length transects by including the "extra" distance observations 
#'   in the detection data frame, with valid
#'   site IDs, but set the length of those site IDs to NA in the site data frame. 
#'
#' @section Point Transect Lengths:
#'   Point transects do not have a physical measurement for length. 
#'   The "length" of point transects is the number of points on the transect. 
#'   Point transects can contain only one point.  
#'   Rdistance treats transects of points as independent 
#'   and bootstrap resamples them to estimate variance. The number of points
#'   on each point transect must exist in the RdistDf and cannot have 
#'   physical measurement units (it is a count, not a distance).
#'   
#'   
#' @return An Rdistance 'abundance estimate' object, which is a list of
#'   class \code{c("abund", "dfunc")}, containing all the components of a "dfunc"
#'   object (see \code{\link{dfuncEstim}}), plus the following: 
#'   
#'   \item{estimates}{A tibble containing fitted coefficients in the
#'   distance function, density in the area(s) surveyed, 
#'   abundance on the study area, the number of groups seen 
#'   between w.lo and w.hi, the number 
#'   of individuals seen between w.lo and w.hi, 
#'   study area size, surveyed area, average group size, and 
#'   average effective detection distance.
#'   }
#'   
#'   \item{B}{If confidence intervals were requested, a tibble 
#'   containing all bootstrap values of coefficients, 
#'   density, abundance, groups seen, individuals seen, 
#'   study area size, surveyed area size, average group size, 
#'   and average effective detection distance.  The number of rows is always 
#'   \code{R}, the requested number of bootstrap 
#'   iterations.  If an iteration fails, the
#'   corresponding row in \code{B} is \code{NA} (hence, use 'na.rm = TRUE' 
#'   when computing summaries). Columns 1 through \code{length(coef(dfunc))}
#'   contain bootstrap realizations of the distance function's coefficients. 
#'   }
#'   
#'   \item{ci}{Confidence level of the confidence intervals}
#'   
#' @references Manly, B.F.J. (1997) \emph{Randomization, bootstrap, and 
#'   Monte-Carlo methods in biology}, London: Chapman and Hall.
#'   
#'   Buckland, S.T., D.R. Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers,
#'    and L. Thomas. (2001) \emph{Introduction to distance sampling: estimating
#'    abundance of biological populations}. Oxford University Press, Oxford, UK.
#'   
#' @seealso \code{\link{dfuncEstim}}, \code{\link{autoDistSamp}}, 
#'  \code{\link{predict.dfunc}} with 'type = "density"'.
#' 
#' @examples
#' # Load example sparrow data (line transect survey type)
#' # sparrowDf <- RdistDf(sparrowSiteData, sparrowDetectionData)
#' data(sparrowDf)
#' 
#' # Fit half-normal detection function
#' dfunc <- sparrowDf |> 
#'   dfuncEstim(formula=dist ~ groupsize(groupsize)
#'            , likelihood="halfnorm"
#'            , w.hi=150 %m%.
#'   )
#' 
#' # Estimate abundance - Convenient for programming 
#' abundDf <- estimateN(dfunc
#'                    , area = 4105 %km^2%.
#'            )
#' 
#' # Same - Nicer output 
#' # Set ci=0.95 (or another value) to estimate bootstrap CI's 
#' fit <- abundEstim(dfunc
#'                 , area = 4105 %km^2%.
#'                 , ci = NULL
#'                 )
#' summary(fit)
#'          
#' @export
abundEstim <- function(object
                     , area = NULL
                     , propUnitSurveyed = 1.0
                     , ci = 0.95
                     , R = 500
                     , plot.bs = FALSE
                     , showProgress = TRUE
                     , parallel = TRUE
                     ){


  # It is okay to have NA in effort, distance, and group size columns. See documentation.
  
  bootstrapping <- !is.null(ci) && !is.na(ci)
  
  # Initial setup for parallel session ----
  parallelRequest <- getNCores( parallel )
  parallel <- parallelRequest$parallel  # T or F
  
  if( parallel ){
    plot.bs <- FALSE
    showProgress <- FALSE
  }
  
  # Initial setup for plotting ----
  if (bootstrapping && plot.bs) {
    graphics::par( xpd=TRUE )
    plotObj <- plot(object)
  } else {
    plotObj <- NULL
  }
  
  # ---- Set Area ----
  if( is.null(area) ){
    # doing this here saves a tiny sliver of time in estimateN
    area <- setUnits(1, object$outputUnits)^2
  }
  
  # ---- Construct Original estimates frame ----
  ests <- estimateN(object = object
                  , area = area
                  , propUnitSurveyed = propUnitSurveyed
                    )
  ests$id <- "Original"
  
  # ---- Prelims: constants for bootstrapping ----

  # pb <- list(tick = function(){}) # NULL tick function for not bootstrapping

  # ---- Add bootstrap indices if called for ----
  # This is essentially what rsample::bootstraps does, but rsample stores the 
  # entire data frame in each 'rsample'.  This saves space. But, entire 
  # (R*nrow(x$data)+1) X 2 data frame must be constructable in memory.
  if ( bootstrapping ) {
    
    B <- bootstrap(
                    object = object
                  , area = area
                  , propUnitSurveyed = propUnitSurveyed
                  , R = R
                  , plot.bs = plot.bs
                  , plotCovValues = plotObj$predCovValues
                  , showProgress = showProgress
                  , parallel = parallel
                  , cores = parallelRequest$cores
                ) 
    
    # Replace varcovar with bootstrap varcovar
    bsCoefs <- B |> 
      dplyr::ungroup() |> 
      dplyr::select(dplyr::all_of(names(stats::coef(object))))
    object$varcovar <- stats::var(bsCoefs)
    object$asymptoticSE <- FALSE
    

  } else {
    ci <- NA
    B <- NULL
  }
  
  # ---- Construct output object ----
  ans <- c(object
          , estimates = list(ests)
          , B = list(B)
          )
  
  # ---- Plot original fit again (over bs lines) ----
  if (bootstrapping && plot.bs) {
    graphics::lines(object
          , newdata = plotObj$predCovValues
          , col = "red"
          , lwd = 3)
  } 

  # ---- Compute confidence intervals ----      
  if( bootstrapping ){
    vec2df <- function(x, nm){
      # only for vectors of length 2, not general but fast
      xx <- data.frame(lo = x[1], hi = x[2])
      names(xx) <- paste0(nm, "_", names(xx))
      xx
    }
    abCI <- Rdistance::bcCI(B$abundance, ests$abundance, ci)
    dnCI <- Rdistance::bcCI(B$density, ests$density, ci)
    efCI <- Rdistance::bcCI(B$avgEffDistance, ests$avgEffDistance, ci) 
    abCI <- vec2df(abCI, "abundance") 
    dnCI <- vec2df(dnCI, "density") 
    efCI <- vec2df(efCI, "avgEffDistance") 
    
    ans$estimates <- ans$estimates |> 
      dplyr::bind_cols( abCI ) |> 
      dplyr::bind_cols( dnCI ) |> 
      dplyr::bind_cols( efCI ) 

    # rearrange columns    
    ans$estimates <- ans$estimates |> 
      dplyr::select("id"
                  , dplyr::all_of(names(stats::coef(object)))  
                  , dplyr::starts_with("density")
                  , dplyr::starts_with("abundance")
                  , dplyr::starts_with("avgEffDistance")
                  , dplyr::everything())
    B <- B |> 
      dplyr::select("id"
                  , dplyr::all_of(names(stats::coef(object)))  
                  , dplyr::starts_with("density")
                  , dplyr::starts_with("abundance")
                  , dplyr::starts_with("avgEffDistance")
                  , dplyr::everything())
    
    if ((object$LhoodType == "parametric") && 
        (any(is.na(B$density))) && 
        showProgress){
      cat(paste( sum(is.na(B$density)), "of", R
                 , "iterations did not converge.\n"))
    }
  }

  # Output
  ans$ci <- ci
  class(ans) <- c("abund", class(object))
    
  return(ans)
} 

#' @title Coefficients of an estimated detection function
#' 
#' @description Extract distance model coefficients from 
#' an estimated detection function object.
#'   
#' @usage \method{coef}{dfunc}(object, \dots)
#' 
#' @inheritParams is.smoothed
#' 
#' @param ... Ignored
#' 
#' @return The estimated coefficient vector for the detection function. 
#' Length and interpretation of values vary 
#' depending on the form of the detection function and expansion terms.
#'   
#' @seealso [AIC()], [dfuncEstim()]
#' 
#' @examples
#' data(sparrowDfuncObserver) # pre-estimated dfunc
#' 
#' # Same as sparrowDfuncObserver$par 
#' coef(sparrowDfuncObserver) 
#' 
#' \dontrun{
#' data(sparrowDf)
#' dfunc <- sparrowDf |> dfuncEstim(dist~bare + observer,
#'                       w.hi = 150 %m%.)
#' coef(dfunc)
#' }
#'  
#' @export

coef.dfunc <- function(object, ...){
  if( object$likelihood == "smu" ){
    # smoothed distance function
    COEF <- NULL
  } else {
    COEF <- object$par
  }
  COEF
}

#' @title Compute off-transect distances from sighting distances and angles
#' 
#' @description Computes off-transect (also called 'perpendicular') distances from measures of sighting distance and sighting angle.
#' 
#' @param sightDist Character, name of column in `data` that contains the observed or sighting distances from the observer to the detected objects.
#' @param sightAngle Character, name of column in `data` that contains the observed or sighting angles from the line transect to the detected 
#'   objects.  Angles must be measured in degrees.
#' @param data data.frame object containing sighting distance and sighting angle.
#' 
#' @details If observers recorded sighting distance and sighting angle (as is often common in line transect surveys), use this function to convert 
#'   to off-transect distances, the required input data for `dfunc.estim`.
#' @return A vector of off-transect (or perpendicular) distances.  Units are the same as `sightDist`.
#' @references Buckland, S.T., Anderson, D.R., Burnham, K.P. and Laake, J.L. 1993. 
#'   *Distance Sampling: Estimating Abundance of Biological Populations*. Chapman and Hall, London.
#'   
#' @seealso [dfuncEstim()]
#' 
#' @examples
#' # Load the example dataset of sparrow detections from package
#' data(sparrowDetectionData)
#' # Compute perpendicular, off-transect distances from the observer's sight distance and angle
#' sparrowDetectionData$perpDist <- perpDists(sightDist="sightdist", sightAngle="sightangle",
#'                                            data=sparrowDetectionData)
#' @export

perpDists <- function(sightDist, sightAngle, data){
  
  # Stop and print error if dist or angle is NA
  if(any(is.na(data[, sightDist])))
    stop("Please remove detections for which sightDist and/or sightAngle is NA.")
  if(any(is.na(data[, sightAngle])))
    stop("Please remove detections for which sightDist and/or sightAngle is NA.")
  
  # Convert degrees to radians, then apply sine function and round off decimal places
  data[, sightDist] * sin(data[, sightAngle]*(pi/180))
    
}

#' @title Negative log likelihood of distances
#' 
#' @description Return the negative log likelihood of 
#' observed detection distances given a likelihood
#'   and the estimated parameters.
#'
#' @param a A vector of likelihood parameter values. Length and 
#' meaning depend on \code{ml$series} and \code{ml$expansions}. If no expansion 
#' terms were called for (i.e., \code{ml$expansions = 0}), the distance 
#' likelihood contain one or two canonical parameters (see Details). 
#' If one or more expansions are called for, coefficients for the 
#' expansion terms follow coefficients for the canonical parameters.  
#' i.e., length of this vector is 
#'   \code{(num Covars incl. intercept) + expansions + 1*(like \%in\% c("hazrate"))}.
#'   
#' @inheritParams startLimits
#' 
#' @param verbosity The level of output produced during estimation. 
#' \code{verbosity} = 0 produces no output. Increasing values 
#' of \code{verbosity} >= 1 produce increasing levels of intermediate details.
#' This is mostly used for de-bugging and checking convergence.
#'  
#' @details
#' \bold{Expansion Terms}: If \code{ml$expansions} = k (k > 0), 
#' the expansion function specified by \code{ml$series} is 
#' called (see for example \code{\link{cosine.expansion}}). 
#' Assuming \eqn{h_{ij}(x)}{h_ij(x)} is the 
#' \eqn{j^{th}}{j-th} expansion term for the 
#' \eqn{i^{th}}{i-th} distance and that 
#' \eqn{c_1, c_2, \dots, c_k}{c(1), c(2), ..., c(k)} 
#' are (estimated) coefficients for the expansion terms, 
#' the likelihood contribution for the \eqn{i^{th}}{i-th} 
#' distance is, \deqn{f(x|a,b,c_1,c_2,\dots,c_k) = 
#' f(x|a,b)(1 + \sum_{j=1}^{k} c_j h_{ij}(x)).}{f(x|a,b,c_1,c_2,...,c_k) 
#' = f(x|a,b)(1 + c(1) h_i1(x) + c(2) h_i2(x) + ... + c(k) h_ik(x)). }
#'   
#' @return A scalar, the negative of the log likelihood evaluated at 
#' parameters \code{a}.
#' 
#' @seealso See \code{\link{halfnorm.like}} and links there; 
#'  \code{\link{dfuncEstim}}
#' 
#' @examples
#' 
#' # A halfnorm distance function 
#' fit <- dfuncEstim(sparrowDf, dist~1, likelihood = "halfnorm")
#' nLL(fit$par, fit, 3)
#' fit$loglik
#' ESW(fit)[1]
#' 
#' # Another way, b/c we have pnorm()
#' d <- distances(fit)
#' ones <- matrix(1, nrow = length(d), ncol = 1)
#' l <- halfnorm.like(fit$par, d, ones)
#' esw <-(pnorm(units::drop_units(fit$w.hi)
#'   , units::drop_units(fit$w.lo)
#'   , sd = exp(l$params)) - 0.5) * sqrt(2*pi) * exp(l$params)
#' -sum(log(l$L.unscaled/esw))
#' 
#' # A third way, b/c we have pnorm() and dnorm(). 
#' l2 <- dnorm(units::drop_units(d), mean = 0, sd = exp(fit$par))
#' scaler <- pnorm(units::drop_units(fit$w.hi), mean = 0, sd = exp(fit$par)) - 0.5 
#' -sum(log(l2/scaler))
#'  
#' @export

nLL <- function(a
                , ml
                , verbosity = 0
                ){
  
  
  # Pull data from input list ----
  # rule is: parameters in 'a' never have units,  
  # even though they could (e.g., sigma of halfnorm)

  # Because of na.pass when building model frame, and 
  # other code in parseModel(), there could be missing distances 
  # ONLY because they are inherently missing (crew did not get a distance
  # on an observation). Observations outside the strip are not present 
  # in the model frame (ml$mf). 

  # Evaluate the "key" function ----
  # I call Buckland's "key" function just "likelihood".
  # Returns one value per observation. Expansions, if any, 
  # are applied below, after this.
  fuzz <- getOption("Rdistance_fuzz")
  f.like <- utils::getFromNamespace(paste0( ml$likelihood, ".like"), "Rdistance")    
  d <- distances(ml) - ml$w.lo # observed distances, n of them
  X <- stats::model.matrix(ml)
  L <- f.like( a = a
             , dist = d
             , covars = X
             , w.hi = ml$w.hi - ml$w.lo # needed for some but not all Likes
             )
  key <- L$L.unscaled  # (n X k matrix)
  parms <- L$params

  if( verbosity >= 3 ){
    likExpan <- paste0(ml$likelihood, "_", ml$expansions, "_", transectType(ml))
    cat(colorize(paste(paste(rep("-", 40), collapse = ""))))
    cat(paste(" "
              , colorize(likExpan)
              , "\n"))
  }
  
  
  if( all(is.na(key)) ){
    # Parameters are out of bounds.  Happens when using optim.
    # Save a bunch of computation by doing this...
    key <- matrix(fuzz, nrow = nrow(key), ncol = ncol(key))
    if( verbosity >= 4 ){
      # for later
      exp.terms <- rep(NA, 5)
      W <- NA
    }
    
  } else {
    # Evaluate and apply the expansions ----
    if( ml$expansions > 0 ){
      W <- expandW(ml = ml
                 , params = parms
                 , k = ncol(key)  # I think this is always 1
                 )
      # The following call to expansionTerms returns matrix size (n X k); here,
      #  either nx1 or nxn.
      exp.terms <- Rdistance::expansionTerms(a = a
                                             , d = d
                                             , series = ml$series
                                             , nexp = ml$expansions
                                             , w = W)
      
      if( ncol(exp.terms) > 1 ){
        # W is len k; so exp.terms is kxk
        exp.terms <- diag(exp.terms)
      }
      
      key <- key * exp.terms
  
      # b/c no monotonicity restraints yet, function can go negative, 
      key[ !is.na(key) & (key <= 0) ] <- getOption("Rdistance_zero")
    }
  
    # Apply d if point survey ----
    if(is.points(ml)){
      key <- d * key  # element-wise
    }

    
    # Integrate under the distance function so can scale to density ----
    # This yields area under g(x), which is ESW, and is used to get f(x) = g(x)/ESW
    parms[,1] <- exp(parms[,1]) # variable param
    if( ml$expansions > 0 ){
      # append expansion coefs to parms; needed during integration
      coefLocs <- length(a) - (ml$expansions:1) + 1 
      parms <- cbind(parms
                      , matrix( a[coefLocs]
                               , nrow = nrow(parms)
                               , ncol = ml$expansions
                               , byrow = TRUE
                      ))  # n X ([#canonical] + nexp)
    }
    
    outArea <- integrateDfuncs( parms, ml )
    
    if( verbosity >= 3 ){
      cat(paste(" "
                , "integral ="
                , colorize(unique(outArea))
                , "\n"))
    }
    
    # Scale g(x) into f(x) ----
    key <- key / outArea

    # Note: Key or d*Key should integrate to 1.0 every iteration
    # Note 2: if POINTS, key has units, remove them b/c nlminb can't handle em.
    key <- dropUnits(key)
  
    # Check for really bad values ----
    #   RULE: Any impossible observations (i.e., key exactly 0) get 
    #   replaced by fuzz so that log(L) does not return -Inf. "exactly 0" is 
    #   less than fuzz. Worst fitting objects now contribute -log(fuzz) ~ 31.5 to LL
    
  
    ind <- is.na(key) | (key < fuzz)
    if( any(ind) ){
      key[ ind ] <- fuzz # replace them
    }

  } # endif - if(all(is.na(key)))  
  
  # Compute neg log likelihood ----
  nLL <- -sum(log(key), na.rm=TRUE)  # Note that distances > w in L are set to NA

  # Rules: 
  #   RULE 1 FOR LIKELIHOODS: No matter how bad the guess at a, you cannot return Inf, -Inf, NA, or NaN
  #   This means f.like can return NA, but not NaN (div by 0), Inf or -Inf for any row of data
  #   Must program the likelihoods to trap these values and return the appropriate .Machine constants
  #
  #   RULE 2 FOR NLL: It is possible for all rows of returned L to be non-Inf and non-NaN, 
  #   but there be enough of them that the sum overflows to Inf. This is unlikely because 
  #   we replace any LL < fuzz with fuzz (above) and log(fuzz) ~ -32. But, nonetheless
  #   trap infinite likelihoods here.
  #
  if( is.infinite(nLL) ){
    nLL <- getOption("Rdistance_posInf") # positive b/c already flipped over by -1: Should not happen
  }
  
  # Diagnostics ----
  if( verbosity >= 1 ){
    cat(paste0("  Coeffs: ", colorize(paste(a, collapse = ", "))))
    cat(paste(" -log(likelihood) =", colorize(nLL), "\n"))
  }
  if( length(unique(outArea)) <= 1 ){
    # check of integral only works w/o covars
    if( verbosity >= 3 ){
      integrateKey(ml, key, f0 = 1 / unique(outArea), plot=TRUE )
    } else if( verbosity >= 2) {
      integrateKey(ml, key, f0 = 1 / unique(outArea), plot=FALSE )
    }
  }
  if( verbosity >= 4 ){
    cat(crayon::bgRed( paste("data[1:5]      =", paste(d[1:5], collapse = ", "), "\n")))
    cat(crayon::bgRed( paste("key[1:5]       =", paste(key[1:5], collapse = ", "), "\n")))
    cat(crayon::bgRed( paste("exp.terms[1:5] =", paste(exp.terms[1:5], collapse = ", "), "\n")))
    cat(crayon::bgRed( paste("W              =", paste(W, collapse = ", "), "\n")))
  }
  
  
  if( verbosity >= 5 ){
    yn <- readline("Abort [y/N]?")
    if( toupper(yn) == "Y" ){
      stop("Aborted by user.")
    }
  }

  
  nLL
}

#' @title Integration of distance functions
#' 
#' @description
#' Integrates under distances functions using exact integrals 
#' when possible.  If exact integrals are not known, numerical 
#' integration is used. 
#' 
#' @details
#' Let K be the integral under distance function g(x) (i.e., the output
#' from this routine). In distance analysis, the observation likelihood 
#' being evaluated for maximization 
#' is the *density*, f(x) = g(x)/K.  K is a key quantity in distance 
#' analysis and is called the "effective sampling distance". 
#' 
#' @inheritParams integrateOneStepPoints 
#' 
#' @inheritParams startLimits 
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @examples
#' # Faking a model frame
#' ml <- list( likelihood = "halfnorm"
#'           , expansions = 0
#'           , w.lo = 0 %m% .
#'           , w.hi = 100 %m% .
#'           , Units = "m"
#'           )
#' attr(ml, "transType") <- "line"
#' 
#' parms <- matrix(75, nrow = 1) 
#' integrateDfuncs(parms, ml)
#' 
#' # check: Normal, 0 to 100, sd = 75, scaled to mode = 1
#' (pnorm(q = 100, mean = 0, sd = 75) - 0.5) * sqrt(2*pi)*75
#'
#' @export 
integrateDfuncs <- function( object
                           , ml ) {
  
  likExpan <- paste0(ml$likelihood, "_", ml$expansions, "_", transectType(ml))
  
  # The following IF cases were implemented to speed calculations 
  # dramatically when we know the integrals (i.e., avoid numerical 
  # integration when we can). 
  
  if( likExpan == "halfnorm_0_line" ){
    # CASE: Halfnormal, 0 expansions, Lines ----
    intType <- "Exact"
    
    outArea <- integrateHalfnormLines(object
                                      , w.lo = ml$w.lo
                                      , w.hi = ml$w.hi
                                      , Units = ml$outputUnits
    )
    
  } else if( likExpan == "halfnorm_0_point" ){
    # CASE: Halfnormal, 0 expansions, Points ----
    intType = "Exact"
    
    outArea <- integrateHalfnormPoints(object
                                       , w.lo = ml$w.lo
                                       , w.hi = ml$w.hi
                                       , Units = ml$outputUnits
    )
    
  } else if( likExpan == "negexp_0_line" ){
    # CASE: Negative exponential, 0 expansions, Lines ----
    intType = "Exact"
    
    outArea <- integrateNegexpLines(object
                                    , w.lo = ml$w.lo
                                    , w.hi = ml$w.hi
                                    , Units = ml$outputUnits
    )
    
  } else if( likExpan == "negexp_0_point" ){
    # CASE: Negative exponential, 0 expansions, Points ----
    intType = "Exact"
    
    outArea <- integrateNegexpPoints(object
                                     , w.lo = ml$w.lo
                                     , w.hi = ml$w.hi
                                     , Units = ml$outputUnits
    )
    
  } else if( likExpan == "hazrate_0_line" ){
    # CASE: Hazrate, 0 expansions, Lines ----
    intType = "Exact"
    
    outArea <- integrateHazrateLines(object
                                     , w.lo = ml$w.lo
                                     , w.hi = ml$w.hi
                                     , Units = ml$outputUnits
    )
    
  } else if( likExpan == "oneStep_0_line" ){
    # CASE: One step, 0 expansions, lines ----
    # Answer is:Theta <- object[,1];p <- object[,2];outArea <- Theta / p
    intType = "Exact"
    
    outArea <- integrateOneStepLines(object, Units = ml$outputUnits)
    
  } else if( likExpan == "oneStep_0_point"){
    # CASE: One step, 0 expansions, points ----
    intType = "Exact"
    
    outArea <- integrateOneStepPoints(object
                                      , w.lo = ml$w.lo
                                      , w.hi = ml$w.hi
                                      , Units=ml$outputUnits)
    
  } else if( grepl("oneStep", likExpan )){
    # CASE: oneStep (point or line) with expansions ----
    # Numeric integration by Trapazoid Rule
    # Must integrate from 0 to Theta, then Theta+ to w.hi
    # We know ml$expansions > 0 in this case
    # For expansion calculation we need expansion coefficients in 'object'
    # Do NOT exp() parameter b/c raw likelihood is called inside integrateOneStepNumeric
    # and the likelihood applies the link function
    intType = "Trapazoid"
    
    object[,1] <- log(object[,1]) 
    
    outArea <- integrateOneStepNumeric(object
                                       , w.lo = ml$w.lo
                                       , w.hi = ml$w.hi
                                       , Units = ml$outputUnits
                                       , expansions = ml$expansions
                                       , series = ml$series
                                       , isPoints = is.points(ml))
    
  } else if( likExpan == "triangle_0_line" ){
    
    # CASE: triangle, 0 expansions, lines ----
    # Answer is:Theta <- object[,1];p <- object[,2];outArea <- Theta / p
    intType = "Exact"
    
    outArea <- integrateTriangleLines(object
                                      , w.lo = ml$w.lo
                                      , w.hi = ml$w.hi
                                      , Units = ml$outputUnits)
    
  } else if( likExpan == "huber_0_line" ){
    
    # CASE: huber, 0 expansions, lines ----
    intType = "Exact"
    
    outArea <- integrateHuberLines(object
                                  , w.lo = ml$w.lo
                                  , w.hi = ml$w.hi
                                  , Units = ml$outputUnits)
    
  } else if( likExpan == "Gamma_0_line"){
    # CASE: Gamma, 0 expansions, lines ----
    intType = "Exact"
    
    outArea <- integrateGammaLines(object
                                   , w.lo = ml$w.lo
                                   , w.hi = ml$w.hi
                                   , Units=ml$outputUnits)
    
  # Trent could not figure out the integral of gamma points, when he does,
  # he should implement it in integrateGammaPoints and uncomment this code:
  # } else if( likExpan == "Gamma_0_point"){
  #   # CASE: Gamma, 0 expansions, lines ----
  #   intType = "Exact"
  # 
  #   outArea <- integrateGammaPoints(object
  #                                  , w.lo = ml$w.lo
  #                                  , w.hi = ml$w.hi
  #                                  , Units=ml$outputUnits)
    
  } else {
    # CASE: All other cases = Numeric integration by Simpson's Rule ----
    # do NOT exp parameters
    intType = "Simpson"
    
    # if( ml$expansions > 0 ){
    #   coefLocs <- (length(ml$par)-(ml$expansions-1)):(length(ml$par))
    #   object <- cbind(object
    #                  , matrix(ml$par[coefLocs]
    #                           , nrow = nrow(object)
    #                           , ncol=ml$expansions
    #                           , byrow = TRUE
    #                  ))  
    # }
    object[,1] <- log(object[,1]) 
    
    outArea <- integrateNumeric(object
                                , w.lo = ml$w.lo
                                , w.hi = ml$w.hi
                                , Units = ml$outputUnits
                                , expansions = ml$expansions
                                , series = ml$series
                                , isPoints = is.points(ml)
                                , likelihood = ml$likelihood
    )
    
  }

  # Done. Final prep of output ----  
  attr(outArea, "interalType") <- intType
  outArea
  
}

#' @title Numeric second derivatives
#' 
#' @description Computes numeric second derivatives (hessian) of an 
#' arbitrary multidimensional function at a particular location.
#' 
#' @param x The location (a vector) where the second derivatives 
#' of \code{FUN} are desired.
#' 
#' @param FUN An R function for which the second derivatives are 
#' sought.  
#' This must be a function of the form FUN <- function(x, ...)\{...\}
#' where x is a vector of variable parameters to FUN at which 
#' to evaluate the 2nd derivative, 
#' and ... are additional parameters needed to evaluate the function.
#' FUN must return a single value (scalar), the height of the 
#' surface above \code{x}, i.e., FUN evaluated at x.  
#' 
#' @param eps A vector of small relative 
#' distances to add to \code{x} 
#' when evaluating derivatives.   This determines the '\eqn{dx}' of 
#' the numerical derivatives.  That is, the function 
#' is evaluated at \code{x}, \code{x+dx}, and \code{x+2*dx}, where 
#' \eqn{dx} = \code{x*eps^0.25}, in order to compute the second 
#' derivative. 
#' \code{eps} defaults to 1e-8 for all 
#' dimensions which equates to setting \eqn{dx} to one percent
#' of each \code{x} (i.e., by default the function is 
#' evaluate at \code{x}, \code{1.01*x} and \code{1.02*x} 
#' to compute the second derivative).  
#' 
#' One might want to change \code{eps} if the scale 
#' of dimensions in \code{x} varies wildly (e.g., kilometers and millimeters), 
#' or if changes between 
#' \code{FUN(x)} and \code{FUN(x*1.01)} are below machine precision.
#' If length of \code{eps} is less than length of \code{x},
#' \code{eps} is replicated to the length of \code{x}. 
#' 
#' @param \dots Any arguments passed to \code{FUN}. 
#' 
#' @details This function uses the "5-point" numeric second derivative 
#' method advocated in numerous numerical recipe texts.  During computation
#' of the 2nd derivative, FUN must be 
#' capable of being evaluated at numerous locations within a hyper-ellipsoid 
#' with cardinal radii 2*\code{x}*(\code{eps})^0.25 = 0.02*\code{x} at the 
#' default value of \code{eps}.   
#' 
#' A handy way to use this function is to call an optimization routine 
#' like \code{nlminb} with FUN, then call this function with the 
#' optimized values (solution) and FUN.  This will yield the hessian 
#' at the solution and this is can produce a better 
#' estimate of the variance-covariance
#' matrix than using the hessian returned by some optimization routines. 
#' Some optimization routines return the hessian evaluated 
#' at the next-to-last step of optimization. 
#' 
#' An estimate of the variance-covariance matrix, which is used in 
#' \code{Rdistance}, is \code{solve(hessian)} where \code{hessian} is
#' \code{secondDeriv(<parameter estimates>, <likelihood>)}.  
#' 
#' @examples 
#' 
#'func <- function(x){-x*x} # second derivative should be -2
#'secondDeriv(0,func)
#'secondDeriv(3,func)
#'
#'func <- function(x){3 + 5*x^2 + 2*x^3} # second derivative should be 10+12x
#'secondDeriv(0,func)
#'secondDeriv(2,func)
#'
#'func <- function(x){x[1]^2 + 5*x[2]^2} # should be rbind(c(2,0),c(0,10))
#'secondDeriv(c(1,1),func)
#'
#' @export

secondDeriv <- function(x
                        , FUN
                        , eps=1e-8
                        , ...
                        ){
  d <- length(x)   # number of dimensions
  if(d > length(eps)){
    eps <- rep(eps,ceiling( d/length(eps) ))[1:d]
  } else if( length(eps) > d){
    eps <- eps[1:d]
  }
  FUN <- match.fun(FUN)
  hess <- matrix(0, nrow=d, ncol=d)
  h <- ifelse(x==0, eps^0.25, (eps^(0.25))*x )
  for(i in 1:d){
    ei <- rep(0,d)
    ei[i] <- 1
    # compute diagonal element
    hess[i,i] <- (-FUN(x+2*h*ei, ...) + 
                   16*FUN(x+h*ei, ...) - 
                   30*FUN(x, ...) +
                   16*FUN(x-h*ei, ...) - 
                   FUN(x-2*h*ei, ...)) / (12*h[i]*h[i])
    if((i+1) <= d){
      # compute off diagonal elements in row i
      for(j in (i+1):d){
        ej <- rep(0,d)
        ej[j] <- 1
        hess[i,j] <- (FUN(x+h*ei+h*ej, ...) - 
                        FUN(x+h*ei-h*ej, ...) -
                        FUN(x-h*ei+h*ej, ...) + 
                        FUN(x-h*ei-h*ej, ...)) / (4*h[i]*h[j])
        # Assume symetric
        hess[j,i] <- hess[i,j]
      }
    }
  }
  hess
}

#' @title Group Sizes
#'
#' @description 
#' Extract the group size information from 
#' an Rdistance model frame.
#' 
#' @inheritParams startLimits
#' 
#' @param ... Ignored
#' 
#' @return A vector containing group sizes contained
#' in the Rdistance model frame or fitted object.
#' 
#' @examples
#' 
#' data("sparrowDf")
#' sparrowModel <- parseModel( sparrowDf, dist ~ observer )
#' stats::model.offset(sparrowModel$mf)
#' groupSizes(sparrowModel)  # same, but future-proof
#'
#' sparrowModel <- parseModel( sparrowDf
#'                  , dist ~ observer + groupsize(groupsize) )
#' groupSizes(sparrowModel)  
#' 
#' @export
#' 
groupSizes <- function(ml, ...){
  stats::model.offset(ml$mf)
}

#' @title Find coordinate of function maximum 
#' 
#' @description Find the x coordinate that maximizes g(x).
#' 
#' @param fit An estimated 'dfunc' object produced by `dfuncEstim`.
#' 
#' @param covars Covariate values to calculate g(x).
#' 
#' @return The value of x that maximizes g(x) in `fit`.
#' 
#' @seealso [dfuncEstim()]
#' 
#' @examples \dontrun{
#' # Fake data
#' set.seed(22223333)
#' x <- rgamma(100, 10, 1)
#' 
#' fit <- dfuncEstim( x, likelihood="Gamma", x.scl="max" )
#' 
#' maximize.g( fit )  # should be near 10.
#' fit$x.scl            # same thing
#' }
#' 
#' @export

maximize.g <- function( fit, covars = NULL ){

g.neg <-  function(x, 
                   params, 
                   covars = NULL, 
                   like, 
                   w.lo, 
                   w.hi, 
                   series, 
                   expansions=0, 
                   pointSurvey = FALSE,
                   correctUnits){
    f.like <- utils::getFromNamespace(paste0( x$likelihood, ".like"), "Rdistance")

    if( x < w.lo ){
      x <- w.lo # somehow, optim occasionally passes in -4.1e-15, essentially zero, but 
             # likelihood returns NA and optim bombs.  I'm not happy with optim. Lower
             # argument to optim is clearly 0
    } else if( x > w.hi ){
      x <- w.hi
    }
    
    # above here, we needed x, w.lo, and w.hi to be unitless
    # because they had to pass through optim(). Now, we need 
    # them to have units because we are about to call the likelihood.
    x <- setUnits(x, correctUnits)
    w.lo <- setUnits(w.lo, correctUnits)
    w.hi <- setUnits(w.hi, correctUnits)
    
    g.x <- f.like( a = params, 
                   dist = x, 
                   covars = covars, 
                   w.lo=w.lo, 
                   w.hi=w.hi, 
                   series = series, 
                   expansions = expansions, 
                   pointSurvey = pointSurvey )

    -g.x * 10000000000
    
}

# Strategy for handling units:  make sure everything is 
# converted to the same units, then drop units because optim
# does not propagate them.  w.lo and w.hi are PROBABLY already 
# in the correct units, but make sure.

correctUnits <- fit$outputUnits

wlo <- setUnits(fit$w.lo, correctUnits)
whi <- setUnits(fit$w.hi, correctUnits)

wlo <- dropUnits(wlo)
whi <- dropUnits(whi)

x.start <- (wlo + whi) / 10 + wlo

x.max <- stats::optim(par = x.start, 
               fn = g.neg,  
               method = "L-BFGS-B", 
               lower=wlo, 
               upper=whi, 
               
               params = fit$parameters, 
               w.lo=wlo,
               w.hi=whi,
               like=fit$like.form,
               expansions=fit$expansions, 
               series=fit$series, 
               covars = covars, 
               pointSurvey = fit$pointSurvey,
               correctUnits = correctUnits)


if( x.max$convergence != 0 ){
    warning(paste("Maximum of g() could not be found. Message=", x.max$message))
    x.max <- NA
} else {
    x.max <- x.max$par
}

x.max
}

#' @title Abundance point estimates
#' 
#' @description Estimate abundance from an Rdistance fitted model. 
#' This function is called internally by \code{abundEstim}.  Most users will call 
#' \code{abundEstim} to estimate abundance unless they are running simulations or 
#' bootstrapping.
#' 
#' @inheritParams predict.dfunc 
#' @inheritParams abundEstim
#'  
#' @inherit abundEstim details
#'   
#' @return A list containing the following components:
#' 
#'    \item{density}{Estimated density in the surveyed area.}
#'    
#'    \item{abundance}{Estimated abundance on the study area. Equals density if 
#'    area is not specified.}
#'    
#'    \item{n.groups}{The number of detected groups (not individuals, unless all group sizes = 1).}
#'    
#'    \item{n.seen}{The number of individuals (sum of group sizes).}
#'    
#'    \item{area}{Total area of inference. Study area size}
#'    
#'    \item{surveyedUnits}{Number of surveyed sites.  This is total transect length
#'    for line-transects or number of points for point-transects. This total transect
#'    length does not include transects with missing lengths.}
#'    
#'    \item{propUnitSurveyed}{Proportion of the standard survey unit
#'    that was observed}
#'    
#'    \item{avg.group.size}{Average group size on non-NA transects}
#'    
#'    \item{w}{Strip width. }
#'    
#'    \item{pDetection}{Probability of detection.}
#'    
#' For line-transects that do not involve covariates, object$density  
#' is object$n.seen / (2 * propUnitSurveyed * object$w * object$pDetection * object$surveyedUnits)
#'    
#'         
#'    
#' @seealso \code{\link{dfuncEstim}}, \code{\link{abundEstim}}
#' 
#' @export 

estimateN <- function(object
                      , area = NULL
                      , propUnitSurveyed = 1.0
                      ){

  w <- object$w.hi - object$w.lo
  
  # ---- Find observations on NA length transects inside the strip ----
  # We need these regardless whether dfunc converges or not
  groupSz <- Rdistance::groupSizes(object) # length = num distance obs (could include NA)
  eff <- Rdistance::effort(object) # length = num non-missing plus missing transects
  totSurveyedUnits <- sum(eff, na.rm = TRUE) # na.rm CRITICAL here: remove transects with NA length
  if( !Rdistance::is.points(object) ){
    if(units(totSurveyedUnits) != object$outputUnits){
      # w has units we want; but, effort came from user and has not been converted yet
      totSurveyedUnits <- setUnits(totSurveyedUnits, object$outputUnits)
    }
  }  # Point effort vector has no units b/c it's number of points
  
  # ---- Estimate numerator of abundance ----
  if( object$convergence == 0 ){
    # REMEMBER: component $mf is the model frame and has been truncated to (w.lo, w.hi) and
    #           potentially has distance observations from transects without lengths
    #           component $data has NOT been truncated to the strip
  
    # esw is always a vector of length n. 
    esw <- effectiveDistance(object)
    
    if (Rdistance::is.points(object)) {
      phat <- (esw / w)^2  # for points
    } else {
      phat <- esw / w  # for lines
    }
  
    # phat should be unit-less; check just to be sure, if so drop "1" units
    # Odd: sometimes phat has units "1", sometimes "m/m". Either way, remove,
    phat <- dropUnits(phat)
    nhat <- groupSz / phat # inflated counts one per detection
    
    # ---- Compute density ----
    # na.rm CRITICAL here; missing groupsizes on missing transects
    if(Rdistance::is.points(object)){
      dens <- sum(nhat, na.rm = TRUE) / (propUnitSurveyed * pi * w^2 * totSurveyedUnits) 
    } else {
      dens <- sum(nhat, na.rm = TRUE) / (propUnitSurveyed * 2 * w * totSurveyedUnits)
    }
    
    # ---- Compute abundance ----
    oneSqUnit <- setUnits(1, object$outputUnits)^2 
    if( is.null(area) ){
      area <- oneSqUnit
    } else if( units(area) != units(oneSqUnit) ){
      area <- setUnits(area, units(oneSqUnit))
    }
    
    nhat.df <- dens * area
    if( units(nhat.df) != units(setUnits(1, "1")) ){
      warning(paste("Units on N are not 1 (unitless). Some units did not convert correctly."
                    , "Manually convert all measurements"
                    , "to the same units outside Rdistance, and re-run."))
    } else {
      nhat.df <- dropUnits(nhat.df)
    }
    
  } else {
    # if we are here, object did not converge
    dens <- NA
    nhat.df <- NA
    phat <- NA
  }

  Coefs <- data.frame(matrix(stats::coef(object), nrow = 1))
  names(Coefs) <- names(stats::coef(object))
  
  if(Rdistance::is.points(object)){
    avgEDD <- mean( sqrt(phat) * w, na.rm = TRUE)
  } else {
    avgEDD <- mean( phat * w, na.rm = TRUE)
  }
  
  # ---- Make output data frame ----
  nhat.df <- tibble::tibble(
    Coefs
    , density = dens
    , abundance = nhat.df
    , nGroups = sum(!is.na(groupSz))
    , nSeen = sum(groupSz, na.rm = TRUE)
    , avgGroupSize = mean(groupSz, na.rm = TRUE)
    , area = area
    , surveyedUnits = totSurveyedUnits
    , propUnitSurveyed = propUnitSurveyed
    , w = w
    , avgEffDistance = avgEDD
  )

  # some interesting tidbits:
  #  sampled area = tot.trans.len * 2 * (dfunc$w.hi - dfunc$w.lo)
  #  sum(1/phat) = what Distance calls "N in covered region".  This is
  #    estimated number of groups in the sampled area.
  #  sum(1/phat)*mean(dfunc$detections$groupsize) = an estimate of individuals
  #    in the sampled area. This is probably how Distance would do it.
  #  sum(dfunc$detections$groupsize/phat) = the HT estimate of individuals 
  #    in the sampled area.  How RDistance does it. This and the Distance 
  #    estimate are very close.  Only difference is ratio of sums or sum of 
  #    ratios.
  #  sum(detectionData$groupsize) / (sum(1/phat)*mean(detectionData$groupsize)) = 
  #    n.indivs / (nhat.groups*mean.grp.size) = n.groups / nhat.groups = 
  #    what Distance calls "Average p".  This is different than mean(phat) 
  #    the way Rdistance calculates it.

  return(nhat.df)
}  

#' @title Start and limit values for the Huber distance function
#' 
#' @description Computes starting values and limits for the 'huber' 
#' distance function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @export
huber.start.limits <- function(ml){

  X <- stats::model.matrix(ml)
  dist <- Rdistance::distances(ml)  
  
  np <- ncol(X)  
  expan <- ml$expansions

  fuzz <- getOption("Rdistance_fuzz")
  zero <- getOption("Rdistance_zero")
  posInf <- getOption("Rdistance_posInf")
  negInf <- getOption("Rdistance_negInf")
  
  wlo <- ml$w.lo
  whi <- ml$w.hi
  w <- dropUnits(whi - wlo)
  
  # Initial values for Theta1, Theta2, and p
  q1  <- dropUnits(stats::quantile(dist, p = 0.33))
  q2  <- dropUnits(stats::quantile(dist, p = 0.67))
  p1 <- 0.05
    
  start <- c(log(q1)
             , rep(zero, np - 1)
             , q2
             , p1
             , rep(zero, expan)
             )
  low   <- c(log(zero)
             , rep(negInf, np - 1 )
             , zero
             , zero
             , rep(negInf, expan)
             )
  high  <- c(log(w * 1.5)
             , rep(posInf, np - 1 )
             , w 
             , 1 - fuzz
             , rep(posInf, expan)
             )
  nms <- c(colnames(X), "theta2", "p")
  
  if(expan > 0){
    nms <- c(nms, paste( "a", 1:expan, sep=""))
  }
  
  names(start) <- nms
  names(low) <- nms
  names(high) <- nms
  
  list( start=start, low=low, high=high, names=nms )
  
}

#' @title Unnest an RdistDf data frame
#' 
#' @description Unnest an RdistDf data frame by expanding 
#' the embedded 'detections' column.  This unnest
#' includes the so-called zero transects (transects without detections).
#' 
#' @inheritParams dfuncEstim
#' 
#' @param ... Additional arguments passed to `tidyr::unnest` if
#' `data` is not an RdistDf.  
#' 
#' @return An expanded data frame, without embedded data frames. Rows 
#' in the return represent with one detection or one transect.  If multiple
#' detections were made on one transect, the transect will appear on multiple
#' rows.  If no detections were made on a transect, it will appear on one 
#' row with NA detection distance. 
#' 
#' @examples
#' data('sparrowDf')
#' 
#' # tidyr::unnest() does not include zero transects
#' detectionDf <- tidyr::unnest(sparrowDf, detections)
#' nrow(detectionDf)
#' any(detectionDf$siteID == "B2")
#' 
#' # Rdistance::unnest() includes zero transects
#' fullDf <- unnest(sparrowDf)
#' nrow(fullDf)
#' any(fullDf$siteID == "B2")
#' 
#' @export
unnest <- function(data, ...){
  
  if( Rdistance::is.RdistDf(data) ){
    
    detectionCol <- attr(data, "detectionColumn")
    rowID <- data |> dplyr::group_vars()
    isNull <- data |> 
      dplyr::mutate(dplyr::across(dplyr::all_of(detectionCol)
                                           , ~ is.null(.x))) |> 
      dplyr::pull(dplyr::all_of(detectionCol))
    
    fullDf <- data |> 
      dplyr::ungroup() |> 
      dplyr::filter(isNull) |> # zero transects
      dplyr::select(-dplyr::all_of(detectionCol)) |> 
      dplyr::bind_rows(tidyr::unnest(data = data
                                   , cols = dplyr::all_of(detectionCol))) |> 
      dplyr::arrange(dplyr::across(dplyr::all_of(rowID)))  
    
  } else {
    warning("Rdistance::unnest called with a non-RdistDf data frame. tidyr::unnest() applied instead.")
    fullDf <- tidyr::unnest(data, ...)
  }
  
  fullDf
}

#' @title Start and limit values for halfnorm distance function
#' 
#' @description Compute starting values and limits for the half normal distance
#' function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @export
halfnorm.start.limits <- function (ml){
  
  X <- stats::model.matrix(ml)
  dist <- Rdistance::distances(ml)  
  
  expan <- ml$expansions
  ncovars <- nCovars(X)

  fuzz <- getOption("Rdistance_fuzz")
  zero <- getOption("Rdistance_zero")
  posInf <- getOption("Rdistance_posInf")
  negInf <- getOption("Rdistance_negInf")
  
  dist2 <- dist - ml$w.lo
  # Only time dist2 will not have units is when user overides requirement
  # otherwise this always runs
  dist2 <- dropUnits(dist2)
  dist2 <- dist2[dist2 > 0]
  sdHalf <- max(sqrt(sum( dist2^2 )/length(dist2)), fuzz)
  
  start <- c(log(sdHalf)
             , rep(zero, ncovars - 1)
             , rep(zero, expan)
             )
  if( ncovars <= 1 ){
    # (Intercept)-only model. Use tighter bounds.
    low <- log(zero)
    high <- log(posInf)
  } else {
    # We have covariates
    low <- rep(negInf, ncovars)
    high <- rep(posInf, ncovars)
  }
  # Add bounds for expansions
  low <- c(low
         , rep(negInf, expan)
           )
  high  <- c(high
           , rep(posInf, expan)
           )
  nms <- colnames(X)
  
  if(expan > 0){
    nms <- c(nms, paste( "a", 1:expan, sep=""))
  }
  
  names(start) <- nms
  names(low) <- nms
  names(high) <- nms
  
  list( start=start, low=low, high=high, names=nms )
  
}

#' @title Integrate Half-normal line surveys
#' 
#' @description
#' Compute integral of the half-normal distance function for 
#' line-transect surveys.
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' Returned integrals are
#' \deqn{\int_0^{w} e^{-x^2/2\sigma_i^2} dx = \sqrt{2\pi}\sigma_i (\Phi(w) - 0.5),}{
#' Integral(e^{-x^2/(2s^2)}) = sqrt(2*pi)*s*(Pnorm(w) - 0.5),}
#' where \eqn{w = w.hi - w.lo}, \eqn{\sigma_i}{s} is the estimated half-normal distance 
#' function parameter for the 
#' i-th observed distance, and \eqn{\Phi}{Pnorm} is the standard normal 
#' cumulative probability function. 
#' Rdistance uses R's base 
#' function `pnorm()`, which for all intents and purposes is exact.
#' 
#' @seealso [integrateNumeric()]; [integrateNegexpLines()]; 
#' [integrateOneStepLines()] 
#' 
#' @examples
#' 
#' # Fake distance function object w/ minimum inputs for integration
#' d <- rep(1,4) %m%. # Only units needed, not values
#' obs <- factor(rep(c("obs1", "obs2"), 2))
#' beta <- c(3.5, -0.5)
#' w.hi <- 125
#' w.lo <- 20
#' ml <- list(
#'     mf = model.frame(d ~ obs)
#'   , par = beta 
#'   , likelihood = "halfnorm"
#'   , w.lo = w.lo %#% "m"
#'   , w.hi = w.hi %#% "m"
#'   , expansions = 0
#' )
#' class(ml) <- "dfunc"
#' integrateHalfnormLines(ml)
#' 
#' # Check: Integral of exp(-x^2/(2*s^2)) from 0 to w.hi-w.lo
#' b <- exp(c(beta[1], beta[1] + beta[2]))
#' intgral <- (pnorm(w.hi, mean=w.lo, sd = b) - 0.5) * sqrt(2*pi)*b
#' intgral
#' 
#' 
#' @export
#' 
integrateHalfnormLines <- function(object
                            , newdata = NULL
                            , w.lo = NULL
                            , w.hi = NULL
                            , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    w.lo <- object$w.lo
    w.hi <- object$w.hi
    object <- stats::predict(object = object
                             , newdata = newdata
                             , type = "parameters"
    )
  } 
  
  # Drop units b/c pnorm hickups when y is vector 
  # (but not matrix, interesting...?)
  # It is safe to drop units b/c we converted everything 
  # to same units in parseModel.
  
  w.lo <- dropUnits(w.lo)
  w.hi <- dropUnits(w.hi)
  
  outArea <- (stats::pnorm(q = w.hi
                  , mean = w.lo
                  , sd = object) - 0.5) * sqrt(2*pi)*object
  
  outArea <- setUnits(outArea, Units)
  
  outArea 
  
}

#' @title Distance function expansion terms
#' 
#' @description
#' Compute "expansion" terms that modify the shape of 
#' a base distance function. 
#' 
#' @param a A vector or matrix of (estimated) coefficients. 
#' `a` has length \eqn{p} + `nexp` (if a vector) or dimension
#' (\eqn{k}, \eqn{p} + `nexp`), where \eqn{p} is the number 
#' of canonical parameters in the likelihood and \eqn{k} is the 
#' number of 'cases' (coefficient vectors = `nrow(a)`) to evaluate. 
#' The first \eqn{p}
#' elements of `a`, or the first \eqn{p} columns if `a`
#' is a matrix, are ignored. I.e., Expansion 
#' term coefficients are the last `nexp` elements or columns 
#' of `a`. 
#'
#' @param d A vector or 1-column matrix of 
#' distances at which to evaluate 
#' the expansion terms.  `d` should be distances 
#' above w.lo, i.e., distances - w.lo. 
#' Parameters `d` and `w`
#' must have compatible measurement units.
#' 
#' @param nexp Number of expansion terms.  Integer from 0 to 5. 
#'
#' @param w A vector specifying strip width for every 'case' 
#' in `a`.  Vector must have length `length(a)` or `nrow(a)`. 
#' In general, this is constant vector containing the range 
#' of sighting distances, i.e., `rep(w.hi - w.low, nrow(a))`. 
#' But, for some likelihoods (e.g., 'oneStep') this vector allows the user to 
#' restrict application of the expansion terms to less than the full range 
#' of distances. For the 'oneStep' likelihood, expansion terms are only 
#' applied between 0 and \eqn{\Theta}{T}, the boundary of the two uniforms, 
#' which varies by 'case' when covariates are present.  
#' Parameters `d` and `w` must 
#' have compatible measurement units.
#' 
#' @inheritParams dE.single 
#' 
#' @details
#' Expansion terms modify the base likelihood function
#' and are used to incorporate "wiggle".  The modified distance function is,
#' `key * expTerms` where `key` is a vector of values in
#' the base likelihood function (e.g., `halfnorm.like()$L.unscaled`) 
#' and `expTerms` is the 
#' matrix returned by this routine. In equation form, 
#'   \deqn{f(x_i|\beta,a_1,a_2,\dots,a_m) = 
#'    f(x_i|\beta)(1 + \sum_{k=1}^{m} a_k h_{k}(x_i/w)).}{f(x(i)|beta,a_1,a_2,...,a_k) = 
#'    f(x(i)|beta)(1 + a(1) h_1(x(i)/w) + a(2) h_2(x(i)/w) + ... + a(m) h_m(x(i)/w)). },
#' where \eqn{m} = the the number of expansions (`nexp`), \eqn{h_{j}(x)}{h_j(x)} 
#' are expansion terms for distance \eqn{x}, and  
#' \eqn{a_1, a_2, \dots, a_m}{a(1), a(2), ..., a(m)} are the (estimated) 
#' expansion term coefficients.
#' 
#' @return If `nexp` equals 0, 1 is returned. 
#' If `nexp` is greater than 0, a matrix of 
#' size \eqn{n}X\eqn{k} containing expansion terms, 
#' where \eqn{n} = `length(d)` and \eqn{k} = `nrow(a)`. The 
#' expansion series associated with row \eqn{j} of `a` 
#' are in column \eqn{j} of the return. i.e., 
#' element (\eqn{i},\eqn{j}) of the return is 
#'   \deqn{1 + \sum_{k=1}^{m} a_{jk} h_{k}(x_{i}/w).}{(1 + a(j1) h_1(x(i)/w) + ... + a(jm) h_m(x(i)/w)).}
#' 
#' 
#' @examples
#' a1 <- c(log(40), 0.5, -.5)
#' a2 <- c(log(40), 0.25, -.5)
#' dists <- seq(0, 100, length = 100) %m%.
#' w = 100 %m%.
#' 
#' expTerms1 <- expansionTerms(a1, dists, "cosine", 2, w)
#' expTerms2 <- expansionTerms(a2, dists, "cosine", 2, w)
#' plot(dists, expTerms2, ylim = c(0,2.5))
#' points(dists, expTerms1, pch = 16)
#' 
#' # Same as above
#' a <- rbind(a1, a2)
#' w <- rep(w, nrow(a))
#' expTerms <- expansionTerms(a, dists, "cosine", 2, w)
#' matlines(dists, expTerms, lwd=2, col=c("red", "blue"), lty=1)
#' 
#' # Showing key and expansions
#' key <- halfnorm.like(log(40), dists, matrix(1,length(dists),1))$L.unscaled
#' plot(dists, key, type = "l", col = "blue", ylim=c(0,1.5))
#' lines(dists, key * expTerms1, col = "red")
#' lines(dists, key * expTerms2, col = "purple")
#' 
#' @export
#'    
expansionTerms <- function(a, d, series, nexp, w){
  
  if( nexp > 0 ){
    # dimensions:
    #  n = length(d) = number of distances
    #  k = length(w) 
    #  nexp = number of expasions
    # Note:
    #  nrow(a) == length(w) for all likes 
    
    dscl <- dropUnits(outer(d, w, "/")) # n X k
    
    indOutside <- dscl > 1  # save this for later

    if (series=="cosine"){
      exp.term <- cosine.expansion( dscl, nexp ) # returns n X k X nexp array
    } else if (series=="sine"){
      exp.term <- sine.expansion( dscl, nexp ) # returns n X k X nexp array
    } else if (series=="hermite"){
      exp.term <- hermite.expansion( dscl, nexp )
    } else if (series == "simple") {
      exp.term <- simple.expansion( dscl, nexp )
    } else if (series == "bspline") {
      exp.term <- bspline.expansion( dscl, nexp )
    } else {
      stop( paste( "Unknown expansion series", series, "requested." ))
    }

    # keep in mind: nrow(expCoeffs) must equal length(w)
    if(is.matrix(a)){
      if( nrow(a) != length(w) ){
        stop("nrow(a) != length(w) in expansionTerms function.")
      }
      coefLocs <- (ncol(a)-(nexp-1)):(ncol(a))
      expCoeffs <- a[, coefLocs, drop = FALSE]  # n X nexp
    } else {
      coefLocs <- (length(a)-(nexp-1)):(length(a))
      expCoeffs <- matrix(a[coefLocs]
                        , nrow = length(w)
                        , ncol = nexp
                        , byrow = TRUE ) # length(w) X nexp
    }
    
    jMat <- kronecker(diag(nexp), matrix(1, 1, nrow(exp.term))) # nexp X (nexp*d)
    bigCoeffs <- expCoeffs %*% jMat # (n X nexp) * (nexp X (nexp*d)) = n X (nexp*d)
    bigCoeffs <- array(bigCoeffs
                     , dim = c(nrow(expCoeffs), nrow(exp.term), nexp))
    bigCoeffs <- aperm(bigCoeffs, c(2,1,3)) # was nXdXnexp; now dXnXnexp; constant w/i pages
    expTerms <- 1 + apply(exp.term * bigCoeffs, c(1,2), sum) # apply(dXnXnExp) = dXn; sums across nexp

    # expTerms <- t(t(expTerms) / expTerms[1,]) # Make sure value for g(0) is 1.0
      
    expTerms[ indOutside ] <- 1 # blank out values > w

  } else {  
    expTerms <- 1
  }
  
  expTerms
  
}

#' @name Rdistance-package
#' 
#' @title Rdistance - Distance Sampling Analyses for Abundance Estimation
#' 
#' @description 
#' `Rdistance` contains functions and associated routines to analyze
#' distance-sampling data collected on point or line transects. 
#' Some of `Rdistance`'s features include:
#' 
#' *  Accommodation of both point and line transect analyses in one routine ([dfuncEstim()]).
#' *  Regression-like formula for inclusion of distance function covariates ([dfuncEstim()]).
#' *  Automatic bootstrap confidence intervals ([abundEstim()]).
#' *  Parallel processing of bootstrap iterations (`parallel` argument of [abundEstim()]).
#' *  Availability of both study-area and site-level abundance estimates (`help("predict.dfunc")`).
#' *  Rigorous physical measurement requirements and automated conversion when necessary (\code{\link{%#%}}, [setUnits()]).
#' *  Classic parametric distance functions ([halfnorm.like()], [hazrate.like()], [negexp.like()]), and
#'    expansion functions ([cosine.expansion()], [hermite.expansion()], [simple.expansion()]).
#' *  Mixture distance functions for non-standard shapes and thresholds ([oneStep.like()], [triangle.like()], and [huber.like()]).
#' *  Automated distance function fitting and selection [autoDistSamp()].
#' *  `print`, `plot`, `predict`, `coef`, and `summary` methods for distance function objects and 
#'    abundance classes.
#' 
#' # Background:
#' Distance-sampling is a popular method for abundance estimation in
#' ecology. Line transect surveys are conducted by traversing
#' randomly placed transects in a study area with the objective of
#' sighting animals and estimating density or abundance.  Data collected
#' during line transect surveys consists of *target* sightings, 
#' either of individuals or groups, and off-transect distances to the 
#' original location of the *target*. 
#' When *targets* are sighted in groups, data include the number 
#' of individuals in the group.
#' 
#' Point transect surveys are similar except that observers stop one 
#' or more times along the transect to observe *targets*.  Point transects are
#' popular avian survey methods where detections are often auditory 
#' cues. Point transects are also for studies using automated auditory detectors
#' or trail cameras.  Point transect data consists of radial distances from 
#' the observer to the *target*.   
#' 
#' The defining feature of distance sampling is the tendency for 
#' probability of detection to decline as
#' off-transect or radial distances increase. Targets far from 
#' the observer are generally  
#' harder to detect than those closer.  In most
#' line transect studies, *targets* on the transect (off-transect
#' distance = 0) are assumed to be sighted with 100% probability.  This
#' assumption allows researchers to estimate the proportion of missed 
#' targets and in turn adjust the number of 
#' sighted *targets* for missed detections. 
#' Some studies utilize two observers searching the same areas and are able to
#' estimate the proportion of individuals missed on the transect line and 
#' thereby eliminate the
#' assumption that all individuals on the line have been observed.
#' 
#' # Purpose: 
#' The author's aims are 
#' to provide an easy-to-use, rigorous, and flexible analysis option in R for 
#' distance-sampling data.  The authors believe that beginning
#' users need easy-to-use and easy-to-understand software,
#' while advanced users require greater flexibility and customization, and 
#' their aim is to meet the demands of both groups. 
#' 
#' # Data sets:
#' `Rdistance` contains the following example data sets: 
#' 
#' *  Line-transect sampling of Brewers sparrows in central Wyoming 
#' ([sparrowDf()]).
#' *  Point-transect sampling of Sage Thrashers in central Wyoming 
#' ([thrasherDf()]).
#' 
#' 
#' # References 
#' 
#' Buckland, S.T., Anderson, D.R., Burnham, K.P. and Laake, J.L.
#'   1993. *Distance Sampling: Estimating Abundance of Biological
#'   Populations*. Chapman and Hall, London.
#' 
#' @aliases Rdistance-package distance Rdistance point-transect line-transect
#' 
#' 
#' @author Main author and maintainer: Trent McDonald
#' <trent@mcdonalddatasciences.com>
#' 
#' Coauthors: Ryan Nielson, Jason Carlisle, and Aidan McDonald
#' 
#' Contributors: Ben Augustine, James Griswald, Joel Reynolds, Pham Quang, Earl
#' Becker, Aaron Christ, Brook Russelland, Patrick McKann, Lacey Jeroue, 
#' Abigail Hoffman, Michael Kleinsasser, and Ried Olson
#' 
#' @keywords package
#' 
#' @import units
#' 
"_PACKAGE"

#' @title Rdistance model matrix
#'
#' @description 
#' Extract the model matrix ("X" matrix) from an Rdistance model object.
#'
#' @usage \method{model.matrix}{dfunc}(object, \dots)
#' 
#' @inheritParams predict.dfunc 
#' 
#' @param ... Ignored
#' 
#' @return A matrix containing covariates for fitting an
#' Rdistance model.
#' 
#' @examples
#' 
#' data(sparrowDf)
#' sparrowModel <- parseModel( sparrowDf, dist ~ observer )
#' model.matrix(sparrowModel)
#' 
#' @export
#' 
model.matrix.dfunc <- function(object, ...){
  # ml$mf is a regular linear model frame, with terms, so this is easy
  stats::model.matrix(stats::terms(object$mf), object$mf)
}

#' @title Hazard rate likelihood 
#' 
#' @description Computes the hazard rate distance function. 
#'
#' @inheritParams halfnorm.like 
#' 
#' @details 
#' The hazard rate likelihood is 
#' \deqn{f(x|\sigma,k) = 1 - \exp(-(x/\sigma)^{-k})}{%
#' f(x|Sigma,k) = 1 - exp(-(x/Sigma)^(-k))} 
#' where \eqn{\sigma}{Sigma} determines location 
#' (i.e., distance at which the function equals 1 - exp(-1) = 0.632), 
#' and \eqn{k}{k} determines slope of the function 
#' at \eqn{\sigma}{Sigma} (i.e., larger k equals steeper 
#' slope at \eqn{\sigma}{Sigma}). For distance analysis, 
#' the valid range for both \eqn{\sigma}{Sigma} and k is
#' \eqn{\geq 0}{>=0}.  
#'   
#'  
#' @inherit halfnorm.like return seealso
#'          
#' @examples
#' d <- seq(0, 100, length=100)
#' covs <- matrix(1,length(d),1)
#' hazrate.like(c(log(20), 5), d, covs)
#' 
#' # Changing location parameter
#' plot(d, hazrate.like(c(log(20), 5), d, covs)$L.unscaled, type="l", col="red")
#' lines(d, hazrate.like(c(log(40), 5), d, covs)$L.unscaled, col="blue")
#' abline(h = 1 - exp(-1), lty = 2)
#' abline(v = c(20,40), lty = 2)
#' 
#' # Changing slope parameter
#' plot(d, hazrate.like(c(log(50), 20), d, covs)$L.unscaled, type="l", col="red")
#' lines(d, hazrate.like(c(log(50), 2), d, covs)$L.unscaled, col="blue")
#' abline(h = 1 - exp(-1), lty = 2)
#' abline(v = 50, lty = 2)
#' 
#'          
#' @export

hazrate.like <- function(a 
                       , dist 
                       , covars
                       , w.hi = NULL
                       ){

  # w.hi is ignored, but needed for compatability in other likelihoods
  
  if(length(dim(dist)) >= 2 && dim(dist)[2] != 1 ){ 
    stop(paste("Argument 'dist' must be a vector or single-column matrix.",
               "Found array with", length(dim(dist)), "dimensions."))
  }
  
  # What's in a? : 
  #   a = [(Intercept), b1, ..., bp, k, <expansion coef>]
  #   or it is a matrix, in which case 
  #   it is n X p containing cannonical parameters on the log scale
  #
  # Dimensions:
  #   m = number of "cases" = nrow(a)
  #   q = number of coefficients in variable part of model = ncol(covars)
  #   n = number of data distances = length(dist) = nrow(covars)
  q <- nCovars(covars) # in Rdistance, not exported
  dist <- dropUnits(dist) # drop units

  if(is.matrix(a)){
    beta <- a[, 1:q, drop = FALSE]  # m X q
    K <- a[1, q+1, drop = TRUE]     # 1 X 1
  } else {
    beta <- matrix(a[1:q], nrow = 1)  # 1 X q
    K <- a[q+1]     # 1 X 1
  }
  s <- covars %*% t(beta) # (nXq) %*% (qXm) = nXm
  sigma <- exp(s)  # link function here

  # take logs here to speed calculations. When logged, we don't have to 
  # expand K or dist to matrix dimensions, which is slow.  They must be vectors
  # of length equal to n = nrow(sigma). This works provided K is constant over 
  # observations (does not depend on covariates).
  # hazrate = 1 - exp(-(dist/sigma)^(-K))
  key <- log( dist ) - log( sigma )  # ((n vector) / nXm) ^ (nXm)
  key <- -K * key
  key <- 1 - exp(-exp(key))

  return( list(L.unscaled = key, 
               params = cbind(s, K)) # return params on log scale
          )  

}

#' @title Integrate Half-normal Point transects
#' 
#' @description
#' Compute integral of the half-normal distance function for 
#' point surveys.
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' Returned integrals are 
#' \deqn{\int_0^{w} xe^{-x^2/2\sigma_i^2} dx = 0.5\sigma_i^2(1 - e^{-w^2/2\sigma_i^2}),}{
#' Integral(xe^{-x^2/(2s^2)}) = 0.5*s^2*(1 - exp(-w^2/(2*s^2))),} 
#' where \eqn{w = w.hi - w.lo} and \eqn{\sigma_i}{s} is the estimated half-normal 
#' distance function parameter for the i-th observed distance.  
#' 
#' @seealso [integrateNumeric()]; [integrateNegexpPoints()]; 
#' [integrateOneStepPoints()] 
#' 
#' @examples
#' 
#' # Fake distance function object w/ minimum inputs for integration
#' d <- rep(1,4) %m%. # Only units needed, not values
#' obs <- factor(rep(c("obs1", "obs2"), 2))
#' beta <- c(3.5, -0.5)
#' w.hi <- 125
#' w.lo <- 20
#' ml <- list(
#'     mf = model.frame(d ~ obs)
#'   , par = beta 
#'   , likelihood = "halfnorm"
#'   , w.lo = w.lo %#% "m"
#'   , w.hi = w.hi %#% "m"
#'   , expansions = 0
#' )
#' class(ml) <- "dfunc"
#' integrateHalfnormPoints(ml)
#' 
#' # Check: Integral of x exp(-x^2/(2*s^2)) from 0 to w = w.hi-w.lo
#' sigma <- exp(c(beta[1], beta[1] + beta[2]))
#' w <- w.hi - w.lo
#' intgral <- sigma^2 * (1 - exp(-w^2 / (2*sigma^2)))
#' intgral
#' 
#' # Effective detection radius
#' sqrt(2 * intgral)
#' 
#' @export
#' 
integrateHalfnormPoints <- function(object
                            , newdata = NULL
                            , w.lo = NULL
                            , w.hi = NULL
                            , Units = NULL
                              ){

  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    w.lo <- object$w.lo
    w.hi <- object$w.hi
    object <- stats::predict(object = object
                             , newdata = newdata
                             , type = "parameters"
    )
  } 
  
  w <- w.hi - w.lo
  object <- setUnits(object, Units)
  
  # exp() can't handle units, even if they are [1], gotta remove em
  s.squared <- object^2 # want units on this one
  wsRatio <- dropUnits(-w^2 / (2*s.squared)) # remove [1] units
  
  outArea <- s.squared * (1 - exp( wsRatio ))
  
  outArea 
  
}

#' @title oneStep likelihood start and limit values
#' 
#' @description Compute starting values and limits 
#' for the oneStep distance function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @seealso \code{\link{oneStep.like}}
#' 
#' @examples
#' # make 'model list' object
#' # Boundary is 10, p is 100 / 120 = 0.833
#' library(Rdistance)
#' whi <- 50
#' x <- c( runif(100, min=0, max=10), runif(20, min=10, max=whi))
#' x <- setUnits(x, "m")
#' detectDf <- data.frame(transect = 1, dist = x)
#' siteDf <- data.frame(transect = 1, length = setUnits(10,"m"))
#' distDf <- RdistDf(siteDf, detectDf)
#' ml <- parseModel(distDf
#'             , formula = dist ~ 1
#'             , w.lo = 0
#'             , w.hi = setUnits(whi, "m")
#'             )
#'             
#'
#' sl <- oneStep.start.limits(ml)
#' hist(x, n = 20)
#' abline(v = exp(sl$start["(Intercept)"]))
#' 
#' 
#' @export
oneStep.start.limits <- function (ml){
  
  X <- stats::model.matrix(ml)
  dist <- Rdistance::distances(ml)  
  
  expan <- ml$expansions
  ncovars <- nCovars(X)
  w.lo <- ml$w.lo
  w.hi <- ml$w.hi

  fuzz <- getOption("Rdistance_fuzz")
  zero <- getOption("Rdistance_zero")
  posInf <- getOption("Rdistance_posInf")
  negInf <- getOption("Rdistance_negInf")
  
  # Save these for later
  min.d <- min(dist, na.rm = T)
  max.d <- max(dist, na.rm = T)
  
  # w.lo and w.hi should always have values, so the following 
  # never fires, but just in case.
  if(is.null(w.lo) | is.na(w.lo)){
    w.lo <- min.d
  }
  if(is.null(w.hi) | is.na(w.hi)){
    w.hi <- max.d
  }

  # Only time dist2 will not have units is when user overides requirement
  # Regardless, convert w.lo and w.hi, then drop units
  w.lo <- setUnits(w.lo, units(dist))
  w.hi <- setUnits(w.hi, units(dist))
  x    <- dropUnits(dist)
  w.lo <- dropUnits(w.lo)
  w.hi <- dropUnits(w.hi)
  
  # scale so 0 <= xx <= 1
  x <- x - w.lo
  x <- x[x >= 0]
  
  x <- x / (w.hi - w.lo) 
  x <- x[x <= 1]
  
  x <- x[!is.na(x)]

  # Remember: 0 <= xx < 1
  # But: values == 0 or 1 are not useful. Remove them.
  x <- x[ fuzz <= x & x <= (1 - fuzz) ]
  n <- length(x)
  
  if( n > 1 ){
    x <- sort( x ) # order statistics

    # MLE estimator ----
    r <- 1:(n-1)
    R1 <- r * (log(r) - log(x[r])) + 
      (n - r) * (log(n - r) - log(1 - x[r])) 
    rPlus1 <- 2:n
    R2 <- r * (log(r) - log(x[rPlus1])) + 
      (n - r) * (log(n - r) - log(1 - x[rPlus1])) 
    R <- pmax(R1, R2)
    p.ind <- which.max(R)
    theta.MLE <- x[p.ind]
    p.MLE <- sum(x <= theta.MLE) / n
    
    theta.MLE <- theta.MLE * (w.hi - w.lo) + w.lo
    
  } else {
    # case with no data
    theta.MLE <- mean(w.lo, w.hi)
    p.MLE <- 0.5
  }
 
  start <- c(log(theta.MLE)            # threshold 
             , rep(zero, ncovars-1)    # any covars
             , p.MLE                   # p = proportion below theta
             , rep(zero, expan))       # any expansions
  
  if( ncovars <= 1 ){
    # (Intercept)-only model. Use tighter bounds.
    low <- log(max(w.lo, min.d) + fuzz)
    high <- log(min(w.hi, max.d) - fuzz)
  } else {
    # We have covariates
    low <- rep(negInf, ncovars)
    high <- rep(posInf, ncovars)
  }
  # Add bounds for expansions
  low <- c(low
         , zero
         , rep(negInf, expan)
           )
  high  <- c(high
           , 1 - fuzz
           , rep(posInf, expan)
           )
  nms <- c(colnames(X), "p")
  
  if(expan > 0){
    nms <- c(nms, paste0( "a", 1:expan))
  }
  
  names(start) <- nms
  names(low) <- nms
  names(high) <- nms
  
  list( start=start, low=low, high=high, names=nms )
  
}

#' @title Predict distance functions
#' 
#' @description Predict either likelihood parameters, 
#' distance functions, site-specific density, or 
#' site-specific abundance from estimated distance function 
#' objects.
#' 
#' @param object An Rdistance model frame or fitted distance function,
#' normally produced by a call to [dfuncEstim()]. 
#' 
#' @param newdata A data frame containing new values of 
#' the covariates at which to evaluate the distance functions. 
#' If `newdata`
#' is NULL, distance functions are evaluated at values of 
#' the observed covariates and results in one prediction 
#' per distance or transect (see parameter `type`). 
#' If `newdata` is not NULL and the model does not contains covariates, 
#' this routine returns one prediction for each row in `newdata`, but 
#' columns and values in `newdata` are ignored. 
#' 
#' @param type The type of predictions desired. 
#' \itemize{
#'   \item **If `type` == "parameters"**: Returned value is 
#'     a matrix of predicted (canonical) parameters of the 
#'     likelihood function. If `newdata` is NULL, return contains 
#'     one parameter value for every detection distance
#'     in `object$mf` (distances in `object$mf` are between 
#'     `object$w.lo` and `object$w.hi` and non-missing). 
#'     If `newdata` is not NULL, returned vector 
#'     has one parameter for every row in `newdata`. Parameter 
#'     `distances` is ignored when `type` == "parameters". 
#'     Canonical parameters (non-expansion terms)
#'     are returned on the response (inverse-link) scale.  Raw 
#'     canonical parameters in `object$par` are stored in 
#'     the link scale.  Expansion term parameters use the identity 
#'     link, so their value in the output equals their value 
#'     in `object$par`. 
#'     
#'   \item **If `type` == "likelihood"**: Returned value is a 
#'     matrix of **unscaled** likelihood values for all  
#'     observed distances in `object$mf`, i.e., raw distance functions
#'     evaluated at the observed distances.  Parameters `newdata` and 
#'     `distances` are ignored when `type` is "likelihood". 
#'     The negative log likelihood of the full data set is
#'     `-sum(log(predict(object,type="likelihood") / 
#'     effectiveDistance(object)))`. 
#'     
#'   \item **If `type` == "dfuncs" or "dfunc"**: Returned  
#'     value is a matrix whose columns contain scaled distance functions. 
#'     The distance functions in each column are evaluated at distances   
#'     in argument `distances`, not at the observed 
#'     distances in `object$mf`. The number of distance functions
#'     returned (i.e., number of columns) depends on `newdata` 
#'     as follows: 
#'     \itemize{
#'        \item If `newdata` is NULL, one distance function 
#'        will be returned for every detection in `object$mf`
#'        that has valid covariate values.    
#'       \item If `newdata` is not NULL, one distance function 
#'       will be returned for each observation (row) in `newdata`. 
#'     }
#'     
#'    \item **If `type` == "density" or "abundance"**: Returned 
#'    object is a tibble containing predicted density and abundance 
#'    on the area surveyed by each transect. 
#'  }  
#'  
#'  If `object` is a smoothed distance function, it does not have parameters
#'  and this routine will only return scaled distance functions, densities, or 
#'  abundances. That is, 
#'  `type` = "parameters" when `object` is smoothed 
#'  does not make sense and the smoothed distance function estimate 
#'  will be returned if `type` does not equal "density" or "abundance". 
#' 
#' @param distances A vector or 1-column matrix of 
#' distances at which to evaluate 
#' distance functions, when distance functions 
#' are requested.  `distances` must have measurement units. 
#' Any distances outside the observation 
#' strip (`object$w.lo` to `object$w.hi`) are discarded.  If 
#' `distances` is NULL, a sequence 
#' of `getOption("Rdistance_intEvalPts")` (default 101) evenly 
#' spaced distances between 
#' `object$w.lo` and `object$w.hi` (inclusive) is used. 
#' 
#' @inheritParams abundEstim
#'
#' @param \dots Included for compatibility with generic `predict` methods.
#' 
#' @return A matrix containing predictions: 
#' \itemize{
#'   \item **If `type` is "parameters"**, the returned matrix 
#'   contains likelihood parameters. The extent of the 
#'   first dimension (rows) in the returned matrix is equal to 
#'   either the number of detection distances 
#'   in the observed strip or number of rows in `newdata`. 
#'   The returned matrix's second dimension (columns) is 
#'   the number of parameters in the likelihood 
#'   plus the number of expansion terms.  
#'   See the help for each likelihoods to interpret  
#'   returned parameter values. All parameters are returned 
#'   on the inverse-link scale; i.e., *exponential* for canonical 
#'   parameters and *identity* for expansion terms. 
#'   
#'   \item **If `type` is "dfuncs" or "dfunc"**, columns of the 
#'   returned matrix contains detection functions (i.e., *g(x)*).  
#'   The extent of the first 
#'   dimension (number of rows) is either the number of distances 
#'   specified in `distances`
#'   or `options()$Rdistance_intEvalPts` if `distances` is 
#'   not specified.
#'   The extent of the second dimension (number of columns) is: 
#'     \itemize{
#'       \item the number of detections with non-missing distances: 
#'       if `newdata` is NULL.
#'       \item the number of rows in `newdata` if 
#'        `newdata` is specified.
#'     }
#'   All distance functions in columns of the return are scaled 
#'   to `object$g.x.scale` at `object$x.scl`. The returned matrix has 
#'   the following additional attributes:
#'    \itemize{
#'       \item `attr(return, "distances")` is the vector of distances used to 
#'       predict the function in `return`.  Either the input `distances` object
#'       or the computed sequence of distances when `distances` is NULL. 
#'       \item `attr(return, "x0")` is the vector of distances at which each 
#'        distance function in `return` was scaled. i.e., the vector of 
#'        `x.scl`.
#'       \item `attr(return, "g.x.scl")` is the height of *g(x)* (the distance 
#'        function) at *x0*. 
#'   }
#'   
#'   \item **If `type` is "density" or "abundance"**, the return is a 
#'   tibble containing density and abundance estimates by transect.  
#'   All transects in the input data (i.e., `object$data`) are 
#'   included, even those with missing lengths. 
#'    Columns in the tibble are:
#'    \itemize{
#'      \item transect ID: the grouping factor of the original RdistDf object.
#'      \item individualsSeen: sum of non-missing group sizes on that transect.
#'      \item avgPdetect: average probability of detection over groups sighted on that transect.
#'      \item effort: size of the area surveyed by that transect.
#'      \item density: density of individuals in the area surveyed by the transect.
#'      \item abundance: abundance of individuals in the area surveyed by the transect.
#'    } 
#' }
#' 
#' @seealso [halfnorm.like()], [negexp.like()], 
#' [hazrate.like()]
#' 
#' @examples
#' 
#' data("sparrowDf")
#'
#' # For dimension checks:
#' nd <- getOption("Rdistance_intEvalPts")
#' 
#' # No covariates
#' dfuncObs <- sparrowDf |> dfuncEstim(formula = dist ~ 1
#'                      , w.hi = units::as_units(100, "m"))
#'                      
#' n  <- nrow(dfuncObs$mf)
#' p <- predict(dfuncObs) # parameters
#' all(dim(p) == c(n, 1)) 
#' 
#' # values in newdata ignored because no covariates
#' p <- predict(dfuncObs, newdata = data.frame(x = 1:5))
#' all(dim(p) == c(5, 1)) 
#' 
#' # Distance functions in columns, one per observation
#' p <- predict(dfuncObs, type = "dfunc") 
#' all(dim(p) == c(nd, n))
#' 
#' d <- setUnits(c(0, 20, 40), "ft")
#' p <- predict(dfuncObs, distances = d, type = "dfunc") 
#' all(dim(p) == c(3, n))
#' 
#' p <- predict(dfuncObs
#'    , newdata = data.frame(x = 1:5)
#'    , distances = d
#'    , type = "dfunc") 
#' all(dim(p) == c(3, 5))
#' 
#' # Covariates
#' data(sparrowDfuncObserver) # pre-estimated object
#' \dontrun{
#' # Command to generate 'sparrowDfuncObserver'
#' sparrowDfuncObserver <- sparrowDf |> 
#'             dfuncEstim(formula = dist ~ observer
#'                      , likelihood = "hazrate")
#' }
#' 
#' predict(sparrowDfuncObserver)  # n X 2
#' 
#' Observers <- data.frame(observer = levels(sparrowDf$observer))
#' predict(sparrowDfuncObserver, newdata = Observers) # 5 X 2
#' 
#' predict(sparrowDfuncObserver, type = "dfunc") # nd X n
#' predict(sparrowDfuncObserver, newdata = Observers, type = "dfunc") # nd X 5
#' d <- setUnits(c(0, 150, 400), "ft")
#' predict(sparrowDfuncObserver
#'   , newdata = Observers
#'   , distances = d
#'   , type = "dfunc") # 3 X 5
#' 
#' # Density and abundance by transect
#' predict(sparrowDfuncObserver
#'   , type = "density")
#'   
#' @export
#' 
#' 
predict.dfunc <- function(object
                        , newdata = NULL
                        , type = c("parameters")
                        , distances = NULL
                        , propUnitSurveyed = 1.0
                        , area = NULL
                        , ...) {

  if (!inherits(object, "dfunc")){ 
    stop("Argument 'object' is not a 'dfunc' object")
  }
  
  isSmooth <- Rdistance::is.smoothed(object)
  
  # Establish the X matrix ----
  # We ALWAYS have covariates
  # We always need parameters, except for smoothed dfuncs
  if( !isSmooth ){
    if ( is.null(newdata) | (type == "likelihood") ) {
      # Case: Use original covars
      X <- stats::model.matrix(object)
    } else {
      # We have NEWDATA to deal with
      Terms <- stats::terms( object$mf )
      Terms <- stats::delete.response( Terms ) # model.frame (below) fails if there's a response, go figure.
      gsName <- all.vars(Terms)[ attr(Terms, "offset") ] # there is always an offset
      if( !(gsName %in% names(newdata)) ){
        # gotta add a fake groupsize to newdata so model.frame (below) works
        # but, model.matrix drops offset, so this is silly
        newdata <- cbind(newdata, data.frame( 1 ))
        names(newdata)[length(names(newdata))] <- gsName
      }
      xLevs <- lapply( object$mf, levels ) # get unspecified levels of factors
      m <- stats::model.frame( formula = Terms
                      , data = newdata
                      , xlev = xLevs )
      X <- stats::model.matrix( object = Terms
                       , data = m
                       , contrasts.arg = attr(object$mf,"contrasts") )
    }
    
    BETA <- stats::coef(object)
    p <- length(BETA)
    q <- ncol(X)
    
    paramsLink <- X %*% BETA[1:q] # could be extra parameters tacked on. e.g., shape or expansion terms
  
    if(q < p){
      extraParams <- matrix(BETA[(q+1):p]
                          , nrow = nrow(paramsLink)
                          , ncol = p-q
                          , byrow = TRUE)
      # types 'dfunc' and 'likelihood' need the extra params attached
      # other types do not; but, attach extras anyway
      paramsLink <- cbind(paramsLink, extraParams)
    } else {
      extraParams <- NULL
    }
    
  } 

  # Dispatch functions to deal with 'type' ----
  if(type == "parameters" & isSmooth ){
    type = "dfuncs"  # smoothed dfuncs have no parameters
  }
  
  return(
    switch(type
         , dfunc = 
         , dfuncs = predDfuncs(object = object
                             , params = paramsLink
                             , distances = distances
                             , isSmooth = isSmooth) 
         , likelihood = predLikelihood(object = object
                                     , params = paramsLink
                                      ) 
         , abundance = 
         , density = predDensity(object = object
                               , propUnitSurveyed = propUnitSurveyed
                                )  
         , {  # the default = parameters
             parms <- cbind(exp(paramsLink[,1]), # All link functions are exp...thus far
                            extraParams )
             
             parmsNames <- likeParamNames(object$likelihood) # cannonical names
             if( object$expansions >= 1 ){
               parmsNames <- c(parmsNames, names(BETA)[p - (object$expansions:1) + 1])
             }
             dimnames(parms) <- list(rownames(newdata), parmsNames)
             parms
           }
    )
  )
}

#' @title Simple polynomial expansion factors
#' 
#' @description Computes simple polynomial expansion terms 
#' for use in distance analysis. The Simple (and other expansions) 
#' allow "wiggle" in estimated distance functions.
#'   
#' @inheritParams cosine.expansion
#' 
#' @details The polynomials computed here are:
#'   \itemize{
#'     \item \bold{First term}: \deqn{h_1(x)=x^4,}{h1(x) = x^4,}
#'     \item \bold{Second term}: \deqn{h_2(x)=x^6,}{h2(x) = x^6,}
#'     \item \bold{Third term}: \deqn{h_3(x)=x^8,}{h3(x) = x^8,}
#'     \item \bold{Fourth term}: \deqn{h_4(x)=x^{10},}{h4(x) = x^10,}
#'   }
#'   The maximum number of expansion terms computed is 4.
#'
#' @inherit cosine.expansion return
#'   
#' @seealso \code{\link{dfuncEstim}}
#' , \code{\link{cosine.expansion}}
#' , \code{\link{sine.expansion}}
#' , \code{\link{hermite.expansion}}.
#'   
#' @examples 
#' x <- matrix(seq(0, 1, length = 200), ncol = 1)
#' simp.expn <- simple.expansion(x, 4)
#' plot(range(x), range(simp.expn), type="n")
#' matlines(x, simp.expn[,1,1:4], col=rainbow(4), lty = 1)
#' 
#' @export

simple.expansion <- function(x, expansions){

    expansions <- as.integer(expansions)  
  
    if (expansions > 4){
        warning("Too many Simple polynomial expansion terms. Only 4 used.")
        expansions = 4
    }

    if( expansions < 1 ) stop( "Number of expansions must be >= 1" )

    expansion = array(dim = c(nrow(x), ncol(x), expansions))
  
    expansion[,,1] = x^4

    if(expansions >= 2){
        expansion[,,2] = x^6
    }
    if(expansions >= 3){
        expansion[,,3] = (x)^8
    }
    if(expansions >= 4){
        expansion[,,4] =  (x)^10
    }

    return(expansion)

}

#' @title triangle likelihood start and limit values
#' 
#' @description Compute starting values and limits 
#' for the triangle distance function. 
#' 
#' @inheritParams startLimits
#' 
#' @inherit startLimits return
#' 
#' @seealso \code{\link{triangle.like}}
#' 
#' @examples
#' # make 'model list' object
#' # Boundary is 10, p is 100 / 120 = 0.833
#' library(Rdistance)
#' whi <- 50
#' x <- c( runif(100, min=0, max=10), runif(20, min=10, max=whi))
#' x <- setUnits(x, "m")
#' detectDf <- data.frame(transect = 1, dist = x)
#' siteDf <- data.frame(transect = 1, length = setUnits(10,"m"))
#' distDf <- RdistDf(siteDf, detectDf)
#' ml <- parseModel(distDf
#'             , formula = dist ~ 1
#'             , w.lo = 0
#'             , w.hi = setUnits(whi, "m")
#'             )
#'             
#'
#' sl <- oneStep.start.limits(ml)
#' hist(x, n = 20)
#' abline(v = exp(sl$start["(Intercept)"]))
#' 
#' 
#' @export
triangle.start.limits <- function (ml){
  
  strtVals <- oneStep.start.limits(ml)
  
  # Change theta high to allow it to exceed w
  ncovars <- nCovars(stats::model.matrix(ml))
  posInf <- getOption("Rdistance_posInf")
  strtVals$high[1:ncovars] <- posInf
  
  # P returned is proportion < theta, change to triangle parameterzation
  Theta <- strtVals$start[1]
  pStar <- strtVals$start[ncovars + 1]
  wStar <- dropUnits(ml$w.hi - setUnits(Theta, ml$outputUnits))
  p <- Theta * (1 - pStar) / (pStar * (Theta + 2 * wStar) - Theta)
  p <- min( p, strtVals$high[ncovars + 1] )
  p <- max( p, strtVals$low[ncovars + 1] )
  strtVals$start[ncovars + 1] <- p
  

  strtVals
}

#' @title Number of covariates
#' 
#' @description
#' Return number of covariates in a distance model
#' 
#' @param X The X matrix of covariates, or a vector. 
#' 
#' @details
#' The reason this routine is needed is that sometimes 
#' we pass one row of covariates to a likelihood function. 
#' If so, it may come in as a normal vector, not a matrix. 
#' If a normal vector, ncol(X) does not work. 
#' 
#' @return An integer scalar
#' 
#'
 
# do not export
nCovars <- function(X){
  
  if( is.matrix(X) ){
    q <- ncol(X)
  } else {
    q <- length(X)
  }
  
  q
}

#' @title B-spline expansion terms
#' 
#' @description 
#' Computes spline, specifically b-spline, expansion terms that modify the 
#' shape of distance likelihood functions. 
#'   
#' @inheritParams cosine.expansion
#' 
#' @inherit cosine.expansion return
#'   
#' @seealso [dfuncEstim()], 
#'          [cosine.expansion()],
#'          [hermite.expansion()], 
#'          [simple.expansion()].
#'   
#' @examples 
#' x <- matrix(seq(0, 1, length = 200), ncol = 1)
#' spl.expn <- bspline.expansion(x, 5)
#' plot(range(x), range(spl.expn), type="n")
#' matlines(x, spl.expn[,1,1:5], col=rainbow(5), lty = 1)
#' 
#' @export
bspline.expansion <- function(x, expansions){

  if( expansions < 1 ) stop( "Number of expansions must be >= 1" )
  
  expansions <- as.integer(expansions)  
  
  if( expansions == 1){
    bs = x
  } else if(expansions == 2){
    bs = splines::bs(x, df = 2, degree = 2)
  } else if(expansions >= 3){
    bs = splines::bs(x, df = expansions, degree = 3)
  }

  bs <- array(bs, dim = c(nrow(x), ncol(x), expansions))
  return(bs)
            
}

#' @title Simpson numerical integration coefficients
#' 
#' @description 
#' Return a vector of Simpson's Composite numerical integration 
#' coefficients. 
#' 
#' @param n Number of coefficients, which is the number of points 
#' at which the function of interest is evaluated.  The number of 
#' intervals is \code{(n-1)/2}.  This number must be odd.
#' 
#' @return A vector of Simpson Composite rule coefficients 
#' suitable for numeric integration. The return is a vector of 
#' integers alternating between 4 and 2, with 1's on each end.
#' 
#' @details
#' Let \code{x} be an vector of equally spaced points in the domain 
#' of a function f (equally spaced is critical). 
#' Let \code{y = f(x)}. The numeric integral of f from \code{min(x)} 
#' to \code{max(x)} is 
#' \code{sum(simpsonCoefs(length(y)) * y) * (x[2] - x[1]) / 3}. 
#' 
#' @examples
#' 
#' x <- seq(0, 9, length=13)
#' y <- x^2
#' 
#' scoefs <- simpsonCoefs(length(x))
#' 
#' # exact integral is 9^3/3 = 243
#' sum( scoefs*y ) * (x[2] - x[1]) / 3
#' 
#' @export
simpsonCoefs <- function( n ){
  
  if( (n %% 2) == 0 ){
    stop(paste("Number of Simpson coefficients must be odd. Found", n))
  }
  intCoefs <- c(rep( c(2,4), ((n-1)/2) ), 1) 
  intCoefs[1] <- 1
  
  intCoefs
}

#' @title Calculations for one bootstrap iteration
#' 
#' @description
#' Performs density and abundance estimation for one bootstrap iteration. 
#' 
#' 
#' @param pb A progress bar created with `progress::progress_bar$new()`.
#' 
#' @param plot.bs Logical. Whether to plot bootstrap estimate of detection function.
#' A plot must already exist because this uses `lines`.
#' 
#' @param plotCovValues Data frame containing values of covariates to plot.
#' Ignored if `plot.bs` is FALSE.
#' 
#' @inheritParams dE.single
#' 
#' @inheritParams abundEstim
#' 
#' @return A data frame containing density and abundance 
#' and other relevant statistics for 
#' one iteration of the bootstrap. 
#' 
#' @seealso [bootstrap()]; [abundEstim()]
#' 
#' @export
oneBsIter <- function(  object
                      , area 
                      , propUnitSurveyed
                      , pb
                      , plot.bs
                      , plotCovValues
                      , warn = FALSE
                      , asymptoticSE = FALSE
){

  # Stratified bootstrap: same rows from each strata, strata = is.na(effort)
  effCol <- attr(object$data, "effortColumn")
  missingTransect <- is.na(object$data[[effCol]]) 
  n <- length(missingTransect)
  noTranInd    <- sample( (1:n)[missingTransect], replace = T )
  withTranInd  <- sample( (1:n)[!missingTransect], replace = T )
  bsdf <- rbind( object$data[noTranInd, ,drop = FALSE]
               , object$data[withTranInd, ,drop = FALSE] )
  
  # I wish rbind or dplyr::bind_rows preserved attributes
  # is.RdistDf(bsdf) must be TRUE; make it so
  attr(bsdf, "detectionColumn") <- attr(object$data, "detectionColumn")
  attr(bsdf, "effortColumn") <- attr(object$data, "effortColumn")
  attr(bsdf, "obsType") <- attr(object$data, "obsType")
  attr(bsdf, "transType" ) <- attr(object$data, "transType")

  # Fit dfunc to bs data
  dfunc.bs <- dfuncEstim(data = bsdf,
                         formula = object$formula,  
                         likelihood = object$likelihood, 
                         w.lo = object$w.lo,
                         w.hi = object$w.hi,
                         expansions = object$expansions, 
                         series = object$series,
                         x.scl = object$x.scl, 
                         g.x.scl = object$g.x.scl,
                         outputUnits = object$outputUnits,
                         warn = warn, 
                         asymptoticSE = asymptoticSE)

  # Note: Convergence is checked in estimateN. If nonConvergent, nEst$density returns NA
  nEst <- estimateN(
      dfunc.bs
    , area = area
    , propUnitSurveyed = propUnitSurveyed
  )

  if ( plot.bs ) {
    graphics::lines(dfunc.bs
          , newdata = plotCovValues
          , col = "blue"
          , lwd = 0.5
    )  
  }
  
  pb$tick()
  
  nEst        
}

#' @title Estimate a distance-based detection function
#' 
#' @description Fits a detection function using maximum likelihood. 
#'
#' @inheritDotParams dE.single formula likelihood w.lo w.hi expansions series x.scl g.x.scl warn outputUnits
#' @inheritDotParams dE.multi  formula likelihood w.lo w.hi expansions series x.scl g.x.scl warn outputUnits
#' 
#' @param data An `RdistDf` data frame. `RdistDf` data frames 
#' contain one line per transect and a list-based column. The list-based
#' column contains a data frame with detection information. 
#' The detection information data frame on each row contains (at least) distances 
#' and group sizes of all targets detected on the transect.
#' Function [RdistDf()] creates `RdistDf` data frames 
#' from separate transect and detection data frames. 
#' [is.RdistDf()] checks whether data frames
#' are `RdistDf`'s. 
#' 
#' @inherit dE.single details
#' 
#' @inheritSection dE.single Group Sizes
#' 
#' @inheritSection dE.single Contrasts
#' 
#' @inheritSection dE.single Measurement Units
#' 
#' @inherit dE.single return
#'     
#' @references Buckland, S.T., D.R. Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers,
#'    and L. Thomas. (2001) *Introduction to distance sampling: estimating
#'    abundance of biological populations*. Oxford University Press, Oxford, UK.
#'     
#' @seealso [abundEstim()], [autoDistSamp()].
#' Likelihood-specific help files (e.g., [halfnorm.like()]). 
#' 
#' @examples 
#' # Sparrow line transect example
#' data(sparrowDetectionData)
#' data(sparrowSiteData)
#' 
#' sparrowDf <- RdistDf(sparrowSiteData, sparrowDetectionData)
#' 
#' dfunc <- dfuncEstim(sparrowDf, 
#'                     formula = dist ~ 1
#'                   )
#' summary(dfunc)
#' 
#'   
#' data(sparrowDfuncObserver) # pre-estimated object
#' \dontrun{                 
#' # Commands that produced 'sparrowDfuncObserver'
#' sparrowDfuncObserver <- sparrowDf |> 
#'          dfuncEstim( 
#'            formula = dist ~ observer
#'          )
#' }     
#' sparrowDfuncObserver
#' summary(sparrowDfuncObserver)
#' plot(sparrowDfuncObserver)
#' plot(sparrowDfuncObserver
#'    , newdata = data.frame(observer = c("obs1", "obs2", "obs3"
#'                                      , "obs4", "obs5")))
#'
#' @export

dfuncEstim <- function (  data, ... ){

  call <- match.call()
  obsType <- Rdistance::observationType(data)
  
  # because Rdistance can override some options, e.g. optimizer
  # for oneStep, save a copy of options so can restore later.
  op <- options()
  op <- op[grepl("Rdistance_", names(op))]

  # Dispatch separate estimation functions based on observer type ----
  res <- switch( obsType
          , "single" = dE.single(data, ...)
          , "1|2"    =
          , "2|1"    =
          , "both"   = dE.multi(data, ...)
          , stop(dfuncEstimErrMessage("observer system", "obsType"))
          )

  # Assign common elements, and class ==== 
  res$call <- call
  res$dataName <- deparse(substitute(data))
  
  # nCovars ----
  # Can always count number of covariates later, in methods, but store 
  # here to save some time and code.
  # nCovars meanings:
  #   nCovars < 0 : model has 'nCovars' covariates but no intercept, 
  #                 e.g., dist ~ -1 + observer + height => nCovars = -2
  #   nCovars == 0 : model has (Intercept) only
  #   nCovars > 0 : model has 'nCovars' besides intercept,
  #                 e.g., dist ~ observer + height => nCovars = 2

  nCovars <- length(attr(stats::terms(res$mf), "term.labels"))
  if( attr(stats::terms(res$mf), "intercept") == 0 ){
    nCovars <- -nCovars  
  }
  res$nCovars <- nCovars
  
  # Estimated function type ----
  if( !( res$likelihood %in% c("Gamma","smu")) ){
    res$LhoodType <- "parametric" # halfnorm, hazrate, etc., all those
  } else {
    res$LhoodType <- res$likelihood  # Gamma and smu
  }

  # restore options, but make sure intEvalPts and intCoeffs match b/c 
  # they could have changed.
  options(op) # restores old intEvalPts and intCoeffs that may not match
  intCoefs <- simpsonCoefs(getOption("Rdistance_intEvalPts"))
  options(Rdistance_intCoefs = intCoefs)

  res 
}

#' @title Print method for distance function object
#' 
#' @description Print method for distance function objects produced 
#' by \code{dfuncEstim}.
#' 
#' @param x An estimated detection function object, normally 
#' produced by calling \code{\link{dfuncEstim}}.
#' 
#' @param \dots Included for compatibility with other print methods.  Ignored here.
#' 
#' @return The input distance function (\code{x}) is returned invisibly.
#' 
#' @seealso \code{\link{dfuncEstim}}, \code{\link{plot.dfunc}}, 
#' \code{\link{print.abund}}, \code{\link{summary.dfunc}}
#' 
#' @examples
#' # Load example sparrow data (line transect survey type)
#' data(sparrowSiteData)
#' data(sparrowDetectionData)
#' 
#' # Fit half-normal detection function
#' sparrowDf <- RdistDf(sparrowSiteData, sparrowDetectionData)
#' dfunc <- sparrowDf |> dfuncEstim(formula=dist~1)
#' 
#' dfunc
#' 
#' @export
print.dfunc <- function( x, ... ){

  # This routine must handle x with estimated coefficients and 
  # x that is just an mf object. 
  
  if( is.null(x$call) ){
    # this is what happens when x is just a model frame
    callLine <- paste("Rdistance Model:"
                  , colorize("No coefficients"))
    cat(paste0(callLine,"\n"))
    callLine <- paste("  "
                    , "Formula:"
                    , colorize(format(x$formula)))
    cat(paste0(callLine,"\n"))
    summary(  object = x$data
            , formula = x$formula
            , w.lo = x$w.lo
            , w.hi = x$w.hi)
    return(invisible(x))
  } 
  
  # If we are here, x has coefficients and is a regular 
  # estimated model.
  callLine <- deparse(x$call)
  callLine <- paste(callLine, collapse = " ")
  callLine <- strwrap(paste0("Call: ",callLine),exdent=2)
  cat(paste0(callLine,"\n"))

  isSmooth <- is.smoothed(x)
  fuzz <- getOption("Rdistance_fuzz")
  
  coefs <- stats::coef(x)
  if( is.null(x$varcovar) && !isSmooth ){
    
    # No bootstraps yet, SE's pending
    coefMat <- cbind(format(coefs))
    dimnames(coefMat)[[2]] <- c("Estimate")
    if( x$convergence == 0 ) {
      mess <- paste0(colorize("Success")
                   , "; "
                   , colorize("SE's pending bootstrap"))
    } else {
      mess <- colorize("FAILURE", col="white", bg = "bgRed")
      mess <- paste0( mess, " (Exit code= ", x$convergence, ", ", x$message, ")")
    }

  } else if ( !isSmooth ) {
    # X has a varcovar, either asymptotic or bootstrap
    vcDiag <- diag(x$varcovar)
    seCoef <- vcDiag
    seCoef[seCoef < fuzz] <- NA # fuzz is positive, so this NAs all negatives
    seCoef <- sqrt(seCoef)      # Avoids warning re sqrt of negative
    waldZ <- coefs / seCoef
    if( x$convergence == 0 ) {
      if( any(is.na(seCoef)) ) {
        mess <- colorize("VARIANCE FAILURE", col = "black", bg = "bgYellow")
        mess <- paste(mess, "(singular variance-covariance matrix)")
      } else {
        mess <- colorize("Success")
        # IF YOU CHANGE EITHER OF THESE MESSAGES, MAKE SURE TO 
        # CHANGE grepl IN IF STATEMENT MARKED WITH **** BELOW! 
        # ALSO! YOU NEED TO CHANGE THE SAME grepl STATEMENT 
        # IN SUMMARY.DFUNC (MARKED BY ****)
        if( is.null(x$B) ){
          mess <- paste0(mess, "; ", colorize("Asymptotic SE's"))
        } else {
          mess <- paste0(mess, "; ", colorize("Bootstrap SE's"))
        }
      }
    } else {
      mess <- colorize("FAILURE", col="white", bg = "bgRed")
      mess <- paste0( mess, " (Exit code= ", x$convergence, ", ", x$message, ")")
    }
    pWaldZ <- 2*stats::pnorm(-abs(waldZ), 0, 1 )
    coefMat <- cbind(format(coefs)
                     , format(seCoef)
                     , format(waldZ)
                     , format(pWaldZ))
    dimnames(coefMat)[[2]] <- c("Estimate", "SE", "z", "p(>|z|)")
  }
  cat("Coefficients:\n")
  print.default(coefMat, print.gap = 2, quote = FALSE)
  
  # **** CHANGE THIS grepl IF SUCCESSFUL MESSAGE CHANGES
  if( !grepl("(Asymptotic|Bootstrap) SE's", mess)  ){ 
    cat("\n")
    cat(paste("Message: ", mess,  "\n", sep=""))
  } 
  
  if( isSmooth ){
    cat(paste(x$fit$call[["kernel"]], "kernel smooth\n"))
    cat(paste(" Bandwidth method:", x$fit$call[["bw"]], "with adjustment factor", 
                format(x$fit$call[["adjust"]]),"\n"))
    cat(paste(" Actual bandwidth =", format(x$fit$bw), "\n"))
  } else if(length(coefs) == 0){
    cat("No coefficients\n")
    coefMat <- NULL
    mess <- "Rdistance data"
  }

  x$coefficients <- coefMat
  x$convMessage <- mess
  invisible(x)
}

#' @title Check RdistDf data frames
#' 
#' @description
#' Checks the validity of `Rdistance` nested data frames. 
#' `Rdistance` data frames 
#' are a particular implementation of rowwise `tibbles` 
#' that contain detections in a list column, and extra attributes 
#' specifying types. 
#' 
#' @param df A data frame to check
#' 
#' @param verbose If TRUE, an explanation of the check that fails is printed. 
#' Otherwise, no information on checks is provided.
#' 
#' @details The following checks are performed (in this order):
#' \itemize{
#'   \item `attr(df, "detectionColumn")` exists and points to a valid 
#'   list-based column in the data frame. 
#'   \item `attr(df, "obsType")` exists and is one of the valid values.
#'   \item `attr(df, "transType")` exists and is one of the valid values.
#'   \item `attr(df, "effortColumn")` exists and points to a column in the 
#'   data frame with 1 dimensional units attached.
#'   \item The data frame is either a 'rowwise_df' or 'grouped_df' 
#'   `tibble`.
#'   \item The data frame has only one row per group. One row per group 
#'   is implied by 'rowwise_df', but not a 'grouped_df', and both are allowed
#'   in `Rdistance`. One row per group ensures rows are uniquely identified 
#'   and hence represents one transect. 
#'   \item No column names in the list-column are duplicated in the non-list 
#'   columns of the data frame. This check ensures that `tidyr::unnest`
#'   executes. 
#' }
#' Other data checks, e.g., for measurement units, are performed 
#' later in [dfuncEstim()], after the model is specified. 
#' 
#' @return TRUE or FALSE invisibly. TRUE means all checks passed. FALSE implies 
#' at least one check failed. Use `verbose` = TRUE to see which. 
#' 
#' @examples
#' 
#' data(sparrowDf)
#' is.RdistDf(sparrowDf)
#' 
#' # Data frame is okay, but no attributes
#' data(sparrowDetectionData)
#' data(sparrowSiteData)
#' sparrowDf <- sparrowDetectionData |> 
#'   dplyr::nest_by( siteID
#'                , .key = "distances") |> 
#'   dplyr::right_join(sparrowSiteData, by = "siteID")
#' is.RdistDf(sparrowDf, verbose = TRUE)
#' 
#' # Manually add required attributes
#' attr(sparrowDf, "detectionColumn") <- "distances"
#' attr(sparrowDf, "effortColumn") <- "length"
#' attr(sparrowDf, "obsType") <- "single"
#' attr(sparrowDf, "transType" ) <- "line"
#' is.RdistDf(sparrowDf, verbose = TRUE)
#' 
#' @export
#' 
is.RdistDf <- function(df, verbose = FALSE){
  
  dfName <- deparse(substitute(df))

  # Check for list column name attribute ----
  distColName <- attr(df, "detectionColumn")[1]  # could be NULL
  hasListColName <- !is.null(distColName)
  if( !hasListColName ){
    if(verbose){
      cat(paste(
        crayon::red(dfName)
        , "must have a"
        , crayon::red("'detectionColumn'")
        , "attribute naming a list-based column that contains detection information."
        , "Assign attributes with statements like"
        , crayon::red(paste0("attr("
                             , dfName
                             , ",'detectionColumn') <- <list column>"))
        , "See help('RdistDf')."
        , "\n"
      ))
    }
    return(FALSE)
  } 
  
  # Check for list-based distance column. ----
  # The && are critical here. Must stop evaluating if distColName 
  # is not in the data frame. 
  hasDistCol <-  
    (distColName %in% names(df)) &&
    (is.list(df[,distColName][[1]]))
  if( !hasDistCol ){
    if(verbose){
      cat(paste0(
        crayon::red(dfName)
        , " must have a list column containing detection information." 
        , " Expected to find a list-based column named "
        , crayon::red(distColName)
        , ". "
        , "See help('RdistDf')."
        , "\n"
      ))
    }
    return(FALSE)
  }
  
  # Check for presence and validity of transType ----
  transType <- Rdistance::transectType(df)
  hasTransType <- !is.null(transType) &&
    (transType %in% c("line", "point"))
  if( !hasTransType ){
    if(verbose){
      cat(paste(
        crayon::red(dfName)
        , "must have a declared transect type, either 'line' or 'point'."
        # , "Transect type must be named in attribute 'transType'."
        , "Assign transect type attribute with statement like"
        , crayon::red(paste0("attr("
                             , dfName
                             , ",'transType') <- '<type>'"))
        , "See help('RdistDf').\n"
      ))
    }
    return(FALSE)
  }

  # Check for presence and validity of obsType ----
  obsType <- Rdistance::observationType(df)
  hasObsType <- !is.null(obsType) &&
    (obsType %in% c("single", "1given2", "2given1", "both"))
  if( !hasObsType ){
    if(verbose){
      cat(paste(
        crayon::red(dfName)
        , "must have a declared observation system, one of 'single', '1given2', '2given1', or 'both'."
        # , " Observation type must be named in attribute 'obsType'."
        , "Assign observation system attribute with statement like"
        , crayon::red(paste0("attr("
                             , dfName
                             , ",'obsType') <- '<type>'"))
        , "See help('RdistDf').\n"
      ))
    }
    return(FALSE)
  }

  # Check for presence and validity of Effort column ----
  effCol <- attr(df, "effortColumn")
  hasEffCol <- !is.null(effCol) &&
    (effCol %in% names(df))
  if( !hasEffCol ){
    if(verbose){
      cat(paste(
        crayon::red(dfName)
        , "must have a valid effort column."
        , "Assign effort with a statement like"
        , crayon::red(paste0("attr("
                             , dfName
                             , ",'effortColumn') <- '<column name>'"))
        , "See help('RdistDf').\n"
      ))
    }
    return(FALSE)
  }
  
  # Check line length has units, nPoints does not ----
  effVec <- df |> 
    dplyr::pull(dplyr::all_of(effCol))
  if( transType == "line" ){
    if( !inherits(effVec, "units") ){
      if(verbose){
        cat(paste(
          "Transect type is 'line' but effort column"
          , crayon::red(effCol)
          , "does not have units. "
          , "Set units with a statement like"
          , crayon::red(paste0(
            dfName
            , "$"
            , effCol
            , " <- setUnits( "
            , dfName
            , "$"
            , effCol
            , ", 'm')\n"
          ))
        ))
      }
      return(FALSE)
    }
  } else {
    # Transect type is 'point'
    if( inherits(effVec, "units") || 
        !is.numeric(effVec)){
      if(verbose){
        cat(paste(
          "Transect type is 'point', but effort column"
          , crayon::red(effCol)
          , "either has units or is not numeric. "
          , "Set effort column to number of points on transects.\n"
          ))
      }
      return(FALSE)
    }
  }
  
  
  # Check for rowwise or grouped data frame ----
  if( !inherits(df, "rowwise_df") && !inherits(df, "grouped_df")){
    if(verbose){
      cat(paste(
        crayon::red(dfName)
        , "must be a 'rowwise_df' or 'grouped_df'."
        , "Use dplyr::nest() or run Rdistance::RdistDf()."
        , "See help('RdistDf').\n"
      ))
    }
    return(FALSE)
  }
  
  # Check that groups uniquely identify rows ----
  # If we are here, the data frame has groups.
  # Note: I am not sure it is possible to have a "rowwise_df" containing groups
  # that have >1 row.  I.e., "rowwise_df" may imply all groups have one row.
  # I don't think so.  Anyway, we check group sizes here. 
  grps <- dplyr::group_size(df)
  if( any(grps > 1) ){
    if(verbose){
      cat(paste(
        "Groups in" 
        , crayon::red(dfName)
        , "must identify unique rows, corresponding"
        , "to unique transects."
        , "Found"
        , sum(grps > 1)
        , "groups containing more than one row, i.e., duplicate row IDs."
        , "Identify duplicate rows using"
        , crayon::red(paste0(dfName, " |> dplyr::summarise(n = dplyr::n())"
        , " |> dplyr::filter(n > 1)."))
        , "See help('RdistDf').\n"
      ))
    }
    return(FALSE)
  }
  
  # Check that tidy::unnest will work ----
  # no duplicate names in list vs out list
  firstList <- df |> 
    dplyr::ungroup() |>  
    dplyr::pull(dplyr::all_of(distColName))
  firstList <- do.call(dplyr::bind_rows, firstList)
  namesInList <- names(firstList)
  namesOutList <- setdiff(names(df), distColName)
  if( any( namesInList %in% namesOutList) ){
    if(verbose){
      cat(paste0(
        "Duplicate names found inside ",
        crayon::red(distColName), 
        " column and in the remainder of " 
        , crayon::red(dfName)
        , ". Make names in detection data frame unique."
        , " Test: "
        , crayon::red(paste0("tidyr::unnest(", dfName, ")"))
        , " should execute."))
      cat(" Duplicate names found: ")
      cat(crayon::red(intersect(namesOutList, namesInList)))
      cat("\n")
    }
    return(FALSE)
  }
  
  # could pull and save the site ID's that identify rows. Columns that 
  # produce unique rows (i.e., the Site ID columns) are: 
  # grps <- dplyr::group_data(df)
  # names(grps)[ names(grps) != ".rows"]
  # But, I don't think we need the actual name of the columns provided 
  # groups of one row each are defined. 
  
  return(TRUE)
  
}

#' @title Construct Rdistance nested data frames
#' 
#' @description Makes an `Rdistance` data frame from 
#' separate transect and detection 
#' data frames. `Rdistance` data frames are nested 
#' data frames with one row per transect. Detection 
#' information for each transect appears in a list-based 
#' column that itself contains a
#' data frame. See **Rdistance Data Frames**. 
#' 
#' `Rdistance` data frames can be constructed using calls to 
#' `dplyr::nest_by` and `dplyr::right_jion`, with subsequent 
#' attribute assignment (see **Examples**). This routine is 
#' a convenient wrapper for those calls.
#' 
#' @param transectDf A data frame with one row per transect and 
#' columns containing information about the entire transect. 
#' At a minimum, this data frame must contain the transect's ID so 
#' it can be merged with `detectionDf`, (see parameter `by`) 
#' and the amount of effort the transect represents 
#' (see parameter `.effortCol`).  
#' All detections are made on a transect, but not all transects 
#' require detections. That is, `transectDf` 
#' should contain rows, and hence
#' ID's and lengths, of all surveyed transects, even those on 
#' which no targets were detected (so-called "zero transects"). 
#' Transect-level covariates, such 
#' as habitat type, elevation, or observer IDs, should appear as variables 
#' in this data frame. 
#' 
#' @param detectionDf A data frame containing   
#' detection information associated with each transect.  
#' At a minimum, each row of this data 
#' frame must contain the following:
#' \itemize{
#'   \item **Transect IDs**: The ID of the transect on 
#'   which a target group was detected so that the 
#'   detection data frame can be merged with `transectDf` 
#'   (see parameter `by`). Multiple detections on the same transect 
#'   are possible and hence multiple rows in `detectonDf` 
#'   can contain the same transect ID. 
#'      
#'   \item **Detection Distances**: The distance at which each 
#'   detection was made. The distance column will eventually be 
#'   specified on the left-hand 
#'   side of `formula` in a call to `dfuncEstim`.  
#'   As of Rdistance version 3.0.0, detection distances must have 
#'   physical measurement units assigned. See 
#'   **Measurement Units**. 
#' }
#' Optional columns in `detectionDf`:
#' \itemize{
#'    \item **Group sizes**:If sighted targets vary in size, or group sizes 
#'          are not all 1, `detectionDf` must 
#'          also contain a column specifying group sizes. Non-unity group sizes 
#'          are specified using `+groupsize(columnName)` on the 
#'          right-hand-side of `formula` in an eventual call to `dfuncEstim`. 
#'          
#'    \item **Detection Level Covariates**: Such as sex, color, habitat, etc. 
#' }
#' 
#' @param pointSurvey If TRUE, observations were
#' made from discrete points (i.e., during a point-transect survey) and distances 
#' are radial from observation point to target.  If FALSE, 
#' observations were made along a continuous transect 
#' (i.e., during a line-transect survey) and distances are from target to nearest 
#' point on the transect (i.e., perpendicular to transect).
#'    
#' @param observer Type of observer system.  Legal values are **"single"** for 
#' single observer systems, **"1given2"** for a double observer system 
#' wherein observations 
#' made by observer 1 are tested against observations made by observer 2,
#' **"2given1"** for a double observer system wherein observations 
#' made by observer 2 are tested against observations made by observer 1, and
#' **"both"** for a double observer system wherein observations 
#' made by both observers are tested against the other and combined.
#' 
#' @param .detectionCol Name of the list column that will
#' contain detection data frames. Default name is "detections".
#' Detection distances (LHS of `dfuncEstim` formula) and 
#' group sizes are normally 
#' columns in the nested detection data frames embedded in 
#' `.detectionCol`. 
#' 
#' @param .effortCol For continuous line transects, 
#' this specifies the name of a column in `transectDf` 
#' containing transect lengths, which must have measurement units.  
#' For point transects, this specifies the name of a column containing 
#' the number of points on each transect.  The effort column for point 
#' transects *cannot* contain 
#' measurement units. Default is "length" for line-transects, "numPoints" for 
#' point-transects. If those names are not found, the first column
#' in the merged data frame whose name contains 'point' 
#' (for point transects) or 'length' (for line transects)
#' is used and a message is printed. Matching is case insensitive, 
#' so for example, 'nPoints' and 
#' 'N_point' and 'numberOfPoints' will all be matched.   If two or more column 
#' names match the effort column search terms, a warning is issued. 
#' See **Transect Lengths** for a description of point and line transects.
#' 
#' @param by A character vector of variables to use in the join. The right-hand
#' side of this join identifies unique transects (unique 
#' rows) in both `transectDf` and the output (see warning in **Details**).
#' If NULL, the join will be 'natural', using all common variables 
#' in `transectDf` and `detectionDf`. To join on 
#' specific variables, specify a character vector of 
#' the variables. For example, by = c("a", "b") 
#' joins `transectDf$a` to `detectionDf$a` and 
#' `transectDf$b` to `detectionDf$b`. If join variable names 
#' differ between `transectDf` and `detectionDf`, 
#' use a named character vector like by = c("a" = "b", 
#' "c" = "d") which joins `transectDf$a` to 
#' `detectionDf$b` and 
#' `transectDf$c` to `detectionDf$d`. 
#' 
#' 
#' @inheritSection dE.single Measurement Units
#' 
#' @return A nested tibble (a generalization of base data frames) 
#' with one row per transect, and detection 
#' information in a list column.  Technically, the return is 
#' a grouped `tibble` from 
#' the `tibble` package with one row per group, and a list 
#' column containing detection information. 
#' Survey type, observer system, and name of the effort column 
#' are recorded 
#' as attributes (`transType`, `obsType`, and `effortColumn`, respectfully). 
#' The return prints nicely using methods 
#' in package `tibble`.  If returned objects print strangely, 
#' attach library `tibble`.  A summary method tailored to distance sampling 
#' is available (i.e., `summary(return)`).
#' 
#' @section Rdistance Data Frames: 
#' 
#' `RdistDf` data frames contain the following information:
#' 
#' \itemize{
#'   \item **Transect Information**: Each row of the 
#'   data frame contains transect id and effort. Effort is 
#'   transect length for line-transects, and number of points for 
#'   point-transects.
#'   Optionally, transect-level covariates (such as habitat or observer
#'   id) appear on each row. 
#'   \item **Detection Information**: Observation distances
#'   (either perpendicular off-transect or radial off-point) appear 
#'   in a data frame stored in a list column.  If detected groups
#'   occasionally included more than one target, a group size column must 
#'   be present in the list-column data frame.
#'   Optionally, detection-level covariates (such as sex or size)
#'   can appear in the data frame of the list column.
#'   \item **Distance Type**: The type of observation distances, either 
#'   perpendicular off-transect (for line-transects studies) or 
#'   radial off-point (for point-transect studies) must appear as an 
#'   attribute of `RdistDf`'s. 
#'   \item **Observer Type**: The type of observation system used, either 
#'   single observer or one of three types of multiple observer systems, must 
#'   appear as an attribute of `RdistDf`'s.
#' }
#'
#' 
#' @details 
#' 
#' For valid bootstrap estimates of confidence intervals (computed in [abundEstim()]), 
#' each row of the nested data frame must represent one transect (more generally, 
#' one sampling unit), and none should
#' be duplicated. The combination of transect columns 
#' in `by` (i.e., the LHS of the merge, or "a" and "b" of 
#' `by = c("a" = "d", "b" = "c")` for example) 
#' should specify *unique* transects and unique rows of 
#' `transectDf`. **Warning:** If `by` 
#' does not specify unique rows of `transectDf`, `dplyr::left_join`, 
#' which is called internally, will perform a many-to-many merge without 
#' warning, and this normally duplicates both 
#' transects and detections.
#'
#' 
#' @section Transect Lengths:
#' Line-transects are continuous paths with targets detectable at 
#' any point.  Point transects
#' consist of one or more discrete points along a path 
#' from which observers search for targets. 
#' The length of a line-transect is it's physical length (e.g., km or miles).
#' The 'length' of a point transect is 
#' the number of points along the transect. Single 
#' points are considered transects of length one. The length of line-transects
#' must have a physical measurement unit (e.g., 'm' or 'ft').  The length of 
#' point-transects must be a unit-less integers (i.e., number of points).
#' 
#'  
#' @seealso [is.RdistDf()]: check validity of RdistDf data frames;
#'  [dfuncEstim()]: estimate distance function.
#'  
#' @examples
#' data(sparrowSiteData)
#' data(sparrowDetectionData)
#' 
#' sparrowDf <- RdistDf( sparrowSiteData, sparrowDetectionData )
#' is.RdistDf(sparrowDf, verbose = T)
#' summary(sparrowDf)
#' summary(sparrowDf
#'       , formula = dist ~ groupsize(groupsize)
#'       , w.hi = 100 %m%.)
#' 
#' # Equivalent to above: 
#' sparrowDf <- sparrowDetectionData |> 
#'   dplyr::nest_by( siteID
#'                , .key = "detections") |> 
#'   dplyr::right_join(sparrowSiteData, by = "siteID") 
#' attr(sparrowDf, "detectionColumn") <- "detections"
#' attr(sparrowDf, "effortColumn") <- "length"
#' attr(sparrowDf, "obsType") <- "single"
#' attr(sparrowDf, "transType") <- "line"
#' is.RdistDf(sparrowDf, verbose = T)
#' summary(sparrowDf, formula = dist ~ groupsize(groupsize))
#'
#' # Condensed view: 1 row per transect (make sure tibble is attached)
#' sparrowDf
#' 
#' # Expansion methods:
#' # (1) use Rdistance::unnest (includes zero transects)
#' df1 <- unnest(sparrowDf)
#' any( df1$siteID == "B2" )  # TRUE
#' 
#' # Use tidyr::unnest(); but, no zero transects
#' df2 <- tidyr::unnest(sparrowDf, cols = "detections")
#' any( df2$siteID == "B2" )  # FALSE
#' 
#' # Use dplyr::reframe for specific transects (e.g., for transect "B3")
#' sparrowDf |> 
#'   dplyr::filter(siteID == "B3") |>
#'   dplyr::reframe(detections)
#'   
#' # Count detections per transect (can't use dplyr::if_else)
#' df3 <- sparrowDf |> 
#'   dplyr::reframe(nDetections = ifelse(is.null(detections), 0, nrow(detections)))
#' sum(df3$nDetections) # Number of detections
#' sum(df3$nDetections == 0) # Number of zero transects
#'     
#' # Point transects
#' data(thrasherDetectionData)
#' data(thrasherSiteData)
#' thrasherDf <- RdistDf( thrasherSiteData
#'                , thrasherDetectionData
#'                , pointSurvey = TRUE
#'                , by = "siteID"
#'                , .detectionCol = "detections")
#' summary(thrasherDf, formula = dist ~ groupsize(groupsize))
#'                
#' @export
#' 
RdistDf <- function( transectDf
                   , detectionDf
                   , by = NULL
                   , pointSurvey = FALSE
                   , observer = "single"
                   , .detectionCol = "detections"
                   , .effortCol = NULL
                   ){
  
  # Check lengths ----
  # Why? Because switch throws error when EXPR > length 1
  if( length(pointSurvey) > 1 ){
    warning(paste0("'pointSurvey' has length", length(pointSurvey), 
                  ". Only its first element was used."))
    pointSurvey <- pointSurvey[1]
  }
  if( length(observer) > 1 ){
    warning(paste0("'observer' has length", length(observer), 
                   ". Only its first element was used."))
    observer <- observer[1]
  }
  
  # translate inputs to class values ----
  obsType <-  factor(observer
                   , levels = c("single", "1given2", "2given1", "both")
                   )
  transType <-  factor(pointSurvey
                       , levels = c(TRUE, FALSE)
                       , labels = c("point", "line")
                       )

  # check inputs ----
  if( is.na(obsType) ){
    stop(paste("Value of 'observer' must be one of 'single', '1given2', '2given1', or 'both'."
               , "Found", paste(observer, collapse = ",")))
  }
  if( is.na(transType) ){
    stop(paste("Value of 'pointSurvey' must be either TRUE or FALSE."
               , "Found", paste(pointSurvey, collapse = ",")))
  }

  # Nest detection data frame first ----
  if( is.null(by) ){
    by <- intersect(names(transectDf), names(detectionDf))
    if( length(by) == 0){
      stop(paste("No variables in common to merge transects and detections."
                 , "Consider using format 'by = c('a' = 'b')' to'
                 , 'join on columns 'a' and 'b'."))
    }
  } 
  
  # 'by' now has a value always, check it is named
  if( is.null(names(by)) ){
    # e.g., by = c("id", "region")
    reverseBy <- by
    names(reverseBy) <- by
  } else {
    # by is specified. b/c we nest first, then right join, we 
    # gotta reverse names and values in 'by'
    reverseBy <- names(by)
    names(reverseBy) <- by
  }
  ans <- detectionDf |> 
    dplyr::nest_by( dplyr::across(dplyr::all_of(unname(by)))
                    , .key = .detectionCol
                    , .keep = FALSE) |> 
    dplyr::right_join(transectDf, by = reverseBy)

  attr(ans, "detectionColumn") <- .detectionCol
  attr(ans, "obsType") <- as.character(obsType)
  attr(ans, "transType") <- as.character(transType)
  
  # Rename ID column: b/c we nested first, then merged, the ID column
  # in the result is names the first element of reverseBy. 
  # Don't need to do this if by == reverseby, but...no harm if so
  ans <- ans |> 
    dplyr::rename_with(
        .fn = function(inNm, nmMap){nmMap}
      , .cols = names(reverseBy), nmMap = reverseBy
    )

  # Figure out effort column ----
  if( is.null(.effortCol) ){
    if( pointSurvey ){
      .effortCol <- "numPoints"
    } else {
      .effortCol <- "length"
    }
  }
  if( !(.effortCol %in% names(ans)) ){
    if( pointSurvey ){
      searchTerm <- "(p|P)(o|O)(i|I)(n|N)(t|T)"
      torp <- "number of points"
    } else {
      searchTerm <- "(l|L)(e|E)(n|N)(g|G)(t|T)(h|H)"
      torp <- "transect length"
    }
    candidates <- grep(searchTerm, names(ans))
    if(length(candidates) == 0){
      # we tried
      stop(paste0("Effort column '", .effortCol, "' not found.", 
                  " Specify effort column using .effortCol input parameter. ",
                  "Potential effort column(s): ", 
                  paste(names(ans)[candidates], collapse = ", ")))
    } else if(length(candidates) > 1){
      warning(paste0("Found two or more potential effort columns. "
                  , "Using "
                  , names(ans)[candidates[1]]
                  , " as "
                  , torp
                  , ". Specify a different effort column using the .effortCol input parameter."
                  ))
    } else {
      cat(paste0(
                 "Using "
                , colorize(names(ans)[candidates[1]])
                , " as "
                , torp
                , ". Specify a different effort column using the .effortCol input parameter."
      ))
    }
    .effortCol <- names(ans)[candidates[1]]
  }
  attr(ans, "effortColumn") <- .effortCol
  
  # Could assign class this way; but, I wish outside ops would preserve
  # the extra classes.  they don't. dplyr::group_by wipes out 
  # obsType and transType class components
  # class(ans) <- c("RdistDf", class(ans))
  # class(ans) <- clsAns
  
  ans
}

#' @title Effective Strip Width (ESW) for line transects
#'   
#' @description Returns effective strip width (ESW) for 
#'   line-transect detection functions. 
#'   See [EDR()] is for point transects.  
#'   
#' @inheritParams effectiveDistance
#' 
#' @details ESW is area under 
#'   a scaled distance function between its
#'   left-truncation limit (`obj$w.lo`) and its right-truncation 
#'   limit (`obj$w.hi`). \if{latex}{I.e., 
#'     \deqn{ESW = \int_{w.lo}^{w.hi} g(x)dx,} 
#'   where \eqn{g(x)} is the distance
#'   function scaled so that \eqn{g(x.scl) = g.x.scl}
#'   and \eqn{w.lo} and \eqn{w.hi} are the lower
#'   and upper truncation limits.  }
#'   
#'   If detection does not decline with distance, 
#'   the detection function is flat (horizontal), and 
#'   area under the detection  
#'   function is  \eqn{g(0)(w.hi - w.lo)}.  
#'   If, in this case, \eqn{g(0) = 1}, effective sampling distance is 
#'   the half-width of the surveys, \eqn{(w.hi - w.lo)}
#'   
#' @inherit effectiveDistance return   
#'   
#' @seealso [dfuncEstim()], [EDR()], 
#' [effectiveDistance()]
#' 
#' @examples
#' data(sparrowDfuncObserver)
#' 
#' ESW(sparrowDfuncObserver) # vector length 356 = number of groups
#' ESW(sparrowDfuncObserver, newdata = data.frame(observer = 
#'    c("obs2", "obs4"))) # vector length 2
#' 
#' @export

ESW <- function( object, newdata = NULL ){
  
  # Issue error if the input detection function was fit to point-transect data

  if( Rdistance::is.points(object) ){
    stop("ESW is for line transects only.  See EDR for the point-transect equivalent.")
  } 

  likExpan <- paste0(object$likelihood
                     , "_"
                     , object$expansions
                     )

  if( likExpan == "halfnorm_0" ){
    esw <- integrateHalfnormLines(object, newdata = newdata)
    
  } else if( likExpan == "hazrate_0" ){
    esw <- integrateHazrateLines(object, newdata = newdata)
    
  } else if( likExpan == "negexp_0" ){
    esw <- integrateNegexpLines(object, newdata = newdata)
    
  } else if( likExpan == "oneStep_0" ){
    esw <- integrateOneStepLines(object, newdata = newdata)
    
  } else if( grepl("oneStep", likExpan) ){
    esw <- integrateOneStepNumeric(object, newdata = newdata)

  } else if( grepl("Gamma_0", likExpan) ){
    esw <- integrateGammaLines(object, newdata = newdata)
    
  } else {
    esw <- integrateNumeric(object, newdata = newdata)
  }
  
  # Scale ESW for g(0)
  esw <- object$g.x.scl * esw
  
  esw
  
}

#' @title Domain of expansion factors
#' 
#' @description
#' The domain to which expansion factors are applied varies by likelihood. 
#' When the domain varies, it depends on parameter values. For example, 
#' `oneStep` expansion factors are applied between 
#' 0 and the likelihood's first parameter (\eqn{\theta}{Theta}), which 
#' varies by covariates.  This function computes the likelihood-specific 
#' expansion domains
#' 
#' @inheritParams startLimits
#' 
#' @param params A matrix of likelihood parameters.  Size is (number 
#' of distances) X ((number of cases) + (number of non-covariate parameter)).
#' 
#' @param k The number of cases.
#' 
#' @return A plain vector of length k containing expansion domains, with units.  
#' 
#' @export
expandW <- function(ml
                  , params
                  , k
                  ){
  # oneStep and triangle and huber have expansion W that varies
  # (expansion W is NOT the real w = strip width)
    
  if(!(ml$likelihood %in% differentiableLikelihoods())){
    # Expansion domain depends on parameters.
    # e.g., Apply expansion between 0 and theta for triangle
    W <- params[1, 1:k] # drop second parameter which is tagged onto end
    W <- setUnits(exp(W), ml$outputUnits) # W always constant w/in columns
    W <- W - ml$w.lo
    if( ml$likelihood %in% c("huber") ){
      # e.g., Apply expansion between 0 and theta for oneStep, triangle
      theta2 <- setUnits(params[1, k+1], ml$outputUnits)
      W <- W + theta2
      W[W > (ml$w.hi - ml$w.lo)] <- ml$w.hi - ml$w.lo
    }

  } else { 
    # Most likelihoods: expansions constant across params
    W <- rep(ml$w.hi - ml$w.lo, k)
  }

  W
}

#' @title Density on transects
#' 
#' @description
#' An internal prediction method for computing density 
#' on the sampled transects. 
#' 
#' @inheritParams predict.dfunc 
#' @inheritParams abundEstim
#' 
#' @return A data frame containing the original data used 
#' to fit the distance function, plus an additional column
#' containing the density of individuals on each transect. 
#' 
#' @examples
#' data(sparrowDfuncObserver)
#' predict(sparrowDfuncObserver, type="density")
#' 
#'  
#' @export
predDensity <- function(object
                      , propUnitSurveyed = 1.0
                       ){
  
  if( any(0 > propUnitSurveyed) | any(propUnitSurveyed > 1) ){
    stop(paste0("Proportion of unit surveyed must be between 0 and 1. "
              , "Found "
              , sum((0 > propUnitSurveyed) | (propUnitSurveyed > 1))
              , " values outside this range."
        ))
  }
  if( length(propUnitSurveyed) != 1 && length(propUnitSurveyed) != nrow(object$data) ){
    stop(paste0("Length of 'propUnitSurveyed' must either be 1 or "
              , nrow(object$data)
              , ". Found length "
              , length(propUnitSurveyed)
              , "."))
  }
  
  # Get siteId column(s) ----
  siteIDs <- object$data |> dplyr::group_vars()

  # Merge counts ----
  # Note: We would like to use object$mf which has observations ONLY
  # inside the observation window (w.hi - w.lo). But, object$mf
  # does not have the siteID columns nor does it have missing 
  # missing distances.  So, we use unnest(object$data), which has 
  # all observations, and we filter to the right ones. 
  
  mt <- stats::terms(object$mf)
  distVar <- all.vars(mt)[attributes(mt)$response]
  groupSizeVar <- all.vars(mt)[attributes(mt)$offset]
  effVar <- attr(object$data, "effortColumn")
  
  # Gotta use object$data because object$mf has no missing distances
  # nor the zero transects. $data has these
  df <- Rdistance::unnest(object$data) 
  
  # Must add group size to df if not specified in formula. ----
  #  If not specified (i.e., assumed 1), fake groupsizes are in object$mf
  #  but not in object$data. 
  
  # Note: to use the [missing distance but sighted group] functionality,
  # users must specify groupsize() in the formula. i.e., set some groupsizes
  # to positive, set their distances to missing, then specify group sizes 
  # in the original formula.  Otherwise, all groups are assumed size 1 and 
  # missing distances (from unnest) are assumed to be zero transects. 
  if( length(groupSizeVar) == 0 || !(groupSizeVar %in% names(df)) ){
    # I don't think length(groupSizeVar) == 0 ever happens b/c there is always an offset
    # BUT, groupSizeVar is not in df if we made up the groupsize in parseModel
    ..groupSizes.. <- dplyr::if_else(
        is.na(df[[distVar]]),
        0, # assumed zero transects, non-missing group sizes overwritten
        1
      ) 
    df[,groupSizeVar] <- ..groupSizes.. # needed for effectiveDistance
  } else {
    # Gotta set missing group sizes to 0, in df, or model.frame inside 
    # predict method (that is inside effectiveDistance(), below) tosses 
    # the missings. ( I have na.action set to 
    # na.omit in model.frame, and group size is in the Terms so any 
    # missing are dropped)
    df <- df |> 
      dplyr::mutate(dplyr::across(dplyr::all_of(groupSizeVar)
                                  , .fns = function(.x){
                                    .x[is.na(.x)] <- 0
                                    .x
                                  }))
    ..groupSizes.. <- df[[groupSizeVar]]
  }
  
  # Note: at this point, df has the group size variable, and 
  # there are no missing groupsizes. Zero transects have 0 groupsize.
  
  # Compute ESW/EDR for every observation
  # at this point, df contains missing distances.
  # missing distances are zero transects and missing distances 
  # coupled with non-missing group sizes
  effDist <- Rdistance::effectiveDistance(object
                         , newdata = df)
  
  # Now can pull columns for convenience
  ..distances.. <- df[[distVar]] # don't call distances() b/c need zero transects
  ..effort.. <- effort(object) # nTransects long
  
  # Compute index of observations in strip OR those with 
  # missing distance but non-missing groupsizes
  ..instrip.. <- (is.na(..distances..) & !is.na(..groupSizes..)) |
             ((object$w.lo <= ..distances..) & (..distances.. <= object$w.hi))
  
  # Compute estimates by transect
  w <- object$w.hi - object$w.lo # length 1
  if(is.points(object)){
    ..pDetect.. <- (effDist / w)^2
    ..effort.. <- pi * propUnitSurveyed * w^2 * ..effort..
  } else {
    ..pDetect.. = effDist / w
    ..effort.. = 2 * propUnitSurveyed * w * ..effort..
  }
  # NOTE: At this point, there cannot be any missing ..groupSizes.. or ..pDetect..
  ..nAdjusted.. <- ..groupSizes.. / ..pDetect..  # nObs+nZero long
  deCounts <- df |>
    dplyr::mutate(..groupSizes.. = ..groupSizes..
                , ..pDetect.. = ..pDetect..
                , ..nAdjusted.. = ..nAdjusted..
                , ..instrip.. = ..instrip.. ) |> 
    dplyr::group_by(dplyr::across(dplyr::all_of(siteIDs))) |> 
    dplyr::summarise(individualsSeen = sum(..groupSizes..[..instrip..])
                     , avgPdetect = dplyr::if_else(
                             sum(..instrip..) > 0
                           , mean(..pDetect..[..instrip..])
                           , mean(..pDetect..)
                         )
                     , ..nAdjusted.. = sum(..nAdjusted..[..instrip..])
    ) 
  deCounts <- deCounts |> 
    dplyr::ungroup() |> 
    dplyr::mutate(density = ..nAdjusted.. / ..effort..
                , effort = ..effort..
    ) |> 
    dplyr::rename("abundance" = "..nAdjusted..")
  
  # NOTE: the "..XXX.." variables and renaming of abundance and 
  # the strange MUTATE statements above are to get around CRAN checks
  # that don't recognize new variables inside mutate statements. 

  # Internal function to remove units from unitless columns ----  
  drop1Units <- function(x){
    unitless <- setUnits(1,"1")
    if(inherits(x, "units")){
      if( units(x) == units(unitless) ){
        x <- dropUnits(x)
      }
    }
    x
  }
  
  # Remove units from unitless columns ----
  deCounts <- deCounts |> 
    dplyr::mutate(dplyr::across(tidyselect::where(is.numeric),
                                .fns = drop1Units))
  
  return( deCounts )  
}

#' @title Perform bootstrap iterations 
#' 
#' @description
#' Performs bootstrap resampling iterations, either in parallel across 
#' CPU cores or in serial on a single core. 
#' 
#' @inheritParams dE.single
#' 
#' @inheritParams abundEstim
#' 
#' @inheritParams oneBsIter
#' 
#' @param parallel Logical scalar. TRUE if we are running iterations 
#' in parallel across CPU cores.  Number of cores specified in \code{cores}.
#' 
#' @param cores Integer scalar.  The number of CPU cores to use during 
#' parallel operations, if requested.  Ignored if \code{parallel == FALSE}.
#' 
#' @return A data frame containing density, abundance,  
#' and other relevant statistics for every bootstrap iteration.
#' Number of rows is \code{R}.  If the model from one iteration failed 
#' for any reason (e.g., non-convergence), the entire row except the ID column
#' is missing.
#' 
#' @seealso \code{\link{abundEstim}}; \code{\link{oneBsIter}}
#' 
#' @export
bootstrap <- function(
                      object
                    , area 
                    , propUnitSurveyed 
                    , R 
                    , plot.bs 
                    , plotCovValues
                    , showProgress 
                    , parallel
                    , cores
                    ){
  
  nDigits <- ceiling(log10(R + 0.1))
  id <- 1:R
  bsData <-  data.frame(
    id = paste0("Bootstrap_",
                formatC(id
                        , format = "f"
                        , digits = 0
                        , width = nDigits
                        , flag = "0"))
  ) |> 
    dplyr::group_by(id) 
  
  # set up progress bar if called for
  if(showProgress){
    pb <- progress::progress_bar$new(
      format = paste0(R, " Bootstraps: [:bar] ETA: :eta")
      , total = R
      , clear = FALSE
    )
  } else {
    pb <- list(tick = function(){})
  }

  # start a timer
  strtTime <- Sys.time()  
  
  # Create cluster if called for
  if( parallel ){
    cat(paste0("Creating CPU cluster with "
               , colorize(cores)
               , " cores..."))
    
    cl <- multidplyr::new_cluster(cores)  
    bsData <- bsData |> 
      multidplyr::partition(cl)
    
    cat("done.\n")
    
    cat("Copying data to cores...")
    multidplyr::cluster_library(cl, "Rdistance")
    multidplyr::cluster_copy(cl, "object")
    multidplyr::cluster_copy(cl, "area")
    multidplyr::cluster_copy(cl, "propUnitSurveyed")
    multidplyr::cluster_copy(cl, "pb") # known NULL b/c showprogress known F
    multidplyr::cluster_copy(cl, "plot.bs")  # known F
    cat("done.\n")
    
    cat(paste0(R, " bootstrap iterations initiated. Standby..."))
    strtTime <- Sys.time()
  } 
  
  # --- Apply estimation to each ID group ----
  B <- bsData |> 
    dplyr::summarise(
      oneBsIter(  object = object
                , area = area
                , propUnitSurveyed = propUnitSurveyed
                , pb = pb
                , plot.bs = plot.bs
                , plotCovValues = plotCovValues
                , warn = FALSE
                , asymptoticSE = FALSE
      )
    )
  
  if( parallel ){
    B <- B |> 
      dplyr::collect()
  }

  # compute run time    
  runTime <- as.numeric(difftime(Sys.time(), strtTime, units = "s"))
  runTimeUnits <- "sec"
  if(runTime > 60){
    runTime <- runTime / 60
    runTimeUnits <- "min"
  }
  if(runTime > 60){
    runTime <- runTime / 60
    runTimeUnits <- "hrs"
  }
  cat(paste0("Run Time: ", round(runTime,3), " ", runTimeUnits, "\n"))

  
  if(showProgress){
    pb$terminate()
  }
  
  B
  
}

#' @title Mixture of two uniforms likelihood
#' 
#' @description
#' Compute likelihood function for a mixture of two uniform
#' distributions. 
#' 
#' @inheritParams halfnorm.like
#' 
#' @inherit halfnorm.like return seealso
#' 
#' @details Rdistance's `oneStep` likelihood is a mixture of two 
#' non-overlapping uniform distributions. The 'oneStep' density function
#' is  
#' \deqn{f(d|p, \theta) = \frac{p}{\theta}I(0 \leq d \leq \theta) + 
#'        \frac{1 - p}{w - \theta}I(\theta \le d \leq w),}{
#'        f(d|p,T) = (p/T)I(0<=d<=T) + ((1-p)/(w-T))I(T<d<=w),}
#' where \eqn{I(x)} is the indicator function for event \eqn{x}, 
#' and \eqn{w} is the nominal strip width (i.e., `w.hi - w.lo` in Rdistance). 
#' The unknown parameters to be estimated 
#' are \eqn{\theta}{T} and \eqn{p}{p}
#' (\eqn{w} is fixed - given by the user). 
#'  
#' Covariates influence values of \eqn{\theta}{T} 
#' via a log link function, i.e., \eqn{\theta = e^{x'b}}{T = exp(x'b)},
#' where \eqn{x} is the vector of covariate values 
#' associated with distance \eqn{d}, and \eqn{b}
#' is the vector of estimated coefficients. 
#' 
#' 
#' 
#' 
#' @references  
#' Peter F. Craigmile & D.M. Tirrerington (1997) "Parameter estimation for 
#' finite mixtures of uniform distributions", 
#' Communications in Statistics - Theory and Methods, 26:8, 1981-1995, 
#' DOI: 10.1080/03610929708832026
#' 
#'  A. Hussein & J. Liu (2009) "Parametric estimation of mixtures of two 
#'  uniform distributions", Journal of Statistical Computation and Simulation, 
#'  79:4, 395-410, DOI:10.1080/00949650701810406
#'  
#' @examples
#' 
#' # Fit oneStep to simulated data
#' whi <- 250
#' T <- 100  # true threshold
#' p <- 0.85
#' n <- 200 
#' x <- c( runif(n*p, min=0, max=T), runif(n*(1-p), min=T, max=whi))
#' x <- setUnits(x, "m")
#' tranID <- sample(rep(1:10, each=n/10), replace=FALSE)
#' detectDf <- data.frame(transect = tranID, dist = x)
#' siteDf <- data.frame(transect = 1:10
#'                , length = rep(setUnits(10,"m"), 10))
#' distDf <- RdistDf(siteDf, detectDf)
#' 
#' # Estimation
#' fit <- dfuncEstim(distDf
#'                  , formula = dist ~ 1
#'                  , likelihood = "oneStep"
#'                  , w.hi = setUnits(whi, "m")
#'                  )
#' plot(fit)
#' thetaHat <- exp(coef(fit)[1]) 
#' pHat <- coef(fit)[2]
#' c(thetaHat, pHat) # should be close to c(100,0.85)
#'
#' summary(abundEstim(fit, ci=NULL)) 
#' 
#' @export
oneStep.like <- function(a
                , dist
                , covars 
                , w.hi = NULL) {
  
  if(length(dim(dist)) >= 2 && dim(dist)[2] != 1 ){ 
    stop(paste("Argument 'dist' must be a vector or single-column matrix.",
               "Found array with", length(dim(dist)), "dimensions."))
  }
  q <- nCovars(covars)
  if(is.matrix(a)){
    beta <- a[,1:q, drop = FALSE]  # k X q
    p <- a[1, q+1, drop = TRUE]     # 1 X 1
  } else {
    beta <- matrix(a[1:q], nrow = 1) # 1 X q
    p <- a[q+1]     # 1 X 1
  }
  s <- covars %*% t(beta) # (nXq) %*% (qXk) = nXk
  theta <- exp(s)  # link function here
  
  # Dropping units of dist is safe b/c checked already
  # 'key' is unit-less
  dist <- dropUnits(dist)

  if(is.null(w.hi)){
    w.hi <- max(dist)  # no units b/c removed above
  } else {
    w.hi <- dropUnits(w.hi) # already checked units
  }

  # or, alternative dist <- matrix(dist,ncol=1) %*% matrix(1,1,length(dist))
  # p <- matrix(p, nrow = nrow(theta), ncol = ncol(theta))
  key <- (0 <= dist & dist <= theta) + 
         (((1-p) * theta) / ((w.hi - theta) * p)) * (theta < dist & dist <= w.hi)

  return( list(L.unscaled = key, 
               params = cbind(s, p)))

}

#' @title Likelihood parameter names
#' 
#' @description Returns names of the likelihood parameters. This is 
#' a helper function and is not
#' necessary for estimation.  It is nice to label some 
#' parameters with descriptive
#' names like "sigma" or "k", depending on the likelihood.
#' 
#' @param like.form A text string naming the form of the likelihood. 
#' An error is thrown if the likelihood is unknown.
#' 
#' @return A vector of parameter names for the likelihood
#' 
#' @export
#' 
likeParamNames <- function(like.form){
  
  switch(like.form
       , halfnorm = c("Sigma")
       , hazrate = c("Sigma", "k")
       , uniform = c("Threshold", "Knee")
       , negexp = c("Alpha")
       , oneStep = c("Theta", "p")
       , Gamma = c("Scale", "Shape")
       , triangle = c("Theta", "p")
       , huber = c("Theta1", "Theta2", "p")
       , {    
           stop(paste("Unknown likelihood. Found", like.form))
         }
  )
  
}

#' @title Check number of numeric integration intervals
#' 
#' @description
#' Check that number of integration intervals is odd and 
#' sufficiently large. Plus, compute and store the Simpson's Composite
#' rule coefficients. 
#' 
#' @param nEvalPts An integer to check.
#' 
#' @return The first element of nEvalPts is returned if it is acceptable.
#' If nEvalPts is not acceptable, an error is thrown. Simpson's composite
#' coefficients are store in options()
#' 
checkNEvalPts <- function(nEvalPts){
  
  if( length(nEvalPts) > 1){
    warning(paste0("Only first element of option 'Rdistance_intEvalPts' used."))  
    nEvalPts <- nEvalPts[1]
  }
  
  if( (nEvalPts %% 2) == 0 ) {
    stop(paste0("Option 'Rdistance_intEvalPts' must be odd for numerical integration. Found ", 
                nEvalPts
                , ". Reset with statement like 'options(Rdistance_intEvalPts = "
                , nEvalPts + 1
                , ")'"))
  }
  
  if( nEvalPts < 29 ){
    warning( paste0("Number of integration intervals is low. "
                    , "Found ", nEvalPts
                    , ". Recommend options('Rdistance_intEvalPts' = 101) or higher. "
                    , " Speed is not dramatically faster for low values "
                    , "but accuracy can be reduced."
                    ))
  }
  
  # Store Simpson coefficients in options() to speed calculations a little
  intCoefs <- simpsonCoefs( nEvalPts )
  options("Rdistance_intCoefs" = intCoefs)
  
  invisible(nEvalPts)
}

#' @title Type of transects
#' 
#' @description Return the type of transects represented in either a fitted 
#' distance function or Rdistance data frame. 
#' 
#' @param x Either an estimated distance function, output by 
#' `dfuncEstim`, or an Rdistance nested data frame, output by 
#' `RdistDf`. 
#' 
#' @details 
#' This function is a simple helper function.  If `x` is an
#' estimated distance object, it polls the `transType` attribute 
#' of `x`'s Rdistance nested data frame. 
#' If `x` is an Rdistance nested data frame, it 
#' polls the `transType` attribute. 
#' 
#' @return A scalar. Either 'line' if `x` contains  
#' continuous line-transect detections, or 'point' if `x` contains 
#' point-transects detections. If transect type has not been assigned, 
#' return is NULL.
#' 
#' @export
transectType <- function(x){
  if( inherits(x, "dfunc") ){
    return( attr(x$data, "transType" ))
  } else {
    return( attr(x, "transType") )
  }
}

#' @title dfuncEstim error messages
#' 
#' @description
#' Utility function to produce error messages suitable for `stop`
#' 
#' @param txt A text string describing the error. 
#' 
#' @param attri An attribute to report.
#' 
#' @return A string
#' 

# Non-exported
dfuncEstimErrMessage <- function(txt, attri){
  paste0( "Unknown "
          , txt
          , ". Set "
          , txt
          , " using 'RdistDf',"
          , " or with "
          , 'attr(data, "'
          , attri
          , '") <- "value".'
          , " See help(RdistDf) for list of values."
  )
} 

#' @title Sine expansion terms
#' 
#' @description 
#' Computes the sine expansion terms that modify the 
#' shape of distance likelihood functions. 
#'   
#' @inheritParams cosine.expansion
#' 
#' @details The sine expansion used here is:
#'   \itemize{
#'     \item **First term**: \deqn{h_1(x)=\sin(2\pi x)/2,}{h1(x) = sin(2*Pi*x)/2,}
#'     \item **Second term**: \deqn{h_2(x)=\sin(3\pi x)/2,}{h2(x) = sin(3*Pi*x)/2,}
#'     \item **Third term**: \deqn{h_3(x)=\sin(4\pi x)/2,}{h3(x) = sin(4*Pi*x)/2,}
#'     \item **Fourth term**: \deqn{h_4(x)=\sin(5\pi x)/2,}{h4(x) = sin(5*Pi*x)/2,}
#'     \item **Fifth term**: \deqn{h_5(x)=\sin(6\pi x)/2,}{h5(x) = sin(6*Pi*x)/2,}
#'   }
#'   The maximum number of expansion terms is 5.
#'   
#' The sine expansion frequency in Rdistance is pi. Each term is one pi more than 
#' the previous. The cosine expansion frequency in Rdistance is 2*pi. Consequently, 
#' the sine and cosine expansions fit different models. 
#'   
#' @inherit cosine.expansion return
#'    
#' @seealso [dfuncEstim()], 
#'          [cosine.expansion()]
#'   
#' @examples 
#' x <- matrix(seq(0, 1, length = 200), ncol = 1)
#' sin.expn <- sine.expansion(x, 5)
#' plot(range(x), range(sin.expn), type="n")
#' matlines(x, sin.expn[,1,1:5], col=rainbow(5), lty = 1)
#' 
#' @export

sine.expansion <- function(x, expansions){

  expansions <- as.integer(expansions)  
  
  if (expansions > 5){
      warning("Too many Sine polynomial expansion terms. Only 5 used.")
      expansions = 5
  }
  
  if( expansions < 1 ) stop( "Number of expansions must be >= 1" )
    
  expansion = array(dim = c(nrow(x), ncol(x), expansions))
  
  expansion[,,1] = sin(2*pi*x)
  
  # I realize I could do this in a for loop, but I think this is faster.    
  if(expansions >= 2){
    expansion[,,2] = sin(4*pi*x)
  }
  if(expansions >= 3){
    expansion[,,3] = sin(6*pi*x)
  }
  if(expansions >= 4){
    expansion[,,4]= sin(8*pi*x)
  }
  if(expansions >= 5){
    expansion[,,5] = sin(10*pi*x)
  }      

  # Sine expansion range = -.5 to .5
  expansion <- expansion / 2
  
  return(expansion)
            
}

#' @usage x \%m\% .
#'
#' @param . Placeholder for the fixed unit assignment operators. Ignored.
#' See Details.
#' 
#' @details
#' The fixed unit assignment operators are designed to behave somewhat like 
#' unary operators (i.e., 1 argument);
#' but, R does not allow 
#' user defined unary operators. 
#' Technically, the fixed unit assignment operators are instances of
#' R's user-defined infix 
#' operator, and as such they require two arguments. 
#' Their syntax must be 
#' \code{x \%<units>\% <something>}; but, the second argument is ignored
#' and '.' is suggested. See Examples.  
#' 
#' @returns 
#' For all the fixed unit assignment operators (i.e., \%<units>\%), 
#' argument x with the respective units assigned. 
#' 
#' 
#' @examples 
#' 
#' # For fixed unit assignment, 2nd argument does not matter
#' # All of the following are equivalent
#' 2 %m%.
#' 2 %m% x
#' 2 %m% 3
#' 2 %m% NULL
#' 2 %m% NA
#' 
#' # Conversion:
#' x <- 10 %#% "ft"
#' x %m%.
#' 
#' @exportPattern `%.+%`
#' 
#' @rdname unitHelpers
#' 
`%m%` <- function(x, . = NULL){
  units::set_units(x, "m")
}

#' @title Predict distance functions
#' 
#' @description
#' An internal prediction function to predict a distance 
#' function.  This version allows for matrix inputs and 
#' uses matrix operations, and is thus faster than earlier
#' looping versions.
#' 
#' @inheritParams predict.dfunc
#' 
#' @param params A matrix of distance function parameters. 
#' Rows are observations, columns contain the set of parameters
#' (canonical and expansion) for each observation. 
#' 
#' @param isSmooth Logical.  TRUE if the distance function 
#' is smoothed (and hence has no parameters).
#' 
#' @return A matrix of distance function values, of size 
#' length(distances) X nrow(params).  Each row of params
#' is associated with a column, i.e., a different distance 
#' function.  Distances are associated with rows, 
#' i.e., use matplot(d,return) to plot values on separate distance 
#' functions specified by rows of params.
#' 
#' @export
predDfuncs <- function(object
                     , params
                     , distances
                     , isSmooth
                       ){
  # DISTANCE function prediction ----

  if( is.null(distances) ){
    distances <- seq( object$w.lo
                      , object$w.hi
                      , length = getOption("Rdistance_intEvalPts"))
  } else {
    # check distances have units
    if( !inherits(distances, "units") ){
      stop("Distances must have measurement units.")
    }
    # Make sure input distances are converted properly b.c. likelihoods drop units.
    distances <- setUnits(distances, object$outputUnits)
  }
  
  if( isSmooth ){
    # unscaled distance function is already stored
    # no covariates.  
    # NEED TO REVISIT THIS
    y <- stats::approx( object$fit$x, object$fit$y, xout = distances )$y
    y <- matrix( y, nrow = 1) 
  } else {
    # After next apply, y is length(distances) x nrow(parms).  
    # Each row is a un-scaled distance function (f(x))
    # We already eval-ed covars, so new X is constant (1), "XIntOnly"
    
    like <- utils::getFromNamespace(paste0( object$likelihood, ".like"), "Rdistance")    
    d <- distances - object$w.lo

    # don't need covars since params are always computed
    XIntOnly <- matrix(1, nrow = length(d), ncol = 1)
    
    y <- like(
             a = params
           , dist = d
           , covars = XIntOnly
           , w.hi = object$w.hi
    )
    y <- y$L.unscaled # (nXk) = (length(d) X nrow(params))
    
    if(object$expansions > 0){
      W <- expandW(ml = object
                   , params = params
                   , k = nrow(params)  
      )
      
      # Dimensions: n = length(d); k = length(W) = nrow(params)
      # The following call to expansionTerms returns matrix size n X k 
      # for all likelihoods

      exp.terms <- Rdistance::expansionTerms(a = params
                                             , d = d 
                                             , series = object$series
                                             , nexp = object$expansions
                                             , w = W)

      y <- y * exp.terms # (kXq) * (kXn) OR (kXq) * (kX1); where q = nrow(params)
                         # for oneStep, q = n
      
      # without monotonicity restraints, function can go negative, 
      # especially in a gap between datapoints. Don't want this in distance
      # sampling and screws up the convergence. In future, could
      # apply monotonicity constraint here.
      y[ !is.na(y) & (y <= 0) ] <- getOption("Rdistance_zero")
      
    }
  }
  
  # At this point, we have unscaled distance functions in columns of y.
  
  # SCALE distance functions ----
  
  if( object$likelihood == "Gamma" & !isSmooth ){
    # Gamma case. Only x.scl allowed is 'max'.
    x0 <- GammaModes( params )
    x0 <- setUnits(x0, object$outputUnits)
    
  } else if( is.character(object$x.scl) && (object$x.scl == "max") ){
    # Technically, we could do this. Just like Gamma, we could 
    # estimate x.max for every distance function in y
    # something like...
    # full.df <- predict.dfunc(x, type="dfuncs", newdata = newdata)
    # maxPos <- apply(full.df, 2, which.max)
    # x0 <- attr(full.df, "distances")[maxPos]
    
    stop(paste0("x.scl = 'max' not currently implemented for non-Gamma likelihoods."))
  } else if( !isSmooth ){
    # Case:  All likelihoods except Gamma
    x0 <- object$x.scl
  } 
  
  # Now that we know x0 (either a scaler or a vector), compute f(x0)
  
  if( !isSmooth ){
    
    # Likelihood functions return g(x). This is what we want; 
    # except, if there are expansions, g(0) != 1

    d <- x0 - object$w.lo
    XIntOnly <- matrix(1, nrow = length(d), ncol = 1)
    
    # here, length(x0) == length(d) could be >1 (e.g., for Gamma)
    f.at.x0 <- like(a = params
                  , dist = d
                  , covars = XIntOnly
                  , w.hi = object$w.hi
                  )$L.unscaled  # should be length(d) X k
    
    if( length(d) > 1 ){
      # Gamma case: b/c max could be a different x values for every obs
      f.at.x0 <- diag(f.at.x0)
    }
    
    if(object$expansions > 0){
      exp.terms <- Rdistance::expansionTerms(a = params
                                             , d = d 
                                             , series = object$series
                                             , nexp = object$expansions
                                             , w = W)
      if( length(d) > 1 ){
        # Gamma case: b/c max could be a different x values for every obs
        exp.terms <- diag(exp.terms)
      }
      
      f.at.x0 <- f.at.x0 * exp.terms # (1Xk) * (1) OR (1Xk) * (1Xk) [if Gamma]
      f.at.x0[ !is.na(f.at.x0) & (f.at.x0 <= 0) ] <- getOption("Rdistance_zero")
    }
    
    
  } else {
    # Case: smoothed distance function.  No covars. 
    # This is so simple we handle everything here
    # I THINK WE JUST NEED TO INTEGRATE UNDER Y, SO I DON'T THINK 
    # WE NEED THIS SPECIAL CASE, BUT CHECK.
    x0 <- object$x.scl
    f.at.x0 <- stats::approx(distances, y, xout = x0)$y 
  }
  
  # ncol(f.at.x0) must equal ncol(y), or error here, which is what we want;
  # hence length(f.at.x0) not ncol(y)
  
  f.at.x0 <- matrix(f.at.x0, nrow(y), length(f.at.x0), byrow = TRUE)
  
  y <- object$g.x.scl * (y / f.at.x0)


  # Did you know that 'scaler' is ESW?  At least for lines. 
  # Makes sense. 1/f(0) = ESW in 
  # the old formulas.
  # For smoothed distance functions, we extended the range of dist by approx 2, and scaler
  # is twice too big. i.e., esw for smu's is approx scaler / 2.
  
  # y <- y * scaler  # length(scalar) == nrow(y), so this works right
  
  attr(y, "distances") <- distances
  attr(y, "x0") <- x0
  attr(y, "g.x.scl") <- object$g.x.scl
  # if(isSmooth){
  #   scaler <- scaler / 2
  # }
  # attr(y, "scaler") <- scaler
  
  return( y )  
}

#' @title Tests for smoothed distance functions
#' 
#' @description
#' Determines whether a distance function is a non-parametric
#' smooth or classic parameterized function.
#' 
#' @inheritParams predict.dfunc
#' 
#' @return TRUE if the model frame or fitted distance function 
#' arises from a non-parametric density smoother. FALSE if the 
#' model frame or distance 
#' function is a parameterized function. 
#' 
#' @export
#' 
is.smoothed <- function(object){
  object$likelihood == "smu"
}
  

#' @title Integrate Gamma line surveys
#' 
#' @description
#' Compute integral of the Gamma distance function for 
#' line-transect surveys.
#' 
#' @inheritParams integrateOneStepPoints
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @details 
#' #' Returned integrals are
#' \deqn{\int_0^{w} \left(\frac{x}{m}\right)^{\alpha -1}  e^{-(x - m)/\sigma_i}dx,}{
#' Integral( (x/m)^(a-1) exp(-(x-m)/s_i) ),}
#' where \eqn{w = w.hi - w.lo}, \eqn{\sigma_i}{s_i} is the i-th estimated scale 
#' parameter for the Gamma distance function, and \eqn{m} is the mode of Gamma
#' (i.e., \eqn{(\alpha - 1)\sigma_i}{(a-1)s_i}. 
#' Rdistance computes the integral using R's base function 
#' \code{pgamma()}, which for all intents and purposes is exact. 
#' See also \code{\link{Gamma.like}}.
#' 
#' @seealso \code{\link{integrateNumeric}}; \code{\link{integrateNegexpLines}}; 
#' \code{\link{integrateOneStepLines}} 
#' 
#' @examples
#' 
#' # Fake distance function object w/ minimum inputs for integration
#' d <- rep(1,4) %m%. # Only units needed, not values
#' obs <- factor(rep(c("obs1", "obs2"), 2))
#' beta <- c(4.0, -0.5, 1.5) # {'Intercept', b_1, shape}
#' w.hi <- 125
#' w.lo <- 20
#' ml <- list(
#'     mf = model.frame(d ~ obs)
#'   , par = beta 
#'   , likelihood = "Gamma"
#'   , w.lo = w.lo %#% "m"
#'   , w.hi = w.hi %#% "m"
#'   , expansions = 0
#' )
#' class(ml) <- "dfunc"
#' integrateGammaLines(ml)
#' 
#' # Check: Integral of Gamma density from 0 to w.hi-w.lo
#' b <- exp(c(beta[1], beta[1] + beta[2]))
#' B <- Rdistance::GammaReparam(shp = beta[3], scl = b)
#' m <- (B$shp - 1)*B$scl
#' f.at.m <- dgamma(m, shape = B$shp, scale = B$scl)
#' intgral <- pgamma(q = w.hi - w.lo, shape = B$shp, scale = B$scl) / f.at.m
#' intgral
#' 
#' 
#' @export
#' 
integrateGammaLines <- function(object
                                , newdata = NULL
                                , w.lo = NULL
                                , w.hi = NULL
                                , Units = NULL
){
  
  if( inherits(object, "dfunc") ){
    Units <- object$outputUnits
    w.lo <- object$w.lo
    w.hi <- object$w.hi
    object <- stats::predict(object = object
                             , newdata = newdata
                             , type = "parameters"
    )
  } 
  
  w <- dropUnits(w.hi) - dropUnits(w.lo)
  
  dgamPars <- GammaReparam(scl = object[,1]
                           , shp = object[,2])
  
  # Note: we use pgamma and dgamma to avoid taking gamma(<big number>)
  # which is Inf. If gamma(dgamPars$shp) is < Inf, the integral is:
  # pgamma(w, shp, scale = scl) * 
  #       scl^shp * gamma(shp) / 
  #       ((shp - 1)*scl)^(shp - 1) * exp(1 - shp)
  
  outArea <-  stats::pgamma(q = w
                            , shape = dgamPars$shp
                            , scale = dgamPars$scl
  ) 
  
  
  # Raw gamma is scaled to max = 1
  m <- (dgamPars$shp - 1)*dgamPars$scl # mode
  f.at.m <- stats::dgamma( m, shape=dgamPars$shp, scale=dgamPars$scl )
  outArea <- outArea / f.at.m
  
  outArea <- setUnits(outArea, Units)
  
  outArea 
  
}

#' @title Numeric Integration
#' 
#' @description
#' Numerically integrate under a distance function. 
#' 
#' @inheritParams integrateOneStepPoints
#' @inheritParams dE.single
#' 
#' @inheritSection integrateOneStepPoints Note
#'  
#' @inherit integrateOneStepPoints return
#' 
#' @param isPoints Boolean. TRUE if integration is for point surveys.  
#' FALSE for line-transect surveys. Line-transect surveys integrate 
#' under the distance function, g(x), while point surveys integrate under 
#' the distance function times distances, xg(x). 
#' 
#' @section Numeric Integration: 
#' Rdistance uses Simpson's composite 1/3 rule to numerically 
#' integrate distance functions from 0 to the maximum sighting distance
#' (\code{w.hi - w.lo}). The number of points evaluated 
#' during numerical integration is controlled by 
#' \code{options(Rdistance_intEvalPts)} (default 101).
#' Option 'Rdistance_intEvalPts' must be odd because Simpson's rule
#' requires an even number of intervals. 
#' Lower values of 'Rdistance_intEvalPts' increase calculation speeds; 
#' but, decrease accuracy.
#' 'Rdistance_intEvalPts' must be >= 5.  A warning is thrown if 
#' 'Rdistance_intEvalPts' < 29. Empirical tests by the author 
#' suggest 'Rdistance_intEvalPts' values >= 30 are accurate 
#' to several decimal points for smooth distance functions
#' (e.g., hazrate, halfnorm, negexp)
#' and that all 'Rdistance_intEvalPts' >= 101 produce 
#' identical results if the distance function is not smooth. 
#'   
#' \emph{Details}: Let \code{n} = \code{options(Rdistance_intEvalPts)}.
#' Evaluate the distance function at \code{n} equal-spaced 
#' locations \{f(x0), f(x1), ..., f(xn)\} between 0 and (w.hi - w.lo). 
#' Simpson's composite approximation to the area under the curve is
#' \deqn{\frac{1}{3}h(f(x_0) + 4f(x_1) + 2f(x_2) + 
#'      4f(x_3) + 2f(x_4) + ... + 2f(x_{n-2}) + 
#'      4f(x_{n-1}) + f(x_{n}))}{(1/3)h(f(x0) + 4f(x1) + 2f(x2) + 
#'      4f(x3) + 2f(x4) + ... + 2f(x(n-2)) + 4f(x(n-1)) + f(xn))}
#' where \eqn{h} is the interval size (w.hi - w.lo) / n.
#' 
#' Physical units on the return values
#' are the original (linear) units if \code{object} contains line-transect data
#' (e.g., [m]), or square of the original units if \code{object} contains
#' point-transect data (e.g., [m^2]). Point-transect units are squared because
#' the likelihood is the product of the detection function (which is unitless) 
#' and distances (which have units).  
#' 
#' 
#' @examples
#' 
#' # A halfnorm distance function 
#' fit <- dfuncEstim(sparrowDf, dist~1, likelihood = "halfnorm")
#' 
#' exact <- integrateHalfnormLines(fit)[1,] # exact area
#' apprx <- integrateNumeric(fit)[1]  # Numeric approx
#' pd <- options(digits = 20)
#' cbind(exact, apprx)
#' absDiff <- abs(apprx - exact) 
#' absDiff
#' options(pd)
#' 
#' # halfnorm approx good to this number of digits
#' round(log10(absDiff),1)  
#'
#' @export
#' 
integrateNumeric <- function(object
                           , newdata = NULL
                           , w.lo = NULL
                           , w.hi = NULL
                           , Units = NULL
                           , expansions = NULL
                           , series = NULL
                           , isPoints = NULL
                           , likelihood = NULL
                             ){

  if( inherits(object, "dfunc") ){
    w.lo <- object$w.lo
    w.hi <- object$w.hi 
    Units <- object$outputUnits 
    expansions <- object$expansions
    series <- object$series
    isPoints <- is.points(object)
    likelihood <- object$likelihood 
    
    # Now convert object to parameters
    object <- stats::predict(object = object
                             , newdata = newdata
                             , type = "parameters"
    )
    object[,1] <- log(object[,1]) # predict returns real param, f.like needs link value
    # If object has expansions, their coefficients come back from predict
  } 

  # This check of nInts and intCoefs slows things down, but integrateNumeric 
  # gets called from ESW/EDR and NLL. User could change nInts or intCoeffs 
  # in between calls to ESW/EDR and NLL, so we have not choice but to check.
  nInts <- getOption("Rdistance_intEvalPts") # already checked it's odd, in parseModel::checkNevalPts
  intCoefs <- getOption("Rdistance_intCoefs") # Next, check intCoefs are correct
  if( nInts != length(intCoefs) || sum(intCoefs) != (3*(nInts-1)) ){
    intCoefs <- simpsonCoefs( nInts ) # oddity checked
    options("Rdistance_intCoefs" = intCoefs )
  }
  zero <- setUnits(0, Units)

  d <- seq(zero, w.hi - w.lo, length=nInts) 
  dx <- d[2] - d[1]  # or (w.hi - w.lo) / (nInts-1); could do diff(dx) if unequal intervals

  
  # don't need covars since params are always computed
  XIntOnly <- matrix(1, nrow = length(d), ncol = 1) 

  f.like <- utils::getFromNamespace(paste0( likelihood, ".like"), "Rdistance")    

  likeObj <- f.like(
      a = object
    , dist = d
    , covars = XIntOnly
    , w.hi = w.hi - w.lo # I don't think we need w.hi in f.like here
  )
  
  y <- likeObj$L.unscaled # (nInts x n) = (length(d) X nrow(object))

  if( expansions > 0 ){
    # a bit awkward here: must construct a 'ml' object
    ml <- list(
        likelihood = likelihood
      , outputUnits = Units
      , w.lo = w.lo
      , w.hi = w.hi
    )
    W <- expandW(ml = ml
               , params = likeObj$params
               , k = ncol(y)
               )

    exp.terms <- Rdistance::expansionTerms(a = object
                                         , d = d
                                         , series = series
                                         , nexp = expansions
                                         , w = W)
    y <- y * exp.terms
    y[ !is.na(y) & (y <= 0) ] <- getOption("Rdistance_zero")

  }

  if(isPoints){
    y <- d * y  # element-wise
  }
  
  outArea <- intCoefs * y  # (n vector) * (n X k)
  outArea <- colSums(outArea) * dx / 3

  outArea 
  
}

#' @title Effective sampling distances
#' 
#' @description Computes Effective Strip Width (ESW) for line-transect detection
#'   functions, or the analogous Effective Detection Radius (EDR) for point-transect
#'   detection functions.
#'
#' @param newdata A data frame containing new values for 
#' covariates at which either
#' ESW's or EDR's will be computed. If NULL and 
#'   `object` contains covariates, the  
#'   covariates stored in
#'   `object` are used (like [predict.lm()]).
#'   If not NULL, covariate values in `newdata`
#'   are used. 
#'   See **Value** section for more information. 
#'  
#' @inheritParams predict.dfunc 
#'   
#' @details Serves as a wrapper for 
#' [ESW()] and [EDR()].
#' 
#' Effective distances are areas under scaled 
#' distance functions (i.e., area under g(x)). Areas are 
#' exact for functions whose integral is known (e.g., negexp). 
#' Numeric integration is used for all others. 
#' 
#' @return If `newdata` is present, the returned value is 
#' a vector of effective sampling distances associated with 
#' covariate values in `newdata`. Length of return 
#' in this case is the number of rows in `newdata`. 
#' If `newdata` is NULL, the returned value is a vector 
#' of effective sampling distances associated with covariate 
#' values in `object`. Length of return in this case 
#' is the number of detected groups.  The returned vector 
#' has measurement units, i.e., `object$outputUnits`.
#' 
#'      
#' @seealso [dfuncEstim()],
#'   [ESW()],
#'   [EDR()], 
#'   [integrateNumeric()],
#'   [integrateNegexpLines()]
#' 
#' @export

effectiveDistance <- function(object, newdata = NULL){
  
  # call ESW for line transects and EDR for point transects

  if (is.points(object)) {
    EDR(object, newdata)
  } else {
    ESW(object, newdata)
  }

}

#' @title Brewer's Sparrow detection function
#' 
#' @name sparrowDfuncObserver 
#' 
#' @description 
#' Pre-estimated Brewer's sparrow detection function that 
#' includes an 'observer' effect.  Included to speed up 
#' example execution times. See 'Examples'.
#' 
#' @docType data
#' 
#' @format An estimated distance function object with 
#' class 'dfunc'. See 'Value' section of 
#' \code{\link{dfuncEstim}} for 
#' description of components. 
#' 
#' @seealso \code{\link{sparrowSiteData}} and 
#' \code{\link{sparrowDetectionData}} for description of the data
#' 
#' 
#' @examples
#' \dontrun{
#' # the following code generated 'sparrowDfuncObserver'
#' data(sparrowDf)
#' sparrowDfuncObserver <- sparrowDf |> 
#'             dfuncEstim(formula = dist ~ observer
#'                      , likelihood = "hazrate")
#' }
#' 
#' 
NULL

#' @title AIC-related fit statistics for detection functions
#' 
#' @description Computes AICc, AIC, or BIC for estimated distance functions.
#' 
#' @inheritParams predict.dfunc
#' 
#' @param criterion String specifying the criterion to compute. Either 
#'   "AICc", "AIC", or "BIC".
#'   
#' @details Regular Akaike's information criterion 
#'   (<https://en.wikipedia.org/wiki/Akaike_information_criterion>) (\eqn{AIC}) is 
#'   \deqn{AIC = LL + 2p,}{AIC = (LL) + 2p,}
#'   where \eqn{LL} is the maximized value of the log likelihood 
#'   (the minimized value of the negative log 
#'   likelihood) and \eqn{p} is the 
#'   number of coefficients estimated in the detection function.  For 
#'   `dfunc` objects, \eqn{AIC} = `obj$loglik + 2*length(coef(obj))`.  
#'   
#'   A correction 
#'   for small sample size, \eqn{AIC_c}{AICc}, is 
#'   \deqn{AIC_c = LL + 2p + \frac{2p(p+1)}{n-p-1},}{AIC_c = LL + 2p + (2p(p+1))/(n-p-1),} 
#'   where \eqn{n} is sample 
#'   size or number of detected groups for distance analyses.  By default, this function 
#'   computes \eqn{AIC_c}{AICc}.   \eqn{AIC_c}{AICc} converges quickly to \eqn{AIC} 
#'   as \eqn{n} increases.
#'   
#'   The Bayesian Information Criterion (BIC) is
#'   \deqn{BIC = LL + log(n)p,}{BIC = (LL) + log(n)p}. 
#'   
#' @return A scalar, the requested fit statistic for `object`.
#' 
#' @references Burnham, K. P., and D. R. Anderson, 2002. 
#' *Model selection and multi-model inference: 
#'    A practical information-theoretic approach, Second ed.* 
#'    Springer-Verlag. ISBN 0-387-95364-7.
#'   
#'   McQuarrie, A. D. R., and Tsai, C.-L., 1998. *Regression and 
#'   time series model selection.* 
#'   World Scientific. ISBN 981023242X
#'   
#' @seealso [coef()], [dfuncEstim()]
#' 
#' @examples 
#' data(sparrowDf)
#' dfunc <- sparrowDf |> dfuncEstim(dist~1)
#'   
#' # Fit statistics
#' AIC(dfunc)  # AICc
#' AIC(dfunc, criterion="AIC")  # AIC
#' AIC(dfunc, criterion="BIC")  # BIC
#' 
#' @export

AIC.dfunc=function (object, ..., criterion="AICc") 
{
  if( criterion == "AIC"){
    k <- 2
    n <- Inf
  } else if( criterion == "BIC"){
    k <- log(nrow(object$mf))
    n <- Inf
  } else {
    k <- 2
    n <- nrow(object$mf)
    criterion <- "AICc"
  }
    
  p <- length(stats::coef(object))
  ans <- -2*object$loglik + (k*p) + (k * p * (p + 1))/(n - p - 1)
  attr(ans, "criterion") <- criterion
  ans
}

#' @title 'optim' optimizer
#' 
#' @description
#' Call R native function 'optim' to perform optimization. 
#' 
#' @inheritParams mlEstimates
#' 
#' @inherit Nlminb return
#' 
# Do Not export

Optim <- function(ml, strt.lims){
  
  contRl <- list(trace = getOption("Rdistance_trace")
                 , maxit = getOption("Rdistance_maxIters")
                 , factr = getOption("Rdistance_likeTol")
                 , pgtol = getOption("Rdistance_likeTol")
  )
  
  verboseLevel <- getOption("Rdistance_verbosity")
  optimMeth <- ml$optimizer
  optimSubMeth <- sub("^.+_", "", optimMeth)
  
  if( verboseLevel >= 1 ){
    cat(colorize(paste(
        optimMeth
      , "maximization"
      , "----\n"), col = "red"))
  }

  if( optimSubMeth %in% c("L-BFGS-B") ){
    # can use limits, else no
    fit <- stats::optim(
        par = strt.lims$start
      , fn = nLL
      , lower = dropUnits(strt.lims$low) # safe
      , upper = dropUnits(strt.lims$high)
      , hessian = TRUE
      , control = contRl
      , method = optimSubMeth
      , ml = ml
      , verbosity = verboseLevel
    )
  } else {
    fit <- stats::optim(
        par = strt.lims$start
      , fn = nLL
      , hessian = TRUE
      , control = contRl
      , method = optimSubMeth
      , ml = ml
      , verbosity = verboseLevel
    )
    
  }
  names(fit$par) <- strt.lims$names
  if( ml$asymptoticSE ){
    fit$varcovar <- Rdistance::varcovarEstim(fit, ml)
  } else {
    fit$varcovar <- NULL
  }
  fit$hessian <- NULL  # erase hessian from fit object

  # final few things ----
  fit$limits <- strt.lims[c("low", "high")]

  # Flip over objective: object$logLike is true logLike, -LL was minimized
  names(fit)[names(fit) == "value"] <- "loglik"
  fit$loglik <- -fit$loglik  
  
  fit$evaluations <- fit$counts[1]
  fit$counts <- NULL
  
  names(fit)[names(fit) == "feval"] <- "evaluations"
  names(fit)[names(fit) == "niter"] <- "iterations"
  
  fit
} 

#' @title Brewer's Sparrow detection data 
#' 
#' @name sparrowDetectionData
#' 
#' @description 
#' Detection data from line transect surveys for 
#' Brewer's sparrow on 72 transects 
#' located on a 4105 km^2 study 
#' area in central Wyoming. Data were collected by 
#' Dr. Jason Carlisle of the Wyoming Cooperative Fish & Wildlife
#' Research Unit in 2012. Each transect was 500 meters long.
#' 
#' @docType data
#' 
#' @format A data.frame containing 356 rows and 5 columns.  Each row represents
#' a detected group of sparrows.  Column descriptions: 
#' \enumerate{ 
#'   \item \code{siteID}: Factor (72 levels), the site or transect where the detection
#'   was made.  
#'   \item \code{groupsize}: Number, the number of individuals within
#'   the detected group.  
#'   \item \code{sightdist}: Number, distance (m) from the observer to the detected group.  
#'   \item \code{sightangle}: Number, the angle (degrees) from the transect 
#'   line to the detected group.  
#'   \item \code{dist}: Number, the perpendicular, off-transect distance (m) from the
#'   transect to the detected group.  This is the distance used in analysis.
#'   Calculated using \code{\link{perpDists}}.  }
#' 
#' @seealso \code{\link{sparrowSiteData}}
#' 
#' @source The Brewer's sparrow data are a subset of the data collected 
#' by Jason Carlisle and various field technicians for his Ph.D. from the 
#' Department of Ecology, University of Wyoming, in 2017.  This portion of 
#' Jason's work was funded by the Wyoming Game and Fish Department through agreements 
#' with the University of Wyoming's  Cooperative Fish & Wildlife
#' Research Unit (2012). 
#' 
#' @references Carlisle, J.D. 2017. The effect of sage-grouse conservation on
#' wildlife species of concern: implications for the umbrella species concept.
#' Dissertation. University of Wyoming, Laramie, Wyoming, USA.
#' 
#' Carlisle, J. D., and A. D. Chalfoun. 2020. The abundance of Greater 
#' Sage-Grouse as a proxy for the abundance of sagebrush-associated songbirds 
#' in Wyoming, USA. \emph{Avian Conservation and Ecology} 15(2):16. 
#' \doi{10.5751/ACE-01702-150216}
#' 
NULL

#' @title Half-normal distance function
#' 
#' @description Evaluate the half-normal distance function, 
#' for sighting distances, potentially including covariates
#' and expansion terms
#'
#'
#' @param a A vector or matrix of covariate 
#' and expansion term 
#' coefficients. If matrix, dimension is 
#' \emph{k} X \emph{p}, where 
#' \emph{k} = \code{nrow(a)}) is the number of coefficient
#' vectors to evaluate (cases) and \emph{p} 
#' = \code{ncol(a)})
#' is the number of covariate and expansion 
#' coefficients in the likelihood (i.e., rows are 
#' cases and columns are covariates). If \code{a} is a 
#' dimensionless vector, it is interpreted as a 
#' single row with \emph{k} = 1. 
#' Covariate coefficients in \code{a} are the first 
#' \emph{q} values (\emph{q} <= \emph{p}), and must 
#' be on a log scale.
#' 
#' @param dist A numeric vector of length \emph{n} or 
#' a single-column matrix (dimension \emph{n}X1) containing 
#' detection distances at which to evaluate the likelihood.
#' 
#' @param covars A numeric vector of length \emph{q} or a
#' matrix of dimension \emph{n}X\emph{q} containing 
#' covariate values 
#' associated with distances in argument \code{dist}. 
#' 
#' @param w.hi A numeric scalar containing maximum 
#' distance. The right-hand cutoff or upper limit. 
#' Ignored by some likelihoods (such as halfnorm, 
#' negexp, and hazrate), but is a fixed parameter
#' in other likelihoods (such as oneStep 
#' and uniform).
#' 
#' @details The half-normal distance function is 
#' \deqn{f(d|\sigma) = \exp(-\frac{d^2}{2\sigma^2})}{f(d|s) = exp(-d^2 / (2*s^2))}
#' where \eqn{\sigma = exp(x'a)}{s = exp(x'a)}, \eqn{x} is a vector of 
#' covariate values associated with distance \eqn{d} 
#' (i.e., a row of \code{covars}), and 
#' \eqn{a} is a vector of the first $q$ (=\code{ncol(covars)}) 
#' values in argument \code{a}.
#' 
#' Some authors parameterize the halfnorm without 
#' the "2" in the denominator of the exponent. 
#' \code{Rdistance} includes 
#' "2" in this denominator to make 
#' quantiles of the half normal agree with 
#' the standard normal. This means that half-normal 
#' coefficients in 
#' Rdistance (i.e., \eqn{\sigma = exp(x'a)}{s = exp(x'a)}) can be 
#' interpreted as normal standard errors.  
#' Approximately 95\% of distances should 
#' occur between 0 and 2\eqn{\sigma}{s}.
#' 
#' @return A list containing the following two components:
#' \itemize{
#'   \item \bold{L.unscaled}: A matrix of size 
#'    \emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
#'    cases = \code{nrow(a)})
#'    containing likelihood values evaluated at 
#'    distances in \code{dist}.
#'    Each row is associated with 
#'    a single distance, and each column is associated with 
#'    a single case (row of \code{a}).  Values in this matrix are  
#'    the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
#'    These values are "unscaled" likelihood values; they must be 
#'    scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi  
#'    to form proper likelihood values. 
#'    
#'  \item \bold{params}: A \emph{n}X\emph{b}X\emph{k}  array 
#'  of the likelihood's (canonical) parameters in link space (i.e., on 
#'  log scale; \emph{b} = number of canonical parameters in 
#'  the likelihood; \emph{k} = number of cases). 
#'  Rows correspond to distances in \code{dist}. Columns 
#'  correspond to parameters (columns of \code{a}), 
#'  and pages correspond to cases (rows of \code{a}). 
#' }
#'  
#' @seealso \code{\link{dfuncEstim}}, 
#'          \code{\link{abundEstim}},
#'          other \code{<likelihood>.like} functions
#'          
#' @examples  
#' d <- seq(0, 100, length=100)
#' covs <- matrix(1,length(d),1)
#' halfnorm.like(log(20), d, covs)
#' 
#' plot(d, halfnorm.like(log(20), d, covs)$L.unscaled, type="l", col="red")
#' lines(d, halfnorm.like(log(40), d, covs)$L.unscaled, col="blue")
#' 
#' # Evaluate 3 functions at once using matrix of coefficients:
#' # sigma ~ 20, 30, 40
#' coefs <- matrix(log(c(7.39,7.33, 4.48,44.80, 2.72,216.54))
#'                , byrow = TRUE
#'                , ncol=2) # (3 coef vectors)X(2 covars)
#' covs <- matrix(c(rep(1,length(d))
#'                , rep(0.5,length(d)))
#'                , nrow = length(d)) # 100 X 2
#' L <- halfnorm.like( coefs, d, covs ) 
#' L$L.unscaled # 100 X (3 coef vectors)
#' L$params     # 100 X (3 coef vectors); ~ log(c(20,30,40))
#' matplot(d, L$L.unscaled, type="l")
#' 
#' @export

halfnorm.like <- function(a
                          , dist
                          , covars
                          , w.hi = NULL
                          ){

  # w.hi is ignored, but needed for compatability in other likelihoods
  # cat(paste("In", crayon::red("halfnorm.like"), "\n"))
  
  if(length(dim(dist)) >= 2 && dim(dist)[2] != 1 ){ 
    stop(paste("Argument 'dist' must be a vector or single-column matrix.",
            "Found array with", length(dim(dist)), "dimensions."))
  }
  
  q <- nCovars(covars)
  if(is.matrix(a)){
    # cat(crayon::red("A is matrix\n"))
    beta <- a[,1:q, drop = FALSE]  # k X q
  } else {
    beta <- matrix(a[1:q], nrow = 1) # 1 X q
  }
  s <- covars %*% t(beta) # (nXq) %*% (qXk) = nXk
  sigma <- exp(s)  # link function here

  dist <- dropUnits(dist)
  key <- drop(-(dist*dist))  # n-vector; use drop() in case dist is matrix
  key <- key / (2*sigma*sigma)  # (n vector) / nXk 
  key <- exp(key)  # exp of density function here, not link.
  
  # Rules for likelihoods:
  #  1. 'key' must be a matrix (not vector), dim(key) should = (length(dist), nrow(a))
  #  2. 'key' must be unscaled. It should not sum to 1. Max should be 1. 
  #     i.e., this is g(x) [not f(x)], or else ESW calculations are wrong.
  #  3. 'key' cannot have units.

  return( list(L.unscaled = key, # MUST be a MATRIX (not vector)
               params = s))  # return params on log scale
    
  
}

#' @title Summary method for Rdistance data frames 
#' 
#' @description Summary method for distance sampling data frames. 
#' \code{Rdistance} data frames are rowwise tibbles. This routine is a 
#' replacement summary method for rowwise_df's that  
#' provides useful distance sampling descriptive statistics. 
#' 
#' @param object An \code{RdistDf} data frame. 
#' 
#' @inheritParams dE.single
#' 
#' @param ...  Other arguments for summary methods.
#' 
#' @return If \code{object} is an \code{RdistDf}, a data frame 
#' containing summary statistics relevant to distance sampling is returned
#' invisibly.
#' If \code{formula} is not specified, the number of distance observations
#' and target detections is not returned because the distances, group sizes,
#' and covariates are not known. 
#' If \code{object} is not an Rdistance data frame, return is the result of 
#' the next summary method.
#' 
#' @examples
#' data(thrasherDf)
#' summary(thrasherDf)
#' summary(thrasherDf
#'         , formula = dist ~ groupsize(groupsize)
#'         , w.hi = setUnits(100,"m")
#'         )
#' 
#' @export
#' 
summary.rowwise_df <- function(object
                             , formula = NULL
                             , w.lo = 0
                             , w.hi = NULL
                             , ...){
  
  if( is.RdistDf(object) ){
    tranType <- Rdistance::transectType(object)
    effVec <- Rdistance::effort(object)
    nTrans <- sum(!is.na(effVec))
    lenTrans <- sum(effVec, na.rm = TRUE)
    if( !is.null(formula) ){
      mf <- Rdistance::parseModel(data = object
                               , formula = formula
                               , w.lo = w.lo
                               , w.hi = w.hi
                               )
      groupSz <- Rdistance::groupSizes(mf)
      dists <- Rdistance::distances(mf)
      
      nDistances <- sum( !is.na(dists) )
      nTargets <- sum(groupSz, na.rm = TRUE)
      nGroups <- sum( !is.na(groupSz) )
    } else {
      nDistances <- NA
      nTargets <- NA
      nGroups <- NA
    }
    
    cat(paste("Transect type: "))
    if( Rdistance::is.points(object) ){
      tType <- "point"
      cat(colorize(tType))
      lenMess <- paste(lenTrans, "[points]")
    } else {
      tType <- "line"
      cat(colorize(tType))
      lenMess <- format(lenTrans)
    }
    cat("\n")
    
    cat("Effort:\n ")
    cat(paste("  " 
            , format( c("Transects:"
                      , "Total length:")
                    , justify = "right")
            , colorize(format( c(nTrans, lenMess)))
            , "\n"
            ))
    
    if(!is.null(formula)){
      cat("Distances:\n ")
      cat(paste0("  "
                , colorize(format(mf$w.lo))
                , " to "
                , colorize(format(mf$w.hi))
                , ": "
                , colorize(nDistances)
                , "\n"))
      cat("Sightings:\n ")
      cat(paste("  "
                 , format(c("Groups:", "Individuals:"), justify = "right")
                 , colorize(c(nGroups, nTargets))
                 , "\n"))
      
    } else {
      cat(colorize("Specify 'formula', 'w.lo', and 'w.hi' to obtain distances, groups, and individuals."))
    }
    
    res <- data.frame(
      statistic = c(
        "transectType"
        , "transects"
        , "transectLength"
        , "distances"
        , "groups"
        , "individuals"
      ),
      value = c(
        tType
        , nTrans
        , lenMess
        , nDistances
        , nGroups
        , nTargets
      ))
  } else {
    res <- NextMethod("summary", object)
  }
  invisible(res)
}

#' @title Automated classical distance analysis
#' 
#' @description Perform automated likelihood, expansion, and series
#' selection for a classic distance sampling analysis.  Estimate 
#' abundance using the best fitting likelihood, expansion, and series. 
#' 
#' @inheritParams dE.single
#' @inheritParams abundEstim
#' @inheritParams dfuncEstim
#' 
#' @param plot Logical scalar specifying whether to plot models during model selection. 
#'   If \code{TRUE}, a histogram with fitted distance function is plotted for every model. 
#'   The function pauses between each plot and prompts the user for whether they want to continue. 
#'   To suppress user prompts, set \code{plot} = \code{FALSE}.
#'   
#' @param criterion A string specifying the criterion to use when assessing model fit.   
#' The best fitting model, as defined by this routine, has the lowest value 
#' of this criterion.  This must be one of "AICc" (the default), 
#' "AIC", or "BIC".  See \code{\link{AIC.dfunc}} for formulas. 
#'   
#' @param likelihoods String vector specifying the likelihoods to fit. 
#' See 'likelihood' parameter of \code{\link{dfuncEstim}}.
#'   
#' @details During distance function selection, all combinations of likelihoods, series, and 
#' number of expansions is fitted. For example, if \code{likelihoods} has 3 elements, 
#' \code{series} has 2 elements, and \code{expansions} has 4 elements, 
#' this routine fits a total of 3 (likelihoods) * 2 (series) * 4 (expansions) 
#' = 24 models.  Default parameters fit 9 detection functions, i.e.,  
#' all combinations of "halfnorm", "hazrate", and "negexp" likelihoods 
#' and 0 through 3 expansions. Other combinations are specified through 
#' values of \code{likelihoods}, \code{series}, and \code{expansions}. 
#' 
#' Suppress all intermediate output using \code{plot.bs=FALSE}, 
#' \code{showProgress=FALSE}, and \code{plot=FALSE}. 
#' 
#' The returned abundance estimate object contains
#' an additional component, the fitting table (a list of models fitted and 
#' criterion values) in component \code{$fitTable}.
#' 
#' @inherit abundEstim return
#' 
#' @seealso \code{\link{dfuncEstim}}, \code{\link{abundEstim}}.
#' @examples 
#' # Load example sparrow data (line transect survey type)
#' data(sparrowDf)
#' 
#' autoDistSamp(data = sparrowDf
#'            , formula = dist ~ groupsize(groupsize)
#'            , likelihoods = c("halfnorm","negexp")
#'            , expansions = 0
#'            , plot = FALSE
#'            , ci = NULL
#'            , area = 1 %ha%.
#' )
#' 
#' \dontrun{
#' autoDistSamp(data = sparrowDf
#'     , formula = dist ~ 1 + groupsize(groupsize)
#'     , ci = 0.95
#'     , area = 1 %ha%.
#' )     
#' }
#' 
#'            
#' @export

autoDistSamp <- function (data
                          , formula
                          , likelihoods = c("halfnorm", "hazrate", "negexp")
                          , w.lo = setUnits(0,"m")
                          , w.hi = NULL
                          , expansions = 0:3
                          , series = c("cosine")
                          , x.scl = w.lo
                          , g.x.scl = 1
                          , warn = TRUE
                          , outputUnits = NULL
                          
                          , area = NULL
                          , propUnitSurveyed = 1.0
                          , ci = 0.95
                          , R = 500
                          , plot.bs = FALSE
                          , showProgress = TRUE
                          , parallel = TRUE
                          
                          , plot = TRUE
                          , criterion = "AICc"
                          ){
  

  if(any(!(criterion %in% c("AIC","AICc","BIC")))) stop(paste0(criterion, " criterion not supported."))
  
  # function to save results
  f.save.result <- function(results
                          , dfunc
                          , like
                          , ser
                          , expan
                          , plot
                          , CRIT
                          , showProgress) {
    
    esw <- Rdistance::effectiveDistance(dfunc)
    
    if (any(!is.na(esw) & (esw > dfunc$w.hi))) {
      scl.ok <- "Not ok"
      scl.ok.flag <- 0
      aic <- NA
    } else {
      scl.ok <- "Ok"
      scl.ok.flag <- 1
      aic <- AIC.dfunc(dfunc, criterion=CRIT) 
    }
    conv <- dfunc$convergence
    if (conv != 0) {
      if (conv == -1) {
        conv.str <- "Bad"
      } else {
        conv.str <- "No"
      }
      aic <- NA
      attr(aic,"criterion")<-CRIT
      scl.ok <- "NA"
      scl.ok.flag <- NA
    } else {
      conv.str <- "Yes"
    }
    results <- rbind(results, data.frame(like = like
                                       , series = ser
                                       , expansions = expan
                                       , converge = conv
                                       , scale = scl.ok.flag
                                       , aic = aic
                                       ))
    
    if(showProgress){
      if (nchar(like) < 8) {
        sep1 <- "\t\t"
      } else {
        sep1 <- "\t"
      } 
      cat(paste(like, sep1, ser, "\t", expan, "\t", conv.str, 
              "\t\t", scl.ok, "\t", colorize(round(aic, 4)), sep = ""))
    }
    
    if (plot) {
      plot(dfunc)  # (jdc) This is the source of "Error: object of type 'symbol' is not subsettable
      k <- readline(" Next?[Enter=y,n]")
      if (length(k) == 0) 
        k <- "y"
    } else if(showProgress) {
      cat("\n")
      k <- "y"
    } else {
      k <- "y"
    }
    
    list(results = results, k = k)
  }
  
  # Fit detection functions (dfuncEstim appears 4 times below)
  
  wwarn <- options()$warn
  options(warn = -1)
  fit.table <- NULL
  
  if(showProgress){  
    cat(paste0("Likelihood\tSeries\tExpans\tConverged?\tScale?\t",criterion,"\n"))
  }
  
  for (like in likelihoods) {
    if (like == "Gamma") {
      dfunc <- Rdistance::dfuncEstim(
                            data = data
                          , formula = formula
                          , likelihood = like
                          , w.lo = w.lo
                          , w.hi = w.hi
                          , expansions = 0
                          , series = "cosine"
                          , x.scl = w.lo
                          , g.x.scl = 1
                          , warn = TRUE
                          , outputUnits = NULL
                          )
      ser <- ""  
      expan <- 0
      fit.table <- f.save.result(fit.table, dfunc, like, 
                                 ser, expan, plot, criterion, showProgress)
      cont <- fit.table$k
      fit.table <- fit.table$results
    } else {
      for (expan in expansions) {
        for(ser in series){
          
          dfunc <- Rdistance::dfuncEstim(
              data = data
            , formula = formula
            , likelihood = like
            , w.lo = w.lo
            , w.hi = w.hi
            , expansions = expan
            , series = ser
            , x.scl = w.lo
            , g.x.scl = g.x.scl
            , warn = TRUE
            , outputUnits = NULL)
          
          fit.table <- f.save.result(fit.table, dfunc, 
                                     like, ser, expan, plot, criterion, 
                                     showProgress)
          cont <- fit.table$k
          fit.table <- fit.table$results
          
          if (cont == "n"){
            break
          }
        }

        if (cont == "n"){ 
          break
        }
      }
    }
    
    if (cont == "n"){
      break
    } 
  }
  
  # Done fitting distance function, Show summary ----
  
  if (sum(fit.table$converge != 0) > 0 & showProgress) {
    cat("Note: Some models did not converge or had parameters at their boundaries.\n")
  }
    
  fit.table$aic <- ifelse(fit.table$converge == 0, fit.table$aic, Inf)
  fit.table <- fit.table[order(fit.table$aic), ]
  fit.table$deltaAIC <- fit.table$aic - min(fit.table$aic, na.rm = TRUE)

  # Refit best dfunc
  dfunc <- Rdistance::dfuncEstim(
      data = data
    , formula = formula
    , likelihood = fit.table$like[1]
    , w.lo = w.lo
    , w.hi = w.hi
    , expansions = fit.table$expansions[1]
    , series = fit.table$series[1]
    , x.scl = w.lo
    , g.x.scl = g.x.scl
    , warn = TRUE
    , outputUnits = NULL)
  if (plot) {
    plot(dfunc)
    graphics::mtext("BEST FITTING FUNCTION", side = 3, cex = 1.5, line = 3)
  }

  # Estimate abundance ----
  
  abund <- abundEstim(object = dfunc
              , area = area
              , propUnitSurveyed = propUnitSurveyed
              , ci = ci
              , R = R
              , plot.bs = plot.bs
              , showProgress = showProgress
              , parallel = parallel
              )
  
  # Store the fitting table, just in case user wants it.
  abund$fitTable <- fit.table

  if(showProgress){
    cat("\n\n------------ Abundance Estimate Based on Top-Ranked Detection Function ------------\n")
    summary(abund, criterion=criterion)
  }
  
  options(warn = wwarn)
  abund
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distances.R
\name{distances}
\alias{distances}
\title{Observation distances}
\usage{
distances(ml, na.rm = TRUE, ...)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance 
'fitted object'.  Both are of class "dfunc". 
Rdistance 'model frames' are lists containing components 
necessary to estimate a distance function, but no estimates. 
Rdistance 'model frames' are typically
produced by calls to \code{\link{parseModel}}. 
Rdistance 'fitted objects'
are typically produced by calls to \code{\link{dfuncEstim}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates, 
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}

\item{na.rm}{Whether to include or exclude missing distance values. 
In \code{ml}, the model list containing the model frame, missing 
values of the response (distance) are potentially present for two 
reasons: (1) they are outside the strip w.lo to w.hi, and (2) they
are missing because the crew did not get a distance for that observation.}

\item{...}{Ignored}
}
\value{
A vector containing observation distances contained
in the Rdistance model frame.
}
\description{
Extract the observation distances (i.e., responses for 
an Rdistance model) from an Rdistance model frame.
}
\examples{

data(sparrowDf)
sparrowModel <- parseModel( sparrowDf, dist ~ observer )
stats::model.response(sparrowModel$mf)
distances(sparrowModel) # same, but future-proof

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateOneStepPoints.R
\name{integrateOneStepPoints}
\alias{integrateOneStepPoints}
\title{Integrate Point-survey One-step function}
\usage{
integrateOneStepPoints(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced 
by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
If a matrix, each row corresponds to a 
distance function and each column is a parameter. The first column is 
the parameter related to sighting covariates and must be transformed 
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
be applied outside this routine).  If \code{object} is a matrix, 
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function 
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for 
the distance function covariates. If NULL and 
\code{object} is a fitted distance function, the  
observed covariates stored in
\code{object} are used (behavior similar to \code{\link{predict.lm}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function. 
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if 
\code{object} is a matrix. Sighting distance units can differ from units 
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in 
\code{object}. 
If \code{object} is a distance function and 
\code{newdata} is specified, the returned vector's length is 
\code{nrow(newdata)}.  If \code{object} is a distance function and 
\code{newdata} is NULL, 
returned vector's length is \code{length(distances(object))}. If 
\code{object} is a matrix, return's length is 
\code{nrow(object)}.
}
\description{
Compute integral of the one-step distance function
for point-surveys.
}
\details{
Returned integrals are
\deqn{\int_0^{w} x(\frac{p}{\theta_i}I(0\leq x \leq \theta_i) + \frac{1-p}{w - \theta_i}I(\theta_i < x \leq w)) dx = \frac{\theta_i}{2p}((1-p)w + \theta_i),}{
Integral(x((p/Theta)I(0<=x<=Theta) + ((1-p)/(w-Theta))I(Theta<x<=w))) = Theta ((1-p)w + Theta) / (2p),} 
where \eqn{w = w.hi - w.lo}, \eqn{\theta_i}{Theta} is the estimated one-step
distance function
threshold for the i-th observed distance, and \eqn{p}{p} is the estimated
one-step proportion.
}
\section{Note}{

Users will not normally call this function. It is called 
internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}.
}

\examples{

fit <- dfuncEstim(thrasherDf, dist~1, likelihood = "oneStep")
integrateOneStepPoints(fit, newdata = data.frame(`(Intercept)`=1))
EDR(fit, newdata = data.frame(`(Intercept)`=1))

# Check: 
Theta <- exp(fit$par[1])
Theta <- setUnits(Theta, "m")
p <- fit$par[2]
w.hi <- fit$w.hi
w.lo <- fit$w.lo
g.at0 <- w.lo
g.atT <- Theta
g.atTPlusFuzz <- (((1-p) * Theta) / ((w.hi - Theta) * p))*Theta
g.atWhi <- (((1-p) * Theta) / ((w.hi - Theta) * p))*w.hi
area.0.to.T <- (Theta - w.lo) * (g.atT - g.at0) / 2 # triangle; Theta^2/2
area.T.to.w <- (w.hi - Theta) * (g.atTPlusFuzz + g.atWhi) / 2 # trapazoid
area <- area.0.to.T + area.T.to.w
edr <- sqrt( 2*area )

}
\seealso{
\code{\link{integrateNumeric}}; \code{\link{integrateOneStepNumeric}}; 
\code{\link{integrateOneStepLines}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparrowDfuncObserver.R
\docType{data}
\name{sparrowDfuncObserver}
\alias{sparrowDfuncObserver}
\title{Brewer's Sparrow detection function}
\format{
An estimated distance function object with 
class 'dfunc'. See 'Value' section of 
\code{\link{dfuncEstim}} for 
description of components.
}
\description{
Pre-estimated Brewer's sparrow detection function that 
includes an 'observer' effect.  Included to speed up 
example execution times. See 'Examples'.
}
\examples{
\dontrun{
# the following code generated 'sparrowDfuncObserver'
data(sparrowDf)
sparrowDfuncObserver <- sparrowDf |> 
            dfuncEstim(formula = dist ~ observer
                     , likelihood = "hazrate")
}


}
\seealso{
\code{\link{sparrowSiteData}} and 
\code{\link{sparrowDetectionData}} for description of the data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effectiveDistance.R
\name{effectiveDistance}
\alias{effectiveDistance}
\title{Effective sampling distances}
\usage{
effectiveDistance(object, newdata = NULL)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{newdata}{A data frame containing new values for
covariates at which either
ESW's or EDR's will be computed. If NULL and
\code{object} contains covariates, the
covariates stored in
\code{object} are used (like \code{\link[=predict.lm]{predict.lm()}}).
If not NULL, covariate values in \code{newdata}
are used.
See \strong{Value} section for more information.}
}
\value{
If \code{newdata} is present, the returned value is
a vector of effective sampling distances associated with
covariate values in \code{newdata}. Length of return
in this case is the number of rows in \code{newdata}.
If \code{newdata} is NULL, the returned value is a vector
of effective sampling distances associated with covariate
values in \code{object}. Length of return in this case
is the number of detected groups.  The returned vector
has measurement units, i.e., \code{object$outputUnits}.
}
\description{
Computes Effective Strip Width (ESW) for line-transect detection
functions, or the analogous Effective Detection Radius (EDR) for point-transect
detection functions.
}
\details{
Serves as a wrapper for
\code{\link[=ESW]{ESW()}} and \code{\link[=EDR]{EDR()}}.

Effective distances are areas under scaled
distance functions (i.e., area under g(x)). Areas are
exact for functions whose integral is known (e.g., negexp).
Numeric integration is used for all others.
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}},
\code{\link[=ESW]{ESW()}},
\code{\link[=EDR]{EDR()}},
\code{\link[=integrateNumeric]{integrateNumeric()}},
\code{\link[=integrateNegexpLines]{integrateNegexpLines()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/huber.cumFunc.R
\name{huber.cumFunc}
\alias{huber.cumFunc}
\title{Huber Cumulative Function}
\usage{
huber.cumFunc(x, t1, t2, p, w)
}
\arguments{
\item{x}{A vector of distances. Can have units or not
(i.e., regular numeric).}

\item{t1}{A vector of values for the \eqn{\theta_1}{T1} parameter 
of the `huber` likelihood. If \code{x} has units, \code{t1} must have 
compatible units.}

\item{t2}{A vector of values for the \eqn{\theta_2}{T2} parameter 
of the `huber` likelihood. If \code{x} has units, \code{t2} must have 
compatible units.}

\item{p}{A vector of values for the \eqn{p} parameter of the 
`huber` likelihood. If \code{x} has units, \code{p} must have 
units of '[1]' (i.e., \code{setUnits(p,1)}).}

\item{w}{A vector of maximum strip widths, the maximum distance. 
If \code{x} has units, \code{x} must have 
compatible units.}
}
\value{
A vector of values from the `huber` cumulative function. 
The `huber` cumulative function is
\deqn{
F(x|\theta_1,\theta_2,p) = \int_0^x f(y|\theta_1,\theta_2,p) dy,}{
F(x|T1,T2,p) = Area Under f(y|T1,T2,p) from 0 to x,
}
where \eqn{f(y|\theta_1,\theta_2,p)}{f(y|T1,T2,p)} is Rdistance's `huber` 
likelihood. The only difference between this \emph{cumulative function}, 
and the \emph{cumulative distribution function}
is the scaling constant. That is, the maximum of the \emph{cumulative 
function} is greater than 1 while the maximum \emph{cumulative distribution function}
is exactly 1.
}
\description{
Computes the cumulative function of the `huber` likelihood. The 
cumulative function is proportional to the `huber` 
cumulative distribution function, differing only by appropriate
scaling constant.
}
\examples{
d <- -10:210

# Cumulative function
fd <- huber.cumFunc(d, 125, 25, .05, 200) 
plot(d, fd, type="l")

# Cumulative distribution function
Fd <- fd / huber.cumFunc(200, 125, 25, .05, 200)

}
\seealso{
\code{\link{huber.like}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thrasherSiteData.R
\docType{data}
\name{thrasherSiteData}
\alias{thrasherSiteData}
\title{Sage Thrasher site data.}
\format{
A data.frame containing 120 rows and 6 columns.  Each row represents
a surveyed site (point).  Column descriptions:
\enumerate{
\item \code{siteID}: Factor (120 levels), the site or point surveyed.
\item \code{observer}: Factor (six levels), identity of the observer who surveyed
the point.
\item \code{bare}: Number, the mean bare ground cover (\%)
within 100 m of each point.
\item \code{herb}: Number, the mean herbaceous
cover (\%) within 100 [m] of each point.
\item \code{shrub}: Number, the mean
shrub cover (\%) within 100 [m] of each point.
\item \code{height}: Number,
the mean shrub height [cm] within 100 [m] of each point.  }
}
\source{
The Sage Thrasher data are a subset of data collected
by Jason Carlisle and field technicians for his Ph.D. from the
Department of Ecology, University of Wyoming, in 2017.  This portion of
Jason's work was funded by the Wyoming Game and Fish Department through agreements
with the University of Wyoming's  Cooperative Fish & Wildlife
Research Unit (2012).
}
\description{
Point transect data collected in central
Wyoming from 120 points surveyed for Sage Thrashers by the Wyoming
Cooperative Fish & Wildlife Research Unit in 2013.
}
\references{
Carlisle, J.D. 2017. The effect of sage-grouse conservation on
wildlife species of concern: implications for the umbrella species concept.
Dissertation. University of Wyoming, Laramie, Wyoming, USA.

Carlisle, J. D., A. D. Chalfoun, K. T. Smith, and J. L. Beck. 2018.
Nontarget effects on songbirds from habitat manipulation for Greater
Sage-Grouse: Implications for the umbrella species concept.
\emph{The Condor: Ornithological Applications} 120:439–455.
\doi{10.1650/CONDOR-17-200.1}
}
\seealso{
\code{\link[=thrasherDetectionData]{thrasherDetectionData()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdistance-package.R
\docType{package}
\name{Rdistance-package}
\alias{Rdistance-package}
\alias{distance}
\alias{Rdistance}
\alias{point-transect}
\alias{line-transect}
\title{Rdistance - Distance Sampling Analyses for Abundance Estimation}
\description{
\code{Rdistance} contains functions and associated routines to analyze
distance-sampling data collected on point or line transects.
Some of \code{Rdistance}'s features include:
\itemize{
\item Accommodation of both point and line transect analyses in one routine (\code{\link[=dfuncEstim]{dfuncEstim()}}).
\item Regression-like formula for inclusion of distance function covariates (\code{\link[=dfuncEstim]{dfuncEstim()}}).
\item Automatic bootstrap confidence intervals (\code{\link[=abundEstim]{abundEstim()}}).
\item Parallel processing of bootstrap iterations (\code{parallel} argument of \code{\link[=abundEstim]{abundEstim()}}).
\item Availability of both study-area and site-level abundance estimates (\code{help("predict.dfunc")}).
\item Rigorous physical measurement requirements and automated conversion when necessary (\code{\link{\%#\%}}, \code{\link[=setUnits]{setUnits()}}).
\item Classic parametric distance functions (\code{\link[=halfnorm.like]{halfnorm.like()}}, \code{\link[=hazrate.like]{hazrate.like()}}, \code{\link[=negexp.like]{negexp.like()}}), and
expansion functions (\code{\link[=cosine.expansion]{cosine.expansion()}}, \code{\link[=hermite.expansion]{hermite.expansion()}}, \code{\link[=simple.expansion]{simple.expansion()}}).
\item Mixture distance functions for non-standard shapes and thresholds (\code{\link[=oneStep.like]{oneStep.like()}}, \code{\link[=triangle.like]{triangle.like()}}, and \code{\link[=huber.like]{huber.like()}}).
\item Automated distance function fitting and selection \code{\link[=autoDistSamp]{autoDistSamp()}}.
\item \code{print}, \code{plot}, \code{predict}, \code{coef}, and \code{summary} methods for distance function objects and
abundance classes.
}
}
\section{Background:}{
Distance-sampling is a popular method for abundance estimation in
ecology. Line transect surveys are conducted by traversing
randomly placed transects in a study area with the objective of
sighting animals and estimating density or abundance.  Data collected
during line transect surveys consists of \emph{target} sightings,
either of individuals or groups, and off-transect distances to the
original location of the \emph{target}.
When \emph{targets} are sighted in groups, data include the number
of individuals in the group.

Point transect surveys are similar except that observers stop one
or more times along the transect to observe \emph{targets}.  Point transects are
popular avian survey methods where detections are often auditory
cues. Point transects are also for studies using automated auditory detectors
or trail cameras.  Point transect data consists of radial distances from
the observer to the \emph{target}.

The defining feature of distance sampling is the tendency for
probability of detection to decline as
off-transect or radial distances increase. Targets far from
the observer are generally
harder to detect than those closer.  In most
line transect studies, \emph{targets} on the transect (off-transect
distance = 0) are assumed to be sighted with 100\% probability.  This
assumption allows researchers to estimate the proportion of missed
targets and in turn adjust the number of
sighted \emph{targets} for missed detections.
Some studies utilize two observers searching the same areas and are able to
estimate the proportion of individuals missed on the transect line and
thereby eliminate the
assumption that all individuals on the line have been observed.
}

\section{Purpose:}{
The author's aims are
to provide an easy-to-use, rigorous, and flexible analysis option in R for
distance-sampling data.  The authors believe that beginning
users need easy-to-use and easy-to-understand software,
while advanced users require greater flexibility and customization, and
their aim is to meet the demands of both groups.
}

\section{Data sets:}{
\code{Rdistance} contains the following example data sets:
\itemize{
\item Line-transect sampling of Brewers sparrows in central Wyoming
(\code{\link[=sparrowDf]{sparrowDf()}}).
\item Point-transect sampling of Sage Thrashers in central Wyoming
(\code{\link[=thrasherDf]{thrasherDf()}}).
}
}

\section{References}{
Buckland, S.T., Anderson, D.R., Burnham, K.P. and Laake, J.L.
1993. \emph{Distance Sampling: Estimating Abundance of Biological
Populations}. Chapman and Hall, London.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://mcdonalddatasciences.com/Rdistance.html}
  \item Report bugs at \url{https://github.com/tmcd82070/Rdistance/issues}
}

}
\author{
Main author and maintainer: Trent McDonald
\href{mailto:trent@mcdonalddatasciences.com}{trent@mcdonalddatasciences.com}

Coauthors: Ryan Nielson, Jason Carlisle, and Aidan McDonald

Contributors: Ben Augustine, James Griswald, Joel Reynolds, Pham Quang, Earl
Becker, Aaron Christ, Brook Russelland, Patrick McKann, Lacey Jeroue,
Abigail Hoffman, Michael Kleinsasser, and Ried Olson
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateNegexpLines.R
\name{integrateNegexpLines}
\alias{integrateNegexpLines}
\title{Integrate Negative exponential}
\usage{
integrateNegexpLines(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced
by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}), or a matrix of canonical
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}).
If a matrix, each row corresponds to a
distance function and each column is a parameter. The first column is
the parameter related to sighting covariates and must be transformed
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must
be applied outside this routine).  If \code{object} is a matrix,
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for
the distance function covariates. If NULL and
\code{object} is a fitted distance function, the
observed covariates stored in
\code{object} are used (behavior similar to \code{\link[=predict.lm]{predict.lm()}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if
\code{object} is a matrix. Sighting distance units can differ from units
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in
\code{object}.
If \code{object} is a distance function and
\code{newdata} is specified, the returned vector's length is
\code{nrow(newdata)}.  If \code{object} is a distance function and
\code{newdata} is NULL,
returned vector's length is \code{length(distances(object))}. If
\code{object} is a matrix, return's length is
\code{nrow(object)}.
}
\description{
Compute integral of the negative exponential distance function.
}
\details{
Returned integrals are
\deqn{\int_0^{w} e^{-a_i x} dx = \frac{1}{a_i} (1 - e^{-a_i w}),}{
Integral( exp(-a*x) ) = (1 - exp(-a*w)) / a,}
where \eqn{w = w.hi - w.lo} and \eqn{a_i}{a} is the estimated
negative exponential distance
function parameter for the
i-th observed distance.
}
\section{Note}{

Users will not normally call this function. It is called
internally by \code{\link[=nLL]{nLL()}} and \code{\link[=effectiveDistance]{effectiveDistance()}}.
}

\examples{

# Fake distance function object w/ minimum inputs for integration
d <- rep(1,4) \%#\% "m" # Only units needed, not values
obs <- factor(rep(c("obs1", "obs2"), 2))
beta <- c(-5, -0.5)
w.hi <- 125
w.lo <- 20
ml <- list(
    mf = model.frame(d ~ obs)
  , par = beta 
  , likelihood = "negexp"
  , w.lo = w.lo \%#\% "m"
  , w.hi = w.hi \%#\% "m"
  , expansions = 0
)
class(ml) <- "dfunc"
integrateNegexpLines(ml)

# Check: Integral of exp(-bx) from 0 to w.hi-w.lo
b <- c(exp(beta[1]), exp(beta[1] + beta[2]))
intgral <- (1 - exp(-b*(w.hi - w.lo))) / b
intgral


}
\seealso{
\code{\link[=integrateNumeric]{integrateNumeric()}}; \code{\link[=integrateNegexpPoints]{integrateNegexpPoints()}};
\code{\link[=integrateOneStepLines]{integrateOneStepLines()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoDistSamp.R
\name{autoDistSamp}
\alias{autoDistSamp}
\title{Automated classical distance analysis}
\usage{
autoDistSamp(
  data,
  formula,
  likelihoods = c("halfnorm", "hazrate", "negexp"),
  w.lo = setUnits(0, "m"),
  w.hi = NULL,
  expansions = 0:3,
  series = c("cosine"),
  x.scl = w.lo,
  g.x.scl = 1,
  warn = TRUE,
  outputUnits = NULL,
  area = NULL,
  propUnitSurveyed = 1,
  ci = 0.95,
  R = 500,
  plot.bs = FALSE,
  showProgress = TRUE,
  parallel = TRUE,
  plot = TRUE,
  criterion = "AICc"
)
}
\arguments{
\item{data}{An \code{RdistDf} data frame. \code{RdistDf} data frames 
contain one line per transect and a list-based column. The list-based
column contains a data frame with detection information. 
The detection information data frame on each row contains (at least) distances 
and group sizes of all targets detected on the transect.
Function \code{\link{RdistDf}} creates \code{RdistDf} data frames 
from separate transect and detection data frames. 
\code{\link{is.RdistDf}} checks whether data frames
are \code{RdistDf}'s.}

\item{formula}{A standard formula object.  For example, \code{dist ~ 1}, 
\code{dist ~ covar1 + covar2}). The left-hand side (before \code{~})
is the name of the vector containing off-transect or radial detection distances.  
The right-hand side contains the names of covariate 
vectors to fit in the detection
function, and potentially group sizes. Group sizes are specified 
by including \code{+ groupsize(<variable>)} in the RHS 
(see 'Group Sizes' section).
Covariates can be either detection level 
or transect level and can appear in  \code{data} or exist in the 
global working environment. Regular R scoping 
rules apply.}

\item{likelihoods}{String vector specifying the likelihoods to fit. 
See 'likelihood' parameter of \code{\link{dfuncEstim}}.}

\item{w.lo}{Lower or left-truncation limit of the distances in distance data. 
This is the minimum possible off-transect distance. Default is 0.  If 
\code{w.lo} is greater than 0, it must have measurement units. 
See \code{help(unitHelpers)} for assistance assigning units.}

\item{w.hi}{Upper or right-truncation limit of the distances 
in \code{dist}. This is the maximum off-transect distance that 
could be observed. If unspecified (i.e., NULL), 
right-truncation is set to the maximum of the observed 
distances.  If \code{w.hi} is specified, it must have  
measurement units.  
See \code{help(unitHelpers)} for assistance assigning units.}

\item{expansions}{A scalar specifying the number of terms 
in \code{series} to compute. Depending on the series, 
this could be 0 through 5.  The default of 0 equates 
to no expansion terms of any type.  No expansion terms 
are allowed (i.e., \code{expansions} is forced to 0) if 
covariates are present in the detection function 
(i.e., right-hand side of \code{formula} includes
something other than \code{1}).}

\item{series}{If \code{expansions} > 0, this string 
specifies the type of expansion to use. Valid values at 
present are 'simple', 'hermite', and 'cosine'.}

\item{x.scl}{The x coordinate (a distance) at which the 
detection function will be scaled.  \code{g.x.scl} can be a distance
or the string "max".  
When \code{x.scl} is specified (i.e., not 0 or "max"), it must have measurement 
units assigned. 
See \code{help(unitHelpers)} for assistance assigning units.}

\item{g.x.scl}{Height of the distance function at coordinate \code{x}. 
The distance function 
will be scaled so that g(\code{x.scl}) = \code{g.x.scl}. 
If \code{g.x.scl} is not 
a data frame, it must be a numeric value (vector of length 1) 
between 0 and 1.}

\item{warn}{A logical scalar specifying whether to issue 
an R warning if the estimation did not converge or if one 
or more parameter estimates are at their boundaries.  
For estimation, \code{warn} should generally be left at
its default value of \code{TRUE}.  When computing bootstrap 
confidence intervals, setting \code{warn = FALSE} 
turns off annoying warnings when an iteration does 
not converge.  Regardless of \code{warn}, after 
completion all messages about 
convergence and boundary conditions are printed 
by \code{print.dfunc}, \code{print.abund}, and 
\code{plot.dfunc}.}

\item{outputUnits}{A string specifying the symbolic measurement 
units for results. Valid units are listed in \code{units::valid_udunits()}.
The strings for common distance symbolic units are: 
"m" - meters, "ft" - feet, "cm" - centimeters, "mm" - 
millimeters, "mi" - miles, "nmile" - 
nautical miles ("nm" is nano meters), "in" - inches, 
"yd" - yards, "km" - kilometers, "fathom" - fathoms, 
"chains" - chains, and "furlong" - furlongs.  
If \code{outputUnits} is unspecified (NULL),
output units will be the same as those on 
distances in \code{data}.}

\item{area}{A scalar containing the total area of inference. Usually, this is 
study area size.  If \code{area} is NULL (the default), 
\code{area} will be set to 1 square unit of the output units and density estimates
will be produced. 
If \code{area} is not NULL, it must have measurement units 
assigned by the \code{units} package. 
The units on \code{area} must be convertible
to squared output units. Units 
on \code{area} must be two-dimensional. 
For example, if output units are "foo", 
units on area must be convertible to "foo^2" by the \code{units}
package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
others are acceptable.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that 
  was surveyed.  If both sides of line transects were observed, 
  \code{propUnitSurveyed}
  = 1.  If only a single side of line transects were observed, set 
  \code{propUnitSurveyed} = 0.5. For point transects, this should be set to 
  the proportion of each circle that was observed. Length must either be
  1 or the total number of transects in \code{x}.}

\item{ci}{A scalar indicating the confidence level of confidence intervals. 
Confidence intervals are computed using a bias corrected bootstrap
method. If \code{ci = NULL} or \code{ci == NA}, confidence intervals 
are not computed.}

\item{R}{The number of bootstrap iterations to conduct when \code{ci} is not
NULL.}

\item{plot.bs}{A logical scalar indicating whether to plot individual
bootstrap iterations.
Ignored unless \code{parallel = FALSE}.}

\item{showProgress}{A logical indicating whether to show a text-based
progress bar during bootstrapping. Default is \code{TRUE}. 
It is handy to shut off the 
progress bar if running this within another function. 
Ignored unless \code{parallel = FALSE}.}

\item{parallel}{A logical scalar, or a positive integer; ignored unless 
confidence intervals are requested (i.e., \code{!is.null(ci)}).
If TRUE, bootstrap iterations are 
run in parallel using the maximum number of CPU cores minus 1. 
The maximum number of CPU cores is reported by \code{parallel::detectCores()}.
If a positive integer (1 <= \code{parallel} <= maximum cores), bootstrap 
iterations are performed in parallel on that many cores. 
If FALSE, bootstrap iterations are performed in series, and progress 
will be shown if \code{showProgress == TRUE}. Parameters 
\code{showProgress} and \code{plot.bs} are ignored when operating
in parallel.}

\item{plot}{Logical scalar specifying whether to plot models during model selection. 
If \code{TRUE}, a histogram with fitted distance function is plotted for every model. 
The function pauses between each plot and prompts the user for whether they want to continue. 
To suppress user prompts, set \code{plot} = \code{FALSE}.}

\item{criterion}{A string specifying the criterion to use when assessing model fit.
The best fitting model, as defined by this routine, has the lowest value 
of this criterion.  This must be one of "AICc" (the default), 
"AIC", or "BIC".  See \code{\link{AIC.dfunc}} for formulas.}
}
\value{
An Rdistance 'abundance estimate' object, which is a list of
  class \code{c("abund", "dfunc")}, containing all the components of a "dfunc"
  object (see \code{\link{dfuncEstim}}), plus the following: 
  
  \item{estimates}{A tibble containing fitted coefficients in the
  distance function, density in the area(s) surveyed, 
  abundance on the study area, the number of groups seen 
  between w.lo and w.hi, the number 
  of individuals seen between w.lo and w.hi, 
  study area size, surveyed area, average group size, and 
  average effective detection distance.
  }
  
  \item{B}{If confidence intervals were requested, a tibble 
  containing all bootstrap values of coefficients, 
  density, abundance, groups seen, individuals seen, 
  study area size, surveyed area size, average group size, 
  and average effective detection distance.  The number of rows is always 
  \code{R}, the requested number of bootstrap 
  iterations.  If an iteration fails, the
  corresponding row in \code{B} is \code{NA} (hence, use 'na.rm = TRUE' 
  when computing summaries). Columns 1 through \code{length(coef(dfunc))}
  contain bootstrap realizations of the distance function's coefficients. 
  }
  
  \item{ci}{Confidence level of the confidence intervals}
}
\description{
Perform automated likelihood, expansion, and series
selection for a classic distance sampling analysis.  Estimate 
abundance using the best fitting likelihood, expansion, and series.
}
\details{
During distance function selection, all combinations of likelihoods, series, and 
number of expansions is fitted. For example, if \code{likelihoods} has 3 elements, 
\code{series} has 2 elements, and \code{expansions} has 4 elements, 
this routine fits a total of 3 (likelihoods) * 2 (series) * 4 (expansions) 
= 24 models.  Default parameters fit 9 detection functions, i.e.,  
all combinations of "halfnorm", "hazrate", and "negexp" likelihoods 
and 0 through 3 expansions. Other combinations are specified through 
values of \code{likelihoods}, \code{series}, and \code{expansions}. 

Suppress all intermediate output using \code{plot.bs=FALSE}, 
\code{showProgress=FALSE}, and \code{plot=FALSE}. 

The returned abundance estimate object contains
an additional component, the fitting table (a list of models fitted and 
criterion values) in component \code{$fitTable}.
}
\examples{
# Load example sparrow data (line transect survey type)
data(sparrowDf)

autoDistSamp(data = sparrowDf
           , formula = dist ~ groupsize(groupsize)
           , likelihoods = c("halfnorm","negexp")
           , expansions = 0
           , plot = FALSE
           , ci = NULL
           , area = 1 \%ha\%.
)

\dontrun{
autoDistSamp(data = sparrowDf
    , formula = dist ~ 1 + groupsize(groupsize)
    , ci = 0.95
    , area = 1 \%ha\%.
)     
}

           
}
\seealso{
\code{\link{dfuncEstim}}, \code{\link{abundEstim}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RdistDf.R
\name{RdistDf}
\alias{RdistDf}
\title{Construct Rdistance nested data frames}
\usage{
RdistDf(
  transectDf,
  detectionDf,
  by = NULL,
  pointSurvey = FALSE,
  observer = "single",
  .detectionCol = "detections",
  .effortCol = NULL
)
}
\arguments{
\item{transectDf}{A data frame with one row per transect and 
columns containing information about the entire transect. 
At a minimum, this data frame must contain the transect's ID so 
it can be merged with \code{detectionDf}, (see parameter \code{by}) 
and the amount of effort the transect represents 
(see parameter \code{.effortCol}).  
All detections are made on a transect, but not all transects 
require detections. That is, \code{transectDf} 
should contain rows, and hence
ID's and lengths, of all surveyed transects, even those on 
which no targets were detected (so-called "zero transects"). 
Transect-level covariates, such 
as habitat type, elevation, or observer IDs, should appear as variables 
in this data frame.}

\item{detectionDf}{A data frame containing
detection information associated with each transect.  
At a minimum, each row of this data 
frame must contain the following:
\itemize{
  \item \bold{Transect IDs}: The ID of the transect on 
  which a target group was detected so that the 
  detection data frame can be merged with \code{transectDf} 
  (see parameter \code{by}). Multiple detections on the same transect 
  are possible and hence multiple rows in \code{detectonDf} 
  can contain the same transect ID. 
     
  \item \bold{Detection Distances}: The distance at which each 
  detection was made. The distance column will eventually be 
  specified on the left-hand 
  side of \code{formula} in a call to \code{dfuncEstim}.  
  As of Rdistance version 3.0.0, detection distances must have 
  physical measurement units assigned. See 
  \bold{Measurement Units}. 
}
Optional columns in `detectionDf`:
\itemize{
   \item \bold{Group sizes}:If sighted targets vary in size, or group sizes 
         are not all 1, \code{detectionDf} must 
         also contain a column specifying group sizes. Non-unity group sizes 
         are specified using \code{+groupsize(columnName)} on the 
         right-hand-side of \code{formula} in an eventual call to \code{dfuncEstim}. 
         
   \item \bold{Detection Level Covariates}: Such as sex, color, habitat, etc. 
}}

\item{by}{A character vector of variables to use in the join. The right-hand
side of this join identifies unique transects (unique 
rows) in both \code{transectDf} and the output (see warning in \bold{Details}).
If NULL, the join will be 'natural', using all common variables 
in \code{transectDf} and \code{detectionDf}. To join on 
specific variables, specify a character vector of 
the variables. For example, by = c("a", "b") 
joins \code{transectDf$a} to \code{detectionDf$a} and 
\code{transectDf$b} to \code{detectionDf$b}. If join variable names 
differ between \code{transectDf} and \code{detectionDf}, 
use a named character vector like by = c("a" = "b", 
"c" = "d") which joins \code{transectDf$a} to 
\code{detectionDf$b} and 
\code{transectDf$c} to \code{detectionDf$d}.}

\item{pointSurvey}{If TRUE, observations were
made from discrete points (i.e., during a point-transect survey) and distances 
are radial from observation point to target.  If FALSE, 
observations were made along a continuous transect 
(i.e., during a line-transect survey) and distances are from target to nearest 
point on the transect (i.e., perpendicular to transect).}

\item{observer}{Type of observer system.  Legal values are \bold{"single"} for 
single observer systems, \bold{"1given2"} for a double observer system 
wherein observations 
made by observer 1 are tested against observations made by observer 2,
\bold{"2given1"} for a double observer system wherein observations 
made by observer 2 are tested against observations made by observer 1, and
\bold{"both"} for a double observer system wherein observations 
made by both observers are tested against the other and combined.}

\item{.detectionCol}{Name of the list column that will
contain detection data frames. Default name is "detections".
Detection distances (LHS of `dfuncEstim` formula) and 
group sizes are normally 
columns in the nested detection data frames embedded in 
`.detectionCol`.}

\item{.effortCol}{For continuous line transects, 
this specifies the name of a column in \code{transectDf} 
containing transect lengths, which must have measurement units.  
For point transects, this specifies the name of a column containing 
the number of points on each transect.  The effort column for point 
transects \emph{cannot} contain 
measurement units. Default is "length" for line-transects, "numPoints" for 
point-transects. If those names are not found, the first column
in the merged data frame whose name contains 'point' 
(for point transects) or 'length' (for line transects)
is used and a message is printed. Matching is case insensitive, 
so for example, 'nPoints' and 
'N_point' and 'numberOfPoints' will all be matched.   If two or more column 
names match the effort column search terms, a warning is issued. 
See \bold{Transect Lengths} for a description of point and line transects.}
}
\value{
A nested tibble (a generalization of base data frames) 
with one row per transect, and detection 
information in a list column.  Technically, the return is 
a grouped \code{tibble} from 
the \code{tibble} package with one row per group, and a list 
column containing detection information. 
Survey type, observer system, and name of the effort column 
are recorded 
as attributes (\code{transType}, \code{obsType}, and \code{effortColumn}, respectfully). 
The return prints nicely using methods 
in package \code{tibble}.  If returned objects print strangely, 
attach library \code{tibble}.  A summary method tailored to distance sampling 
is available (i.e., \code{summary(return)}).
}
\description{
Makes an \code{Rdistance} data frame from 
separate transect and detection 
data frames. \code{Rdistance} data frames are nested 
data frames with one row per transect. Detection 
information for each transect appears in a list-based 
column that itself contains a
data frame. See \bold{Rdistance Data Frames}. 

\code{Rdistance} data frames can be constructed using calls to 
\code{dplyr::nest_by} and \code{dplyr::right_jion}, with subsequent 
attribute assignment (see \bold{Examples}). This routine is 
a convenient wrapper for those calls.
}
\details{
For valid bootstrap estimates of confidence intervals (computed in \code{\link{abundEstim}}), 
each row of the nested data frame must represent one transect (more generally, 
one sampling unit), and none should
be duplicated. The combination of transect columns 
in \code{by} (i.e., the LHS of the merge, or "a" and "b" of 
\code{by = c("a" = "d", "b" = "c")} for example) 
should specify \emph{unique} transects and unique rows of 
\code{transectDf}. \bold{Warning:} If \code{by} 
does not specify unique rows of \code{transectDf}, \code{dplyr::left_join}, 
which is called internally, will perform a many-to-many merge without 
warning, and this normally duplicates both 
transects and detections.
}
\section{Rdistance Data Frames}{
 

\code{RdistDf} data frames contain the following information:

\itemize{
  \item \bold{Transect Information}: Each row of the 
  data frame contains transect id and effort. Effort is 
  transect length for line-transects, and number of points for 
  point-transects.
  Optionally, transect-level covariates (such as habitat or observer
  id) appear on each row. 
  \item \bold{Detection Information}: Observation distances
  (either perpendicular off-transect or radial off-point) appear 
  in a data frame stored in a list column.  If detected groups
  occasionally included more than one target, a group size column must 
  be present in the list-column data frame.
  Optionally, detection-level covariates (such as sex or size)
  can appear in the data frame of the list column.
  \item \bold{Distance Type}: The type of observation distances, either 
  perpendicular off-transect (for line-transects studies) or 
  radial off-point (for point-transect studies) must appear as an 
  attribute of \code{RdistDf}'s. 
  \item \bold{Observer Type}: The type of observation system used, either 
  single observer or one of three types of multiple observer systems, must 
  appear as an attribute of \code{RdistDf}'s.
}
}

\section{Transect Lengths}{

Line-transects are continuous paths with targets detectable at 
any point.  Point transects
consist of one or more discrete points along a path 
from which observers search for targets. 
The length of a line-transect is it's physical length (e.g., km or miles).
The 'length' of a point transect is 
the number of points along the transect. Single 
points are considered transects of length one. The length of line-transects
must have a physical measurement unit (e.g., 'm' or 'ft').  The length of 
point-transects must be a unit-less integers (i.e., number of points).
}

\section{Measurement Units}{
 
As of \code{Rdistance} version 3.0.0, measurement units are 
require on all physical distances. 
Requiring units ensures that internal calculations and results 
(e.g., ESW and abundance) are correct 
and that output units are clear.   
Physical distances are required on
off-transect distances, radial distances, truncation distances 
(\code{w.lo}, unless it is zero; and \code{w.hi}, unless it is NULL), 
scale locations (\code{x.scl}, unless it is zero),
line-transect lengths, and study area size. All units are 
1-dimensional except those on study area, which are 2-dimensional. 

Physical measurement units can vary. For example, 
off-transect distances can be meters ("m"), \code{w.hi} can be inches ("in"), 
and \code{w.lo} can be kilometers ("km").  Internally, all distances are 
converted to the units specified by \code{outputUnits} 
(or the units of input distances if 
\code{outputUnits} is NULL), and 
all output is reported 
in units of \code{outputUnits}. Valid conversions must exist between 
units or an error is thrown.  For example, meters cannot be converted
into hectares.
  
Measurement units can be assigned using one of Rdistance's 
unit helper routines (see \code{help(unitHelpers)}), or from 
routines in the \code{units} package (e.g., 
\code{x <- units::set_units(x, "<units>")}). 
See \code{units::}\code{\link[units]{valid_udunits}}
for a list of valid symbolic units. 

If measurements are truly unit-less, or measurement units are unknown, 
set \code{options(Rdist_requireUnits = FALSE)}.  This suppresses 
all unit checks and conversions.  Users are on their own 
to make sure inputs are scaled correctly and that output units are known.
}

\examples{
data(sparrowSiteData)
data(sparrowDetectionData)

sparrowDf <- RdistDf( sparrowSiteData, sparrowDetectionData )
is.RdistDf(sparrowDf, verbose = T)
summary(sparrowDf)
summary(sparrowDf
      , formula = dist ~ groupsize(groupsize)
      , w.hi = 100 \%m\%.)

# Equivalent to above: 
sparrowDf <- sparrowDetectionData |> 
  dplyr::nest_by( siteID
               , .key = "detections") |> 
  dplyr::right_join(sparrowSiteData, by = "siteID") 
attr(sparrowDf, "detectionColumn") <- "detections"
attr(sparrowDf, "effortColumn") <- "length"
attr(sparrowDf, "obsType") <- "single"
attr(sparrowDf, "transType") <- "line"
is.RdistDf(sparrowDf, verbose = T)
summary(sparrowDf, formula = dist ~ groupsize(groupsize))

# Condensed view: 1 row per transect (make sure tibble is attached)
sparrowDf

# Expansion methods:
# (1) use Rdistance::unnest (includes zero transects)
df1 <- unnest(sparrowDf)
any( df1$siteID == "B2" )  # TRUE

# Use tidyr::unnest(); but, no zero transects
df2 <- tidyr::unnest(sparrowDf, cols = "detections")
any( df2$siteID == "B2" )  # FALSE

# Use dplyr::reframe for specific transects (e.g., for transect "B3")
sparrowDf |> 
  dplyr::filter(siteID == "B3") |>
  dplyr::reframe(detections)
  
# Count detections per transect (can't use dplyr::if_else)
df3 <- sparrowDf |> 
  dplyr::reframe(nDetections = ifelse(is.null(detections), 0, nrow(detections)))
sum(df3$nDetections) # Number of detections
sum(df3$nDetections == 0) # Number of zero transects
    
# Point transects
data(thrasherDetectionData)
data(thrasherSiteData)
thrasherDf <- RdistDf( thrasherSiteData
               , thrasherDetectionData
               , pointSurvey = TRUE
               , by = "siteID"
               , .detectionCol = "detections")
summary(thrasherDf, formula = dist ~ groupsize(groupsize))
               
}
\seealso{
\code{\link{is.RdistDf}}: check validity of RdistDf data frames;
 \code{\link{dfuncEstim}}: estimate distance function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/huber.start.limits.R
\name{huber.start.limits}
\alias{huber.start.limits}
\title{Start and limit values for the Huber distance function}
\usage{
huber.start.limits(ml)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}
}
\value{
A list containing the following components
\item{start}{Vector of starting values for parameters of the likelihood and expansion terms. }
\item{lowlimit}{Vector of lower limits for the likelihood parameters and expansion terms.}
\item{uplimit}{Vector of upper limits for the likelihood parameters and expansion terms.}
\item{names}{Vector of names for the likelihood parameters and expansion terms.}

The length of each vector in the return is:
\verb{(Num expansions) + 1 + 1*(like \%in\% c("hazrate")) + (Num Covars)}.
}
\description{
Computes starting values and limits for the 'huber'
distance function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlEstimates.R
\name{mlEstimates}
\alias{mlEstimates}
\title{Distance function maximum likelihood estimates}
\usage{
mlEstimates(ml, strt.lims)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}

\item{strt.lims}{A list containing start, low, and high limits for
parameters of the requested likelihood. This list is typically produced
by a call to \code{\link[=startLimits]{startLimits()}}.}
}
\value{
An Rdistance fitted model object. This object contains the
raw object returned by the optimization routine (e.g., \code{nlming}),
and additional components specific to Rdistance.
}
\description{
Estimate parameters of a distance function using maximum likelihood.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perpDists.R
\name{perpDists}
\alias{perpDists}
\title{Compute off-transect distances from sighting distances and angles}
\usage{
perpDists(sightDist, sightAngle, data)
}
\arguments{
\item{sightDist}{Character, name of column in \code{data} that contains the observed or sighting distances from the observer to the detected objects.}

\item{sightAngle}{Character, name of column in \code{data} that contains the observed or sighting angles from the line transect to the detected
objects.  Angles must be measured in degrees.}

\item{data}{data.frame object containing sighting distance and sighting angle.}
}
\value{
A vector of off-transect (or perpendicular) distances.  Units are the same as \code{sightDist}.
}
\description{
Computes off-transect (also called 'perpendicular') distances from measures of sighting distance and sighting angle.
}
\details{
If observers recorded sighting distance and sighting angle (as is often common in line transect surveys), use this function to convert
to off-transect distances, the required input data for \code{dfunc.estim}.
}
\examples{
# Load the example dataset of sparrow detections from package
data(sparrowDetectionData)
# Compute perpendicular, off-transect distances from the observer's sight distance and angle
sparrowDetectionData$perpDist <- perpDists(sightDist="sightdist", sightAngle="sightangle",
                                           data=sparrowDetectionData)
}
\references{
Buckland, S.T., Anderson, D.R., Burnham, K.P. and Laake, J.L. 1993.
\emph{Distance Sampling: Estimating Abundance of Biological Populations}. Chapman and Hall, London.
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupSizes.R
\name{groupSizes}
\alias{groupSizes}
\title{Group Sizes}
\usage{
groupSizes(ml, ...)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}

\item{...}{Ignored}
}
\value{
A vector containing group sizes contained
in the Rdistance model frame or fitted object.
}
\description{
Extract the group size information from
an Rdistance model frame.
}
\examples{

data("sparrowDf")
sparrowModel <- parseModel( sparrowDf, dist ~ observer )
stats::model.offset(sparrowModel$mf)
groupSizes(sparrowModel)  # same, but future-proof

sparrowModel <- parseModel( sparrowDf
                 , dist ~ observer + groupsize(groupsize) )
groupSizes(sparrowModel)  

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/secondDeriv.R
\name{secondDeriv}
\alias{secondDeriv}
\title{Numeric second derivatives}
\usage{
secondDeriv(x, FUN, eps = 1e-08, ...)
}
\arguments{
\item{x}{The location (a vector) where the second derivatives 
of \code{FUN} are desired.}

\item{FUN}{An R function for which the second derivatives are 
sought.  
This must be a function of the form FUN <- function(x, ...)\{...\}
where x is a vector of variable parameters to FUN at which 
to evaluate the 2nd derivative, 
and ... are additional parameters needed to evaluate the function.
FUN must return a single value (scalar), the height of the 
surface above \code{x}, i.e., FUN evaluated at x.}

\item{eps}{A vector of small relative 
distances to add to \code{x} 
when evaluating derivatives.   This determines the '\eqn{dx}' of 
the numerical derivatives.  That is, the function 
is evaluated at \code{x}, \code{x+dx}, and \code{x+2*dx}, where 
\eqn{dx} = \code{x*eps^0.25}, in order to compute the second 
derivative. 
\code{eps} defaults to 1e-8 for all 
dimensions which equates to setting \eqn{dx} to one percent
of each \code{x} (i.e., by default the function is 
evaluate at \code{x}, \code{1.01*x} and \code{1.02*x} 
to compute the second derivative).  

One might want to change \code{eps} if the scale 
of dimensions in \code{x} varies wildly (e.g., kilometers and millimeters), 
or if changes between 
\code{FUN(x)} and \code{FUN(x*1.01)} are below machine precision.
If length of \code{eps} is less than length of \code{x},
\code{eps} is replicated to the length of \code{x}.}

\item{\dots}{Any arguments passed to \code{FUN}.}
}
\description{
Computes numeric second derivatives (hessian) of an 
arbitrary multidimensional function at a particular location.
}
\details{
This function uses the "5-point" numeric second derivative 
method advocated in numerous numerical recipe texts.  During computation
of the 2nd derivative, FUN must be 
capable of being evaluated at numerous locations within a hyper-ellipsoid 
with cardinal radii 2*\code{x}*(\code{eps})^0.25 = 0.02*\code{x} at the 
default value of \code{eps}.   

A handy way to use this function is to call an optimization routine 
like \code{nlminb} with FUN, then call this function with the 
optimized values (solution) and FUN.  This will yield the hessian 
at the solution and this is can produce a better 
estimate of the variance-covariance
matrix than using the hessian returned by some optimization routines. 
Some optimization routines return the hessian evaluated 
at the next-to-last step of optimization. 

An estimate of the variance-covariance matrix, which is used in 
\code{Rdistance}, is \code{solve(hessian)} where \code{hessian} is
\code{secondDeriv(<parameter estimates>, <likelihood>)}.
}
\examples{

func <- function(x){-x*x} # second derivative should be -2
secondDeriv(0,func)
secondDeriv(3,func)

func <- function(x){3 + 5*x^2 + 2*x^3} # second derivative should be 10+12x
secondDeriv(0,func)
secondDeriv(2,func)

func <- function(x){x[1]^2 + 5*x[2]^2} # should be rbind(c(2,0),c(0,10))
secondDeriv(c(1,1),func)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/halfnorm.like.R
\name{halfnorm.like}
\alias{halfnorm.like}
\title{Half-normal distance function}
\usage{
halfnorm.like(a, dist, covars, w.hi = NULL)
}
\arguments{
\item{a}{A vector or matrix of covariate 
and expansion term 
coefficients. If matrix, dimension is 
\emph{k} X \emph{p}, where 
\emph{k} = \code{nrow(a)}) is the number of coefficient
vectors to evaluate (cases) and \emph{p} 
= \code{ncol(a)})
is the number of covariate and expansion 
coefficients in the likelihood (i.e., rows are 
cases and columns are covariates). If \code{a} is a 
dimensionless vector, it is interpreted as a 
single row with \emph{k} = 1. 
Covariate coefficients in \code{a} are the first 
\emph{q} values (\emph{q} <= \emph{p}), and must 
be on a log scale.}

\item{dist}{A numeric vector of length \emph{n} or 
a single-column matrix (dimension \emph{n}X1) containing 
detection distances at which to evaluate the likelihood.}

\item{covars}{A numeric vector of length \emph{q} or a
matrix of dimension \emph{n}X\emph{q} containing 
covariate values 
associated with distances in argument \code{dist}.}

\item{w.hi}{A numeric scalar containing maximum 
distance. The right-hand cutoff or upper limit. 
Ignored by some likelihoods (such as halfnorm, 
negexp, and hazrate), but is a fixed parameter
in other likelihoods (such as oneStep 
and uniform).}
}
\value{
A list containing the following two components:
\itemize{
  \item \bold{L.unscaled}: A matrix of size 
   \emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
   cases = \code{nrow(a)})
   containing likelihood values evaluated at 
   distances in \code{dist}.
   Each row is associated with 
   a single distance, and each column is associated with 
   a single case (row of \code{a}).  Values in this matrix are  
   the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
   These values are "unscaled" likelihood values; they must be 
   scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi  
   to form proper likelihood values. 
   
 \item \bold{params}: A \emph{n}X\emph{b}X\emph{k}  array 
 of the likelihood's (canonical) parameters in link space (i.e., on 
 log scale; \emph{b} = number of canonical parameters in 
 the likelihood; \emph{k} = number of cases). 
 Rows correspond to distances in \code{dist}. Columns 
 correspond to parameters (columns of \code{a}), 
 and pages correspond to cases (rows of \code{a}). 
}
}
\description{
Evaluate the half-normal distance function, 
for sighting distances, potentially including covariates
and expansion terms
}
\details{
The half-normal distance function is 
\deqn{f(d|\sigma) = \exp(-\frac{d^2}{2\sigma^2})}{f(d|s) = exp(-d^2 / (2*s^2))}
where \eqn{\sigma = exp(x'a)}{s = exp(x'a)}, \eqn{x} is a vector of 
covariate values associated with distance \eqn{d} 
(i.e., a row of \code{covars}), and 
\eqn{a} is a vector of the first $q$ (=\code{ncol(covars)}) 
values in argument \code{a}.

Some authors parameterize the halfnorm without 
the "2" in the denominator of the exponent. 
\code{Rdistance} includes 
"2" in this denominator to make 
quantiles of the half normal agree with 
the standard normal. This means that half-normal 
coefficients in 
Rdistance (i.e., \eqn{\sigma = exp(x'a)}{s = exp(x'a)}) can be 
interpreted as normal standard errors.  
Approximately 95\% of distances should 
occur between 0 and 2\eqn{\sigma}{s}.
}
\examples{
 
d <- seq(0, 100, length=100)
covs <- matrix(1,length(d),1)
halfnorm.like(log(20), d, covs)

plot(d, halfnorm.like(log(20), d, covs)$L.unscaled, type="l", col="red")
lines(d, halfnorm.like(log(40), d, covs)$L.unscaled, col="blue")

# Evaluate 3 functions at once using matrix of coefficients:
# sigma ~ 20, 30, 40
coefs <- matrix(log(c(7.39,7.33, 4.48,44.80, 2.72,216.54))
               , byrow = TRUE
               , ncol=2) # (3 coef vectors)X(2 covars)
covs <- matrix(c(rep(1,length(d))
               , rep(0.5,length(d)))
               , nrow = length(d)) # 100 X 2
L <- halfnorm.like( coefs, d, covs ) 
L$L.unscaled # 100 X (3 coef vectors)
L$params     # 100 X (3 coef vectors); ~ log(c(20,30,40))
matplot(d, L$L.unscaled, type="l")

}
\seealso{
\code{\link{dfuncEstim}}, 
         \code{\link{abundEstim}},
         other \code{<likelihood>.like} functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predDensity.R
\name{predDensity}
\alias{predDensity}
\title{Density on transects}
\usage{
predDensity(object, propUnitSurveyed = 1)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that 
  was surveyed.  If both sides of line transects were observed, 
  \code{propUnitSurveyed}
  = 1.  If only a single side of line transects were observed, set 
  \code{propUnitSurveyed} = 0.5. For point transects, this should be set to 
  the proportion of each circle that was observed. Length must either be
  1 or the total number of transects in \code{x}.}
}
\value{
A data frame containing the original data used 
to fit the distance function, plus an additional column
containing the density of individuals on each transect.
}
\description{
An internal prediction method for computing density 
on the sampled transects.
}
\examples{
data(sparrowDfuncObserver)
predict(sparrowDfuncObserver, type="density")

 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateKey.R
\name{integrateKey}
\alias{integrateKey}
\title{Compute and print distance function integration}
\usage{
integrateKey(ml, key, f0, plot = FALSE)
}
\arguments{
\item{ml}{The fitted object or model list}

\item{key}{The scaled distance function.}

\item{f0}{The value of f(0) when integrating line transects. 
This should be the ESW for the case.}

\item{plot}{Logical scalar.  If TRUE, plot a diagnostic consisting
of the distance function and approximating points we use for quadrature.}
}
\value{
Nothing.  Prints information on integrals to the screen.
}
\description{
Check several integration characteristics, 
and report them to screen.  This was designed to print 
info when user asks for higher verbose level. The scaled
distance function should integrate to 1.0 every iteration of 
maximization, and this function checks that.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparrowDf.R
\docType{data}
\name{sparrowDf}
\alias{sparrowDf}
\title{Brewer's Sparrow detection data frame
in Rdistance >4.0.0 format.}
\format{
A rowwise tibble containing 72 rows and 9 columns, one of which 
is nested data frame of detections.  Each row represents
one transect.  The embedded data frame in column \code{detections} 
contains the detections made on the transect represented on that row. 

Column descriptions: 
\enumerate{ 
  \item \code{siteID}: Factor (72 levels), the transect identifier 
  for that row of the data frame. 
  \item \code{length}: The length, in meters [m], of each transect.  
  \item \code{observer}: Identity of the observer who surveyed the transect.
  \item \code{bare}: The mean bare ground cover (\%) within 100 [m] of the transect.
  \item \code{herb}: The mean herbaceous cover (\%) within 100 [m] of the transect.
  \item \code{shrub}: The mean shrub cover (\%) within 100 [m] of the transect.
  \item \code{height}: The mean shrub height [cm] within 100 [m] of the transect.
  \item \code{shrubclass}: Shrub class factor.  Either "Low"" when 
  shrub cover is < 10\%, or "High" if cover >= 10\%.  
}
 
The embedded data frame in column \code{detections} contains the following
variables:
\enumerate{
  \item \code{groupsize}: The number of individuals in the detected group.  
  \item \code{sightdist}: Distance [m] from observer to the detected group.  
  \item \code{sightangle}: Angle [degrees] from the transect 
  line to the detected group. Not bearing. Range 0 [degrees] to 90 [degrees].
  \item \code{dist}: Perpendicular, off-transect distance [m], from the
  transect to the detected group.  This is the distance used in analysis.
  Calculated using \code{\link{perpDists}}.  
}
}
\source{
The Brewer's sparrow data are a subset of data collected 
by Jason Carlisle and various field technicians for his Ph.D. from the 
Department of Ecology, University of Wyoming, in 2017.  This portion of 
Jason's work was funded by the Wyoming Game and Fish Department through agreements 
with the University of Wyoming's  Cooperative Fish & Wildlife
Research Unit (2012).
}
\description{
Detection data from line transect surveys for Brewer's sparrow on 72 transects 
located on a 4105 km^2 study area in central Wyoming collected by 
Dr. Jason Carlisle as part of his graduate work in the Wyoming Cooperative Fish & Wildlife
Research Unit in 2012. Each transect was 500 meters long.
}
\examples{
\dontrun{
# The following code generated 'sparrowDf'
data(sparrowDetectionData)
data(sparrowSiteData)
sparrowDf <- RdistDf(transectDf = sparrowSiteData
                   , detectionDf = sparrowDetectionData
                   , by = "siteID"
                   , pointSurvey = FALSE
                   , .effortCol = "length"
                    )
}

data(sparrowDf)
tidyr::unnest(sparrowDf, detections)  # only non-zero transects
Rdistance::unnest(sparrowDf) # with zero transects at the bottom
summary(sparrowDf,
  formula = dist ~ groupsize(groupsize)
)

}
\references{
Carlisle, J.D. 2017. The effect of sage-grouse conservation on
wildlife species of concern: implications for the umbrella species concept.
Dissertation. University of Wyoming, Laramie, Wyoming, USA.

Carlisle, J. D., and A. D. Chalfoun. 2020. The abundance of Greater 
Sage-Grouse as a proxy for the abundance of sagebrush-associated songbirds 
in Wyoming, USA. \emph{Avian Conservation and Ecology} 15(2):16. 
\doi{10.5751/ACE-01702-150216}
}
\seealso{
\code{\link{sparrowSiteData}}, \code{\link{sparrowDetectionData}},
\code{\link{RdistDf}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.RdistDf.R
\name{is.RdistDf}
\alias{is.RdistDf}
\title{Check RdistDf data frames}
\usage{
is.RdistDf(df, verbose = FALSE)
}
\arguments{
\item{df}{A data frame to check}

\item{verbose}{If TRUE, an explanation of the check that fails is printed.
Otherwise, no information on checks is provided.}
}
\value{
TRUE or FALSE invisibly. TRUE means all checks passed. FALSE implies
at least one check failed. Use \code{verbose} = TRUE to see which.
}
\description{
Checks the validity of \code{Rdistance} nested data frames.
\code{Rdistance} data frames
are a particular implementation of rowwise \code{tibbles}
that contain detections in a list column, and extra attributes
specifying types.
}
\details{
The following checks are performed (in this order):
\itemize{
\item \code{attr(df, "detectionColumn")} exists and points to a valid
list-based column in the data frame.
\item \code{attr(df, "obsType")} exists and is one of the valid values.
\item \code{attr(df, "transType")} exists and is one of the valid values.
\item \code{attr(df, "effortColumn")} exists and points to a column in the
data frame with 1 dimensional units attached.
\item The data frame is either a 'rowwise_df' or 'grouped_df'
\code{tibble}.
\item The data frame has only one row per group. One row per group
is implied by 'rowwise_df', but not a 'grouped_df', and both are allowed
in \code{Rdistance}. One row per group ensures rows are uniquely identified
and hence represents one transect.
\item No column names in the list-column are duplicated in the non-list
columns of the data frame. This check ensures that \code{tidyr::unnest}
executes.
}
Other data checks, e.g., for measurement units, are performed
later in \code{\link[=dfuncEstim]{dfuncEstim()}}, after the model is specified.
}
\examples{

data(sparrowDf)
is.RdistDf(sparrowDf)

# Data frame is okay, but no attributes
data(sparrowDetectionData)
data(sparrowSiteData)
sparrowDf <- sparrowDetectionData |> 
  dplyr::nest_by( siteID
               , .key = "distances") |> 
  dplyr::right_join(sparrowSiteData, by = "siteID")
is.RdistDf(sparrowDf, verbose = TRUE)

# Manually add required attributes
attr(sparrowDf, "detectionColumn") <- "distances"
attr(sparrowDf, "effortColumn") <- "length"
attr(sparrowDf, "obsType") <- "single"
attr(sparrowDf, "transType" ) <- "line"
is.RdistDf(sparrowDf, verbose = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/startLimits.R
\name{startLimits}
\alias{startLimits}
\title{Distance function starting values and limits}
\usage{
startLimits(ml)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}
}
\value{
A list containing the following components
\item{start}{Vector of starting values for parameters of the likelihood and expansion terms. }
\item{lowlimit}{Vector of lower limits for the likelihood parameters and expansion terms.}
\item{uplimit}{Vector of upper limits for the likelihood parameters and expansion terms.}
\item{names}{Vector of names for the likelihood parameters and expansion terms.}

The length of each vector in the return is:
\verb{(Num expansions) + 1 + 1*(like \%in\% c("hazrate")) + (Num Covars)}.
}
\description{
Returns starting values and limits (boundaries) for the parameters of
distance functions.  This function is called by
other routines in \code{Rdistance}, and is not intended to
be called by the user. Replace this function in the global
environment to change boundaries and starting values.
}
\examples{
  data(sparrowDf)
  
  # Half-normal start limits
  modList <- parseModel(
       data = sparrowDf
     , formula = dist ~ 1
     , likelihood = "halfnorm"
  )
  startLimits(modList)
  
  # Half-normal with expansions
  modList <- parseModel(
       data = sparrowDf
     , formula = dist ~ 1
     , likelihood = "halfnorm"
     , expansions = 3
  )
  startLimits(modList)
  
  # Hazard rate start limits
  modList$likelihood <- "hazrate"
  startLimits(modList)
  
  # Neg exp start limits
  modList$likelihood <- "negexp"
  startLimits(modList)
  
  
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abundEstim.R
\name{abundEstim}
\alias{abundEstim}
\title{Distance Sampling Abundance Estimates}
\usage{
abundEstim(
  object,
  area = NULL,
  propUnitSurveyed = 1,
  ci = 0.95,
  R = 500,
  plot.bs = FALSE,
  showProgress = TRUE,
  parallel = TRUE
)
}
\arguments{
\item{object}{A fitted Rdistance distance function (class \code{dfunc})
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{area}{A scalar containing the total area of inference. Usually, this is
study area size.  If \code{area} is NULL (the default),
\code{area} will be set to 1 square unit of the output units and density estimates
will be produced.
If \code{area} is not NULL, it must have measurement units
assigned by the \code{units} package.
The units on \code{area} must be convertible
to squared output units. Units
on \code{area} must be two-dimensional.
For example, if output units are "foo",
units on area must be convertible to "foo^2" by the \code{units}
package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
others are acceptable.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that
was surveyed.  If both sides of line transects were observed,
\code{propUnitSurveyed}
= 1.  If only a single side of line transects were observed, set
\code{propUnitSurveyed} = 0.5. For point transects, this should be set to
the proportion of each circle that was observed. Length must either be
1 or the total number of transects in \code{x}.}

\item{ci}{A scalar indicating the confidence level of confidence intervals.
Confidence intervals are computed using a bias corrected bootstrap
method. If \code{ci = NULL} or \code{ci == NA}, confidence intervals
are not computed.}

\item{R}{The number of bootstrap iterations to conduct when \code{ci} is not
NULL.}

\item{plot.bs}{A logical scalar indicating whether to plot individual
bootstrap iterations.
Ignored unless \code{parallel = FALSE}.}

\item{showProgress}{A logical indicating whether to show a text-based
progress bar during bootstrapping. Default is \code{TRUE}.
It is handy to shut off the
progress bar if running this within another function.
Ignored unless \code{parallel = FALSE}.}

\item{parallel}{A logical scalar, or a positive integer; ignored unless
confidence intervals are requested (i.e., \code{!is.null(ci)}).
If TRUE, bootstrap iterations are
run in parallel using the maximum number of CPU cores minus 1.
The maximum number of CPU cores is reported by \code{parallel::detectCores()}.
If a positive integer (1 <= \code{parallel} <= maximum cores), bootstrap
iterations are performed in parallel on that many cores.
If FALSE, bootstrap iterations are performed in series, and progress
will be shown if \code{showProgress == TRUE}. Parameters
\code{showProgress} and \code{plot.bs} are ignored when operating
in parallel.}
}
\value{
An Rdistance 'abundance estimate' object, which is a list of
class \code{c("abund", "dfunc")}, containing all the components of a "dfunc"
object (see \code{\link[=dfuncEstim]{dfuncEstim()}}), plus the following:

\item{estimates}{A tibble containing fitted coefficients in the
distance function, density in the area(s) surveyed,
abundance on the study area, the number of groups seen
between w.lo and w.hi, the number
of individuals seen between w.lo and w.hi,
study area size, surveyed area, average group size, and
average effective detection distance.
}

\item{B}{If confidence intervals were requested, a tibble
containing all bootstrap values of coefficients,
density, abundance, groups seen, individuals seen,
study area size, surveyed area size, average group size,
and average effective detection distance.  The number of rows is always
\code{R}, the requested number of bootstrap
iterations.  If an iteration fails, the
corresponding row in \code{B} is \code{NA} (hence, use \code{na.rm = TRUE}
when computing summaries). Columns 1 through \code{length(coef(dfunc))}
contain bootstrap realizations of the distance function's coefficients.
}

\item{ci}{Confidence level of the confidence intervals}
}
\description{
Estimate abundance (or density) from an estimated detection
function and supplemental information on observed group sizes, transect
lengths, area surveyed, etc.  Computes confidence intervals on
abundance (or density) using a the bias corrected bootstrap method.
}
\details{
The abundance estimate for line-transect surveys (if no covariates
are included in the detection function and both sides of the transect
are observed) is
\deqn{N =\frac{n(A)}{2(ESW)(L)}}{%
         N = n*A / (2*ESW*L)}
where \emph{n} is total number of sighted individuals
(i.e., \code{sum(groupSizes(dfunc))}), \emph{L} is the total length of
surveyed transect (i.e., \code{sum(effort(dfunc))}),
and \emph{ESW} is effective strip width
computed from the estimated distance function (i.e., \code{ESW(dfunc)}).
If only one side of transects were observed, the "2" in the denominator
is not present (or, replaced with a "1").

The abundance estimate for point transect surveys (if no covariates are
included) is
\deqn{N =\frac{n(A)}{\pi(ESR^2)(P)}}{%
         N = n*A / ((3.1415)*ESR^2*(P))}
where \emph{n} is total number of sighted individuals (i.e., \code{sum(groupSizes(dfunc))}),
\emph{P} is the total number of surveyed points (i.e., \code{sum(effort(dfunc))}),
and \emph{ESR} is effective search radius
computed from the estimated distance function (i.e., \code{ESR(dfunc)}).

This routine, \code{abundEstim}, estimates abundance on the
entire study area.  Site-specific density estimates
are computed by
\code{predict(x, type = "density")}, which returns a
tibble containing density and abundance on the area surveyed by every
transect.
}
\section{Bootstrap Confidence Intervals:}{
Rdistance's nested data frames (produced by \code{\link[=RdistDf]{RdistDf()}})
contain all information required to estimate bootstrap CIs.
To compute bootstrap CIs, Rdistance resamples, with replacement,
the rows of the \verb{$data} component contained in Rdistance
fitted models. Rdistance assumes each row of \verb{$data}
contains information on one transect.
The \verb{$data} component also contains
information on which observations inform the
detection function, which observations should be counted as
detected targets,
and which transects count toward transect length.
After resampling rows of \verb{$data}, Rdistance
refits the distance function using non-missing distances,
recomputes the detected number of targets using non-missing
group sizes on transects with non-missing length,
and re-computes total transect length from transects
with non-missing lengths.
By default, \code{R} = 500 bootstrap iterations are
performed, after which bias
corrected confidence intervals are computed (Manly, 1997, section 3.4).

The distance function is not re-selected during bootstrap resampling. The
model of the input object is re-fitted every iteration.

During bootstrap iterations, the distance function can fail.
An iteration can fail for a two reasons:
(1) no detections on the iteration, and (2) a bad configuration
of distances that push the distance function's parameters to their
limits. When an iteration fails, Rdistance
skips the iteration and effectively ignores the
failed iterations.
If the proportion of failed iterations is small
(less than 20\% by default), the resulting abundance confidence interval
is probably valid and no warning is issued.  If the proportion of
non-convergent iterations
is not small (exceeds 20\% by default), a warning is issued.
The warning can be modified
by re-setting option \code{"Rdistance_maxBSFailPropForWarning"} to
the acceptable proportion of failures..
Setting \code{options(Rdistance_masBSFailPropForWarning = 1.0)} will turn
suppress the warning.
Setting \code{options(Rdistance_masBSFailPropForWarning = 0.0)} will
warn if any iteration failed.  Results (density and effective
sampling distance)
from all successful iterations are contained in the
non-NA rows of data frame 'B' in the output object.
}

\section{Missing Transect Lengths}{


Transect lengths can be missing in the RdistDf object.
Missing length transects are equivalent
to 0 [m] transects and do not count toward total surveyed units,
nor do group sizes on these transects count toward total
detected individuals.
Use NA-length transects to include their associated distances
during distance function estimation, but not when estimating abundance.
For example, this allows estimation of abundance on one
study area using off-transect distances from another.  This allows
sightability to be estimated using two or more similar targets (e.g.,
two similar species), but abundance to be estimated separate for each
target type.
Include NA-length transects by including the "extra" distance observations
in the detection data frame, with valid
site IDs, but set the length of those site IDs to NA in the site data frame.
}

\section{Point Transect Lengths}{

Point transects do not have a physical measurement for length.
The "length" of point transects is the number of points on the transect.
Point transects can contain only one point.
Rdistance treats transects of points as independent
and bootstrap resamples them to estimate variance. The number of points
on each point transect must exist in the RdistDf and cannot have
physical measurement units (it is a count, not a distance).
}

\examples{
# Load example sparrow data (line transect survey type)
# sparrowDf <- RdistDf(sparrowSiteData, sparrowDetectionData)
data(sparrowDf)

# Fit half-normal detection function
dfunc <- sparrowDf |> 
  dfuncEstim(formula=dist ~ groupsize(groupsize)
           , likelihood="halfnorm"
           , w.hi=150 \%m\%.
  )

# Estimate abundance - Convenient for programming 
abundDf <- estimateN(dfunc
                   , area = 4105 \%km^2\%.
           )

# Same - Nicer output 
# Set ci=0.95 (or another value) to estimate bootstrap CI's 
fit <- abundEstim(dfunc
                , area = 4105 \%km^2\%.
                , ci = NULL
                )
summary(fit)
         
}
\references{
Manly, B.F.J. (1997) \emph{Randomization, bootstrap, and
Monte-Carlo methods in biology}, London: Chapman and Hall.

Buckland, S.T., D.R. Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers,
and L. Thomas. (2001) \emph{Introduction to distance sampling: estimating
abundance of biological populations}. Oxford University Press, Oxford, UK.
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}, \code{\link[=autoDistSamp]{autoDistSamp()}},
\code{\link[=predict.dfunc]{predict.dfunc()}} with 'type = "density"'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dfuncEstim.R
\name{dfuncEstim}
\alias{dfuncEstim}
\title{Estimate a distance-based detection function}
\usage{
dfuncEstim(data, ...)
}
\arguments{
\item{data}{An \code{RdistDf} data frame. \code{RdistDf} data frames
contain one line per transect and a list-based column. The list-based
column contains a data frame with detection information.
The detection information data frame on each row contains (at least) distances
and group sizes of all targets detected on the transect.
Function \code{\link[=RdistDf]{RdistDf()}} creates \code{RdistDf} data frames
from separate transect and detection data frames.
\code{\link[=is.RdistDf]{is.RdistDf()}} checks whether data frames
are \code{RdistDf}'s.}

\item{...}{
  Arguments passed on to \code{\link[=dE.single]{dE.single}}, \code{\link[=dE.multi]{dE.multi}}
  \describe{
    \item{\code{formula}}{A standard formula object.  For example, \code{dist ~ 1},
\code{dist ~ covar1 + covar2}). The left-hand side (before \code{~})
is the name of the vector containing off-transect or radial detection distances.
The right-hand side contains the names of covariate
vectors to fit in the detection
function, and potentially group sizes. Group sizes are specified
by including \verb{+ groupsize(<variable>)} in the RHS
(see 'Group Sizes' section).
Covariates can be either detection level
or transect level and can appear in  \code{data} or exist in the
global working environment. Regular R scoping
rules apply.}
    \item{\code{likelihood}}{String specifying the likelihood to fit. Built-in
likelihoods at present are "halfnorm", "hazrate", and "negexp".}
    \item{\code{w.lo}}{Lower or left-truncation limit of the distances in distance data.
This is the minimum possible off-transect distance. Default is 0.  If
\code{w.lo} is greater than 0, it must have measurement units.
See \code{help(unitHelpers)} for assistance assigning units.}
    \item{\code{w.hi}}{Upper or right-truncation limit of the distances
in \code{dist}. This is the maximum off-transect distance that
could be observed. If unspecified (i.e., NULL),
right-truncation is set to the maximum of the observed
distances.  If \code{w.hi} is specified, it must have
measurement units.
See \code{help(unitHelpers)} for assistance assigning units.}
    \item{\code{expansions}}{A scalar specifying the number of terms
in \code{series} to compute. Depending on the series,
this could be 0 through 5.  The default of 0 equates
to no expansion terms of any type.  No expansion terms
are allowed (i.e., \code{expansions} is forced to 0) if
covariates are present in the detection function
(i.e., right-hand side of \code{formula} includes
something other than \code{1}).}
    \item{\code{series}}{If \code{expansions} > 0, this string
specifies the type of expansion to use. Valid values at
present are 'simple', 'hermite', and 'cosine'.}
    \item{\code{x.scl}}{The x coordinate (a distance) at which the
detection function will be scaled.  \code{g.x.scl} can be a distance
or the string "max".
When \code{x.scl} is specified (i.e., not 0 or "max"), it must have measurement
units assigned.
See \code{help(unitHelpers)} for assistance assigning units.}
    \item{\code{g.x.scl}}{Height of the distance function at coordinate \code{x}.
The distance function
will be scaled so that g(\code{x.scl}) = \code{g.x.scl}.
If \code{g.x.scl} is not
a data frame, it must be a numeric value (vector of length 1)
between 0 and 1.}
    \item{\code{warn}}{A logical scalar specifying whether to issue
an R warning if the estimation did not converge or if one
or more parameter estimates are at their boundaries.
For estimation, \code{warn} should generally be left at
its default value of \code{TRUE}.  When computing bootstrap
confidence intervals, setting \code{warn = FALSE}
turns off annoying warnings when an iteration does
not converge.  Regardless of \code{warn}, after
completion all messages about
convergence and boundary conditions are printed
by \code{print.dfunc}, \code{print.abund}, and
\code{plot.dfunc}.}
    \item{\code{outputUnits}}{A string specifying the symbolic measurement
units for results. Valid units are listed in \code{units::valid_udunits()}.
The strings for common distance symbolic units are:
"m" - meters, "ft" - feet, "cm" - centimeters, "mm" -
millimeters, "mi" - miles, "nmile" -
nautical miles ("nm" is nano meters), "in" - inches,
"yd" - yards, "km" - kilometers, "fathom" - fathoms,
"chains" - chains, and "furlong" - furlongs.
If \code{outputUnits} is unspecified (NULL),
output units will be the same as those on
distances in \code{data}.}
  }}
}
\value{
An object of class 'dfunc' with the following components:
\itemize{
\item \code{par}: The vector of estimated parameter values.
Length of this vector is the sum of the following:
\enumerate{
\item The number of columns of the design matrix. This equals the
number of covariates in the distance function plus one for the
intercept, assuming an intercept is included.
\item The number of constant parameters in the distance function.
Constant parameters are those not related to covariates.  For example,
the exponent 'k' parameter for hazard rate likelihood, or
the mixing fraction 'p' for the oneStep likelihood. This can be zero.
\item The number of expansion functions called for.  This equals
the input \code{expansions}.
}
\item \code{loglik}: The maximized value of the log likelihood.
\item \code{convergence}: The convergence code. This code
is returned by the optimizing routine (e.g., \code{optim} or \code{nlminb}).
Values other than 0 indicate suspect convergence.
\item \code{message}: If maximization did not converge (\code{convergence != 0}),
this is the reason given by the optimizing routine.
\item \code{varcovar}: The variance-covariance matrix for coefficients
of the distance function, either estimated by the inverse of
the fit's Hessian or by bootstrapping.
If the likelihood is smooth (i.e., those listed by
\verb{Rdistance:::differentiableLikelihoods())},
Rdistance initially estimates the variance-covariance matrix using the
second derivative of the log likelihood surface
at the final estimates, where second derivatives are estimated by
numeric differentiation (in routine \code{\link[=secondDeriv]{secondDeriv()}}.
The variance-covariance matrix is re-set to NULL
if the Hessian is not positive-definite.  If bootstrap resampling
has been performed (using \code{\link[=abundEstim]{abundEstim()}}), the variance-covariance
matrix is re-estimated using the bootstrap values of parameters
and automatically reset.
Error estimates derived from bootstrapping are generally
preferable to the asymptotic estimates, hence the automatic
re-set.
\item \code{limits}: A list containing the lower and upper limits of parameters.
\item \code{evaluations}: The number of likelihood evaluations performed by the
optimizer.
\item \code{mf}: An R 'model frame' containing the detections (within the strip
or circle) used in the fit, covariates specified in the formula,
and groupsizes.  Column 'dist' contains the
observed distances. The intercept, if included in the model, is not
included as a column in this model frame. (Test whether an intercept
is included using \code{attr(terms(return$mf), "intercept")}).
Column \code{offset(...)} contains group sizes associated with
the values of \code{dist}. Name of the group size column is "offset(...)",
not "groupsize(...)", so that group sizes can be treated offsets in
other R routines.  The \code{mf} component is a proper \code{model.frame} and contains
both terms and contrasts attributes. This model frame
contains only non-missing distances between \code{w.lo} and \code{w.hi}.
\item \code{data}: The original nested data frame subset to information required
to complete distance estimation.  This data frame contains information
on replication (i.e., rows are sites and are re-sampled during bootstrapping),
missing distances, missing transect lengths, and distances outside the observation
strip (below \code{w.lo} or above \code{w.hi}).
\item \code{formula}: The distance function's formula.
\item \code{dataName}: Name of the original nested data frame.
\item \code{likelihood}: The name of the likelihood fitted to observation
distances.
\item \code{w.lo}: Left-truncation value used during the fit.
\item \code{w.hi}: Right-truncation value used during the fit.
\item \code{expansions}: The number of expansion terms.
\item \code{series}: The type of expansion used during estimation. This is
only relevant if \code{expansions > 0}.
\item \code{x.scl}: The distance at which the function has been scaled to some value.
This is the \emph{x} at which g(\emph{x}) = \code{g.x.scl}.
\item \code{g.x.scl}: The height of the distance function at distance \code{x.scl}.
\item \code{outputUnits}: A list of type \code{symbolic_units} containing the
physical measurement units used during estimation.
\item \code{asymptoticSE}: A logical scalar indication whether the
variance-covariance matrix in component \code{varcovar} is
asymptotic (TRUE; estimated from the Hessian) or bootstrap (FALSE;
estimated by bootstrap resampling).
\item \code{optimizer}: The optimizing routine used.
\item \code{call}: The original function call.
\item \code{nCovars}: The number of exogenous covariates fitted in the
distance function. Does not include the intercept.
\item \code{LhoodType}: The type of likelihood fitted. Currently, only 'parametric'
types are fitted.
}
}
\description{
Fits a detection function using maximum likelihood.
}
\details{
Optimization and estimation controls can be modified using \code{options()}.
See \code{\link[=RdistanceControls]{RdistanceControls()}}.
}
\section{Group Sizes}{

To specify non-unity group sizes,  use \code{groupsize()}
on the RHS of \code{formula}. When group sizes are not all 1, they must appear in a column
of the 'detections' list-column of \code{data}.
For example, \code{d ~ habitat + groupsize(number)} specifies
distances in column \code{d}, one covariate
named \code{habitat}, and that column \code{number}
contains the number of individuals
associated with each detection.  If group sizes are not specified,
all group sizes are assumed to be 1.
}

\section{Contrasts}{

Factor contrasts in \code{Rdistance} are specified
the same way as in \code{lm} or \code{glm}.
By default, \code{Rdistance} uses
contrasts in \code{getOption("contrasts")}.  To change contrasts, use a statement
like \code{options(contrasts = c(unordered = "contr.SAS",  ordered = "contr.poly"))}.  Or, to set contrasts for a
specific factor in the input data frame, use
\code{contrasts(df$A) <- "contr.sum"} or similar.
See \code{\link[=contrasts]{contrasts()}} or the \code{contrasts.arg}
of \code{\link[=model.matrix]{model.matrix()}}.
}

\section{Measurement Units}{

As of \code{Rdistance} version 3.0.0, measurement units are
require on all physical distances.
Requiring units ensures that internal calculations and results
(e.g., ESW and abundance) are correct
and that output units are clear.
Physical distances are required on
off-transect distances, radial distances, truncation distances
(\code{w.lo}, unless it is zero; and \code{w.hi}, unless it is NULL),
scale locations (\code{x.scl}, unless it is zero),
line-transect lengths, and study area size. All units are
1-dimensional except those on study area, which are 2-dimensional.

Physical measurement units can vary. For example,
off-transect distances can be meters ("m"), \code{w.hi} can be inches ("in"),
and \code{w.lo} can be kilometers ("km").  Internally, all distances are
converted to the units specified by \code{outputUnits}
(or the units of input distances if
\code{outputUnits} is NULL), and
all output is reported
in units of \code{outputUnits}. Valid conversions must exist between
units or an error is thrown (e.g., meters cannot convert
into hectares).

Measurement units can be assigned using one of Rdistance's
unit helper routines (see \code{help(unitHelpers)}), Rdistance's
\code{\link[=setUnits]{setUnits()}} function, or \code{\link[units:units]{units::set_units()}}
See \code{\link[units:valid_udunits]{units::valid_udunits()}}
for a list of valid symbolic units.

If measurements are truly unit-less, or measurement units are unknown,
set \code{options(Rdist_requireUnits = FALSE)}.  This suppresses
all unit checks and conversions.  Users are on their own here
and must make sure all inputs are scaled correctly so that internal
computations are correct and output units are known.
}

\examples{
# Sparrow line transect example
data(sparrowDetectionData)
data(sparrowSiteData)

sparrowDf <- RdistDf(sparrowSiteData, sparrowDetectionData)

dfunc <- dfuncEstim(sparrowDf, 
                    formula = dist ~ 1
                  )
summary(dfunc)

  
data(sparrowDfuncObserver) # pre-estimated object
\dontrun{                 
# Commands that produced 'sparrowDfuncObserver'
sparrowDfuncObserver <- sparrowDf |> 
         dfuncEstim( 
           formula = dist ~ observer
         )
}     
sparrowDfuncObserver
summary(sparrowDfuncObserver)
plot(sparrowDfuncObserver)
plot(sparrowDfuncObserver
   , newdata = data.frame(observer = c("obs1", "obs2", "obs3"
                                     , "obs4", "obs5")))

}
\references{
Buckland, S.T., D.R. Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers,
and L. Thomas. (2001) \emph{Introduction to distance sampling: estimating
abundance of biological populations}. Oxford University Press, Oxford, UK.
}
\seealso{
\code{\link[=abundEstim]{abundEstim()}}, \code{\link[=autoDistSamp]{autoDistSamp()}}.
Likelihood-specific help files (e.g., \code{\link[=halfnorm.like]{halfnorm.like()}}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.points.R
\name{is.points}
\alias{is.points}
\title{Tests for point surveys}
\usage{
is.points(x)
}
\arguments{
\item{x}{Either an estimated distance function, output by
\code{dfuncEstim}, or an Rdistance nested data frame, output by
\code{RdistDf}.}
}
\value{
TRUE if the model frame or fitted distance function
contains point surveys.  FALSE if the model frame or distance
function contains line transect surveys.
}
\description{
Determines whether a distance function is for a point survey
or line survey.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effort.R
\name{effort}
\alias{effort}
\title{Effort information}
\usage{
effort(x, ...)
}
\arguments{
\item{x}{Either an estimated distance function, output by
\code{dfuncEstim}, or an Rdistance nested data frame, output by
\code{RdistDf}.}

\item{...}{Ignored}
}
\value{
A vector containing effort. If line-transects, return is
length of transects, with units.  If point-transects, return is
number of points (integers, no units).  Vector length is
number of transects. If input is not an RdistDf or estimated distance
function, return is NULL.
}
\description{
Extract effort information from an Rdistance data frame. Effort is
length of line-transects or number of points on point-transects.
}
\examples{

data(sparrowDf)
effort(sparrowDf)
fit <- dfuncEstim(sparrowDf, dist ~ 1)
effort(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AIC.dfunc.R
\name{AIC.dfunc}
\alias{AIC.dfunc}
\title{AIC-related fit statistics for detection functions}
\usage{
\method{AIC}{dfunc}(object, ..., criterion = "AICc")
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{...}{Included for compatibility with generic \code{predict} methods.}

\item{criterion}{String specifying the criterion to compute. Either 
"AICc", "AIC", or "BIC".}
}
\value{
A scalar, the requested fit statistic for \code{object}.
}
\description{
Computes AICc, AIC, or BIC for estimated distance functions.
}
\details{
Regular Akaike's information criterion 
  (\url{https://en.wikipedia.org/wiki/Akaike_information_criterion}) (\eqn{AIC}) is 
  \deqn{AIC = LL + 2p,}{AIC = (LL) + 2p,}
  where \eqn{LL} is the maximized value of the log likelihood 
  (the minimized value of the negative log 
  likelihood) and \eqn{p} is the 
  number of coefficients estimated in the detection function.  For 
  \code{dfunc} objects, \eqn{AIC} = \code{obj$loglik + 2*length(coef(obj))}.  
  
  A correction 
  for small sample size, \eqn{AIC_c}{AICc}, is 
  \deqn{AIC_c = LL + 2p + \frac{2p(p+1)}{n-p-1},}{AIC_c = LL + 2p + (2p(p+1))/(n-p-1),} 
  where \eqn{n} is sample 
  size or number of detected groups for distance analyses.  By default, this function 
  computes \eqn{AIC_c}{AICc}.   \eqn{AIC_c}{AICc} converges quickly to \eqn{AIC} 
  as \eqn{n} increases.
  
  The Bayesian Information Criterion (BIC) is
  \deqn{BIC = LL + log(n)p,}{BIC = (LL) + log(n)p}.
}
\examples{
data(sparrowDf)
dfunc <- sparrowDf |> dfuncEstim(dist~1)
  
# Fit statistics
AIC(dfunc)  # AICc
AIC(dfunc, criterion="AIC")  # AIC
AIC(dfunc, criterion="BIC")  # BIC

}
\references{
Burnham, K. P., and D. R. Anderson, 2002. 
\emph{Model selection and multi-model inference: 
   A practical information-theoretic approach, Second ed.} 
   Springer-Verlag. ISBN 0-387-95364-7.
  
  McQuarrie, A. D. R., and Tsai, C.-L., 1998. \emph{Regression and 
  time series model selection.} 
  World Scientific. ISBN 981023242X
}
\seealso{
\code{\link{coef}}, \code{\link{dfuncEstim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkNEvalPts.R
\name{checkNEvalPts}
\alias{checkNEvalPts}
\title{Check number of numeric integration intervals}
\usage{
checkNEvalPts(nEvalPts)
}
\arguments{
\item{nEvalPts}{An integer to check.}
}
\value{
The first element of nEvalPts is returned if it is acceptable.
If nEvalPts is not acceptable, an error is thrown. Simpson's composite
coefficients are store in options()
}
\description{
Check that number of integration intervals is odd and 
sufficiently large. Plus, compute and store the Simpson's Composite
rule coefficients.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oneStep.start.limits.R
\name{oneStep.start.limits}
\alias{oneStep.start.limits}
\title{oneStep likelihood start and limit values}
\usage{
oneStep.start.limits(ml)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance 
'fitted object'.  Both are of class "dfunc". 
Rdistance 'model frames' are lists containing components 
necessary to estimate a distance function, but no estimates. 
Rdistance 'model frames' are typically
produced by calls to \code{\link{parseModel}}. 
Rdistance 'fitted objects'
are typically produced by calls to \code{\link{dfuncEstim}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates, 
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}
}
\value{
A list containing the following components
  \item{start}{Vector of starting values for parameters of the likelihood and expansion terms. }
  \item{lowlimit}{Vector of lower limits for the likelihood parameters and expansion terms.}
  \item{uplimit}{Vector of upper limits for the likelihood parameters and expansion terms.}
  \item{names}{Vector of names for the likelihood parameters and expansion terms.}
  
The length of each vector in the return is:
  \code{(Num expansions) + 1 + 1*(like \%in\% c("hazrate")) + (Num Covars)}.
}
\description{
Compute starting values and limits 
for the oneStep distance function.
}
\examples{
# make 'model list' object
# Boundary is 10, p is 100 / 120 = 0.833
library(Rdistance)
whi <- 50
x <- c( runif(100, min=0, max=10), runif(20, min=10, max=whi))
x <- setUnits(x, "m")
detectDf <- data.frame(transect = 1, dist = x)
siteDf <- data.frame(transect = 1, length = setUnits(10,"m"))
distDf <- RdistDf(siteDf, detectDf)
ml <- parseModel(distDf
            , formula = dist ~ 1
            , w.lo = 0
            , w.hi = setUnits(whi, "m")
            )
            

sl <- oneStep.start.limits(ml)
hist(x, n = 20)
abline(v = exp(sl$start["(Intercept)"]))


}
\seealso{
\code{\link{oneStep.like}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\title{Perform bootstrap iterations}
\usage{
bootstrap(
  object,
  area,
  propUnitSurveyed,
  R,
  plot.bs,
  plotCovValues,
  showProgress,
  parallel,
  cores
)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{area}{A scalar containing the total area of inference. Usually, this is 
study area size.  If \code{area} is NULL (the default), 
\code{area} will be set to 1 square unit of the output units and density estimates
will be produced. 
If \code{area} is not NULL, it must have measurement units 
assigned by the \code{units} package. 
The units on \code{area} must be convertible
to squared output units. Units 
on \code{area} must be two-dimensional. 
For example, if output units are "foo", 
units on area must be convertible to "foo^2" by the \code{units}
package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
others are acceptable.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that 
  was surveyed.  If both sides of line transects were observed, 
  \code{propUnitSurveyed}
  = 1.  If only a single side of line transects were observed, set 
  \code{propUnitSurveyed} = 0.5. For point transects, this should be set to 
  the proportion of each circle that was observed. Length must either be
  1 or the total number of transects in \code{x}.}

\item{R}{The number of bootstrap iterations to conduct when \code{ci} is not
NULL.}

\item{plot.bs}{A logical scalar indicating whether to plot individual
bootstrap iterations.
Ignored unless \code{parallel = FALSE}.}

\item{plotCovValues}{Data frame containing values of covariates to plot.
Ignored if \code{plot.bs} is FALSE.}

\item{showProgress}{A logical indicating whether to show a text-based
progress bar during bootstrapping. Default is \code{TRUE}. 
It is handy to shut off the 
progress bar if running this within another function. 
Ignored unless \code{parallel = FALSE}.}

\item{parallel}{Logical scalar. TRUE if we are running iterations 
in parallel across CPU cores.  Number of cores specified in \code{cores}.}

\item{cores}{Integer scalar.  The number of CPU cores to use during 
parallel operations, if requested.  Ignored if \code{parallel == FALSE}.}
}
\value{
A data frame containing density, abundance,  
and other relevant statistics for every bootstrap iteration.
Number of rows is \code{R}.  If the model from one iteration failed 
for any reason (e.g., non-convergence), the entire row except the ID column
is missing.
}
\description{
Performs bootstrap resampling iterations, either in parallel across 
CPU cores or in serial on a single core.
}
\seealso{
\code{\link{abundEstim}}; \code{\link{oneBsIter}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predLikelihood.R
\name{predLikelihood}
\alias{predLikelihood}
\title{Distance function values at observations}
\usage{
predLikelihood(object, params)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{params}{A matrix of distance function parameters. 
Rows are observations, columns contain the set of parameters
(canonical and expansion) for each observation.}
}
\value{
A vector of distance function values, of length 
n = number of observed distances = length(distances(x)). 
Elements in \code{distances(x)} correspond, in order, 
to values in the returned vector.
}
\description{
An internal prediction function to predict (compute) 
the values of distance functions at a set of observed values. 
Unlike \code{predDfuncs}, which evaluates distance
functions at EVERY input distance, this routine evaluates 
distance functions at only ONE distance. This is what's 
appropriate for likelihood computation.
This version allows for matrix inputs and 
uses matrix operations, and is thus faster than earlier
looping versions.
}
\details{
Assuming \code{L} is the vector returned by this function, 
the negative log likelihood is \code{-sum(log(L / I), na.rm=T)}, 
where \code{I} is the integration constant, or 
area under the likelihood between 
\code{w.lo} and \code{w.hi}. 
Note that returned likelihood values for distances less 
than \code{w.lo} or greater than \code{w.hi} are \code{NA}; 
hence, \code{na.rm=TRUE} in the sum.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple.expansion.r
\name{simple.expansion}
\alias{simple.expansion}
\title{Simple polynomial expansion factors}
\usage{
simple.expansion(x, expansions)
}
\arguments{
\item{x}{A numeric matrix of distances at which to evaluate 
the expansion series. For distance analysis, \code{x} should
be the proportion of the maximum sighting distance
at which a group was sighted, i.e., \eqn{x = d/w}, where \eqn{d}
is sighting distance and \eqn{w} is maximum sighting distance.}

\item{expansions}{A scalar specifying the number of 
expansion terms to compute. Must be one of the 
integers 1, 2, 3, 4, or 5.}
}
\value{
A 3D array of size \code{nrow(x)} X \code{ncol(x)} X \code{expansions}.
  The 'pages' (3rd dimension) of this array are the cosine expansions of 
  \code{x}. i.e., page 1 is the first expansion term of \code{x}, 
  page 2 is the second expansion term of \code{x}, etc.
}
\description{
Computes simple polynomial expansion terms 
for use in distance analysis. The Simple (and other expansions) 
allow "wiggle" in estimated distance functions.
}
\details{
The polynomials computed here are:
  \itemize{
    \item \bold{First term}: \deqn{h_1(x)=x^4,}{h1(x) = x^4,}
    \item \bold{Second term}: \deqn{h_2(x)=x^6,}{h2(x) = x^6,}
    \item \bold{Third term}: \deqn{h_3(x)=x^8,}{h3(x) = x^8,}
    \item \bold{Fourth term}: \deqn{h_4(x)=x^{10},}{h4(x) = x^10,}
  }
  The maximum number of expansion terms computed is 4.
}
\examples{
x <- matrix(seq(0, 1, length = 200), ncol = 1)
simp.expn <- simple.expansion(x, 4)
plot(range(x), range(simp.expn), type="n")
matlines(x, simp.expn[,1,1:4], col=rainbow(4), lty = 1)

}
\seealso{
\code{\link{dfuncEstim}}
, \code{\link{cosine.expansion}}
, \code{\link{sine.expansion}}
, \code{\link{hermite.expansion}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.matrix.dfunc.R
\name{model.matrix.dfunc}
\alias{model.matrix.dfunc}
\title{Rdistance model matrix}
\usage{
\method{model.matrix}{dfunc}(object, \dots)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{...}{Ignored}
}
\value{
A matrix containing covariates for fitting an
Rdistance model.
}
\description{
Extract the model matrix ("X" matrix) from an Rdistance model object.
}
\examples{

data(sparrowDf)
sparrowModel <- parseModel( sparrowDf, dist ~ observer )
model.matrix(sparrowModel)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorize.R
\name{colorize}
\alias{colorize}
\title{Add color to result if terminal accepts it}
\usage{
colorize(STR, col = NULL, bg = NULL)
}
\arguments{
\item{STR}{A vector of strings to colorize.}

\item{col}{A string specifying the desired foreground color.
This is passed straight to \code{crayon::style} and so must be 
recognized as one of the 8 base crayon colors. i.e., 
"black", "red", "green", "yellow", "blue", "magenta", "cyan", 
"white", and "silver" (silver = gray). By default, numbers 
are styled in "green".}

\item{bg}{A string specifying the desired background color. 
Must be one of "bgBlack", "bgRed", "bgGreen", "bgYellow", "bgBlue"
"bgMagenta", "bgCyan", or "bgWhite".  By default, no background 
is applied.}
}
\value{
If color is not allowed in the terminal, the input 
string is returned unperturbed.  If color is allowed, the input 
string is returned with color and background ANSI code surrounding 
the initial part of the string from character 1 to the character 
before the [ in the first pair of [].
}
\description{
Add ANSI color to a string using the 
\code{crayon} package, if the R environment accepts color. 
This function is needed because some  
output cannot be colorized.  Color determination 
is made by \code{crayon::has_color()}. 

Rdistance results often include units, e.g., "25 [m]". 
Only colorize the numbers, not the units.  Everything between the 
first set of square brackets (\code{[...]}) is NOT colorized. 
Subsequent sets of brackets (e.g., "25 [m] + 30 [ft]") ARE colorized
(i.e., "[ft]" is color).
}
\seealso{
\code{crayon::style}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.rowwise_df.R
\name{summary.rowwise_df}
\alias{summary.rowwise_df}
\title{Summary method for Rdistance data frames}
\usage{
\method{summary}{rowwise_df}(object, formula = NULL, w.lo = 0, w.hi = NULL, ...)
}
\arguments{
\item{object}{An \code{RdistDf} data frame.}

\item{formula}{A standard formula object.  For example, \code{dist ~ 1},
\code{dist ~ covar1 + covar2}). The left-hand side (before \code{~})
is the name of the vector containing off-transect or radial detection distances.
The right-hand side contains the names of covariate
vectors to fit in the detection
function, and potentially group sizes. Group sizes are specified
by including \verb{+ groupsize(<variable>)} in the RHS
(see 'Group Sizes' section).
Covariates can be either detection level
or transect level and can appear in  \code{data} or exist in the
global working environment. Regular R scoping
rules apply.}

\item{w.lo}{Lower or left-truncation limit of the distances in distance data.
This is the minimum possible off-transect distance. Default is 0.  If
\code{w.lo} is greater than 0, it must have measurement units.
See \code{help(unitHelpers)} for assistance assigning units.}

\item{w.hi}{Upper or right-truncation limit of the distances
in \code{dist}. This is the maximum off-transect distance that
could be observed. If unspecified (i.e., NULL),
right-truncation is set to the maximum of the observed
distances.  If \code{w.hi} is specified, it must have
measurement units.
See \code{help(unitHelpers)} for assistance assigning units.}

\item{...}{Other arguments for summary methods.}
}
\value{
If \code{object} is an \code{RdistDf}, a data frame
containing summary statistics relevant to distance sampling is returned
invisibly.
If \code{formula} is not specified, the number of distance observations
and target detections is not returned because the distances, group sizes,
and covariates are not known.
If \code{object} is not an Rdistance data frame, return is the result of
the next summary method.
}
\description{
Summary method for distance sampling data frames.
\code{Rdistance} data frames are rowwise tibbles. This routine is a
replacement summary method for rowwise_df's that
provides useful distance sampling descriptive statistics.
}
\examples{
data(thrasherDf)
summary(thrasherDf)
summary(thrasherDf
        , formula = dist ~ groupsize(groupsize)
        , w.hi = setUnits(100,"m")
        )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GammaModes.R
\name{GammaModes}
\alias{GammaModes}
\title{Modes of the Gamma distribution}
\usage{
GammaModes(params)
}
\arguments{
\item{params}{A matrix of Gamma distribution parameters. First column
is the scale parameter, and is a function of covariates.
Second column is the shape parameter.}
}
\value{
A vector of the locations of the gamma modes associated
with each row in the \code{params} matrix.
}
\description{
Compute mode (i.e., maximum) of Rdistance's version of the gamma
distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EDR.R
\name{EDR}
\alias{EDR}
\title{Effective Detection Radius (EDR) for point transects}
\usage{
EDR(object, newdata = NULL)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{newdata}{A data frame containing new values of 
the covariates at which to evaluate the distance functions. 
If \code{newdata}
is NULL, distance functions are evaluated at values of 
the observed covariates and results in one prediction 
per distance or transect (see parameter \code{type}). 
If \code{newdata} is not NULL and the model does not contains covariates, 
this routine returns one prediction for each row in \code{newdata}, but 
columns and values in \code{newdata} are ignored.}
}
\value{
If \code{newdata} is present, the returned value is 
a vector of effective sampling distances associated with 
covariate values in \code{newdata}. Length of return 
in this case is the number of rows in \code{newdata}. 
If \code{newdata} is NULL, the returned value is a vector 
of effective sampling distances associated with covariate 
values in \code{object}. Length of return in this case 
is the number of detected groups.  The returned vector 
has measurement units, i.e., \code{object$outputUnits}.
}
\description{
Computes Effective Detection Radius (EDR) for estimated 
  detection functions on point transects.  
  See \code{\link{ESW}} is for line transects.
}
\details{
Effective Detection Radius is the integral under the 
detection function times distance. \if{latex}{I.e., 
    \deqn{EDR = \int_{w.lo}^{w.hi} xg(x)dx,} 
  where \eqn{g(x)} is the distance
  function scaled so that \eqn{g(x.scl) = g.x.scl}
  and \eqn{w.lo} and \eqn{w.hi} are the lower
  and upper truncation limits.  }
}
\examples{
# Load example thrasher data (point transect survey type)
data(thrasherDf)

# Fit half-normal detection function
dfunc <- thrasherDf |> dfuncEstim(formula=dist~bare)

# Compute effective detection radius (EDR)
EDR(dfunc) # vector length 192
effectiveDistance(dfunc) # same
EDR(dfunc, newdata = data.frame(bare=30)) # vector length 1
  
}
\seealso{
\code{\link{dfuncEstim}}, \code{\link{ESW}}, 
\code{\link{effectiveDistance}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simpsonCoefs.R
\name{simpsonCoefs}
\alias{simpsonCoefs}
\title{Simpson numerical integration coefficients}
\usage{
simpsonCoefs(n)
}
\arguments{
\item{n}{Number of coefficients, which is the number of points
at which the function of interest is evaluated.  The number of
intervals is \code{(n-1)/2}.  This number must be odd.}
}
\value{
A vector of Simpson Composite rule coefficients
suitable for numeric integration. The return is a vector of
integers alternating between 4 and 2, with 1's on each end.
}
\description{
Return a vector of Simpson's Composite numerical integration
coefficients.
}
\details{
Let \code{x} be an vector of equally spaced points in the domain
of a function f (equally spaced is critical).
Let \code{y = f(x)}. The numeric integral of f from \code{min(x)}
to \code{max(x)} is
\code{sum(simpsonCoefs(length(y)) * y) * (x[2] - x[1]) / 3}.
}
\examples{

x <- seq(0, 9, length=13)
y <- x^2

scoefs <- simpsonCoefs(length(x))

# exact integral is 9^3/3 = 243
sum( scoefs*y ) * (x[2] - x[1]) / 3

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oneBsIter.R
\name{oneBsIter}
\alias{oneBsIter}
\title{Calculations for one bootstrap iteration}
\usage{
oneBsIter(
  object,
  area,
  propUnitSurveyed,
  pb,
  plot.bs,
  plotCovValues,
  warn = FALSE,
  asymptoticSE = FALSE
)
}
\arguments{
\item{object}{A fitted Rdistance distance function (class \code{dfunc})
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{area}{A scalar containing the total area of inference. Usually, this is
study area size.  If \code{area} is NULL (the default),
\code{area} will be set to 1 square unit of the output units and density estimates
will be produced.
If \code{area} is not NULL, it must have measurement units
assigned by the \code{units} package.
The units on \code{area} must be convertible
to squared output units. Units
on \code{area} must be two-dimensional.
For example, if output units are "foo",
units on area must be convertible to "foo^2" by the \code{units}
package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
others are acceptable.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that
was surveyed.  If both sides of line transects were observed,
\code{propUnitSurveyed}
= 1.  If only a single side of line transects were observed, set
\code{propUnitSurveyed} = 0.5. For point transects, this should be set to
the proportion of each circle that was observed. Length must either be
1 or the total number of transects in \code{x}.}

\item{pb}{A progress bar created with \code{progress::progress_bar$new()}.}

\item{plot.bs}{Logical. Whether to plot bootstrap estimate of detection function.
A plot must already exist because this uses \code{lines}.}

\item{plotCovValues}{Data frame containing values of covariates to plot.
Ignored if \code{plot.bs} is FALSE.}

\item{warn}{A logical scalar specifying whether to issue
an R warning if the estimation did not converge or if one
or more parameter estimates are at their boundaries.
For estimation, \code{warn} should generally be left at
its default value of \code{TRUE}.  When computing bootstrap
confidence intervals, setting \code{warn = FALSE}
turns off annoying warnings when an iteration does
not converge.  Regardless of \code{warn}, after
completion all messages about
convergence and boundary conditions are printed
by \code{print.dfunc}, \code{print.abund}, and
\code{plot.dfunc}.}

\item{asymptoticSE}{Logical variable for whether to calculate
asymptotic standard errors. The default (TRUE) estimates an
asymptotic variance-covariance matrix for parameters based on the
likelihood's Hessian (2nd derivative). If maximization
has been performed by Nlminb or HookesJeeves, the asymptotic
Hessian is estimated using numeric second derivatives
of the likelihood at the maximum likelihood solution. If
maximization was performed by Optim, the last Hessian of
the maximization is returned
by Optim and used
(see \code{\link[=varcovarEstim]{varcovarEstim()}} and \code{\link[=secondDeriv]{secondDeriv()}}).
Asymptotic standard errors will not be estimated if
\code{asymptoticSE = FALSE}. If not estimated,
bootstrap iterations will run faster because the numeric Hessian,
which is discarded during bootstrapping,
will not be calculated every iteration.}
}
\value{
A data frame containing density and abundance
and other relevant statistics for
one iteration of the bootstrap.
}
\description{
Performs density and abundance estimation for one bootstrap iteration.
}
\seealso{
\code{\link[=bootstrap]{bootstrap()}}; \code{\link[=abundEstim]{abundEstim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setOptimizer.R
\name{setOptimizer}
\alias{setOptimizer}
\title{Set optimizing routine}
\usage{
setOptimizer(ml)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}
}
\value{
If Rdistance options are changed, a list of the options and
their original values.  The return can be used outside this routine
to set options back to their state when this routine was entered.
If no options changed, the return in NULL.
}
\description{
Checks that the maximum likelihood optimizing routine is appropriate
for the requested likelihood function. That is, checks that gradient
based maximization routines are only applied to smooth likelihoods,
and that gradient-free methods are used for non-smooth likelihoods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dfunc.para.R
\name{plot.dfunc.para}
\alias{plot.dfunc.para}
\title{Plot parametric distance functions}
\usage{
\method{plot}{dfunc.para}(
  x,
  include.zero = FALSE,
  nbins = "Sturges",
  newdata = NULL,
  legend = TRUE,
  vertLines = TRUE,
  plotBars = TRUE,
  circles = FALSE,
  density = -1,
  angle = 45,
  xlab = NULL,
  ylab = NULL,
  border = TRUE,
  col = "grey85",
  col.dfunc = NULL,
  lty.dfunc = NULL,
  lwd.dfunc = NULL,
  ...
)
}
\arguments{
\item{x}{An estimated detection function object, normally 
produced by calling \code{\link{dfuncEstim}}.}

\item{include.zero}{Boolean value specifying whether to include 0 on the x-axis 
of the plot.  A value of TRUE will include 0 on the left hand end of the x-axis
regardless of the range of distances.  A value of FALSE will plot only the
observation strip (\code{w.lo} to \code{w.hi}).}

\item{nbins}{Internally, this function uses \code{hist} to compute histogram
bars for the plot. This argument is the \code{breaks} argument to
\code{hist}.  This can be either a vector giving the breakpoints between
bars, the suggested number of bars (a single number), a string naming
an algorithm to compute the number of bars, or a function to compute the
number of bars.  See \code{\link{hist}} for all options.}

\item{newdata}{Data frame (similar to \code{newdata} parameter 
of \code{\link{lm}}) containing new values for covariates in 
the distance function.
One distance function is computed and plotted for each row in the data frame. 
If \code{newdata} is NULL, a single distance function is plotted 
for mean values of all numeric covariates and mode values for all 
factor covariates.}

\item{legend}{Logical scalar for whether to include a legend. 
If TRUE, a legend will be included on the plot detailing
the covariate values used to generate the plot.}

\item{vertLines}{Logical scalar specifying whether to plot vertical 
lines at \code{w.lo} and \code{w.hi} from 0 to the  
distance function.}

\item{plotBars}{Logical scalar for whether to plot the histogram 
of distances behind the distance function.  If FALSE, no histogram 
is plotted, only the distance function line(s).}

\item{circles}{Logical scalar requesting the location of detection distances
be plotted. If TRUE, open circles are plotted at predicted distance 
function heights associated with all detection distances. For computational
simplicity, all distances are plotted for EVERY covariate class even though
observed distances belong to only one covariate class. If FALSE, circles 
are not plotted.}

\item{density}{If \code{plotBars=TRUE}, a vector giving the density of 
shading lines, in lines per inch, for the bars underneath 
the distance function, repeated as necessary to exceed the number 
of bars. Values of NULL or a number strictly less than 0 
mean solid fill using colors from parameter \code{col}. 
If \code{density = 0}, bars are not filled and only the borders are rendered. 
If \code{density} > 0, bars are shaded with colors and angles from 
parameters \code{col} and \code{angle}.}

\item{angle}{When \code{density} > 0, the slope of bar shading lines, 
given as an angle in degrees (counter-clockwise), repeated as necessary
to exceed the number of bars.}

\item{xlab}{Label for the x-axis}

\item{ylab}{Label for the y-axis}

\item{border}{The color of bar borders when bars are plotted, 
repeated as necessary to exceed the number of bars. A 
value of NA or FALSE means no borders. If bars are shaded with lines 
(i.e., \code{density>0}), \code{border = TRUE} uses the same 
color for the border as for the shading lines.  Otherwise, fill color
or shaded line color are specified in \code{col} while 
border color is specified in \code{border}.}

\item{col}{A vector of bar fill colors or line colors when bars are 
drawn and \code{density != 0}, repeated as necessary to exceed the number
of bars. Also used for the bar borders when
\code{border = TRUE}.}

\item{col.dfunc}{Color of the distance function(s).
If only one distance function (one line) is being plotted, 
the default color is "red".
If covariates or \code{newdata} are present, 
the default value uses \code{graphics::rainbow(n)}, 
where \code{n} is the number 
of plotted distance functions.  Otherwise, \code{col.dfunc} 
is replicated to the required length.  Plot all 
distance functions in the same color by setting 
\code{col.dfunc} to
a scalar. Plot blue-red pairs of distance functions 
by setting \code{col.dfunc} = \code{c("blue", "red")}. Etc.}

\item{lty.dfunc}{Line type of the distance function(s).
If covariates or \code{newdata} is present, 
the default uses line types  
to \code{1:n}, where \code{n} is the number 
of plotted distance functions.  Otherwise, \code{lty.dfunc} 
is replicated to the required length. Plot solid lines
by specifying \code{lty.dfunc = 1}. Plot solid-dashed line pairs
by specifying \code{lty.dfunc = c(1,2)}. Etc.}

\item{lwd.dfunc}{Line width of the distance function(s), replicated 
to the required length. Default is 2 for all lines.}

\item{\dots}{When bars are plotted, this routine 
uses \code{graphics::barplot} to draw the 
plotting region and bars. When bars are not plotted,
this routine sets up the plot with \code{graphics::plot}.
\dots can be any argument to \code{barplot} or \code{plot} EXCEPT  
\code{width}, \code{ylim}, \code{xlim}, 
\code{density}, \code{angle}, and \code{space}. For example, 
set the main title with \code{main = "Main Title"}.}
}
\value{
The input distance function is returned, with two additional
  components than can be used to reconstruct the plotted bars.  (To 
  obtain values of the plotted distance functions, use \code{predict}
  with \code{type = "distances"}.) 
  The additional components are:
  \item{barHeights}{A vector containing the scaled bar heights drawn 
  on the plot.}
  \item{barWidths}{A vector or scalar of the bar widths drawn on 
  the plot, with measurement units.  }
  Re-plot the bars with \code{barplot( return$barHeights, 
  width = return$barWidths )}.
}
\description{
Plot method for parametric line and point transect distance functions.
}
\examples{

# Simulated data
set.seed(87654)
x <- rnorm(1000, mean=0, sd=20)
x <- x[x >= 0]
x <- setUnits(x, "ft")
Df <- data.frame(transectID = "A"
               , distance = x
                ) |> 
  dplyr::nest_by( transectID
               , .key = "detections") |> 
  dplyr::mutate(length = setUnits(1,"mi"))
attr(Df, "detectionColumn") <- "detections"
attr(Df, "obsType") <- "single"
attr(Df, "transType") <- "line"
attr(Df, "effortColumn") <- "length"
is.RdistDf(Df) # TRUE

# Estimation
dfunc <- dfuncEstim(Df
                  , formula = distance~1
                  , likelihood="halfnorm")
plot(dfunc)
plot(dfunc, nbins=25)

}
\seealso{
\code{\link{plot.dfunc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RdistanceControls.R
\name{RdistanceControls}
\alias{RdistanceControls}
\alias{control}
\alias{controls}
\title{Rdistance optimization control parameters.}
\description{
Optimization control parameters
are set by calls to \code{options()} (see examples).
Optimization parameters used in
\code{Rdistance} are the following:

\itemize{
\item \code{Rdistance_maxIters}: The maximum number of optimization
iterations allowed.

\item \code{Rdistance_evalMax}: The maximum number of objective function
evaluations allowed.

\item \code{Rdistance_likeTol}: Minimum change in the likelihood
between iterations required optimization to continue.
If the likelihood changes by less than this amount,
optimization stops and a solution is declared. Iteration
continues when likelihood changes exceed this value.

\item \code{Rdistance_coefTol}: Minimum change in model coefficients
between iterations for optimization to continue.
If the sum of squared coefficient differences changes
by less than this amount between iterations,
optimization stops and a solution is declared.

\item \code{Rdistance_optimizer}: Normally, this option is not set and
the default optimizer routines are used. The default optimization
routine for smooth likelihoods (i.e., listed by
\code{\link[=differentiableLikelihoods]{differentiableLikelihoods()}}) is the gradient-based
method in \code{\link[stats:nlminb]{stats::nlminb()}}. The default routine for
non-smooth likelihoods is the Nelder-Mead method
in \code{\link[stats:optim]{stats::optim()}}.

Not often, but occasionally optimizers fail on problem. Switching
algorithms can make a poorly behaved distance function converge,
particularly when parameters are near their boundaries.
If users wish to override the default optimization routine,
set this option to one of the following:
\itemize{
\item "optim_\<method\>": Use the "\<method\>" method in stats::optim.
For example, "optim_Nelder-Mead" uses
the Nelder-Mead. See \code{\link[stats:optim]{stats::optim()}} for
documentation of the six available methods.
\item "nlminb": Uses \code{\link[stats:nlminb]{stats::nlminb()}}, a finite-difference
gradient based approach.
\item "hookeJeeves": Uses \code{\link[dfoptim:hookejeeves]{dfoptim::hjkb()}}, a
derivative-free approach for continuous and discontinuous
likelihoods.
}

\item \code{Rdistance_hessEps}: A vector of parameter distances used during
computation of numeric second derivatives. These distances control
and determine variance estimates, and they may need revision when
the maximum likelihood solution is near a parameter boundary.
Should have length
1 or the number of parameters in the model. See function
\code{\link[=secondDeriv]{secondDeriv()}} for further details.

\item \code{Rdistance_trace}: Integer scalar for the level
of information printed to the console by the optimization
routine during maximization of the likelihood. All optimizer
routines interpret a value of 0 as 'do not print any information'
or silent.  Higher values produce more information.  The
information produced varies among optimization routines.

\item \code{Rdistance_requireUnits}: A logical specifying whether measurement
units are required on distances and areas.  If TRUE,
measurement units are required on off-transect and radial
distances in the input data frame.  Likewise, measurement
units are required on truncation distances, scale location,
transect lengths, and study area size. If FALSE, no units are
required and input values are used as is.  The FALSE options is
provided for rare cases when \code{Rdistance} functions are called
from other functions and the calling functions do not accommodate
units.

Assign units with statement like \code{units(detectionDf$dist) <- "m"}
or \code{setUnits(w.hi, "km")} or \code{w.hi <- 150 \%#\% "m"} or
\code{w.hi <- 150 \%m\%.}.
Measurement units of
the various physical quantities need not
be equal because appropriate conversions occur internally.
An error is thrown if differing units are not compatible.
For example, "m" (meters) cannot be converted into "ha" (hectares),
but "acres" can be converted into "ha".
\code{Rdistance} recognizes units listed in \verb{units::}\code{\link[units:valid_udunits]{units::valid_udunits()}}.

\item \code{Rdistance_maxBSFailPropForWarning}: The proportion of bootstrap
iterations that can fail without a warning. If the proportion
of non-convergent bootstrap iterations exceeds this
parameter, a warning about the validity of CI's is issued in
the abundance print method.

}
}
\examples{
# increase number of iterations
options(Rdistance_maxIters=2000)

# change optimizer and decrease tolerance
op <- options(list(Rdistance_optimizer="optim_Nelder-Mead"
                 , Rdistance_likeTol=1e-6)) 

# change back
options(op)
}
\concept{control optimization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dfunc.r
\name{plot.dfunc}
\alias{plot.dfunc}
\title{Plot method for distance (detection) functions}
\usage{
\method{plot}{dfunc}(x, ...)
}
\arguments{
\item{x}{An estimated detection function object, normally
produced by calling \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{...}{
  Arguments passed on to \code{\link[=plot.dfunc.para]{plot.dfunc.para}}
  \describe{
    \item{\code{include.zero}}{Boolean value specifying whether to include 0 on the x-axis
of the plot.  A value of TRUE will include 0 on the left hand end of the x-axis
regardless of the range of distances.  A value of FALSE will plot only the
observation strip (\code{w.lo} to \code{w.hi}).}
    \item{\code{nbins}}{Internally, this function uses \code{hist} to compute histogram
bars for the plot. This argument is the \code{breaks} argument to
\code{hist}.  This can be either a vector giving the breakpoints between
bars, the suggested number of bars (a single number), a string naming
an algorithm to compute the number of bars, or a function to compute the
number of bars.  See \code{\link[=hist]{hist()}} for all options.}
    \item{\code{newdata}}{Data frame (similar to \code{newdata} parameter
of \code{\link[=lm]{lm()}}) containing new values for covariates in
the distance function.
One distance function is computed and plotted for each row in the data frame.
If \code{newdata} is NULL, a single distance function is plotted
for mean values of all numeric covariates and mode values for all
factor covariates.}
    \item{\code{legend}}{Logical scalar for whether to include a legend.
If TRUE, a legend will be included on the plot detailing
the covariate values used to generate the plot.}
    \item{\code{plotBars}}{Logical scalar for whether to plot the histogram
of distances behind the distance function.  If FALSE, no histogram
is plotted, only the distance function line(s).}
    \item{\code{xlab}}{Label for the x-axis}
    \item{\code{ylab}}{Label for the y-axis}
    \item{\code{density}}{If \code{plotBars=TRUE}, a vector giving the density of
shading lines, in lines per inch, for the bars underneath
the distance function, repeated as necessary to exceed the number
of bars. Values of NULL or a number strictly less than 0
mean solid fill using colors from parameter \code{col}.
If \code{density = 0}, bars are not filled and only the borders are rendered.
If \code{density} > 0, bars are shaded with colors and angles from
parameters \code{col} and \code{angle}.}
    \item{\code{angle}}{When \code{density} > 0, the slope of bar shading lines,
given as an angle in degrees (counter-clockwise), repeated as necessary
to exceed the number of bars.}
    \item{\code{col}}{A vector of bar fill colors or line colors when bars are
drawn and \code{density != 0}, repeated as necessary to exceed the number
of bars. Also used for the bar borders when
\code{border = TRUE}.}
    \item{\code{border}}{The color of bar borders when bars are plotted,
repeated as necessary to exceed the number of bars. A
value of NA or FALSE means no borders. If bars are shaded with lines
(i.e., \code{density>0}), \code{border = TRUE} uses the same
color for the border as for the shading lines.  Otherwise, fill color
or shaded line color are specified in \code{col} while
border color is specified in \code{border}.}
    \item{\code{vertLines}}{Logical scalar specifying whether to plot vertical
lines at \code{w.lo} and \code{w.hi} from 0 to the
distance function.}
    \item{\code{col.dfunc}}{Color of the distance function(s).
If only one distance function (one line) is being plotted,
the default color is "red".
If covariates or \code{newdata} are present,
the default value uses \code{graphics::rainbow(n)},
where \code{n} is the number
of plotted distance functions.  Otherwise, \code{col.dfunc}
is replicated to the required length.  Plot all
distance functions in the same color by setting
\code{col.dfunc} to
a scalar. Plot blue-red pairs of distance functions
by setting \code{col.dfunc} = \code{c("blue", "red")}. Etc.}
    \item{\code{lty.dfunc}}{Line type of the distance function(s).
If covariates or \code{newdata} is present,
the default uses line types
to \code{1:n}, where \code{n} is the number
of plotted distance functions.  Otherwise, \code{lty.dfunc}
is replicated to the required length. Plot solid lines
by specifying \code{lty.dfunc = 1}. Plot solid-dashed line pairs
by specifying \code{lty.dfunc = c(1,2)}. Etc.}
    \item{\code{lwd.dfunc}}{Line width of the distance function(s), replicated
to the required length. Default is 2 for all lines.}
  }}
}
\value{
The input distance function is returned, with two additional
components than can be used to reconstruct the plotted bars.  (To
obtain values of the plotted distance functions, use \code{predict}
with \code{type = "distances"}.)
The additional components are:
\item{barHeights}{A vector containing the scaled bar heights drawn
on the plot.}
\item{barWidths}{A vector or scalar of the bar widths drawn on
the plot, with measurement units.  }
Re-plot the bars with \code{barplot( return$barHeights,  width = return$barWidths )}.
}
\description{
Plot method for objects of class '\code{dfunc}'.  Objects of
class '\code{dfunc}' are estimated distance functions produced by
\code{\link[=dfuncEstim]{dfuncEstim()}}.
}
\details{
If \code{plotBars} is TRUE, a scaled histogram is plotted
and the estimated distance function
is plotted over the top of it.  When bars are plotted,
this routine uses \code{graphics::barplot}
for setting up the initial plotting region and
most parameters to \code{graphics::barplot} can
be specified (exceptions noted above in description of '\dots').

The form of the likelihood and any series
expansions is printed in the main title (overwrite this with
\code{main="<my title>"}). Convergence of the distance
function is checked.  If the distance function did not converge, a warning
is printed over the top of the histogram.  If one or more parameter
estimates are at their limits (likely indicating non-convergence or poor
fit), another warning is printed.
}
\examples{
# Simulated RdistDf
set.seed(87654)
x <- rnorm(1000, mean=0, sd=20)
x <- x[x >= 0]
x <- setUnits(x, "ft")
Df <- data.frame(transectID = "A"
               , distance = x
                ) |> 
  dplyr::nest_by( transectID
               , .key = "detections") |> 
  dplyr::mutate(length = setUnits(1,"mi"))
attr(Df, "detectionColumn") <- "detections"
attr(Df, "obsType") <- "single"
attr(Df, "transType") <- "line"
attr(Df, "effortColumn") <- "length"
is.RdistDf(Df) # TRUE

dfunc <- Df |> dfuncEstim(distance ~ 1, likelihood="halfnorm")
plot(dfunc)
plot(dfunc, nbins=25)

# showing effects of plot parameters
plot(dfunc
  , col=c("red","blue","orange")
  , border="black"
  , xlab="Off-transect distance"
  , ylab="Prob"
  , vertLines = FALSE
  , main="Showing plot params")
 
plot(dfunc
   , col="purple"
   , density=30
   , angle=c(-45,0,45)
   , cex.axis=1.5
   , cex.lab=2
   , ylab="Probability") 

plot(dfunc
   , col=c("grey","lightgrey")
   , border=NA) 

plot(dfunc
   , col="grey"
   , border=0
   , col.dfunc="blue"
   , lty.dfunc=2
   , lwd.dfunc=4
   , vertLines=FALSE)

plot(dfunc
   , plotBars=FALSE
   , cex.axis=1.5
   , col.axis="blue")
rug(distances(dfunc))

# un-equal bin widths, nbins must span distances
plot(dfunc
   , nbins = c(0,2.5,5,7.5,10,15,25,50,70)
)

# Plot showing f(0)
hist(distances(dfunc)
   , n = 40
   , border = NA
   , prob = TRUE)
x <- seq(dfunc$w.lo, dfunc$w.hi, length=200)
g <- predict(dfunc, type="dfunc", distances = x, newdata = data.frame(a=1))
f <- g[,1] / ESW(dfunc)[1]

# Check integration:
sum(diff(x)*(f[-1] + f[-length(f)]) / 2) # Trapazoid rule; should be 1.0
lines(x, f) # hence, 1/f(0) = ESW

# Covariates: detection by observer
data(sparrowDfuncObserver) # pre-estimated model

obsLevs <- levels(sparrowDfuncObserver$data$observer)
plot(sparrowDfuncObserver
   , newdata = data.frame(observer = obsLevs)
   , vertLines = FALSE
   , col.dfunc = heat.colors(length(obsLevs))
   , col = c("grey","lightgrey")
   , border=NA
   , main="Detection by observer")

}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}, \code{\link[=print.dfunc]{print.dfunc()}},
\code{\link[=print.abund]{print.abund()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cosine.expansion.R
\name{cosine.expansion}
\alias{cosine.expansion}
\title{Cosine expansion terms}
\usage{
cosine.expansion(x, expansions)
}
\arguments{
\item{x}{A numeric matrix of distances at which to evaluate
the expansion series. For distance analysis, \code{x} should
be the proportion of the maximum sighting distance
at which a group was sighted, i.e., \eqn{x = d/w}, where \eqn{d}
is sighting distance and \eqn{w} is maximum sighting distance.}

\item{expansions}{A scalar specifying the number of
expansion terms to compute. Must be one of the
integers 1, 2, 3, 4, or 5.}
}
\value{
A 3D array of size \code{nrow(x)} X \code{ncol(x)} X \code{expansions}.
The 'pages' (3rd dimension) of this array are the cosine expansions of
\code{x}. i.e., page 1 is the first expansion term of \code{x},
page 2 is the second expansion term of \code{x}, etc.
}
\description{
Computes the cosine expansion terms that modify the
shape of distance likelihood functions.
}
\details{
The cosine expansion used here is:
\itemize{
\item \strong{First term}: \deqn{h_1(x)=\cos(2\pi x),}{h1(x) = cos(2*Pi*x),}
\item \strong{Second term}: \deqn{h_2(x)=\cos(4\pi x),}{h2(x) = cos(4*Pi*x),}
\item \strong{Third term}: \deqn{h_3(x)=\cos(6\pi x),}{h3(x) = cos(6*Pi*x),}
\item \strong{Fourth term}: \deqn{h_4(x)=\cos(8\pi x),}{h4(x) = cos(8*Pi*x),}
\item \strong{Fifth term}: \deqn{h_5(x)=\cos(10\pi x),}{h5(x) = cos(10*Pi*x),}
}
The maximum number of expansion terms is 5.

The cosine expansion frequency in Rdistance is 2*pi. Each term is two pi more than
the previous. The sine expansion frequency in Rdistance is pi. Consequently,
the sine and cosine expansions fit different models.
}
\examples{
x <- matrix(seq(0, 1, length = 200), ncol = 1)
cos.expn <- cosine.expansion(x, 5)
plot(range(x), range(cos.expn), type="n")
matlines(x, cos.expn[,1,1:5], col=rainbow(5), lty = 1)

}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}},
\code{\link[=hermite.expansion]{hermite.expansion()}},
\code{\link[=simple.expansion]{simple.expansion()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.abund.R
\name{summary.abund}
\alias{summary.abund}
\title{Summarize abundance estimates}
\usage{
\method{summary}{abund}(object, criterion = "AICc", ...)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{criterion}{A string specifying the model fit criterion to print.
Must be one of "AICc" (the default),
"AIC", or "BIC".  See \code{\link[=AIC.dfunc]{AIC.dfunc()}} for formulas.}

\item{\dots}{Included for compatibility to other print methods.
Ignored here.}
}
\value{
0 is invisibly returned.
}
\description{
Summarize an object of class \code{c("abund","dfunc")}
that is output by \code{abundEstim}.
}
\details{
If the proportion of bootstrap iterations that failed is
greater than \code{getOption("Rdistance_maxBSFailPropForWarning")},
a warning about the validity of CI's is issued and
a diagnostic message printed.  Increasing this option to a number greater
than 1 will kill the warning (e.g., \code{options(Rdistance_maxBSFailPropForWarning = 1.3)}),
but ignoring a large number of non-convergent
bootstrap iterations may be a bad idea (i.e., validity of the CI is
questionable). The default value for \code{Rdistance_maxBSFailPropForWarning}
is 0.2.
}
\examples{
# Load example sparrow data (line transect survey type)
data(sparrowDf)

# Fit half-normal detection function
dfunc <- sparrowDf |> dfuncEstim(formula=dist ~ 1 + offset(groupsize))

# Estimate abundance given the detection function
fit <- abundEstim(dfunc
                , area = setUnits(4105, "km^2")
                , ci=NULL)
summary(fit) # No confidence intervals
                
\dontrun{
# With bootstrap confidence intervals 
# Requires ~3 min to complete
fit <- abundEstim(dfunc
                , area = setUnits(4105, "km^2")
                , ci=0.95)

summary(fit)
}
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}, \code{\link[=abundEstim]{abundEstim()}},
\code{\link[=summary.dfunc]{summary.dfunc()}}, \code{\link[=print.dfunc]{print.dfunc()}},
\code{\link[=print.abund]{print.abund()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gxEstim.r
\name{gxEstim}
\alias{gxEstim}
\title{Estimate g(0) or g(x)}
\usage{
gxEstim(fit)
}
\arguments{
\item{fit}{An estimated \code{dfunc} object.  See \code{dfuncEstim}.}
}
\value{
A list comprised of the following components:
\item{x.scl}{The value of x (distance) at which g() is evaluated.  }
\item{comp2 }{The estimated value of g() when evaluated at \code{x.scl}.  }
}
\description{
Estimate distance function scaling factor
, g(0) or g(x), for a specified distance function.
}
\details{
This routine scales sightability such that
g(\code{x.scl}) = \code{g.x.scl}, where g() is the sightability function.
Specification of \code{x.scl} and \code{g.x.scl} covers several estimation cases:
\enumerate{
\item \strong{g(0) = 1} : (the default) Inputs are \code{x.scl} = 0, \code{g.x.scl} = 1.
If \code{w.lo} > 0, \code{x.scl} will be set to \code{w.lo}
so technically this case is g(\code{w.low}) = 1.

\item \strong{User supplied probability at specified distance}: Inputs are
\code{x.scl} = a number greater than or equal
to \code{w.lo}, \code{g.x.scl} = a number between 0 and 1.  This case
covers situations where sightability on the transect (distance 0) is
not perfect.  This case  assumes researchers have an independent
estimate of sightability at distance
\code{x.scl} off the transect.  For example, researchers could be
using multiple
observers to estimate that sightability at distance \code{x.scl}
is \code{g.x.scl}.

\item \strong{Maximum sightability specified}: Inputs
are \code{x.scl}="max", \code{g.x.scl} =  a number
between 0 and 1.  In this case,
g() is scaled such that its maximum value is \code{g.x.scl}.
This routine computes the distance at which g() is maximum, sets
g()'s height there to \code{g.x.scl}, and returns \code{x.max} where
x.max is the distance at which g is maximized. This case covers the
common aerial survey situation where maximum sightability is slightly
off the transect, but the distance at which the maximum occurs
is unknown.

\item \strong{Double observer system}: Inputs are
\code{x.scl}="max", \code{g.x.scl} = \<a data frame\>.
In this case, g(\emph{x}) = \emph{h}, where \emph{x} is the distance that
maximizes g and \emph{h} is the height of g() at \emph{x}
computed from the double observer data frame (see below for
structure of the double observer data frame).

\item \strong{Distance of independence specified, height computed from double
observer system}: Inputs are
\code{x.scl} = a number greater than or equal to \code{w.lo}
\code{g.x.scl} = a data frame.  In this case, g(\code{x.scl}) = \emph{h},
where \emph{h} is computed from the double observer data frame
(see below for structure of the double observer data frame).

}

When \code{x.scl}, \code{g.x.scl}, or \code{observer} are NULL, the routine
will look for and use \verb{$call.x.scl}, or \verb{$call.g.x.scl}, or
\verb{$call.observer} components of the \code{fit} object for whichever
of these three parameters is missing.  Later, different
values can be specified in a direct call to \code{F.gx.estim}
without having to re-estimate the distance function. Because of this feature,
the default values in \code{dfuncEstim} are \code{x.scl} = 0 and
\code{g.x.scl} = 1 and \code{observer} = "both".
}
\section{Structure of the double observer data frame}{

When \code{g.x.scl} is a data frame, it is assumed to contain
the components \verb{$obsby.1} and \verb{$obsby.2} (no flexibility on names).
Each row in the data frame contains data from one sighted target.
The \verb{$obsby.1} and \verb{$obsby.2} components are
TRUE/FALSE (logical) vectors indicating whether
observer 1 (\code{obsby.1}) or observer 2 (\code{obsby.2}) spotted the target.
}

\examples{

data(sparrowDf)
fit <- dfuncEstim(sparrowDf, dist ~ groupsize(groupsize))
gxEstim(fit)
  
fit <- dfuncEstim(sparrowDf, dist ~ groupsize(groupsize)
                , x.scl = 50 \%m\%.
                , g.x.scl = 0.75)
gxEstim(fit)
plot(fit)
abline(h=0.75)
abline(v=50\%m\%.)
  
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.dfunc.R
\name{summary.dfunc}
\alias{summary.dfunc}
\title{Summarize a distance function object}
\usage{
\method{summary}{dfunc}(object, criterion = "AICc", ...)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{criterion}{A string specifying the model fit criterion to print.
Must be one of "AICc" (the default), 
"AIC", or "BIC".  See \code{\link{AIC.dfunc}} for formulas.}

\item{\dots}{Included for compatibility with other print methods.  Ignored here.}
}
\value{
The input distance function object (\code{object}), invisibly, 
with the following additional components:
\itemize{
  \item \code{convMessage}: The convergence message. If the distance function
  is smoothed, the convergence message is NULL.
  \item \code{effDistance}: The ESW or EDR.
  \item \code{pDetect}: Probability of detection in the strip.
  \item \code{AIC}: AICc, AIC, or BIC of the fit, whichever was requested.
  \item \code{coefficients}: If the distance function has coefficients, this 
  is the coefficient matrix with standard errors, Wald Z values, and p values. 
  If the distance function is smoothed, it has no coefficients and this component 
  is NULL.
}
}
\description{
A summary method for distance functions. 
Distance functions are produced by 
\code{dfuncEstim} (class \code{dfunc}).
}
\details{
This function prints the following 
quantities:
\itemize{  
  \item \samp{Call} : The original function call.
  \item \samp{Coefficients} : A matrix of estimated coefficients, their 
  standard errors, and Wald Z tests.
  \item \samp{Strip} : The left (\code{w.lo}) and right (\code{w.hi}) truncation values.
  \item \samp{Effective strip width or detection radius} : ESW or EDR as computed by \code{effectiveDistance}.
  \item \samp{Probability of Detection} : Probability of detecting a single target in the strip. 
  \item \samp{Scaling} : The horizontal and vertical coordinates used to scale the distance function. 
  Usually, the horizontal coordinate is 0 and the vertical coordinate is 1 (i.e., g(0) = 1).
  \item \samp{Log likelihood} : Value of the maximized log likelihood.
  \item \samp{Criterion} : Value of the specified fit criterion (AIC, AICc, or BIC).
}
The number of digits used in the printout is 
controlled by \code{options()$digits}.
}
\examples{
# Load example sparrow data (line transect survey type)
data(sparrowDf)

# Fit half-normal detection function
dfunc <- sparrowDf |> dfuncEstim(formula=dist~1)

# Print results
summary(dfunc)
summary(dfunc, criterion="BIC")

}
\seealso{
\code{\link{dfuncEstim}}, \code{\link{plot.dfunc}}, 
\code{\link{print.abund}}, \code{\link{print.abund}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expandW.R
\name{expandW}
\alias{expandW}
\title{Domain of expansion factors}
\usage{
expandW(ml, params, k)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}

\item{params}{A matrix of likelihood parameters.  Size is (number
of distances) X ((number of cases) + (number of non-covariate parameter)).}

\item{k}{The number of cases.}
}
\value{
A plain vector of length k containing expansion domains, with units.
}
\description{
The domain to which expansion factors are applied varies by likelihood.
When the domain varies, it depends on parameter values. For example,
\code{oneStep} expansion factors are applied between
0 and the likelihood's first parameter (\eqn{\theta}{Theta}), which
varies by covariates.  This function computes the likelihood-specific
expansion domains
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transectType.R
\name{transectType}
\alias{transectType}
\title{Type of transects}
\usage{
transectType(x)
}
\arguments{
\item{x}{Either an estimated distance function, output by 
\code{dfuncEstim}, or an Rdistance nested data frame, output by 
\code{RdistDf}.}
}
\value{
A scalar. Either 'line' if \code{x} contains  
continuous line-transect detections, or 'point' if \code{x} contains 
point-transects detections. If transect type has not been assigned, 
return is NULL.
}
\description{
Return the type of transects represented in either a fitted 
distance function or Rdistance data frame.
}
\details{
This function is a simple helper function.  If \code{x} is an
estimated distance object, it polls the \code{transType} attribute 
of \code{x}'s Rdistance nested data frame. 
If \code{x} is an Rdistance nested data frame, it 
polls the \code{transType} attribute.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.Unitless.R
\name{is.Unitless}
\alias{is.Unitless}
\title{Test whether object is unitless}
\usage{
is.Unitless(obj)
}
\arguments{
\item{obj}{A numeric scalar or vector, with or without units.}
}
\value{
TRUE if \code{obj} has units and they
are either [1] or the denominator units equal
the numerator units.  Otherwise, return FALSE.
If \code{obj} does not have units, this routine
returns TRUE.
}
\description{
Tests whether a 'units' object is actually
unitless.
Unitless objects, such as ratios, should be assigned
units of [1].  Often they are, but
sometimes unitless ratios are assigned units like [m/m].
The \code{units} package should always convert [m/m] to
[1], but it does not always.
Sometimes units like [m/m] mess things up, so it is
better to remove them before calculations.
}
\examples{
a <- setUnits(2, "m")
b <- a / a
is.Unitless(a)
is.Unitless(b)
is.Unitless(3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateHuberLines.R
\name{integrateHuberLines}
\alias{integrateHuberLines}
\title{Integrate Line-transect Huber function}
\usage{
integrateHuberLines(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced 
by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
If a matrix, each row corresponds to a 
distance function and each column is a parameter. The first column is 
the parameter related to sighting covariates and must be transformed 
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
be applied outside this routine).  If \code{object} is a matrix, 
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function 
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for 
the distance function covariates. If NULL and 
\code{object} is a fitted distance function, the  
observed covariates stored in
\code{object} are used (behavior similar to \code{\link{predict.lm}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function. 
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if 
\code{object} is a matrix. Sighting distance units can differ from units 
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in 
\code{object}. 
If \code{object} is a distance function and 
\code{newdata} is specified, the returned vector's length is 
\code{nrow(newdata)}.  If \code{object} is a distance function and 
\code{newdata} is NULL, 
returned vector's length is \code{length(distances(object))}. If 
\code{object} is a matrix, return's length is 
\code{nrow(object)}.
}
\description{
Compute exact integral of the 'huber' distance function for line 
transects.
}
\section{Note}{

Users will not normally call this function. It is called 
internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}.
}

\examples{

w <- 250
T1 <- 160
T2 <- 20
p <- 0.03
obj <- matrix(c(T1,T2,p), 1, 3)

integrateHuberLines(obj
  , w.lo = units::set_units(0,"m")
  , w.hi = units::set_units(w,"m")
  , Units = "m")
  
# same
huber.cumFunc(w,T1,T2,p,w)

# check by numeric integration
hubLike <- function(d, T1, T2, p, wl, wh) { 
  y <- huber.like(a = c(log(T1), T2, p)
           , dist = d - wl
           , covars = matrix(1, length(d))
           , w.hi = wh - wl)$L.unscaled
  y
}
integrate(hubLike, lower = 0, upper = w, T1 = T1
        , T2 = T2, p = p, wl = 0, wh = w)
  
}
\seealso{
\code{\link{integrateNumeric}}; \code{\link{integrateNegexpLines}}; 
\code{\link{integrateHalfnormLines}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateOneStepNumeric.R
\name{integrateOneStepNumeric}
\alias{integrateOneStepNumeric}
\title{Numeric Integration of One-step Function}
\usage{
integrateOneStepNumeric(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL,
  expansions = NULL,
  series = NULL,
  isPoints = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced
by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}), or a matrix of canonical
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}).
If a matrix, each row corresponds to a
distance function and each column is a parameter. The first column is
the parameter related to sighting covariates and must be transformed
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must
be applied outside this routine).  If \code{object} is a matrix,
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for
the distance function covariates. If NULL and
\code{object} is a fitted distance function, the
observed covariates stored in
\code{object} are used (behavior similar to \code{\link[=predict.lm]{predict.lm()}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if
\code{object} is a matrix. Sighting distance units can differ from units
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}

\item{expansions}{A scalar specifying the number of terms
in \code{series} to compute. Depending on the series,
this could be 0 through 5.  The default of 0 equates
to no expansion terms of any type.  No expansion terms
are allowed (i.e., \code{expansions} is forced to 0) if
covariates are present in the detection function
(i.e., right-hand side of \code{formula} includes
something other than \code{1}).}

\item{series}{If \code{expansions} > 0, this string
specifies the type of expansion to use. Valid values at
present are 'simple', 'hermite', and 'cosine'.}

\item{isPoints}{Boolean. TRUE if integration is for point surveys.
FALSE for line-transect surveys. Line-transect surveys integrate
under the distance function, g(x), while point surveys integrate under
the distance function times distances, xg(x).}
}
\value{
A vector of areas under the distance functions represented in
\code{object}.
If \code{object} is a distance function and
\code{newdata} is specified, the returned vector's length is
\code{nrow(newdata)}.  If \code{object} is a distance function and
\code{newdata} is NULL,
returned vector's length is \code{length(distances(object))}. If
\code{object} is a matrix, return's length is
\code{nrow(object)}.
}
\description{
Compute integral of the one-step distance function
using numeric integration. This function is only called for
oneStep functions that contain expansion factors.
}
\details{
The \code{\link[=oneStep.like]{oneStep.like()}} function has an extremely large
discontinuity at Theta. Accurate numeric integration requires
inserting Theta and Theta+ (a value just larger than Theta)
into the series of points being evaluated. Because this creates
variable sized intervals, the Trapezoid rule must be used.
Rdistance's Simpson's rule routine
(\code{\link[=integrateNumeric]{integrateNumeric()}}) will not work for oneStep likelihoods
that have expansions.
}
\section{Note}{

Users will not normally call this function. It is called
internally by \code{\link[=nLL]{nLL()}} and \code{\link[=effectiveDistance]{effectiveDistance()}}.
}

\seealso{
\code{\link[=integrateNumeric]{integrateNumeric()}};
\code{\link[=integrateOneStepLines]{integrateOneStepLines()}}; \code{\link[=integrateOneStepPoints]{integrateOneStepPoints()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/huber.like.R
\name{huber.like}
\alias{huber.like}
\title{Huber distance function}
\usage{
huber.like(a, dist, covars, w.hi = NULL)
}
\arguments{
\item{a}{A vector or matrix of covariate
and expansion term
coefficients. If matrix, dimension is
\emph{k} X \emph{p}, where
\emph{k} = \code{nrow(a)}) is the number of coefficient
vectors to evaluate (cases) and \emph{p}
= \code{ncol(a)})
is the number of covariate and expansion
coefficients in the likelihood (i.e., rows are
cases and columns are covariates). If \code{a} is a
dimensionless vector, it is interpreted as a
single row with \emph{k} = 1.
Covariate coefficients in \code{a} are the first
\emph{q} values (\emph{q} <= \emph{p}), and must
be on a log scale.}

\item{dist}{A numeric vector of length \emph{n} or
a single-column matrix (dimension \emph{n}X1) containing
detection distances at which to evaluate the likelihood.}

\item{covars}{A numeric vector of length \emph{q} or a
matrix of dimension \emph{n}X\emph{q} containing
covariate values
associated with distances in argument \code{dist}.}

\item{w.hi}{A numeric scalar containing maximum
distance. The right-hand cutoff or upper limit.
Ignored by some likelihoods (such as halfnorm,
negexp, and hazrate), but is a fixed parameter
in other likelihoods (such as oneStep
and uniform).}
}
\value{
A list containing the following two components:
\itemize{
\item \strong{L.unscaled}: A matrix of size
\emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
cases = \code{nrow(a)})
containing likelihood values evaluated at
distances in \code{dist}.
Each row is associated with
a single distance, and each column is associated with
a single case (row of \code{a}).  Values in this matrix are
the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
These values are "unscaled" likelihood values; they must be
scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi
to form proper likelihood values.

\item \strong{params}: A \emph{n}X\emph{b}X\emph{k}  array
of the likelihood's (canonical) parameters in link space (i.e., on
log scale; \emph{b} = number of canonical parameters in
the likelihood; \emph{k} = number of cases).
Rows correspond to distances in \code{dist}. Columns
correspond to parameters (columns of \code{a}),
and pages correspond to cases (rows of \code{a}).
}
}
\description{
Computes the Huber likelihood for use as a distance function.
The Huber likelihood is a mixture of an inverted Huber loss
and a uniform distribution.
The distance function is quadratic from the lowest distance to
its first parameter, linear from the first parameter
to the sum of its first and second parameter, and constant
after that.
}
\details{
The 'huber' likelihood is an inverted version of the
Huber loss function, mixed with a uniform at the upper end, and contains
three canonical parameters.
The 'huber' distance function is a negative quadratic
between \code{0} and its first parameter \eqn{\theta_1}{T1},
linear between \eqn{\theta_1}{T1} and \eqn{\theta_1 + \theta_2}{T1 + T2}, and
constant  after \eqn{\theta_1 + \theta_2}{T1 + T2}.
Specifically, the 'huber' likelihood is,
\deqn{
   f(d|\theta_1,\theta_2, p) = \left(1 - \frac{(1-p)h(d)}{m}\right) \,
     I(0 \leq d \leq \Theta) \  + \  p \, I(\Theta < d \leq w)
}
where
\eqn{\Theta = \theta_1 + \theta_2}{T = T1 + T2}, \eqn{w} = \code{w.hi - w.lo},
\eqn{
 m = \theta_1(\Theta - 0.5\theta_1)
}{
 m = T1(T - 0.5(T1))
}
and \emph{h(d)} is Huber loss between 0 and \eqn{\Theta}{T}, i.e.,
\deqn{
 h(d)  = 0.5d^2\, I(0 \leq d \leq \theta_1) \ + \ 
         \theta_1(d - 0.5\theta_1)\, I(\theta_1 < d \leq \Theta).
}

The first parameter, \eqn{\theta_1}{T1}, is related to covariates,
i.e., \eqn{log(\theta_1) = \beta_0 + \beta_1x_1 + \ldots + \beta_qx_q}{
log(T_1) = B_0 + B_1(x_1) + ... + B_q(x_q)}.  \eqn{\theta_2}{T2} and \eqn{p}
are constant across covariate values.
}
\examples{

t1 <- c(65,80,120)
t2 <- 60
p  <- 0.05
a <- matrix(c(log(t1)
       , rep(t2,3)
       , rep(p,3))
       , 3,3)
d <- seq(0, 200, length=201)
X <- matrix(1,length(d),1)
y <- huber.like(a, d, X)

# Plot showing covariate effects 
plot(range(d), range(y$L.unscaled)
  , type = "n", xlab = "d", ylab = "Huber(d)")
for(i in 1:3){
  # Distance functions
  lines(d
    , y$L.unscaled[,i]
    , col = i
    , lwd= 2)
  # Quadradic to linear transitions
  points(exp(a[i,1])
    , y$L.unscaled[(t1[i]-0.1) < d & d < (t1[i]+0.01),i]
    , pch = 16
    , col = i )
  # Linear to constant transition
  Theta <- exp(a[i,1]) + a[i,2]
  points(Theta
    , y$L.unscaled[(Theta-0.1) < d & d < (Theta+0.1),i]
    , pch = 15
    , col = i )
}

}
\seealso{
See \href{https://mcdonalddatasciences.com/Rdistance.html}{Rdistance tutorials}
for a method to generate random observations from the Huber likelihood.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/observationType.R
\name{observationType}
\alias{observationType}
\title{Type of observations}
\usage{
observationType(x)
}
\arguments{
\item{x}{Either an estimated distance function, output by 
\code{dfuncEstim}, or an Rdistance nested data frame, output by 
\code{RdistDf}.}
}
\value{
One of the following values: 
"single", "1given2", "2given1", or "both". If observation type 
has not been assigned, 
return is NULL.
}
\description{
Return the type of observations (single or multiple observers) 
represented in either a fitted distance function or Rdistance data frame.
}
\details{
This function is a simple helper function.  If \code{x} is an
estimated distance object, it polls the \code{obsType}
attribute of the object's Rdistance data frame. If \code{x} is an Rdistance nested data frame, it 
polls the \code{obsType} attribute.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lines.dfunc.R
\name{lines.dfunc}
\alias{lines.dfunc}
\title{lines.dfunc - Line plotting method for distance functions}
\usage{
\method{lines}{dfunc}(x, newdata = NULL, prob = NULL, ...)
}
\arguments{
\item{x}{An estimated detection function object, normally 
produced by calling \code{\link{dfuncEstim}}.}

\item{newdata}{A data frame containing new values of 
the covariates at which to evaluate the distance functions. 
If \code{newdata}
is NULL, distance functions are evaluated at values of 
the observed covariates and results in one prediction 
per distance or transect (see parameter \code{type}). 
If \code{newdata} is not NULL and the model does not contains covariates, 
this routine returns one prediction for each row in \code{newdata}, but 
columns and values in \code{newdata} are ignored.}

\item{prob}{Logical scalar for whether to scale the distance function 
to be a density function (integrates to one). Default behavior is designed 
to be compatible with the plot method for distance functions 
(\code{\link{plot.dfunc}}). By default, line transect distance 
functions are not scaled to a density and integrate to the effective strip width. 
In contrast, point transects distance functions are scaled to be densities 
by default.}

\item{\dots}{Parameters passed to \code{lines.default} that control attributes like 
color, line width, line type, etc.}
}
\value{
A data frame containing the x and y coordinates of the 
plotted line(s) is returned invisibly.  X coordinates in the 
return are names \code{x}. Y coordinates in the return are named 
\code{y1, y2, ..., yn}, i.e., one column per returned 
distance function.
}
\description{
Line plot method for objects of class '\code{dfunc}' 
that adds distance functions to an existing plot.
}
\examples{
# a simulated RdistDf
set.seed(87654)
x <- rnorm(1000, mean=0, sd=20)
x <- x[x >= 0]
x <- setUnits(x, "mi")
Df <- data.frame(transectID = "A"
               , distance = x
                ) |> 
  dplyr::nest_by( transectID
               , .key = "detections") |> 
  dplyr::mutate(length = setUnits(100,"km"))              
attr(Df, "detectionColumn") <- "detections"
attr(Df, "obsType") <- "single"
attr(Df, "transType") <- "line"
attr(Df,'effortColumn') <- "length"
is.RdistDf(Df)  # TRUE

dfunc <- Df |> dfuncEstim(distance ~ 1, likelihood="halfnorm")
plot(dfunc, nbins = 40, col="lightgrey", border=NA, vertLines=FALSE)
lines(dfunc, col="grey30", lwd=15)
lines(dfunc, col="grey90", lwd=5, lty = 2)

# Multiple lines 
data(sparrowDfuncObserver)
obsLevs <- levels(sparrowDfuncObserver$data$observer)
plot(sparrowDfuncObserver
   , vertLines = FALSE
   , lty = 0
   , plotBars = FALSE
   , main="Detection by observer"
   , legend = FALSE)
y <- lines(sparrowDfuncObserver
   , newdata = data.frame(observer = obsLevs)
   , col = palette.colors(length(obsLevs))
   , lty = 1
   , lwd = 4)
head(y) # values returned, with distances as column


}
\seealso{
\code{\link{dfuncEstim}}, \code{\link{plot.dfunc}},
  \code{\link{print.abund}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateDfuncs.R
\name{integrateDfuncs}
\alias{integrateDfuncs}
\title{Integration of distance functions}
\usage{
integrateDfuncs(object, ml)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced 
by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
If a matrix, each row corresponds to a 
distance function and each column is a parameter. The first column is 
the parameter related to sighting covariates and must be transformed 
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
be applied outside this routine).  If \code{object} is a matrix, 
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function 
parameters themselves never have units.}

\item{ml}{Either a Rdistance 'model frame' or an Rdistance 
'fitted object'.  Both are of class "dfunc". 
Rdistance 'model frames' are lists containing components 
necessary to estimate a distance function, but no estimates. 
Rdistance 'model frames' are typically
produced by calls to \code{\link{parseModel}}. 
Rdistance 'fitted objects'
are typically produced by calls to \code{\link{dfuncEstim}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates, 
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}
}
\value{
A vector of areas under the distance functions represented in 
\code{object}. 
If \code{object} is a distance function and 
\code{newdata} is specified, the returned vector's length is 
\code{nrow(newdata)}.  If \code{object} is a distance function and 
\code{newdata} is NULL, 
returned vector's length is \code{length(distances(object))}. If 
\code{object} is a matrix, return's length is 
\code{nrow(object)}.
}
\description{
Integrates under distances functions using exact integrals 
when possible.  If exact integrals are not known, numerical 
integration is used.
}
\details{
Let K be the integral under distance function g(x) (i.e., the output
from this routine). In distance analysis, the observation likelihood 
being evaluated for maximization 
is the \emph{density}, f(x) = g(x)/K.  K is a key quantity in distance 
analysis and is called the "effective sampling distance".
}
\section{Note}{

Users will not normally call this function. It is called 
internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}.
}

\examples{
# Faking a model frame
ml <- list( likelihood = "halfnorm"
          , expansions = 0
          , w.lo = 0 \%m\% .
          , w.hi = 100 \%m\% .
          , Units = "m"
          )
attr(ml, "transType") <- "line"

parms <- matrix(75, nrow = 1) 
integrateDfuncs(parms, ml)

# check: Normal, 0 to 100, sd = 75, scaled to mode = 1
(pnorm(q = 100, mean = 0, sd = 75) - 0.5) * sqrt(2*pi)*75

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.dfunc.r
\name{coef.dfunc}
\alias{coef.dfunc}
\title{Coefficients of an estimated detection function}
\usage{
\method{coef}{dfunc}(object, \dots)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{...}{Ignored}
}
\value{
The estimated coefficient vector for the detection function. 
Length and interpretation of values vary 
depending on the form of the detection function and expansion terms.
}
\description{
Extract distance model coefficients from 
an estimated detection function object.
}
\examples{
data(sparrowDfuncObserver) # pre-estimated dfunc

# Same as sparrowDfuncObserver$par 
coef(sparrowDfuncObserver) 

\dontrun{
data(sparrowDf)
dfunc <- sparrowDf |> dfuncEstim(dist~bare + observer,
                      w.hi = 150 \%m\%.)
coef(dfunc)
}
 
}
\seealso{
\code{\link{AIC}}, \code{\link{dfuncEstim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nCovars.R
\name{nCovars}
\alias{nCovars}
\title{Number of covariates}
\usage{
nCovars(X)
}
\arguments{
\item{X}{The X matrix of covariates, or a vector.}
}
\value{
An integer scalar

# do not export
}
\description{
Return number of covariates in a distance model
}
\details{
The reason this routine is needed is that sometimes 
we pass one row of covariates to a likelihood function. 
If so, it may come in as a normal vector, not a matrix. 
If a normal vector, ncol(X) does not work.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.dfunc.R
\name{predict.dfunc}
\alias{predict.dfunc}
\title{Predict distance functions}
\usage{
\method{predict}{dfunc}(
  object,
  newdata = NULL,
  type = c("parameters"),
  distances = NULL,
  propUnitSurveyed = 1,
  area = NULL,
  ...
)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{newdata}{A data frame containing new values of
the covariates at which to evaluate the distance functions.
If \code{newdata}
is NULL, distance functions are evaluated at values of
the observed covariates and results in one prediction
per distance or transect (see parameter \code{type}).
If \code{newdata} is not NULL and the model does not contains covariates,
this routine returns one prediction for each row in \code{newdata}, but
columns and values in \code{newdata} are ignored.}

\item{type}{The type of predictions desired.
\itemize{
\item \strong{If \code{type} == "parameters"}: Returned value is
a matrix of predicted (canonical) parameters of the
likelihood function. If \code{newdata} is NULL, return contains
one parameter value for every detection distance
in \code{object$mf} (distances in \code{object$mf} are between
\code{object$w.lo} and \code{object$w.hi} and non-missing).
If \code{newdata} is not NULL, returned vector
has one parameter for every row in \code{newdata}. Parameter
\code{distances} is ignored when \code{type} == "parameters".
Canonical parameters (non-expansion terms)
are returned on the response (inverse-link) scale.  Raw
canonical parameters in \code{object$par} are stored in
the link scale.  Expansion term parameters use the identity
link, so their value in the output equals their value
in \code{object$par}.

\item \strong{If \code{type} == "likelihood"}: Returned value is a
matrix of \strong{unscaled} likelihood values for all
observed distances in \code{object$mf}, i.e., raw distance functions
evaluated at the observed distances.  Parameters \code{newdata} and
\code{distances} are ignored when \code{type} is "likelihood".
The negative log likelihood of the full data set is
\code{-sum(log(predict(object,type="likelihood") /  effectiveDistance(object)))}.

\item \strong{If \code{type} == "dfuncs" or "dfunc"}: Returned
value is a matrix whose columns contain scaled distance functions.
The distance functions in each column are evaluated at distances
in argument \code{distances}, not at the observed
distances in \code{object$mf}. The number of distance functions
returned (i.e., number of columns) depends on \code{newdata}
as follows:
\itemize{
\item If \code{newdata} is NULL, one distance function
will be returned for every detection in \code{object$mf}
that has valid covariate values.
\item If \code{newdata} is not NULL, one distance function
will be returned for each observation (row) in \code{newdata}.
}

\item \strong{If \code{type} == "density" or "abundance"}: Returned
object is a tibble containing predicted density and abundance
on the area surveyed by each transect.
}

If \code{object} is a smoothed distance function, it does not have parameters
and this routine will only return scaled distance functions, densities, or
abundances. That is,
\code{type} = "parameters" when \code{object} is smoothed
does not make sense and the smoothed distance function estimate
will be returned if \code{type} does not equal "density" or "abundance".}

\item{distances}{A vector or 1-column matrix of
distances at which to evaluate
distance functions, when distance functions
are requested.  \code{distances} must have measurement units.
Any distances outside the observation
strip (\code{object$w.lo} to \code{object$w.hi}) are discarded.  If
\code{distances} is NULL, a sequence
of \code{getOption("Rdistance_intEvalPts")} (default 101) evenly
spaced distances between
\code{object$w.lo} and \code{object$w.hi} (inclusive) is used.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that
was surveyed.  If both sides of line transects were observed,
\code{propUnitSurveyed}
= 1.  If only a single side of line transects were observed, set
\code{propUnitSurveyed} = 0.5. For point transects, this should be set to
the proportion of each circle that was observed. Length must either be
1 or the total number of transects in \code{x}.}

\item{area}{A scalar containing the total area of inference. Usually, this is
study area size.  If \code{area} is NULL (the default),
\code{area} will be set to 1 square unit of the output units and density estimates
will be produced.
If \code{area} is not NULL, it must have measurement units
assigned by the \code{units} package.
The units on \code{area} must be convertible
to squared output units. Units
on \code{area} must be two-dimensional.
For example, if output units are "foo",
units on area must be convertible to "foo^2" by the \code{units}
package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
others are acceptable.}

\item{\dots}{Included for compatibility with generic \code{predict} methods.}
}
\value{
A matrix containing predictions:
\itemize{
\item \strong{If \code{type} is "parameters"}, the returned matrix
contains likelihood parameters. The extent of the
first dimension (rows) in the returned matrix is equal to
either the number of detection distances
in the observed strip or number of rows in \code{newdata}.
The returned matrix's second dimension (columns) is
the number of parameters in the likelihood
plus the number of expansion terms.
See the help for each likelihoods to interpret
returned parameter values. All parameters are returned
on the inverse-link scale; i.e., \emph{exponential} for canonical
parameters and \emph{identity} for expansion terms.

\item \strong{If \code{type} is "dfuncs" or "dfunc"}, columns of the
returned matrix contains detection functions (i.e., \emph{g(x)}).
The extent of the first
dimension (number of rows) is either the number of distances
specified in \code{distances}
or \code{options()$Rdistance_intEvalPts} if \code{distances} is
not specified.
The extent of the second dimension (number of columns) is:
\itemize{
\item the number of detections with non-missing distances:
if \code{newdata} is NULL.
\item the number of rows in \code{newdata} if
\code{newdata} is specified.
}
All distance functions in columns of the return are scaled
to \code{object$g.x.scale} at \code{object$x.scl}. The returned matrix has
the following additional attributes:
\itemize{
\item \code{attr(return, "distances")} is the vector of distances used to
predict the function in \code{return}.  Either the input \code{distances} object
or the computed sequence of distances when \code{distances} is NULL.
\item \code{attr(return, "x0")} is the vector of distances at which each
distance function in \code{return} was scaled. i.e., the vector of
\code{x.scl}.
\item \code{attr(return, "g.x.scl")} is the height of \emph{g(x)} (the distance
function) at \emph{x0}.
}

\item \strong{If \code{type} is "density" or "abundance"}, the return is a
tibble containing density and abundance estimates by transect.
All transects in the input data (i.e., \code{object$data}) are
included, even those with missing lengths.
Columns in the tibble are:
\itemize{
\item transect ID: the grouping factor of the original RdistDf object.
\item individualsSeen: sum of non-missing group sizes on that transect.
\item avgPdetect: average probability of detection over groups sighted on that transect.
\item effort: size of the area surveyed by that transect.
\item density: density of individuals in the area surveyed by the transect.
\item abundance: abundance of individuals in the area surveyed by the transect.
}
}
}
\description{
Predict either likelihood parameters,
distance functions, site-specific density, or
site-specific abundance from estimated distance function
objects.
}
\examples{

data("sparrowDf")

# For dimension checks:
nd <- getOption("Rdistance_intEvalPts")

# No covariates
dfuncObs <- sparrowDf |> dfuncEstim(formula = dist ~ 1
                     , w.hi = units::as_units(100, "m"))
                     
n  <- nrow(dfuncObs$mf)
p <- predict(dfuncObs) # parameters
all(dim(p) == c(n, 1)) 

# values in newdata ignored because no covariates
p <- predict(dfuncObs, newdata = data.frame(x = 1:5))
all(dim(p) == c(5, 1)) 

# Distance functions in columns, one per observation
p <- predict(dfuncObs, type = "dfunc") 
all(dim(p) == c(nd, n))

d <- setUnits(c(0, 20, 40), "ft")
p <- predict(dfuncObs, distances = d, type = "dfunc") 
all(dim(p) == c(3, n))

p <- predict(dfuncObs
   , newdata = data.frame(x = 1:5)
   , distances = d
   , type = "dfunc") 
all(dim(p) == c(3, 5))

# Covariates
data(sparrowDfuncObserver) # pre-estimated object
\dontrun{
# Command to generate 'sparrowDfuncObserver'
sparrowDfuncObserver <- sparrowDf |> 
            dfuncEstim(formula = dist ~ observer
                     , likelihood = "hazrate")
}

predict(sparrowDfuncObserver)  # n X 2

Observers <- data.frame(observer = levels(sparrowDf$observer))
predict(sparrowDfuncObserver, newdata = Observers) # 5 X 2

predict(sparrowDfuncObserver, type = "dfunc") # nd X n
predict(sparrowDfuncObserver, newdata = Observers, type = "dfunc") # nd X 5
d <- setUnits(c(0, 150, 400), "ft")
predict(sparrowDfuncObserver
  , newdata = Observers
  , distances = d
  , type = "dfunc") # 3 X 5

# Density and abundance by transect
predict(sparrowDfuncObserver
  , type = "density")
  
}
\seealso{
\code{\link[=halfnorm.like]{halfnorm.like()}}, \code{\link[=negexp.like]{negexp.like()}},
\code{\link[=hazrate.like]{hazrate.like()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gamma.like.R
\name{Gamma.like}
\alias{Gamma.like}
\title{Gamma distance function}
\usage{
Gamma.like(a, dist, covars, w.hi = NULL)
}
\arguments{
\item{a}{A vector or matrix of covariate 
and expansion term 
coefficients. If matrix, dimension is 
\emph{k} X \emph{p}, where 
\emph{k} = \code{nrow(a)}) is the number of coefficient
vectors to evaluate (cases) and \emph{p} 
= \code{ncol(a)})
is the number of covariate and expansion 
coefficients in the likelihood (i.e., rows are 
cases and columns are covariates). If \code{a} is a 
dimensionless vector, it is interpreted as a 
single row with \emph{k} = 1. 
Covariate coefficients in \code{a} are the first 
\emph{q} values (\emph{q} <= \emph{p}), and must 
be on a log scale.}

\item{dist}{A numeric vector of length \emph{n} or 
a single-column matrix (dimension \emph{n}X1) containing 
detection distances at which to evaluate the likelihood.}

\item{covars}{A numeric vector of length \emph{q} or a
matrix of dimension \emph{n}X\emph{q} containing 
covariate values 
associated with distances in argument \code{dist}.}

\item{w.hi}{A numeric scalar containing maximum 
distance. The right-hand cutoff or upper limit. 
Ignored by some likelihoods (such as halfnorm, 
negexp, and hazrate), but is a fixed parameter
in other likelihoods (such as oneStep 
and uniform).}
}
\value{
A list containing the following two components:
\itemize{
  \item \bold{L.unscaled}: A matrix of size 
   \emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
   cases = \code{nrow(a)})
   containing likelihood values evaluated at 
   distances in \code{dist}.
   Each row is associated with 
   a single distance, and each column is associated with 
   a single case (row of \code{a}).  Values in this matrix are  
   the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
   These values are "unscaled" likelihood values; they must be 
   scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi  
   to form proper likelihood values. 
   
 \item \bold{params}: A \emph{n}X\emph{b}X\emph{k}  array 
 of the likelihood's (canonical) parameters in link space (i.e., on 
 log scale; \emph{b} = number of canonical parameters in 
 the likelihood; \emph{k} = number of cases). 
 Rows correspond to distances in \code{dist}. Columns 
 correspond to parameters (columns of \code{a}), 
 and pages correspond to cases (rows of \code{a}). 
}
}
\description{
Evaluate the gamma distance function 
for sighting distances, potentially including covariates
and expansion terms
}
\details{
The Rdistance implementation of a Gamma distance function follows 
Becker and Quang (2009). Rdistance's Gamma distance function is 
\deqn{f(d|\alpha, \sigma) = \left(\frac{d}{m}\right)^{\alpha - 1}e^{-(d-m)/\sigma},}{
f(d|a,s) =  (d/m)^{a-1} exp(-(d-m)/s),
}
where \eqn{\alpha}{a} is the \bold{shape} parameter, \eqn{\sigma}{s} is 
the \bold{scale} parameter, and \eqn{m = (\alpha-1)\sigma}{m = (a-1)s}.
\eqn{m} is the mode of the Gamma function, and in Rdistance it's 
scaled to have a maximum of 1.0 at \eqn{m}.  
The \bold{scale} parameter is a function of the shape parameter 
\emph{and} sighting covariates, i.e., 
\deqn{\sigma = k [exp(x'\beta)],}{s = k*exp(x'b),}
where \eqn{x} is a vector of covariate values associated with distance \eqn{d} 
(i.e., a row of \code{covars}), \eqn{\beta}{b} is a vector of the 
first \eqn{q} (=\code{ncol(covars)}) values of the first argument 
of the function (\code{a}), and 
\eqn{k} is a function of the shape parameter, i.e., 
\deqn{k = \frac{1}{\Gamma(\alpha)}  \left(\frac{a - 1}{e^1} \right)^{a - 1}.}{
k = (1/Gamma(a)) * (((a - 1)/exp(1))^(a - 1)).}  
The shape parameter \eqn{\alpha}{a} is the 
\eqn{q+1}-st value in the function's first argument and is constrained to 
be strictly greater than 1.0.
 
See Examples for use of \code{\link{GammaReparam}} to compute \eqn{\alpha}{a}
and \eqn{\sigma}{s} from fitted object coefficients.
}
\examples{
x <- seq(0, 100, length=100)
covars <- matrix(1,100,1)

# Plots showing changes in scale
plot(x, Gamma.like(c(log(20),2.5), x, covars)$L.unscaled, type="l", col="red")
lines(x, Gamma.like(c(log(40),2.5), x, covars)$L.unscaled, col="blue")

# Plots showing changes in shape
plot(x, Gamma.like(c(log(20),1.5), x, covars)$L.unscaled, type="l", col="red")
lines(x, Gamma.like(c(log(20),2.5), x, covars)$L.unscaled, col="blue")
lines(x, Gamma.like(c(log(20),4.5), x, covars)$L.unscaled, col="green")

# Roll-your-own plot, showing "re-parameterization":
# Assume fitted object coefficients are c(log(20), 4.5)
fit <- list(par = c(log(20), 4.5))

# The distance function is then,
gammaPar <- GammaReparam( scl = exp(fit$par[1])
                        , shp = fit$par[2] ) # returns scl=k*exp(x'B)
scl <- gammaPar$scl
shp <- gammaPar$shp
m <- (shp - 1) * scl
g <- (x / m)^(shp - 1) * exp(-(x - m) / scl) # distance function
lines(x, g, lwd = 3, lty = 2, col="green3")

}
\references{
Becker, E. F., and P. X. Quang, 2009. \emph{A Gamma-Shaped Detection Function for Line-Transect Surveys with Mark-Recapture and Covariate Data.}
  Journal of Agricultural, Biological, and Environmental Statistics 14(2):207-223.
}
\seealso{
\code{\link{dfuncEstim}}, 
         \code{\link{abundEstim}},
         other \code{<likelihood>.like} functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentiableLikelihoods.R
\name{differentiableLikelihoods}
\alias{differentiableLikelihoods}
\title{Differentiable likelihoods in Rdistance}
\usage{
differentiableLikelihoods()
}
\value{
A character vector of differentiable
likelihoods.
}
\description{
Return a character vector of
likelihoods which are differentiable and hence
the second derivative method can be used to estimate
variance-covariance.  Any likelihoods not in this list
must use bootstrapping. This vector is polled in
a few Rdistance routines, notably \code{parseModel}
and \code{varcovarEstim}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HookesJeeves.R
\name{HookeJeeves}
\alias{HookeJeeves}
\title{'nlminb' optimizer}
\usage{
HookeJeeves(ml, strt.lims)
}
\arguments{
\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}

\item{strt.lims}{A list containing start, low, and high limits for
parameters of the requested likelihood. This list is typically produced
by a call to \code{\link[=startLimits]{startLimits()}}.}
}
\value{
A list with following named components:
\itemize{
\item par = parameters
\item loglik = objective function value at minimum
\item convergence = 0 for yes, other for no
\item iterations = number of iterations
\item evaluations = function evaluations
\item message = a convergence message
\item varcovar = a variance covariance matrix of parameters
\item limits = low and high limits
}
}
\description{
Call R native function 'nlminb' to perform optimization.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangle.like.R
\name{triangle.like}
\alias{triangle.like}
\title{Mixture of triangle and uniform likelihood}
\usage{
triangle.like(a, dist, covars, w.hi = NULL)
}
\arguments{
\item{a}{A vector or matrix of covariate 
and expansion term 
coefficients. If matrix, dimension is 
\emph{k} X \emph{p}, where 
\emph{k} = \code{nrow(a)}) is the number of coefficient
vectors to evaluate (cases) and \emph{p} 
= \code{ncol(a)})
is the number of covariate and expansion 
coefficients in the likelihood (i.e., rows are 
cases and columns are covariates). If \code{a} is a 
dimensionless vector, it is interpreted as a 
single row with \emph{k} = 1. 
Covariate coefficients in \code{a} are the first 
\emph{q} values (\emph{q} <= \emph{p}), and must 
be on a log scale.}

\item{dist}{A numeric vector of length \emph{n} or 
a single-column matrix (dimension \emph{n}X1) containing 
detection distances at which to evaluate the likelihood.}

\item{covars}{A numeric vector of length \emph{q} or a
matrix of dimension \emph{n}X\emph{q} containing 
covariate values 
associated with distances in argument \code{dist}.}

\item{w.hi}{A numeric scalar containing maximum 
distance. The right-hand cutoff or upper limit. 
Ignored by some likelihoods (such as halfnorm, 
negexp, and hazrate), but is a fixed parameter
in other likelihoods (such as oneStep 
and uniform).}
}
\value{
A list containing the following two components:
\itemize{
  \item \bold{L.unscaled}: A matrix of size 
   \emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
   cases = \code{nrow(a)})
   containing likelihood values evaluated at 
   distances in \code{dist}.
   Each row is associated with 
   a single distance, and each column is associated with 
   a single case (row of \code{a}).  Values in this matrix are  
   the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
   These values are "unscaled" likelihood values; they must be 
   scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi  
   to form proper likelihood values. 
   
 \item \bold{params}: A \emph{n}X\emph{b}X\emph{k}  array 
 of the likelihood's (canonical) parameters in link space (i.e., on 
 log scale; \emph{b} = number of canonical parameters in 
 the likelihood; \emph{k} = number of cases). 
 Rows correspond to distances in \code{dist}. Columns 
 correspond to parameters (columns of \code{a}), 
 and pages correspond to cases (rows of \code{a}). 
}
}
\description{
Compute likelihood function for a mixture of a triangle and 
uniform distributions.
}
\details{
Rdistance's \code{triangle} likelihood is a mixture of a 
triangle and uniform distribution. The 'triangle' density function
is  
\deqn{f(d|\theta, p) = (1 - \frac{1-p}{\theta}d) I(0 \leq d \leq \theta) + 
       p I(\theta \le d \leq w),}{
       f(d|T) = (1 - (1-p)*d/T)I(0<=d<=T) + p*I(T<d<=w),}
where 
\eqn{I(x)} is the indicator function for event \eqn{x}, 
and \eqn{w} is the nominal strip width 
(i.e., \code{w.hi - w.lo} in Rdistance). 
The unknown parameters to be estimated 
are \eqn{\theta}{T} and \eqn{p} 
(\eqn{w} is fixed - given by the user). 
 
Covariates influence values of \eqn{\theta}{T} 
via a log link function, i.e., \eqn{\theta = e^{x'b}}{T = exp(x'b)},
where \eqn{x} is the vector of covariate values 
associated with distance \eqn{d}, and \eqn{b}
is the vector of estimated coefficients.
}
\examples{

w <- 250
Theta <- 160
p <- 0.4
d <- seq(0,w,length = w+1)
y <- (1 - ((1-p)/Theta)*d)*(d <= Theta) + p*(d > Theta)
plot(d, y, type="l", ylim = c(0,1), xlab = "Distance", ylab = "Probability")
points(Theta, p, pch=16, col = "red")
lines(c(-10,Theta), c(p,p), lty = 2, col = "grey")
axis(2, at=p, label = "p", line = 1, srt = 0, tick = FALSE)
lines(c(Theta,Theta), c(-1,p), lty = 2, col = "grey")
axis(1, at=Theta, label = "Theta", line = 2, tick = FALSE)

Theta <- 25
p <- 0.2
y <- (1 - ((1-p)/Theta)*d)*(d <= Theta) + p*(d > Theta)
lines(d, y)
points(Theta, p, pch=16, col = "red")
lines(c(-10,Theta), c(p,p), lty = 2, col = "grey")
axis(2, at=p, label = "p", line = 1, srt = 0, tick = FALSE)
lines(c(Theta,Theta), c(-1,p), lty = 2, col = "grey")
axis(1, at=Theta, label = "Theta", line = 2, tick = FALSE)

# same as above
y <- triangle.like(a = c(log(Theta), p)
                 , dist = d
                 , covars = matrix(1, length(d))
                 , w.hi = 250)
lines(d, y$L.unscaled, col = "green")

}
\seealso{
\code{\link{dfuncEstim}}, 
         \code{\link{abundEstim}},
         other \code{<likelihood>.like} functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkUnits.R
\name{checkUnits}
\alias{checkUnits}
\title{Check for the presence of units}
\usage{
checkUnits(ml)
}
\arguments{
\item{ml}{An \code{Rdistance} model list produced by 
\code{\link{parseModel}} containing a list
of parameters for the distance model.}
}
\value{
The input \code{ml} list, with units of various 
quantities converted to common units.  If a check fails, for example, 
a quantity does not have units, an error is thrown.
}
\description{
Check for the presence of physical measurement units on key
columns of  
an \code{RdistDf} data frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varcovarEstim.R
\name{varcovarEstim}
\alias{varcovarEstim}
\title{Estimate variance-covariance}
\usage{
varcovarEstim(x, ml)
}
\arguments{
\item{x}{An estimated detection function object, normally 
produced by calling \code{\link{dfuncEstim}}.}

\item{ml}{Either a Rdistance 'model frame' or an Rdistance 
'fitted object'.  Both are of class "dfunc". 
Rdistance 'model frames' are lists containing components 
necessary to estimate a distance function, but no estimates. 
Rdistance 'model frames' are typically
produced by calls to \code{\link{parseModel}}. 
Rdistance 'fitted objects'
are typically produced by calls to \code{\link{dfuncEstim}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates, 
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}
}
\value{
A square symmetric matrix estimating the 
variance-covariance matrix of parameters in \code{x}. 
Dimension of return is p X p, where p = \code{length(x$par)}.
}
\description{
Estimate the variance-covariance matrix of parameters 
in the distance function. If the likelihood is 
differentiable, the variance-covariance matrix is 
estimated from the second derivative of the likelihood 
(i.e., the hessian).  If the likelihood is not 
differentiable, the variance-covariance matrix is 
a matrix of 0's that are interpreted as "pending" (i.e., 
pending bootstrapping).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateOneStepLines.R
\name{integrateOneStepLines}
\alias{integrateOneStepLines}
\title{Integrate Line-transect One-step function}
\usage{
integrateOneStepLines(object, newdata = NULL, Units = NULL)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced 
by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
If a matrix, each row corresponds to a 
distance function and each column is a parameter. The first column is 
the parameter related to sighting covariates and must be transformed 
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
be applied outside this routine).  If \code{object} is a matrix, 
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function 
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for 
the distance function covariates. If NULL and 
\code{object} is a fitted distance function, the  
observed covariates stored in
\code{object} are used (behavior similar to \code{\link{predict.lm}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{Units}{Physical units of sighting distances if 
\code{object} is a matrix. Sighting distance units can differ from units 
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in 
\code{object}. 
If \code{object} is a distance function and 
\code{newdata} is specified, the returned vector's length is 
\code{nrow(newdata)}.  If \code{object} is a distance function and 
\code{newdata} is NULL, 
returned vector's length is \code{length(distances(object))}. If 
\code{object} is a matrix, return's length is 
\code{nrow(object)}.
}
\description{
Compute exact integral of the one-step distance function for line 
transects.
}
\details{
Returned integrals are
\deqn{\int_0^{w} (\frac{p}{\theta_i}I(0\leq x \leq \theta_i) + 
   \frac{1-p}{w - \theta_i}I(\theta_i < x \leq w)) dx = \frac{\theta_i}{p},}{
Integral((p/Theta)I(0<=x<=Theta) + ((1-p)/(w-Theta))I(Theta<x<=w)) = Theta / p,} 
where \eqn{w = w.hi - w.lo}, \eqn{\theta_i}{Theta} is the estimated one-step
distance function
threshold for the i-th observed distance, and \eqn{p}{p} is the estimated 
one-step proportion.
}
\section{Note}{

Users will not normally call this function. It is called 
internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}.
}

\examples{

# A oneStep distance function on simulated data
whi <- 250
T <- 100  # true threshold
p <- 0.85 # true proportion <T
n <- 200  # num simulated points
x <- c( runif(n*p, min=0, max=T), runif(n*(1-p), min=T, max=whi))
x <- setUnits(x, "m")
tranID <- sample(rep(1:10, each=n/10), replace=FALSE)
detectDf <- data.frame(transect = tranID, dist = x)
siteDf <- data.frame(transect = 1:10
               , length = rep(setUnits(10,"m"), 10))
distDf <- RdistDf(siteDf, detectDf)

# Estimation
fit <- dfuncEstim(distDf
                 , formula = dist ~ 1
                 , likelihood = "oneStep"
                 , w.hi = setUnits(whi, "m")
                 )
table(integrateOneStepLines(fit))
table(ESW(fit))

# Check:
T.hat <- exp(fit$par[1])
p.hat <- fit$par[2]
gAtT <- ((1-p.hat) * T.hat) / (p.hat * (whi - T.hat))

plot(fit)
abline(h = gAtT, col="blue")

areaLE.T <- (1.0) * T.hat
areaGT.T <- gAtT * (whi - T.hat)
areaLE.T + areaGT.T  # ESW

# Equivalent
T.hat / p.hat

}
\seealso{
\code{\link{integrateNumeric}}; \code{\link{integrateNegexpLines}}; 
\code{\link{integrateHalfnormLines}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likeParamNames.r
\name{likeParamNames}
\alias{likeParamNames}
\title{Likelihood parameter names}
\usage{
likeParamNames(like.form)
}
\arguments{
\item{like.form}{A text string naming the form of the likelihood.
An error is thrown if the likelihood is unknown.}
}
\value{
A vector of parameter names for the likelihood
}
\description{
Returns names of the likelihood parameters. This is
a helper function and is not
necessary for estimation.  It is nice to label some
parameters with descriptive
names like "sigma" or "k", depending on the likelihood.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insertOneStepBreaks.R
\name{insertOneStepBreaks}
\alias{insertOneStepBreaks}
\title{Insert oneStep Likelihood breaks}
\usage{
insertOneStepBreaks(obj, newData, xseq)
}
\arguments{
\item{obj}{A fitted Rdistance model object}

\item{newData}{A data frame containing covariate values to
use in prediction.}

\item{xseq}{A vector of distances into which the break points
are inserted.}
}
\value{
A vector like \code{xseq}, but with the break points
inserted.
}
\description{
Compute break points in a oneStep likelihood
and insert them into a sequence of distances.  The idea
is to insert a point just left and just right of the breaks
so that they plot as vertical lines.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getNCores.R
\name{getNCores}
\alias{getNCores}
\title{Set number of cores}
\usage{
getNCores(parallel)
}
\arguments{
\item{parallel}{A logical scalar, or a positive integer; ignored unless
confidence intervals are requested (i.e., \code{!is.null(ci)}).
If TRUE, bootstrap iterations are
run in parallel using the maximum number of CPU cores minus 1.
The maximum number of CPU cores is reported by \code{parallel::detectCores()}.
If a positive integer (1 <= \code{parallel} <= maximum cores), bootstrap
iterations are performed in parallel on that many cores.
If FALSE, bootstrap iterations are performed in series, and progress
will be shown if \code{showProgress == TRUE}. Parameters
\code{showProgress} and \code{plot.bs} are ignored when operating
in parallel.}
}
\value{
A list with components $parallel and $cores. $parallel
is logical, T for parallel operations, F o.w.  $cores is the
number of cores to use during parallel operations.  $cores is
integer in the range 1, 2, ..., max(Available cores).
}
\description{
Set the number of cores for parallel operations.
Also convert integer 'parallel' to TRUE-FALSE for ease.
}
\details{
Input \code{parallel} <= 0 is converted to 1.
Input \code{parallel} > maxCores is converted to maxCores.
If input \code{parallel} is numeric, is first converted to integer
by rounding down.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GammaReparam.R
\name{GammaReparam}
\alias{GammaReparam}
\title{Reparameterise Gamma parameters for use in dgamma}
\usage{
GammaReparam(shp, scl)
}
\arguments{
\item{shp}{Rdistance's shape parameter}

\item{scl}{Rdistance's scale parameter.  This parameter 
is related to covariates via exp(x'B).}
}
\value{
A list with components $shp and $scl, which are the 
re-parameterized versions of the input parameters suitable for 
us in R::dgamma().
}
\description{
Transform Rdistance's version of the Gamma distribution
parameters, which is that of Becker and Quan, into the version for 
use in R::dgamma() and elsewhere.
}
\examples{
# Rdistance Gamma parameters
Rd.scl <- 50  # must be >0
Rd.shp <- 1.5 # must be >1

# dgamma parameters
dgParams <- GammaReparam(Rd.shp, Rd.scl)
dgParams

# Gamma distribution with (Rd.scl, Rd.shp) from 0 to 100
curve(dgamma(x, shape=dgParams$shp, scale = dgParams$scl)
           , from = 0
           , to = 100)
 
# Rdistance's version: same curve but scaled so maximum = 1
x <- seq(0, 100, length = 200) 
scl <- dgParams$scl
shp <- dgParams$shp
m <- (shp - 1) * scl
g <- (x / m)^(shp - 1) * exp(-(x - m) / scl) # distance function
plot(x, g, type = "l")

}
\seealso{
'Details' section of \code{\link{Gamma.like}}  
for Rdistance's Gamma distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.dfunc.r
\name{print.dfunc}
\alias{print.dfunc}
\title{Print method for distance function object}
\usage{
\method{print}{dfunc}(x, ...)
}
\arguments{
\item{x}{An estimated detection function object, normally
produced by calling \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{\dots}{Included for compatibility with other print methods.  Ignored here.}
}
\value{
The input distance function (\code{x}) is returned invisibly.
}
\description{
Print method for distance function objects produced
by \code{dfuncEstim}.
}
\examples{
# Load example sparrow data (line transect survey type)
data(sparrowSiteData)
data(sparrowDetectionData)

# Fit half-normal detection function
sparrowDf <- RdistDf(sparrowSiteData, sparrowDetectionData)
dfunc <- sparrowDf |> dfuncEstim(formula=dist~1)

dfunc

}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}, \code{\link[=plot.dfunc]{plot.dfunc()}},
\code{\link[=print.abund]{print.abund()}}, \code{\link[=summary.dfunc]{summary.dfunc()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nLL.R
\name{nLL}
\alias{nLL}
\title{Negative log likelihood of distances}
\usage{
nLL(a, ml, verbosity = 0)
}
\arguments{
\item{a}{A vector of likelihood parameter values. Length and
meaning depend on \code{ml$series} and \code{ml$expansions}. If no expansion
terms were called for (i.e., \code{ml$expansions = 0}), the distance
likelihood contain one or two canonical parameters (see Details).
If one or more expansions are called for, coefficients for the
expansion terms follow coefficients for the canonical parameters.
i.e., length of this vector is
\verb{(num Covars incl. intercept) + expansions + 1*(like \%in\% c("hazrate"))}.}

\item{ml}{Either a Rdistance 'model frame' or an Rdistance
'fitted object'.  Both are of class "dfunc".
Rdistance 'model frames' are lists containing components
necessary to estimate a distance function, but no estimates.
Rdistance 'model frames' are typically
produced by calls to \code{\link[=parseModel]{parseModel()}}.
Rdistance 'fitted objects'
are typically produced by calls to \code{\link[=dfuncEstim]{dfuncEstim()}}.
'Fitted objects' are 'model frames'
with additional components such as the parameters estimates,
log likelihood value, convergence information, and the variance-
covariance matrix of the parameters.}

\item{verbosity}{The level of output produced during estimation.
\code{verbosity} = 0 produces no output. Increasing values
of \code{verbosity} >= 1 produce increasing levels of intermediate details.
This is mostly used for de-bugging and checking convergence.}
}
\value{
A scalar, the negative of the log likelihood evaluated at
parameters \code{a}.
}
\description{
Return the negative log likelihood of
observed detection distances given a likelihood
and the estimated parameters.
}
\details{
\strong{Expansion Terms}: If \code{ml$expansions} = k (k > 0),
the expansion function specified by \code{ml$series} is
called (see for example \code{\link[=cosine.expansion]{cosine.expansion()}}).
Assuming \eqn{h_{ij}(x)}{h_ij(x)} is the
\eqn{j^{th}}{j-th} expansion term for the
\eqn{i^{th}}{i-th} distance and that
\eqn{c_1, c_2, \dots, c_k}{c(1), c(2), ..., c(k)}
are (estimated) coefficients for the expansion terms,
the likelihood contribution for the \eqn{i^{th}}{i-th}
distance is, \deqn{f(x|a,b,c_1,c_2,\dots,c_k) = 
f(x|a,b)(1 + \sum_{j=1}^{k} c_j h_{ij}(x)).}{f(x|a,b,c_1,c_2,...,c_k) 
= f(x|a,b)(1 + c(1) h_i1(x) + c(2) h_i2(x) + ... + c(k) h_ik(x)). }
}
\examples{

# A halfnorm distance function 
fit <- dfuncEstim(sparrowDf, dist~1, likelihood = "halfnorm")
nLL(fit$par, fit, 3)
fit$loglik
ESW(fit)[1]

# Another way, b/c we have pnorm()
d <- distances(fit)
ones <- matrix(1, nrow = length(d), ncol = 1)
l <- halfnorm.like(fit$par, d, ones)
esw <-(pnorm(units::drop_units(fit$w.hi)
  , units::drop_units(fit$w.lo)
  , sd = exp(l$params)) - 0.5) * sqrt(2*pi) * exp(l$params)
-sum(log(l$L.unscaled/esw))

# A third way, b/c we have pnorm() and dnorm(). 
l2 <- dnorm(units::drop_units(d), mean = 0, sd = exp(fit$par))
scaler <- pnorm(units::drop_units(fit$w.hi), mean = 0, sd = exp(fit$par)) - 0.5 
-sum(log(l2/scaler))
 
}
\seealso{
See \code{\link[=halfnorm.like]{halfnorm.like()}} and links there;
\code{\link[=dfuncEstim]{dfuncEstim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateHalfnormPoints.R
\name{integrateHalfnormPoints}
\alias{integrateHalfnormPoints}
\title{Integrate Half-normal Point transects}
\usage{
integrateHalfnormPoints(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced 
by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
If a matrix, each row corresponds to a 
distance function and each column is a parameter. The first column is 
the parameter related to sighting covariates and must be transformed 
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
be applied outside this routine).  If \code{object} is a matrix, 
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function 
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for 
the distance function covariates. If NULL and 
\code{object} is a fitted distance function, the  
observed covariates stored in
\code{object} are used (behavior similar to \code{\link{predict.lm}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function. 
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if 
\code{object} is a matrix. Sighting distance units can differ from units 
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in 
\code{object}. 
If \code{object} is a distance function and 
\code{newdata} is specified, the returned vector's length is 
\code{nrow(newdata)}.  If \code{object} is a distance function and 
\code{newdata} is NULL, 
returned vector's length is \code{length(distances(object))}. If 
\code{object} is a matrix, return's length is 
\code{nrow(object)}.
}
\description{
Compute integral of the half-normal distance function for 
point surveys.
}
\details{
Returned integrals are 
\deqn{\int_0^{w} xe^{-x^2/2\sigma_i^2} dx = 0.5\sigma_i^2(1 - e^{-w^2/2\sigma_i^2}),}{
Integral(xe^{-x^2/(2s^2)}) = 0.5*s^2*(1 - exp(-w^2/(2*s^2))),} 
where \eqn{w = w.hi - w.lo} and \eqn{\sigma_i}{s} is the estimated half-normal 
distance function parameter for the i-th observed distance.
}
\section{Note}{

Users will not normally call this function. It is called 
internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}.
}

\examples{

# Fake distance function object w/ minimum inputs for integration
d <- rep(1,4) \%m\%. # Only units needed, not values
obs <- factor(rep(c("obs1", "obs2"), 2))
beta <- c(3.5, -0.5)
w.hi <- 125
w.lo <- 20
ml <- list(
    mf = model.frame(d ~ obs)
  , par = beta 
  , likelihood = "halfnorm"
  , w.lo = w.lo \%#\% "m"
  , w.hi = w.hi \%#\% "m"
  , expansions = 0
)
class(ml) <- "dfunc"
integrateHalfnormPoints(ml)

# Check: Integral of x exp(-x^2/(2*s^2)) from 0 to w = w.hi-w.lo
sigma <- exp(c(beta[1], beta[1] + beta[2]))
w <- w.hi - w.lo
intgral <- sigma^2 * (1 - exp(-w^2 / (2*sigma^2)))
intgral

# Effective detection radius
sqrt(2 * intgral)

}
\seealso{
\code{\link{integrateNumeric}}; \code{\link{integrateNegexpPoints}}; 
\code{\link{integrateOneStepPoints}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unnest.R
\name{unnest}
\alias{unnest}
\title{Unnest an RdistDf data frame}
\usage{
unnest(data, ...)
}
\arguments{
\item{data}{An \code{RdistDf} data frame. \code{RdistDf} data frames
contain one line per transect and a list-based column. The list-based
column contains a data frame with detection information.
The detection information data frame on each row contains (at least) distances
and group sizes of all targets detected on the transect.
Function \code{\link[=RdistDf]{RdistDf()}} creates \code{RdistDf} data frames
from separate transect and detection data frames.
\code{\link[=is.RdistDf]{is.RdistDf()}} checks whether data frames
are \code{RdistDf}'s.}

\item{...}{Additional arguments passed to \code{tidyr::unnest} if
\code{data} is not an RdistDf.}
}
\value{
An expanded data frame, without embedded data frames. Rows
in the return represent with one detection or one transect.  If multiple
detections were made on one transect, the transect will appear on multiple
rows.  If no detections were made on a transect, it will appear on one
row with NA detection distance.
}
\description{
Unnest an RdistDf data frame by expanding
the embedded 'detections' column.  This unnest
includes the so-called zero transects (transects without detections).
}
\examples{
data('sparrowDf')

# tidyr::unnest() does not include zero transects
detectionDf <- tidyr::unnest(sparrowDf, detections)
nrow(detectionDf)
any(detectionDf$siteID == "B2")

# Rdistance::unnest() includes zero transects
fullDf <- unnest(sparrowDf)
nrow(fullDf)
any(fullDf$siteID == "B2")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intercept.only.R
\name{intercept.only}
\alias{intercept.only}
\title{Detect intercept-only distance function}
\usage{
intercept.only(object)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}
}
\value{
TRUE if \code{object} contains an intercept-only.
FALSE if \code{object} contains at least one detection-level
or transect-level covariate in the detection function.
}
\description{
Utility function to detect whether a distance function has
covariates beyond the intercept. If the model contains an
intercept-only, effective distance is constant across detections
and short-cuts can be implemented in code.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateNegexpPoints.R
\name{integrateNegexpPoints}
\alias{integrateNegexpPoints}
\title{Integrate Negative exponential point surveys}
\usage{
integrateNegexpPoints(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced 
by a call to \code{\link{dfuncEstim}}), or a matrix of canonical 
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}). 
If a matrix, each row corresponds to a 
distance function and each column is a parameter. The first column is 
the parameter related to sighting covariates and must be transformed 
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must 
be applied outside this routine).  If \code{object} is a matrix, 
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function 
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for 
the distance function covariates. If NULL and 
\code{object} is a fitted distance function, the  
observed covariates stored in
\code{object} are used (behavior similar to \code{\link{predict.lm}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function. 
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object} 
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if 
\code{object} is a matrix. Sighting distance units can differ from units 
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in 
\code{object}. 
If \code{object} is a distance function and 
\code{newdata} is specified, the returned vector's length is 
\code{nrow(newdata)}.  If \code{object} is a distance function and 
\code{newdata} is NULL, 
returned vector's length is \code{length(distances(object))}. If 
\code{object} is a matrix, return's length is 
\code{nrow(object)}.
}
\description{
Compute integral of the negative exponential distance function
for point surveys
}
\details{
Returned integrals are
\deqn{\int_0^{w} xe^{-a_i x} dx = \frac{1 - e^{-a_i w} (a_i w + 1)}{a_i^2},}{
Integral( x exp(-a*x) ) = (1 - exp(-a*w)(a*w + 1)) / a^2,}
where \eqn{w = w.hi - w.lo} and \eqn{a_i}{a} is the estimated 
negative exponential distance 
function parameter for the 
i-th observed distance.
}
\section{Note}{

Users will not normally call this function. It is called 
internally by \code{\link{nLL}} and \code{\link{effectiveDistance}}.
}

\examples{

# Fake distance function object w/ minimum inputs for integration
d <- rep(1,4) \%#\% "m" # Only units needed, not values
obs <- factor(rep(c("obs1", "obs2"), 2))
beta <- c(-5, -0.5)
w.hi <- 125
w.lo <- 20
ml <- list(
    mf = model.frame(d ~ obs)
  , par = beta 
  , likelihood = "negexp"
  , w.lo = w.lo \%#\% "m"
  , w.hi = w.hi \%#\% "m"
  , expansions = 0
)
class(ml) <- "dfunc"
integrateNegexpPoints(ml)

# Check: Integral of x*exp(-bx) from 0 to w.hi-w.lo
b <- c(exp(beta[1]), exp(beta[1] + beta[2]))
intgral <- (1 - exp(-b*(w.hi - w.lo)) * (b*(w.hi - w.lo) + 1)) / (b^2)
intgral


}
\seealso{
\code{\link{integrateNumeric}}; \code{\link{integrateNegexpLines}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dfuncEstimErrMessage.R
\name{dfuncEstimErrMessage}
\alias{dfuncEstimErrMessage}
\title{dfuncEstim error messages}
\usage{
dfuncEstimErrMessage(txt, attri)
}
\arguments{
\item{txt}{A text string describing the error.}

\item{attri}{An attribute to report.}
}
\value{
A string
}
\description{
Utility function to produce error messages suitable for \code{stop}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.smoothed.R
\name{is.smoothed}
\alias{is.smoothed}
\title{Tests for smoothed distance functions}
\usage{
is.smoothed(object)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}
}
\value{
TRUE if the model frame or fitted distance function 
arises from a non-parametric density smoother. FALSE if the 
model frame or distance 
function is a parameterized function.
}
\description{
Determines whether a distance function is a non-parametric
smooth or classic parameterized function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.abund.r
\name{print.abund}
\alias{print.abund}
\title{Print abundance estimates}
\usage{
\method{print}{abund}(x, ...)
}
\arguments{
\item{x}{An object output by \code{abundEstim}.  This is a distance 
function object augmented with abundance estimates, and has 
class \code{c("abund", "dfunc")}.}

\item{\dots}{Included for compatibility to other print methods.
Ignored here.}
}
\value{
0 is invisibly returned
}
\description{
Print an object of class \code{c("abund","dfunc")} 
produced by \code{abundEstim}.
}
\examples{
# Load example sparrow data (line transect survey type)
data(sparrowDf)

# Fit half-normal detection function
dfunc <- sparrowDf |> dfuncEstim(formula=dist~groupsize(groupsize))

# Estimate abundance given a detection function
fit <- abundEstim(object = dfunc
                , area = setUnits(4105, "km^2")
                , ci = NULL)
print(fit)
summary(fit)

\dontrun{
# Bootstrap confidence intervals (500 iterations)
# Requires ~4 min
fit <- abundEstim(object = dfunc
                , area = setUnits(4105, "km^2")
                , ci = 0.95
                , plot.bs = TRUE
                , showProgress = TRUE)
print(fit)
summary(fit)
}

}
\seealso{
\code{\link{dfuncEstim}}, \code{\link{abundEstim}}, 
\code{\link{summary.dfunc}}, \code{\link{print.dfunc}}, 
\code{\link{summary.abund}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hazrate.like.R
\name{hazrate.like}
\alias{hazrate.like}
\title{Hazard rate likelihood}
\usage{
hazrate.like(a, dist, covars, w.hi = NULL)
}
\arguments{
\item{a}{A vector or matrix of covariate 
and expansion term 
coefficients. If matrix, dimension is 
\emph{k} X \emph{p}, where 
\emph{k} = \code{nrow(a)}) is the number of coefficient
vectors to evaluate (cases) and \emph{p} 
= \code{ncol(a)})
is the number of covariate and expansion 
coefficients in the likelihood (i.e., rows are 
cases and columns are covariates). If \code{a} is a 
dimensionless vector, it is interpreted as a 
single row with \emph{k} = 1. 
Covariate coefficients in \code{a} are the first 
\emph{q} values (\emph{q} <= \emph{p}), and must 
be on a log scale.}

\item{dist}{A numeric vector of length \emph{n} or 
a single-column matrix (dimension \emph{n}X1) containing 
detection distances at which to evaluate the likelihood.}

\item{covars}{A numeric vector of length \emph{q} or a
matrix of dimension \emph{n}X\emph{q} containing 
covariate values 
associated with distances in argument \code{dist}.}

\item{w.hi}{A numeric scalar containing maximum 
distance. The right-hand cutoff or upper limit. 
Ignored by some likelihoods (such as halfnorm, 
negexp, and hazrate), but is a fixed parameter
in other likelihoods (such as oneStep 
and uniform).}
}
\value{
A list containing the following two components:
\itemize{
  \item \bold{L.unscaled}: A matrix of size 
   \emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
   cases = \code{nrow(a)})
   containing likelihood values evaluated at 
   distances in \code{dist}.
   Each row is associated with 
   a single distance, and each column is associated with 
   a single case (row of \code{a}).  Values in this matrix are  
   the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
   These values are "unscaled" likelihood values; they must be 
   scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi  
   to form proper likelihood values. 
   
 \item \bold{params}: A \emph{n}X\emph{b}X\emph{k}  array 
 of the likelihood's (canonical) parameters in link space (i.e., on 
 log scale; \emph{b} = number of canonical parameters in 
 the likelihood; \emph{k} = number of cases). 
 Rows correspond to distances in \code{dist}. Columns 
 correspond to parameters (columns of \code{a}), 
 and pages correspond to cases (rows of \code{a}). 
}
}
\description{
Computes the hazard rate distance function.
}
\details{
The hazard rate likelihood is 
\deqn{f(x|\sigma,k) = 1 - \exp(-(x/\sigma)^{-k})}{%
f(x|Sigma,k) = 1 - exp(-(x/Sigma)^(-k))} 
where \eqn{\sigma}{Sigma} determines location 
(i.e., distance at which the function equals 1 - exp(-1) = 0.632), 
and \eqn{k}{k} determines slope of the function 
at \eqn{\sigma}{Sigma} (i.e., larger k equals steeper 
slope at \eqn{\sigma}{Sigma}). For distance analysis, 
the valid range for both \eqn{\sigma}{Sigma} and k is
\eqn{\geq 0}{>=0}.
}
\examples{
d <- seq(0, 100, length=100)
covs <- matrix(1,length(d),1)
hazrate.like(c(log(20), 5), d, covs)

# Changing location parameter
plot(d, hazrate.like(c(log(20), 5), d, covs)$L.unscaled, type="l", col="red")
lines(d, hazrate.like(c(log(40), 5), d, covs)$L.unscaled, col="blue")
abline(h = 1 - exp(-1), lty = 2)
abline(v = c(20,40), lty = 2)

# Changing slope parameter
plot(d, hazrate.like(c(log(50), 20), d, covs)$L.unscaled, type="l", col="red")
lines(d, hazrate.like(c(log(50), 2), d, covs)$L.unscaled, col="blue")
abline(h = 1 - exp(-1), lty = 2)
abline(v = 50, lty = 2)

         
}
\seealso{
\code{\link{dfuncEstim}}, 
         \code{\link{abundEstim}},
         other \code{<likelihood>.like} functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sine.expansion.R
\name{sine.expansion}
\alias{sine.expansion}
\title{Sine expansion terms}
\usage{
sine.expansion(x, expansions)
}
\arguments{
\item{x}{A numeric matrix of distances at which to evaluate
the expansion series. For distance analysis, \code{x} should
be the proportion of the maximum sighting distance
at which a group was sighted, i.e., \eqn{x = d/w}, where \eqn{d}
is sighting distance and \eqn{w} is maximum sighting distance.}

\item{expansions}{A scalar specifying the number of
expansion terms to compute. Must be one of the
integers 1, 2, 3, 4, or 5.}
}
\value{
A 3D array of size \code{nrow(x)} X \code{ncol(x)} X \code{expansions}.
The 'pages' (3rd dimension) of this array are the cosine expansions of
\code{x}. i.e., page 1 is the first expansion term of \code{x},
page 2 is the second expansion term of \code{x}, etc.
}
\description{
Computes the sine expansion terms that modify the
shape of distance likelihood functions.
}
\details{
The sine expansion used here is:
\itemize{
\item \strong{First term}: \deqn{h_1(x)=\sin(2\pi x)/2,}{h1(x) = sin(2*Pi*x)/2,}
\item \strong{Second term}: \deqn{h_2(x)=\sin(3\pi x)/2,}{h2(x) = sin(3*Pi*x)/2,}
\item \strong{Third term}: \deqn{h_3(x)=\sin(4\pi x)/2,}{h3(x) = sin(4*Pi*x)/2,}
\item \strong{Fourth term}: \deqn{h_4(x)=\sin(5\pi x)/2,}{h4(x) = sin(5*Pi*x)/2,}
\item \strong{Fifth term}: \deqn{h_5(x)=\sin(6\pi x)/2,}{h5(x) = sin(6*Pi*x)/2,}
}
The maximum number of expansion terms is 5.

The sine expansion frequency in Rdistance is pi. Each term is one pi more than
the previous. The cosine expansion frequency in Rdistance is 2*pi. Consequently,
the sine and cosine expansions fit different models.
}
\examples{
x <- matrix(seq(0, 1, length = 200), ncol = 1)
sin.expn <- sine.expansion(x, 5)
plot(range(x), range(sin.expn), type="n")
matlines(x, sin.expn[,1,1:5], col=rainbow(5), lty = 1)

}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}},
\code{\link[=cosine.expansion]{cosine.expansion()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseModel.R
\name{parseModel}
\alias{parseModel}
\title{Parse Rdistance model}
\usage{
parseModel(
  data,
  formula = NULL,
  likelihood = "halfnorm",
  w.lo = 0,
  w.hi = NULL,
  expansions = 0,
  series = "cosine",
  x.scl = w.lo,
  g.x.scl = 1,
  outputUnits = NULL,
  asymptoticSE = TRUE
)
}
\arguments{
\item{data}{An \code{RdistDf} data frame. \code{RdistDf} data frames 
contain one line per transect and a list-based column. The list-based
column contains a data frame with detection information. 
The detection information data frame on each row contains (at least) distances 
and group sizes of all targets detected on the transect.
Function \code{\link{RdistDf}} creates \code{RdistDf} data frames 
from separate transect and detection data frames. 
\code{\link{is.RdistDf}} checks whether data frames
are \code{RdistDf}'s.}

\item{formula}{A standard formula object.  For example, \code{dist ~ 1}, 
\code{dist ~ covar1 + covar2}). The left-hand side (before \code{~})
is the name of the vector containing off-transect or radial detection distances.  
The right-hand side contains the names of covariate 
vectors to fit in the detection
function, and potentially group sizes. Group sizes are specified 
by including \code{+ groupsize(<variable>)} in the RHS 
(see 'Group Sizes' section).
Covariates can be either detection level 
or transect level and can appear in  \code{data} or exist in the 
global working environment. Regular R scoping 
rules apply.}

\item{likelihood}{String specifying the likelihood to fit. Built-in 
likelihoods at present are "halfnorm", "hazrate", and "negexp".}

\item{w.lo}{Lower or left-truncation limit of the distances in distance data. 
This is the minimum possible off-transect distance. Default is 0.  If 
\code{w.lo} is greater than 0, it must have measurement units. 
See \code{help(unitHelpers)} for assistance assigning units.}

\item{w.hi}{Upper or right-truncation limit of the distances 
in \code{dist}. This is the maximum off-transect distance that 
could be observed. If unspecified (i.e., NULL), 
right-truncation is set to the maximum of the observed 
distances.  If \code{w.hi} is specified, it must have  
measurement units.  
See \code{help(unitHelpers)} for assistance assigning units.}

\item{expansions}{A scalar specifying the number of terms 
in \code{series} to compute. Depending on the series, 
this could be 0 through 5.  The default of 0 equates 
to no expansion terms of any type.  No expansion terms 
are allowed (i.e., \code{expansions} is forced to 0) if 
covariates are present in the detection function 
(i.e., right-hand side of \code{formula} includes
something other than \code{1}).}

\item{series}{If \code{expansions} > 0, this string 
specifies the type of expansion to use. Valid values at 
present are 'simple', 'hermite', and 'cosine'.}

\item{x.scl}{The x coordinate (a distance) at which the 
detection function will be scaled.  \code{g.x.scl} can be a distance
or the string "max".  
When \code{x.scl} is specified (i.e., not 0 or "max"), it must have measurement 
units assigned. 
See \code{help(unitHelpers)} for assistance assigning units.}

\item{g.x.scl}{Height of the distance function at coordinate \code{x}. 
The distance function 
will be scaled so that g(\code{x.scl}) = \code{g.x.scl}. 
If \code{g.x.scl} is not 
a data frame, it must be a numeric value (vector of length 1) 
between 0 and 1.}

\item{outputUnits}{A string specifying the symbolic measurement 
units for results. Valid units are listed in \code{units::valid_udunits()}.
The strings for common distance symbolic units are: 
"m" - meters, "ft" - feet, "cm" - centimeters, "mm" - 
millimeters, "mi" - miles, "nmile" - 
nautical miles ("nm" is nano meters), "in" - inches, 
"yd" - yards, "km" - kilometers, "fathom" - fathoms, 
"chains" - chains, and "furlong" - furlongs.  
If \code{outputUnits} is unspecified (NULL),
output units will be the same as those on 
distances in \code{data}.}

\item{asymptoticSE}{Logical variable for whether to calculate 
asymptotic standard errors. The default (TRUE) estimates an
asymptotic variance-covariance matrix for parameters based on the 
likelihood's Hessian (2nd derivative). If maximization 
has been performed by Nlminb or HookesJeeves, the asymptotic 
Hessian is estimated using numeric second deriviatives 
of the likelihood at the maximum likelihood solution. If 
maximization was performed by Optim, the last Hessian of 
the maximization is returned 
by Optim and used
(see \code{\link{varcovarEstim}} and \code{\link{secondDeriv}}). 
Asymptotic standard errors will not be estimated if 
\code{asymptoticSE = FALSE}. If not estimated, 
bootstrap iterations will run faster because the numeric Hessian, 
which is discarded during bootstrapping,
will not be calculated every iteration.}
}
\value{
An Rdistance model frame, which is an object of class 
"dfunc". 
Rdistance model frames are lists containing distance 
model components but not estimates.  Model frames contain 
everything necessary to fit an Rdistance mode, such as covariates, 
minimum and maximum distances, the form of the likelihood, 
number of expansions, etc.  Rdistance model frames contain a 
subset of fitted Rdistance model components.
}
\description{
Parse an 'Rdistance' formula and produce a list containing all model 
parameters.
This routine is not normally called directly by the user, but 
it might be helpful in simulations.  It is called 
internally from the model estimation routines.
}
\examples{

data(sparrowDf)
   
ml <- Rdistance::parseModel(sparrowDf
   , formula = dist ~ 1 + observer + groupsize(groupsize)
   , likelihood = "halfnorm"
   , w.lo = 0
   , w.hi = NULL
   , series = "cosine"
   , x.scl = 0
   , g.x.scl = 1
   , outputUnits = "m"
   )
class(ml)  # 'dfunc', but no estimated coefficients
print(ml)
print.default(ml)

}
\seealso{
\code{\link{RdistDf}}, which returns an 
Rdistance \emph{data} frame;
\code{\link{dfuncEstim}}, which returns an 
Rdistance \emph{fitted} model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateNumeric.R
\name{integrateNumeric}
\alias{integrateNumeric}
\title{Numeric Integration}
\usage{
integrateNumeric(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL,
  expansions = NULL,
  series = NULL,
  isPoints = NULL,
  likelihood = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced
by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}), or a matrix of canonical
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}).
If a matrix, each row corresponds to a
distance function and each column is a parameter. The first column is
the parameter related to sighting covariates and must be transformed
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must
be applied outside this routine).  If \code{object} is a matrix,
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for
the distance function covariates. If NULL and
\code{object} is a fitted distance function, the
observed covariates stored in
\code{object} are used (behavior similar to \code{\link[=predict.lm]{predict.lm()}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if
\code{object} is a matrix. Sighting distance units can differ from units
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}

\item{expansions}{A scalar specifying the number of terms
in \code{series} to compute. Depending on the series,
this could be 0 through 5.  The default of 0 equates
to no expansion terms of any type.  No expansion terms
are allowed (i.e., \code{expansions} is forced to 0) if
covariates are present in the detection function
(i.e., right-hand side of \code{formula} includes
something other than \code{1}).}

\item{series}{If \code{expansions} > 0, this string
specifies the type of expansion to use. Valid values at
present are 'simple', 'hermite', and 'cosine'.}

\item{isPoints}{Boolean. TRUE if integration is for point surveys.
FALSE for line-transect surveys. Line-transect surveys integrate
under the distance function, g(x), while point surveys integrate under
the distance function times distances, xg(x).}

\item{likelihood}{String specifying the likelihood to fit. Built-in
likelihoods at present are "halfnorm", "hazrate", and "negexp".}
}
\value{
A vector of areas under the distance functions represented in
\code{object}.
If \code{object} is a distance function and
\code{newdata} is specified, the returned vector's length is
\code{nrow(newdata)}.  If \code{object} is a distance function and
\code{newdata} is NULL,
returned vector's length is \code{length(distances(object))}. If
\code{object} is a matrix, return's length is
\code{nrow(object)}.
}
\description{
Numerically integrate under a distance function.
}
\section{Numeric Integration}{

Rdistance uses Simpson's composite 1/3 rule to numerically
integrate distance functions from 0 to the maximum sighting distance
(\code{w.hi - w.lo}). The number of points evaluated
during numerical integration is controlled by
\code{options(Rdistance_intEvalPts)} (default 101).
Option \code{Rdistance_intEvalPts} must be odd because Simpson's rule
requires an even number of intervals.
Lower values of \code{Rdistance_intEvalPts} increase calculation speeds;
but, decrease accuracy.
\code{Rdistance_intEvalPts} must be >= 5.  A warning is thrown if
\code{Rdistance_intEvalPts} < 29. Empirical tests by the author
suggest \code{Rdistance_intEvalPts} values >= 30 are accurate
to several decimal points for smooth distance functions
(e.g., hazrate, halfnorm, negexp)
and that all \code{Rdistance_intEvalPts} >= 101 produce
identical results if the distance function is not smooth.

\emph{Details}: Let \code{n} = \code{options(Rdistance_intEvalPts)}.
Evaluate the distance function at \code{n} equal-spaced
locations \{f(x0), f(x1), ..., f(xn)\} between 0 and (w.hi - w.lo).
Simpson's composite approximation to the area under the curve is
\deqn{\frac{1}{3}h(f(x_0) + 4f(x_1) + 2f(x_2) + 
     4f(x_3) + 2f(x_4) + ... + 2f(x_{n-2}) + 
     4f(x_{n-1}) + f(x_{n}))}{(1/3)h(f(x0) + 4f(x1) + 2f(x2) + 
     4f(x3) + 2f(x4) + ... + 2f(x(n-2)) + 4f(x(n-1)) + f(xn))}
where \eqn{h} is the interval size (w.hi - w.lo) / n.

Physical units on the return values
are the original (linear) units if \code{object} contains line-transect data
(e.g., [m]), or square of the original units if \code{object} contains
point-transect data (e.g., [m^2]). Point-transect units are squared because
the likelihood is the product of the detection function (which is unitless)
and distances (which have units).
}

\section{Note}{

Users will not normally call this function. It is called
internally by \code{\link[=nLL]{nLL()}} and \code{\link[=effectiveDistance]{effectiveDistance()}}.
}

\examples{

# A halfnorm distance function 
fit <- dfuncEstim(sparrowDf, dist~1, likelihood = "halfnorm")

exact <- integrateHalfnormLines(fit)[1,] # exact area
apprx <- integrateNumeric(fit)[1]  # Numeric approx
pd <- options(digits = 20)
cbind(exact, apprx)
absDiff <- abs(apprx - exact) 
absDiff
options(pd)

# halfnorm approx good to this number of digits
round(log10(absDiff),1)  

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oneStep.like.R
\name{oneStep.like}
\alias{oneStep.like}
\title{Mixture of two uniforms likelihood}
\usage{
oneStep.like(a, dist, covars, w.hi = NULL)
}
\arguments{
\item{a}{A vector or matrix of covariate
and expansion term
coefficients. If matrix, dimension is
\emph{k} X \emph{p}, where
\emph{k} = \code{nrow(a)}) is the number of coefficient
vectors to evaluate (cases) and \emph{p}
= \code{ncol(a)})
is the number of covariate and expansion
coefficients in the likelihood (i.e., rows are
cases and columns are covariates). If \code{a} is a
dimensionless vector, it is interpreted as a
single row with \emph{k} = 1.
Covariate coefficients in \code{a} are the first
\emph{q} values (\emph{q} <= \emph{p}), and must
be on a log scale.}

\item{dist}{A numeric vector of length \emph{n} or
a single-column matrix (dimension \emph{n}X1) containing
detection distances at which to evaluate the likelihood.}

\item{covars}{A numeric vector of length \emph{q} or a
matrix of dimension \emph{n}X\emph{q} containing
covariate values
associated with distances in argument \code{dist}.}

\item{w.hi}{A numeric scalar containing maximum
distance. The right-hand cutoff or upper limit.
Ignored by some likelihoods (such as halfnorm,
negexp, and hazrate), but is a fixed parameter
in other likelihoods (such as oneStep
and uniform).}
}
\value{
A list containing the following two components:
\itemize{
\item \strong{L.unscaled}: A matrix of size
\emph{n}X\emph{k} (\emph{n} = length \code{dist}; \emph{k} = number of
cases = \code{nrow(a)})
containing likelihood values evaluated at
distances in \code{dist}.
Each row is associated with
a single distance, and each column is associated with
a single case (row of \code{a}).  Values in this matrix are
the distance function \eqn{g(d)} which generally have \eqn{g(0) = 1}.
These values are "unscaled" likelihood values; they must be
scaled (divided by) with the area under \eqn{g(x)} between w.lo and w.hi
to form proper likelihood values.

\item \strong{params}: A \emph{n}X\emph{b}X\emph{k}  array
of the likelihood's (canonical) parameters in link space (i.e., on
log scale; \emph{b} = number of canonical parameters in
the likelihood; \emph{k} = number of cases).
Rows correspond to distances in \code{dist}. Columns
correspond to parameters (columns of \code{a}),
and pages correspond to cases (rows of \code{a}).
}
}
\description{
Compute likelihood function for a mixture of two uniform
distributions.
}
\details{
Rdistance's \code{oneStep} likelihood is a mixture of two
non-overlapping uniform distributions. The 'oneStep' density function
is
\deqn{f(d|p, \theta) = \frac{p}{\theta}I(0 \leq d \leq \theta) + 
       \frac{1 - p}{w - \theta}I(\theta \le d \leq w),}{
       f(d|p,T) = (p/T)I(0<=d<=T) + ((1-p)/(w-T))I(T<d<=w),}
where \eqn{I(x)} is the indicator function for event \eqn{x},
and \eqn{w} is the nominal strip width (i.e., \code{w.hi - w.lo} in Rdistance).
The unknown parameters to be estimated
are \eqn{\theta}{T} and \eqn{p}{p}
(\eqn{w} is fixed - given by the user).

Covariates influence values of \eqn{\theta}{T}
via a log link function, i.e., \eqn{\theta = e^{x'b}}{T = exp(x'b)},
where \eqn{x} is the vector of covariate values
associated with distance \eqn{d}, and \eqn{b}
is the vector of estimated coefficients.
}
\examples{

# Fit oneStep to simulated data
whi <- 250
T <- 100  # true threshold
p <- 0.85
n <- 200 
x <- c( runif(n*p, min=0, max=T), runif(n*(1-p), min=T, max=whi))
x <- setUnits(x, "m")
tranID <- sample(rep(1:10, each=n/10), replace=FALSE)
detectDf <- data.frame(transect = tranID, dist = x)
siteDf <- data.frame(transect = 1:10
               , length = rep(setUnits(10,"m"), 10))
distDf <- RdistDf(siteDf, detectDf)

# Estimation
fit <- dfuncEstim(distDf
                 , formula = dist ~ 1
                 , likelihood = "oneStep"
                 , w.hi = setUnits(whi, "m")
                 )
plot(fit)
thetaHat <- exp(coef(fit)[1]) 
pHat <- coef(fit)[2]
c(thetaHat, pHat) # should be close to c(100,0.85)

summary(abundEstim(fit, ci=NULL)) 

}
\references{
Peter F. Craigmile & D.M. Tirrerington (1997) "Parameter estimation for
finite mixtures of uniform distributions",
Communications in Statistics - Theory and Methods, 26:8, 1981-1995,
DOI: 10.1080/03610929708832026

A. Hussein & J. Liu (2009) "Parametric estimation of mixtures of two
uniform distributions", Journal of Statistical Computation and Simulation,
79:4, 395-410, DOI:10.1080/00949650701810406
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}},
\code{\link[=abundEstim]{abundEstim()}},
other \verb{<likelihood>.like} functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thrasherDetectionData.R
\docType{data}
\name{thrasherDetectionData}
\alias{thrasherDetectionData}
\title{Sage Thrasher detection data}
\format{
A data.frame containing 193 rows and 3 columns.  Each row represents
a detected group of thrashers.  Column descriptions:
\enumerate{
\item \code{siteID}: Factor (120 levels), the site or point where the detection
was made.
\item \code{groupsize}: Number, the number of individuals within
the detected group.
\item \code{dist}: Number, the radial distance (m) from
the transect to the detected group.  This is the distance used in analysis.
}
}
\source{
The Sage Thrasher data are a subset of the data collected
by Jason Carlisle and various field technicians for his Ph.D. from the
Department of Ecology, University of Wyoming, in 2017.  This portion of
Jason's work was funded by the Wyoming Game and Fish Department through agreements
with the University of Wyoming's  Cooperative Fish & Wildlife
Research Unit (2012).
}
\description{
Point transect data collected in central Wyoming from 120 points
surveyed for Sage Thrashers by the Wyoming Cooperative Fish & Wildlife Research Unit in 2013.
}
\references{
Carlisle, J.D. 2017. The effect of sage-grouse conservation on
wildlife species of concern: implications for the umbrella species concept.
Dissertation. University of Wyoming, Laramie, Wyoming, USA.

Carlisle, J. D., A. D. Chalfoun, K. T. Smith, and J. L. Beck. 2018.
Nontarget effects on songbirds from habitat manipulation for Greater
Sage-Grouse: Implications for the umbrella species concept.
\emph{The Condor: Ornithological Applications} 120:439–455.
\doi{10.1650/CONDOR-17-200.1}
}
\seealso{
\code{\link[=thrasherSiteData]{thrasherSiteData()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparrowDetectionData.R
\docType{data}
\name{sparrowDetectionData}
\alias{sparrowDetectionData}
\title{Brewer's Sparrow detection data}
\format{
A data.frame containing 356 rows and 5 columns.  Each row represents
a detected group of sparrows.  Column descriptions: 
\enumerate{ 
  \item \code{siteID}: Factor (72 levels), the site or transect where the detection
  was made.  
  \item \code{groupsize}: Number, the number of individuals within
  the detected group.  
  \item \code{sightdist}: Number, distance (m) from the observer to the detected group.  
  \item \code{sightangle}: Number, the angle (degrees) from the transect 
  line to the detected group.  
  \item \code{dist}: Number, the perpendicular, off-transect distance (m) from the
  transect to the detected group.  This is the distance used in analysis.
  Calculated using \code{\link{perpDists}}.  }
}
\source{
The Brewer's sparrow data are a subset of the data collected 
by Jason Carlisle and various field technicians for his Ph.D. from the 
Department of Ecology, University of Wyoming, in 2017.  This portion of 
Jason's work was funded by the Wyoming Game and Fish Department through agreements 
with the University of Wyoming's  Cooperative Fish & Wildlife
Research Unit (2012).
}
\description{
Detection data from line transect surveys for 
Brewer's sparrow on 72 transects 
located on a 4105 km^2 study 
area in central Wyoming. Data were collected by 
Dr. Jason Carlisle of the Wyoming Cooperative Fish & Wildlife
Research Unit in 2012. Each transect was 500 meters long.
}
\references{
Carlisle, J.D. 2017. The effect of sage-grouse conservation on
wildlife species of concern: implications for the umbrella species concept.
Dissertation. University of Wyoming, Laramie, Wyoming, USA.

Carlisle, J. D., and A. D. Chalfoun. 2020. The abundance of Greater 
Sage-Grouse as a proxy for the abundance of sagebrush-associated songbirds 
in Wyoming, USA. \emph{Avian Conservation and Ecology} 15(2):16. 
\doi{10.5751/ACE-01702-150216}
}
\seealso{
\code{\link{sparrowSiteData}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predDfuncs.R
\name{predDfuncs}
\alias{predDfuncs}
\title{Predict distance functions}
\usage{
predDfuncs(object, params, distances, isSmooth)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link{dfuncEstim}}.}

\item{params}{A matrix of distance function parameters. 
Rows are observations, columns contain the set of parameters
(canonical and expansion) for each observation.}

\item{distances}{A vector or 1-column matrix of 
distances at which to evaluate 
distance functions, when distance functions 
are requested.  \code{distances} must have measurement units. 
Any distances outside the observation 
strip (\code{object$w.lo} to \code{object$w.hi}) are discarded.  If 
\code{distances} is NULL, a sequence 
of \code{getOption("Rdistance_intEvalPts")} (default 101) evenly 
spaced distances between 
\code{object$w.lo} and \code{object$w.hi} (inclusive) is used.}

\item{isSmooth}{Logical.  TRUE if the distance function 
is smoothed (and hence has no parameters).}
}
\value{
A matrix of distance function values, of size 
length(distances) X nrow(params).  Each row of params
is associated with a column, i.e., a different distance 
function.  Distances are associated with rows, 
i.e., use matplot(d,return) to plot values on separate distance 
functions specified by rows of params.
}
\description{
An internal prediction function to predict a distance 
function.  This version allows for matrix inputs and 
uses matrix operations, and is thus faster than earlier
looping versions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maximize.g.r
\name{maximize.g}
\alias{maximize.g}
\title{Find coordinate of function maximum}
\usage{
maximize.g(fit, covars = NULL)
}
\arguments{
\item{fit}{An estimated 'dfunc' object produced by \code{dfuncEstim}.}

\item{covars}{Covariate values to calculate g(x).}
}
\value{
The value of x that maximizes g(x) in \code{fit}.
}
\description{
Find the x coordinate that maximizes g(x).
}
\examples{
\dontrun{
# Fake data
set.seed(22223333)
x <- rgamma(100, 10, 1)

fit <- dfuncEstim( x, likelihood="Gamma", x.scl="max" )

maximize.g( fit )  # should be near 10.
fit$x.scl            # same thing
}

}
\seealso{
\code{\link{dfuncEstim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcCI.R
\name{bcCI}
\alias{bcCI}
\title{Bias corrected bootstraps}
\usage{
bcCI(x.bs, x, ci = 0.95)
}
\arguments{
\item{x.bs}{A vector of bootstrap estimates of some quantity.}

\item{x}{A scalar of the original estimate of the quantity.}

\item{ci}{A scalar of the desired confidence interval 
coverage.}
}
\value{
A named vector containing the lower and upper endpoints of
the bias-corrected bootstrap confidence interval.
}
\description{
Calculate bias-corrected confidence intervals for bootstrap data
using methods in Manly textbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/errDataUnk.R
\name{errDataUnk}
\alias{errDataUnk}
\title{Unknown error message}
\usage{
errDataUnk(txt, attri)
}
\arguments{
\item{txt}{Text.  The "unknown" we are looking for.}

\item{attri}{Attribute description we are looking for.}
}
\value{
A descriptive string, suitable for warning or error.
}
\description{
Constructs a string stating what is "unknown" that is 
suitable for use in warning and error functions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ESW.r
\name{ESW}
\alias{ESW}
\title{Effective Strip Width (ESW) for line transects}
\usage{
ESW(object, newdata = NULL)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{newdata}{A data frame containing new values for
covariates at which either
ESW's or EDR's will be computed. If NULL and
\code{object} contains covariates, the
covariates stored in
\code{object} are used (like \code{\link[=predict.lm]{predict.lm()}}).
If not NULL, covariate values in \code{newdata}
are used.
See \strong{Value} section for more information.}
}
\value{
If \code{newdata} is present, the returned value is
a vector of effective sampling distances associated with
covariate values in \code{newdata}. Length of return
in this case is the number of rows in \code{newdata}.
If \code{newdata} is NULL, the returned value is a vector
of effective sampling distances associated with covariate
values in \code{object}. Length of return in this case
is the number of detected groups.  The returned vector
has measurement units, i.e., \code{object$outputUnits}.
}
\description{
Returns effective strip width (ESW) for
line-transect detection functions.
See \code{\link[=EDR]{EDR()}} is for point transects.
}
\details{
ESW is area under
a scaled distance function between its
left-truncation limit (\code{obj$w.lo}) and its right-truncation
limit (\code{obj$w.hi}). \if{latex}{I.e., 
    \deqn{ESW = \int_{w.lo}^{w.hi} g(x)dx,} 
  where \eqn{g(x)} is the distance
  function scaled so that \eqn{g(x.scl) = g.x.scl}
  and \eqn{w.lo} and \eqn{w.hi} are the lower
  and upper truncation limits.  }

If detection does not decline with distance,
the detection function is flat (horizontal), and
area under the detection
function is  \eqn{g(0)(w.hi - w.lo)}.
If, in this case, \eqn{g(0) = 1}, effective sampling distance is
the half-width of the surveys, \eqn{(w.hi - w.lo)}
}
\examples{
data(sparrowDfuncObserver)

ESW(sparrowDfuncObserver) # vector length 356 = number of groups
ESW(sparrowDfuncObserver, newdata = data.frame(observer = 
   c("obs2", "obs4"))) # vector length 2

}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}, \code{\link[=EDR]{EDR()}},
\code{\link[=effectiveDistance]{effectiveDistance()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitHelper_%#%.R, R/unitHelper_%acre%.R,
%   R/unitHelper_%cm%.R, R/unitHelper_%ft%.R, R/unitHelper_%ha%.R,
%   R/unitHelper_%inches%.R, R/unitHelper_%km%.R, R/unitHelper_%km^2%.R,
%   R/unitHelper_%m%.R, R/unitHelper_%m^2%.R, R/unitHelper_%mi%.R,
%   R/unitHelper_%mi^2%.R, R/unitHelper_%yd%.R, R/unitHelper_dropUnits.R,
%   R/unitHelper_setUnits.R
\name{\%#\%}
\alias{\%#\%}
\alias{unitHelpers}
\alias{\%acre\%}
\alias{\%cm\%}
\alias{\%ft\%}
\alias{\%ha\%}
\alias{\%inches\%}
\alias{\%km\%}
\alias{\%km^2\%}
\alias{\%m\%}
\alias{\%m^2\%}
\alias{\%mi\%}
\alias{\%mi^2\%}
\alias{\%yd\%}
\alias{dropUnits}
\alias{setUnits}
\title{Unit assignment helpers}
\usage{
x \%#\% u

x \%acre\% .

x \%cm\% .

x \%ft\% .

x \%ha\% .

x \%inches\% .

x \%km\% .

x \%km^2\% .

x \%m\% .

x \%m^2\% .

x \%mi\% .

x \%mi^2\% .

x \%yd\% .

dropUnits(x)

setUnits(x, u)
}
\arguments{
\item{x}{A numeric vector or matrix.}

\item{u}{A string representing physical measurement units to
assign to \code{x}, e.g., "m", "km", "m^2".  Valid units are
listed in columns "(symbol|name)" of \code{\link[units:valid_udunits]{units::valid_udunits()}}.}

\item{.}{Placeholder for the fixed unit assignment operators. Ignored.
See Details.}
}
\value{
For \verb{\%#\%} and \code{setUnits}, argument \code{x} with
units \code{u} attached.

For all the fixed unit assignment operators (i.e., \%units\%),
argument x with the respective units assigned.

For \code{dropUnits}, argument \code{x} with no units. If
input \code{x} has no units, \code{x} is returned unchanged.
}
\description{
Helper functions for assigning physical measurement units in Rdistance.
All are convenience wrappers for \verb{units::}\code{\link[units:units]{units::set_units()}}.
}
\details{
The fixed unit assignment operators are designed to behave like
unary operators (i.e., 1 argument);
but, R does not allow
user defined unary operators.
Technically, these fixed unit assignment operators are instances of
R's user-defined infix
operator, and as such they require two arguments.
Their syntax must be
\verb{x \%<units>\% <something>}; but, the second argument is ignored
and '.' is suggested. See Examples.
}
\examples{

2 \%#\% "m" 
setUnits(2,"km")
x <- 2 \%#\% "km^2"
10 \%#\% units(x)
2 \%#\% "km^2" \%#\% "acres" # Convert km^2 to acres
x \%#\% "acres"            # Same
x \%#\% NULL    # Drop units
dropUnits(x)  # Same

# \%#\%'s precedence is below "^" but above "+" and "*"
# The following fails: 
# 2 \%#\% "m" / (2 \%#\% "ha") \%#\% "in/acre"
# The following succeeds:
(2 \%#\% "m" / (2 \%#\% "ha")) \%#\% "in/acre"
1 \%m\%. ^ 2      # [m]
(1 \%m\%.) ^ 2    # [m^2]


# For fixed unit assignment, 2nd argument does not matter
# All of the following are equivalent
2 \%m\%.
2 \%m\% x
2 \%m\% 3
2 \%m\% NULL
2 \%m\% NA

# Conversion:
x <- 10 \%#\% "ft"
x \%m\%.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bspline.expansion.R
\name{bspline.expansion}
\alias{bspline.expansion}
\title{B-spline expansion terms}
\usage{
bspline.expansion(x, expansions)
}
\arguments{
\item{x}{A numeric matrix of distances at which to evaluate
the expansion series. For distance analysis, \code{x} should
be the proportion of the maximum sighting distance
at which a group was sighted, i.e., \eqn{x = d/w}, where \eqn{d}
is sighting distance and \eqn{w} is maximum sighting distance.}

\item{expansions}{A scalar specifying the number of
expansion terms to compute. Must be one of the
integers 1, 2, 3, 4, or 5.}
}
\value{
A 3D array of size \code{nrow(x)} X \code{ncol(x)} X \code{expansions}.
The 'pages' (3rd dimension) of this array are the cosine expansions of
\code{x}. i.e., page 1 is the first expansion term of \code{x},
page 2 is the second expansion term of \code{x}, etc.
}
\description{
Computes spline, specifically b-spline, expansion terms that modify the
shape of distance likelihood functions.
}
\examples{
x <- matrix(seq(0, 1, length = 200), ncol = 1)
spl.expn <- bspline.expansion(x, 5)
plot(range(x), range(spl.expn), type="n")
matlines(x, spl.expn[,1,1:5], col=rainbow(5), lty = 1)

}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}},
\code{\link[=cosine.expansion]{cosine.expansion()}},
\code{\link[=hermite.expansion]{hermite.expansion()}},
\code{\link[=simple.expansion]{simple.expansion()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thrasherDf.R
\docType{data}
\name{thrasherDf}
\alias{thrasherDf}
\title{Sage Thrasher detection data frame in Rdistance >4.0.0 format}
\format{
A rowwise tibble containing 120 rows and 8 columns, one of which 
(i.e., 'detections') contains nested data frames of detections.  
Each row represents
one transect of one point.

A data.frame containing 120 rows and 6 columns.  Each row represents
a surveyed site.  Each surveyed site is considered one transect of one point.  
Column descriptions: 
\enumerate{ 
\item \code{siteID}: Factor (120 levels), the site or point surveyed.  
\item \code{detections}: An embedded (nested) data frame containing 
detections made at that point.  Columns in the embedded data frame contain:
  \enumerate{
    \item \code{groupsize}: The number of individuals in
    the detected group.  
    \item \code{dist}: The radial distance (m) from
    the transect to the detected group.  
  }
\item \code{observer}: Factor (six levels), identity of the observer who surveyed
the point.  
\item \code{bare}: Number, the mean bare ground cover (\%)
within 100 m of each point.  
\item \code{herb}: Number, the mean herbaceous
cover (\%) within 100 m of each point.  
\item \code{shrub}: Number, the mean
shrub cover (\%) within 100 m of each point.  
\item \code{height}: Number,
the mean shrub height (cm) within 100 m of each point.  
\item \code{npoints}: The number of point counts on the transect.
}
}
\source{
The sage thrasher data are a subset of data collected 
by Jason Carlisle and various field technicians for his Ph.D. from the 
Department of Ecology, University of Wyoming, in 2017.  This portion of 
Jason's work was funded by the Wyoming Game and Fish Department 
through agreements 
with the University of Wyoming's  Cooperative Fish & Wildlife
Research Unit (2012).
}
\description{
Point transect data collected in central Wyoming on 120 points
surveyed for Sage Thrashers by the Wyoming Cooperative Fish & Wildlife Research Unit in 2013.
}
\examples{
data(thrasherDf)

is.RdistDf(thrasherDf)

summary(thrasherDf,
  formula = dist ~ groupsize(groupsize)
)

}
\references{
Carlisle, J.D. 2017. The effect of sage-grouse conservation on
wildlife species of concern: implications for the umbrella species concept.
Dissertation. University of Wyoming, Laramie, Wyoming, USA.

Carlisle, J. D., A. D. Chalfoun, K. T. Smith, and J. L. Beck. 2018. 
Nontarget effects on songbirds from habitat manipulation for Greater 
Sage-Grouse: Implications for the umbrella species concept. 
\emph{The Condor: Ornithological Applications} 120:439–455. 
\doi{10.1650/CONDOR-17-200.1}
}
\seealso{
\code{\link{thrasherSiteData}}, \code{\link{thrasherDetectionData}},
\code{\link{RdistDf}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expansionTerms.R
\name{expansionTerms}
\alias{expansionTerms}
\title{Distance function expansion terms}
\usage{
expansionTerms(a, d, series, nexp, w)
}
\arguments{
\item{a}{A vector or matrix of (estimated) coefficients. 
\code{a} has length \eqn{p} + \code{nexp} (if a vector) or dimension
(\eqn{k}, \eqn{p} + \code{nexp}), where \eqn{p} is the number 
of canonical parameters in the likelihood and \eqn{k} is the 
number of 'cases' (coefficient vectors = \code{nrow(a)}) to evaluate. 
The first \eqn{p}
elements of \code{a}, or the first \eqn{p} columns if \code{a}
is a matrix, are ignored. I.e., Expansion 
term coefficients are the last \code{nexp} elements or columns 
of \code{a}.}

\item{d}{A vector or 1-column matrix of 
distances at which to evaluate 
the expansion terms.  \code{d} should be distances 
above w.lo, i.e., distances - w.lo. 
Parameters \code{d} and \code{w}
must have compatible measurement units.}

\item{series}{If \code{expansions} > 0, this string 
specifies the type of expansion to use. Valid values at 
present are 'simple', 'hermite', and 'cosine'.}

\item{nexp}{Number of expansion terms.  Integer from 0 to 5.}

\item{w}{A vector specifying strip width for every 'case' 
in \code{a}.  Vector must have length \code{length(a)} or \code{nrow(a)}. 
In general, this is constant vector containing the range 
of sighting distances, i.e., \code{rep(w.hi - w.low, nrow(a))}. 
But, for some likelihoods (e.g., 'oneStep') this vector allows the user to 
restrict application of the expansion terms to less than the full range 
of distances. For the 'oneStep' likelihood, expansion terms are only 
applied between 0 and \eqn{\Theta}{T}, the boundary of the two uniforms, 
which varies by 'case' when covariates are present.  
Parameters \code{d} and \code{w} must 
have compatible measurement units.}
}
\value{
If \code{nexp} equals 0, 1 is returned. 
If \code{nexp} is greater than 0, a matrix of 
size \eqn{n}X\eqn{k} containing expansion terms, 
where \eqn{n} = \code{length(d)} and \eqn{k} = \code{nrow(a)}. The 
expansion series associated with row \eqn{j} of \code{a} 
are in column \eqn{j} of the return. i.e., 
element (\eqn{i},\eqn{j}) of the return is 
  \deqn{1 + \sum_{k=1}^{m} a_{jk} h_{k}(x_{i}/w).}{(1 + a(j1) h_1(x(i)/w) + ... + a(jm) h_m(x(i)/w)).}
}
\description{
Compute "expansion" terms that modify the shape of 
a base distance function.
}
\details{
Expansion terms modify the base likelihood function
and are used to incorporate "wiggle".  The modified distance function is,
\code{key * expTerms} where \code{key} is a vector of values in
the base likelihood function (e.g., \code{halfnorm.like()$L.unscaled}) 
and \code{expTerms} is the 
matrix returned by this routine. In equation form, 
  \deqn{f(x_i|\beta,a_1,a_2,\dots,a_m) = 
   f(x_i|\beta)(1 + \sum_{k=1}^{m} a_k h_{k}(x_i/w)).}{f(x(i)|beta,a_1,a_2,...,a_k) = 
   f(x(i)|beta)(1 + a(1) h_1(x(i)/w) + a(2) h_2(x(i)/w) + ... + a(m) h_m(x(i)/w)). },
where \eqn{m} = the the number of expansions (\code{nexp}), \eqn{h_{j}(x)}{h_j(x)} 
are expansion terms for distance \eqn{x}, and  
\eqn{a_1, a_2, \dots, a_m}{a(1), a(2), ..., a(m)} are the (estimated) 
expansion term coefficients.
}
\examples{
a1 <- c(log(40), 0.5, -.5)
a2 <- c(log(40), 0.25, -.5)
dists <- seq(0, 100, length = 100) \%m\%.
w = 100 \%m\%.

expTerms1 <- expansionTerms(a1, dists, "cosine", 2, w)
expTerms2 <- expansionTerms(a2, dists, "cosine", 2, w)
plot(dists, expTerms2, ylim = c(0,2.5))
points(dists, expTerms1, pch = 16)

# Same as above
a <- rbind(a1, a2)
w <- rep(w, nrow(a))
expTerms <- expansionTerms(a, dists, "cosine", 2, w)
matlines(dists, expTerms, lwd=2, col=c("red", "blue"), lty=1)

# Showing key and expansions
key <- halfnorm.like(log(40), dists, matrix(1,length(dists),1))$L.unscaled
plot(dists, key, type = "l", col = "blue", ylim=c(0,1.5))
lines(dists, key * expTerms1, col = "red")
lines(dists, key * expTerms2, col = "purple")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimateN.r
\name{estimateN}
\alias{estimateN}
\title{Abundance point estimates}
\usage{
estimateN(object, area = NULL, propUnitSurveyed = 1)
}
\arguments{
\item{object}{An Rdistance model frame or fitted distance function,
normally produced by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}.}

\item{area}{A scalar containing the total area of inference. Usually, this is
study area size.  If \code{area} is NULL (the default),
\code{area} will be set to 1 square unit of the output units and density estimates
will be produced.
If \code{area} is not NULL, it must have measurement units
assigned by the \code{units} package.
The units on \code{area} must be convertible
to squared output units. Units
on \code{area} must be two-dimensional.
For example, if output units are "foo",
units on area must be convertible to "foo^2" by the \code{units}
package. Units of "km^2", "cm^2", "ha", "m^2", "acre", "mi^2", and several
others are acceptable.}

\item{propUnitSurveyed}{A scalar or vector of real numbers between 0 and 1.
The proportion of the default sampling unit that
was surveyed.  If both sides of line transects were observed,
\code{propUnitSurveyed}
= 1.  If only a single side of line transects were observed, set
\code{propUnitSurveyed} = 0.5. For point transects, this should be set to
the proportion of each circle that was observed. Length must either be
1 or the total number of transects in \code{x}.}
}
\value{
A list containing the following components:

\item{density}{Estimated density in the surveyed area.}

\item{abundance}{Estimated abundance on the study area. Equals density if
area is not specified.}

\item{n.groups}{The number of detected groups (not individuals, unless all group sizes = 1).}

\item{n.seen}{The number of individuals (sum of group sizes).}

\item{area}{Total area of inference. Study area size}

\item{surveyedUnits}{Number of surveyed sites.  This is total transect length
for line-transects or number of points for point-transects. This total transect
length does not include transects with missing lengths.}

\item{propUnitSurveyed}{Proportion of the standard survey unit
that was observed}

\item{avg.group.size}{Average group size on non-NA transects}

\item{w}{Strip width. }

\item{pDetection}{Probability of detection.}

For line-transects that do not involve covariates, object$density
is object$n.seen / (2 * propUnitSurveyed * object$w * object$pDetection * object$surveyedUnits)
}
\description{
Estimate abundance from an Rdistance fitted model.
This function is called internally by \code{abundEstim}.  Most users will call
\code{abundEstim} to estimate abundance unless they are running simulations or
bootstrapping.
}
\details{
The abundance estimate for line-transect surveys (if no covariates
are included in the detection function and both sides of the transect
are observed) is
\deqn{N =\frac{n(A)}{2(ESW)(L)}}{%
         N = n*A / (2*ESW*L)}
where \emph{n} is total number of sighted individuals
(i.e., \code{sum(groupSizes(dfunc))}), \emph{L} is the total length of
surveyed transect (i.e., \code{sum(effort(dfunc))}),
and \emph{ESW} is effective strip width
computed from the estimated distance function (i.e., \code{ESW(dfunc)}).
If only one side of transects were observed, the "2" in the denominator
is not present (or, replaced with a "1").

The abundance estimate for point transect surveys (if no covariates are
included) is
\deqn{N =\frac{n(A)}{\pi(ESR^2)(P)}}{%
         N = n*A / ((3.1415)*ESR^2*(P))}
where \emph{n} is total number of sighted individuals (i.e., \code{sum(groupSizes(dfunc))}),
\emph{P} is the total number of surveyed points (i.e., \code{sum(effort(dfunc))}),
and \emph{ESR} is effective search radius
computed from the estimated distance function (i.e., \code{ESR(dfunc)}).

This routine, \code{abundEstim}, estimates abundance on the
entire study area.  Site-specific density estimates
are computed by
\code{predict(x, type = "density")}, which returns a
tibble containing density and abundance on the area surveyed by every
transect.
}
\seealso{
\code{\link[=dfuncEstim]{dfuncEstim()}}, \code{\link[=abundEstim]{abundEstim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrateHazrateLines.R
\name{integrateHazrateLines}
\alias{integrateHazrateLines}
\title{Integrate Hazard-rate line survey distance functions}
\usage{
integrateHazrateLines(
  object,
  newdata = NULL,
  w.lo = NULL,
  w.hi = NULL,
  Units = NULL
)
}
\arguments{
\item{object}{Either an Rdistance fitted distance function
(an object that inherits from class "dfunc"; usually produced
by a call to \code{\link[=dfuncEstim]{dfuncEstim()}}), or a matrix of canonical
distance function parameters (e.g., \code{matrix(exp(fit$par),1)}).
If a matrix, each row corresponds to a
distance function and each column is a parameter. The first column is
the parameter related to sighting covariates and must be transformed
to the "real" space (i.e., inverse link, which is \eqn{exp()}, must
be applied outside this routine).  If \code{object} is a matrix,
it should not have measurement units because
only derived quantities (e.g., ESW) have units; Rdistance function
parameters themselves never have units.}

\item{newdata}{A data frame containing new values for
the distance function covariates. If NULL and
\code{object} is a fitted distance function, the
observed covariates stored in
\code{object} are used (behavior similar to \code{\link[=predict.lm]{predict.lm()}}).
Argument \code{newdata} is ignored if \code{object} is a matrix.}

\item{w.lo}{Minimum sighting distance or left-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{w.hi}{Maximum sighting distance or right-truncation value
if \code{object} is a matrix.
Ignored if \code{object}
is a fitted distance function.
Must have physical measurement units.}

\item{Units}{Physical units of sighting distances if
\code{object} is a matrix. Sighting distance units can differ from units
of \code{w.lo} or \code{w.hi}.   Ignored if \code{object}
is a fitted distance function.}
}
\value{
A vector of areas under the distance functions represented in
\code{object}.
If \code{object} is a distance function and
\code{newdata} is specified, the returned vector's length is
\code{nrow(newdata)}.  If \code{object} is a distance function and
\code{newdata} is NULL,
returned vector's length is \code{length(distances(object))}. If
\code{object} is a matrix, return's length is
\code{nrow(object)}.
}
\description{
Compute integral of the hazard-rate distance function for
line-transect surveys.
}
\details{
Returned integrals are
\deqn{\int_0^{w} (1 - e^{-(x/\sigma_i)^{-k}}) dx = w - \frac{\sigma_i}{k} \Gamma(-\frac{1}{k}, {\frac{\sigma_i}{w}}^{k}),}{
Integral( 1 - e^{-(x/\sigma_i)^(-k)} ) = w - (s/k)(Gamma(-1/k, (s/w)^k),}
where \eqn{w = w.hi - w.lo}, \eqn{\sigma_i}{s} and \eqn{k}{k} are estimated
hazard-rate distance
function parameters for the
i-th observed distance, and \eqn{\Gamma()}{Gamma()} is the incomplete gamma
function.
Rdistance uses the
incomplete gamma function implemented in
\code{\link[expint:gammainc]{expint::gammainc()}}, which for
all intents and purposes is exact.
}
\section{Note}{

Users will not normally call this function. It is called
internally by \code{\link[=nLL]{nLL()}} and \code{\link[=effectiveDistance]{effectiveDistance()}}.
}

\examples{

# A pre-estimated hazard rate distance function: sparrowDfuncObserver
fit <- sparrowDfuncObserver
table(ESW(fit))
table(integrateHazrateLines(fit))

# Check: Integral of 1 - exp(-(x/s)^(-k)) from 0 to w.hi-w.lo
w <- dropUnits(fit$w.hi - fit$w.lo)
params <- predict(fit)
sigma <- params[,1]
minusk <- -params[,2]

outArea <- w + sigma * 
           expint::gammainc(1/minusk, (w/sigma)^(minusk)) / minusk
table(outArea)

}
\seealso{
\code{\link[=integrateNumeric]{integrateNumeric()}}; \code{\link[=integrateNegexpLines]{integrateNegexpLines()}};
\code{\link[=integrateOneStepLines]{integrateOneStepLines()}}
}
